Package: ExpHunterSuite
Title: Package For The Comprehensive Analysis Of Transcriptomic Data
Version: 1.17.0
Authors@R: 
    c(person(given = "James",
           family = "Perkins",
           role = c("aut", "cre"),
           email = "jimrperkins@gmail.com",
           comment = c(ORCID = "0000-0003-4108-096X")),
    person(given = "Pedro",
           family = "Seoane Zonjic",
           role = c("aut"),
           email = "seoanezonjic@uma.es",
           comment = c(ORCID = "0000-0002-3020-1415")),
    person(given = "Fernando",
           family = "Moreno Jabato",
           role = c("aut"),
           email = "fmjabato@gmail.com",
           comment = c(ORCID = "0000-0001-7498-1962")),     
    person(given = "José",
           family = "Córdoba Caballero",
           role = c("aut"),
           email = "josecordoba1995@gmail.com",
           comment = c(ORCID = "0000-0002-1821-5742")),      
    person(given = "Elena",
           family = "Rojano Rivera",
           role = c("aut"),
           email = "elenarojano@outlook.com",
           comment = c(ORCID = "0000-0002-2678-710X")),
    person(given = "Rocio",
           family = "Bautista Moreno",
           role = c("aut"),
           email = "rociobm@uma.es",
           comment = c(ORCID = "0000-0003-1685-8119")),
    person(given = "M. Gonzalo",
           family = "Claros",
           role = c("aut"),
           email = "claros@uma.es",
           comment = c(ORCID = "0000-0002-0112-3550")),          
    person(given = "Isabel",
           family = "Gonzalez Gayte",
           role = c("aut"),
           email = "isabel.ggayte@gmail.com"),         
    person(given = "Juan Antonio",
           family = "García Ranea",
           role = c("aut"),
           email = "ranea@uma.es",
           comment = c(ORCID = "0000-0003-0327-1837"))
    )
Description: The ExpHunterSuite implements a comprehensive protocol for the analysis of transcriptional data using established *R* packages and combining their results. It covers all key steps in DEG detection, CEG detection and functional analysis for RNA-seq data. It has been implemented as an R package containing functions that can be run interactively. In addition, it also contains scripts that wrap the functions and can be run directly from the command line.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
biocViews: GeneExpressionWorkflow
Suggests: optparse, PerformanceAnalytics, naivebayes, reshape2,
        org.Hs.eg.db, org.Mm.eg.db, testthat (>= 3.0.0)
VignetteBuilder: knitr
Imports: ReactomePA, limma, edgeR, NOISeq, biomaRt, topGO, diffcoexp,
        DT, ggplot2, stringr, WGCNA, dplyr, AnnotationDbi,
        BiocGenerics, enrichplot, rmarkdown, stats, Biobase, DESeq2,
        ROCR, data.table, knitr, magrittr, SummarizedExperiment,
        miRBaseVersions.db, grDevices, graphics, utils, BiocParallel,
        MKinfer, matrixStats, ggupset, rlang, plyr, tidyr, GO.db,
        Matrix, fastcluster, DOSE, heatmaply, EnhancedVolcano, ggrepel,
        clusterProfiler, GenomicRanges, GenomicFeatures, tximport,
        annotatr, ggridges, FactoInvestigate, FactoMineR
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/ExpHunterSuite
git_branch: devel
git_last_commit: d31b29e
git_last_commit_date: 2025-04-15
Repository: Bioconductor 3.22
Date/Publication: 2025-06-27
NeedsCompilation: no
Packaged: 2025-06-27 17:07:50 UTC; biocbuild
Author: James Perkins [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4108-096X>),
  Pedro Seoane Zonjic [aut] (ORCID:
    <https://orcid.org/0000-0002-3020-1415>),
  Fernando Moreno Jabato [aut] (ORCID:
    <https://orcid.org/0000-0001-7498-1962>),
  José Córdoba Caballero [aut] (ORCID:
    <https://orcid.org/0000-0002-1821-5742>),
  Elena Rojano Rivera [aut] (ORCID:
    <https://orcid.org/0000-0002-2678-710X>),
  Rocio Bautista Moreno [aut] (ORCID:
    <https://orcid.org/0000-0003-1685-8119>),
  M. Gonzalo Claros [aut] (ORCID:
    <https://orcid.org/0000-0002-0112-3550>),
  Isabel Gonzalez Gayte [aut],
  Juan Antonio García Ranea [aut] (ORCID:
    <https://orcid.org/0000-0003-0327-1837>)
Maintainer: James Perkins <jimrperkins@gmail.com>

# Generated by roxygen2: do not edit by hand

export(acc)
export(degsynth)
export(fmeasure)
export(get_org_db)
export(get_organism_table)
export(get_stats)
export(get_stats_from_cm)
export(install_DEgenes_hunter)
export(load_and_parse_gmt)
export(load_synth_dataset)
export(main_clusters_to_enrichment)
export(main_degenes_Hunter)
export(main_functional_hunter)
export(multienricher_gsea)
export(multienricher_ora)
export(multienricher_topGO)
export(parse_pseudocounts)
export(ppv)
export(recall)
export(rtable2measures)
export(spc)
export(target_generation)
export(write_clusters_to_enrichment)
export(write_df_list_as_tables)
export(write_enrich_files)
export(write_expression_report)
export(write_functional_report)
importClassesFrom(topGO,topGOdata)
importFrom(AnnotationDbi,keys)
importFrom(AnnotationDbi,select)
importFrom(Biobase,assayData)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpok)
importFrom(BiocParallel,bptry)
importFrom(DESeq2,DESeq)
importFrom(DESeq2,DESeqDataSetFromMatrix)
importFrom(DESeq2,counts)
importFrom(DESeq2,results)
importFrom(DESeq2,rlog)
importFrom(DOSE,setReadable)
importFrom(DT,datatable)
importFrom(FactoInvestigate,dimRestrict)
importFrom(FactoInvestigate,eigenRef)
importFrom(FactoMineR,HCPC)
importFrom(FactoMineR,PCA)
importFrom(FactoMineR,dimdesc)
importFrom(GO.db,GOBPANCESTOR)
importFrom(GO.db,GOBPPARENTS)
importFrom(GO.db,GOCCANCESTOR)
importFrom(GO.db,GOCCPARENTS)
importFrom(GO.db,GOMFANCESTOR)
importFrom(GO.db,GOMFPARENTS)
importFrom(GO.db,GOOBSOLETE)
importFrom(GO.db,GOTERM)
importFrom(GenomicFeatures,makeTxDbFromGFF)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(MKinfer,boot.t.test)
importFrom(Matrix,forceSymmetric)
importFrom(NOISeq,noiseqbio)
importFrom(NOISeq,readData)
importFrom(NOISeq,tmm)
importFrom(ROCR,performance)
importFrom(ROCR,prediction)
importFrom(ReactomePA,enrichPathway)
importFrom(ReactomePA,gsePathway)
importFrom(SummarizedExperiment,assay)
importFrom(WGCNA,allowWGCNAThreads)
importFrom(WGCNA,binarizeCategoricalVariable)
importFrom(WGCNA,blockwiseModules)
importFrom(WGCNA,corPvalueStudent)
importFrom(WGCNA,labels2colors)
importFrom(WGCNA,moduleEigengenes)
importFrom(WGCNA,orderMEs)
importFrom(WGCNA,pickSoftThreshold)
importFrom(WGCNA,plotDendroAndColors)
importFrom(annotatr,annotate_regions)
importFrom(annotatr,build_annotations)
importFrom(clusterProfiler,enrichGO)
importFrom(clusterProfiler,enrichKEGG)
importFrom(clusterProfiler,gseGO)
importFrom(clusterProfiler,gseKEGG)
importFrom(clusterProfiler,merge_result)
importFrom(clusterProfiler,simplify)
importFrom(data.table,as.data.table)
importFrom(data.table,merge.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,setnames)
importFrom(diffcoexp,diffcoexp)
importFrom(dplyr,arrange)
importFrom(dplyr,between)
importFrom(dplyr,desc)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,row_number)
importFrom(edgeR,DGEList)
importFrom(edgeR,calcNormFactors)
importFrom(edgeR,cpm)
importFrom(edgeR,estimateDisp)
importFrom(edgeR,exactTest)
importFrom(edgeR,glmQLFTest)
importFrom(edgeR,glmQLFit)
importFrom(edgeR,topTags)
importFrom(enrichplot,cnetplot)
importFrom(enrichplot,dotplot)
importFrom(enrichplot,emapplot)
importFrom(enrichplot,pairwise_termsim)
importFrom(fastcluster,hclust)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,fortify)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_count)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guides)
importFrom(ggplot2,scale_colour_gradientn)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_light)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylim)
importFrom(ggrepel,geom_text_repel)
importFrom(ggridges,geom_density_ridges)
importFrom(grDevices,dev.control)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,recordPlot)
importFrom(graphics,abline)
importFrom(graphics,par)
importFrom(graphics,text)
importFrom(heatmaply,heatmaply)
importFrom(knitr,knit)
importFrom(knitr,knit_expand)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,topTable)
importFrom(limma,voom)
importFrom(magrittr,"%>%")
importFrom(matrixStats,rowDiffs)
importFrom(matrixStats,rowRanges)
importFrom(matrixStats,rowVars)
importFrom(miRBaseVersions.db,miRBaseVersions.db)
importFrom(plyr,rbind.fill)
importFrom(rlang,.data)
importFrom(rmarkdown,render)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,ecdf)
importFrom(stats,formula)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,pchisq)
importFrom(stats,phyper)
importFrom(stats,pt)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rlnorm)
importFrom(stats,rnbinom)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stringr,str_remove)
importFrom(tidyr,unite)
importFrom(topGO,annFUN)
importFrom(tximport,tximport)
importFrom(utils,assignInNamespace)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(utils,write.table)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ExpHunterSuite

<!-- badges: start -->

[![R-CMD-check](https://github.com/seoanezonjic/ExpHunterSuite/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/seoanezonjic/ExpHunterSuite/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

## DOCUMENTATION - VIGNETTES

[Link to ExpHunterSuite Vignettes](https://bioconductor.org/packages/3.18/workflows/vignettes/ExpHunterSuite/inst/doc/hunter.html)

## DOCUMENTATION - README

# DEgenes Hunter

> Protocol to undertake the key steps in the analysis of transcriptomics
> experiments, including differentially expressed gene detection, a
> co-expression analysis and the functional enrichment.

## REQUIREMENTS

  - R version 4

## INSTALLATION

### Install latest R-Version

Go to page <https://cran.r-project.org> and install the latest R version
on your computer.

### Install DEgenes Hunter from console (using devtools)

To install DEgenes Hunter from console, we use the devtools utility to
install R packages from GitHub. Please type the following
commands:

``` bash
    Rscript -e 'install.packages("devtools", repos="http://cran.us.r-project.org")'
    Rscript -e 'devtools::install_github("seoanezonjic/ExpHunterSuite", dependencies=TRUE)'
```

Sometimes reactome.db download fails because this package is too large
(\>400 Mb) and R has a download timeout of 60 seconds by default.

``` bash
    Rscript -e "getOption('timeout')"
        [1] 60
```

This error can be solved by installing reactome.db from source or
extending the timeout threshold before installing the ExpHunterSuite
package with the
command:

``` bash
    Rscript -e 'options(timeout=1000); devtools::install_github("seoanezonjic/ExpHunterSuite")'
```

Then, please create the folder where you want to install the command
line scripts, copy the scripts there and make them command line
accesible using these commands:

``` bash
    mkdir install_folder
    Rscript -e "ExpHunterSuite::install_DEgenes_hunter('install_folder')"
    export PATH=path_to_install_folder:$PATH
```

This export PATH can also be added to the .bashrc or .bash\_profile
files.

## REFERENCES AND CITATION

Please cite ExpHunterSuite as:



Jabato, F., Córdoba-Caballero, J., Rojano, E., Romá-Mateo, C., Sanz, P., Pérez, B., Gallego, D., Seoane, P., Ranea, J., and Perkins, J. 2021. Gene expression analysis method integration and co-expression module detection applied to rare glucide metabolism disorders using ExpHunterSuite. *Scientific Reports* 2021 11:1, 11(1), p.1–12. <doi:http://dx.doi.org/0.1038/s41598-021-94343-w>


Isabel González Gayte, Rocío Bautista Moreno, Pedro Seoane Zonjic and Manuel G. Claros (2017). DEgenes Hunter - A Flexible R Pipeline for Automated RNA-seq Studies in Organisms without Reference Genome. *Genomics And Computational Biology* , 3(3), e31. <doi:http://dx.doi.org/10.18547/gcb.2017.vol3.iss3.e31>

#! /usr/bin/env Rscript

#' @author refactor by Fernando Moreno Jabato. 
#' Original authors Isabel Gonzalez Gayte

################################################################################
############################### DEgenes Hunter #################################
################################################################################


############################################################
##                      SETUP PROGRAM                     ##
############################################################
options(warn=1)
if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){
  # Obtain this script directory
  full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile), 
                 error=function(e) # works when using R CMD
                normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                  commandArgs())], '='))[2]))
  main_path_script <- dirname(full.fpath)
  root_path <- file.path(main_path_script, '..', '..')
  # Load custom libraries
  custom_libraries <- c('main_degenes_Hunter.R', 'io_handling.R', 
    'general_functions.R', 'dif_expression_packages.R', 
    'qc_and_benchmarking_functions.R', 'correlation_packages.R', 
    'plotting_functions.R', 'write_report.R', "statistics_functions.R", "factor_mining.R")
  for (lib in custom_libraries){
    source(file.path(root_path, 'R', lib))
  }
  template_folder <- file.path(root_path, 'inst', 'templates')
}else{
  require('ExpHunterSuite')
  root_path <- find.package('ExpHunterSuite')
  template_folder <- file.path(root_path, 'templates')
}

# Prepare command line input 
option_list <- list(
  optparse::make_option(c("-i", "--input_file"), type="character", default=NULL,
    help="Input file with read counts"),
  optparse::make_option(c("--pseudocounts"), type="logical", default=FALSE, 
    action = "store_true",
    help="Use this option in case the mapping has been done with salmon, kallisto, StringTie or RSEM"),
  optparse::make_option(c("-C", "--Control_columns"), type="character", 
    default=NULL,
    help=paste0("Control columns. Please indicate column names of control",
      " samples separated by commas")),
  optparse::make_option(c("-T", "--Treatment_columns"), type="character", 
    default=NULL,
    help=paste0("Treatment columns. Please indicate column names of treatment",
      " samples separated by commas")),  
  optparse::make_option(c("-r", "--reads"), type="numeric", default=2,
    help=paste0("Used in filtering. Reads (counts per million mapped reads)",
      " per gene per sample must be higher than this value to count towards",
      " the --minlibraries value. Default=%default. 0 = No filtering")),
  optparse::make_option(c("--count_var_quantile"), type="double", default=0, 
    help=paste0("Proportion (0-1 value) of low variance genes to filter out. ",
      "Default=%default")),
  optparse::make_option(c("-l", "--minlibraries"), type="integer", default=2,
    help=paste0("For each gene, the minimum number of samples that must have",
      " more than --reads. Default=%default")),
  optparse::make_option(c("-F", "--filter_type"), type="character", 
    default="separate",
    help=paste0("Should filtering be done taking into account treatment and",
      " control samples separately (value=separate), possible combinations of factors ",
      "(combined), or across all factors (global). In the case of combined factors, ",
      "the factors should be specified using \"combined:factor1,factor2,...\". Default=%default")),
  optparse::make_option(c("-o", "--output_files"), type="character", 
    default="hunter_DE_results",
    help="Output path. Default=%default"),
  optparse::make_option(c("-p", "--p_val_cutoff"), type="double", default=0.05,
    help=paste0("Adjusted p-value cutoff for the differential expression",
      " analysis. Default=%default")),
  optparse::make_option(c("-f", "--lfc"), type="double", default=1,
    help=paste0("Minimum log2 fold change in expression. Note this is on a",
      " log2 scale, so a value of 1 would mean a 2 fold change.",
      " Default=%default")),
  optparse::make_option(c("-m", "--modules"), type="character", 
    default=c("DELNW"), #D = DESeq2, E = edgeR, L = limma, N = NOISeq W = WGCNA.
    help=paste0("Differential expression packages to able/disable ",
      "(D = DESeq2, E = edgeR, L = limma, N = NOISeq, W = WGCNA, P = PCIT,",
      " X = diffcoexp, F = external_DEA_file.). By default the following ",
      "modules Default=%default are performed")),
  optparse::make_option(c("-c", "--minpack_common"), type="integer", default=4,
    help="Number of minimum package to consider a gene as a 'PREVALENT' DEG"),
  optparse::make_option(c("-t", "--target_file"), type="character",
    default=NULL,
    help=paste0("Sample descriptions: one column must be named treat and",
      " contain values of Treat or Ctrl. This file will take precedent over",
      " the -T and -C sample flags.")),
  optparse::make_option(c("-e", "--external_DEA_file"), type="character", 
    default=NULL,
    help=paste0("External data file containing preanalysed DE data.",
      " Must consist of four columns with the following names and",
      " corresponding information: Entrez (or other gene id supported by",
      " functional hunter), P.Value, logFC and adj.P.Val IN THAT ORDER")),
  optparse::make_option(c("-v", "--model_variables"), type="character", 
    default="",
    help=paste0("Variables to include in the model. Must be comma separated",
      " and each variable must be a column in the target_file")),
  optparse::make_option(c("-n", "--numerics_as_factors"), type="logical", 
    default=FALSE,
    help=paste0("If TRUE, numeric variables in the target file specified",
      " in the DEG model be treated as distinct factors, i.e. categories.",
      " If FALSE, they will be considered as numeric and their values will be",
      " taken into account in the DEG model.")),
  optparse::make_option(c("-S", "--string_factors"), type="character", 
    default="",
    help=paste0("Columns in the target file to be used as categorical",
      " factors for the correlation analysis. If more than one to be used,",
      " should be comma separated")),
  optparse::make_option(c("-N", "--numeric_factors"), type="character", 
    default="",
    help=paste0("Columns in the target file to be used as numeric (continuous)",
      " factors for the correlation analysis. If more than one to be used,",
      " should be comma separated")),
  optparse::make_option(c("-b", "--WGCNA_memory"), type="integer", default=5000,
    help=paste0("Maximum block size value, to be passed to the",
      " blockwiseModules function of WGCNA as the maxBlockSize argument.",
      " Default=%default")),
  optparse::make_option(c("--WGCNA_norm_method"), type="character", 
    default="DESeq2",
    help=paste0("Method used to normalized the table of counts for WGCNA.",
      " Must also run this method in the --modules argument.",
      " Default=%default")),
  optparse::make_option("--WGCNA_deepsplit", type="integer", default=2,
    help=paste0("This option control the module building process and is",
      " defined as 1,2,3 and 4 values. 1 for rough clustering and 4 for",
      " accurate clustering. Default=%default")),
  optparse::make_option("--WGCNA_min_genes_cluster", type="integer", default=20,
    help="Mnimum number of genes to keep a cluster. Default=%default"),  
  optparse::make_option("--WGCNA_detectcutHeight", type="double", default=0.995,
    help="Cut height to split modules. Default=%default"),
  optparse::make_option("--WGCNA_mergecutHeight", type="double", default=0.25,
    help=paste0("Value to merge two similar modules: Maximum dissimilarity",
      " (i.e., 1-correlation). Default=%default")),
  optparse::make_option(c("-w", "--WGCNA_all"), type="logical", default=FALSE, 
    action = "store_true",
    help=paste0("Run WGCNA for treated only, control only, and both as 3",
      " separate runs. Needed if using PCIT. If false, WGCNA runs once, on the",
      " table including treament and control")),
  optparse::make_option(c("--WGCNA_blockwiseNetworkType"), type="character", 
    default="signed",
    help=paste0("NetworkType option to be passed to blockwiseModules function",
      " (unsigned, signed, signed hybrid).")),
  optparse::make_option(c("--WGCNA_blockwiseTOMType"), type="character", 
    default="signed",
    help=paste0("TOMType option to be passed to blockwiseModules function ",
      "(none, unsigned, signed, signed Nowick, unsigned 2, signed 2 and",
      " signed Nowick 2). If none, adjacency will be used for clustering.",
      " Default=%default")),
  optparse::make_option(c("--WGCNA_minCoreKME"), type="double", default=0.7, 
    help=paste0("Minimum module membership threshold to define module core.",
      " Modules under the threshold will not be considered. Default=%default")),
  optparse::make_option(c("--WGCNA_minCoreKMESize"), type="integer", 
    default=NULL, 
    help=paste0("Minimun genes that belong to module core. Modules under",
      " the threshold will not be considered. Default min(20,",
      " WGCNA_min_genes_cluster/3).")),
  optparse::make_option(c("--WGCNA_minKMEtoStay"), type="double", default=0.5, 
    help=paste0("Minimun module membership of a gene to be kept in module.",
      " Default=%default")),
  optparse::make_option("--WGCNA_corType", type="character", default="pearson",
    help="Set the correlation method to perform WGCNA_algorithm. 'pearson' and 'bicor' are the allowed options"),
  optparse::make_option(c("--multifactorial"), type="character", default="", 
        help=paste0("Currently only a 2x2 or factorial design is possible for interactions, and 2xn for group effects. Nested designs can",
          "also be specified (i.e. Group-specific condition effects, individuals nested within groups) The required contrast must be specified in the following manner: ",
          "FactorA,FactorB:contrast. Contrast can be either",
      "\"interaction,baseA,baseB\" if we are interested in the interaction between the two factors, where baseA and baseB should be the base levels for each factor. ",
      "FC would represent [numA_numB - baseA_numB] - [numA_baseB - baseA_baseB] with numA/B representing the non-base levels for the factorA.",
      "Alternatively, Contrast can be specificed in the form \"effect,baseA,groupB\", where the baseA should be the level in FactorA that should be used as the base for FC calculation, ",
      "and groupB represents the level in Factor B that is the group we are looking for the change in. For effect, FactorB can have more than 2 groups, allowing 2xn designs. ",
      "Finally, if nested is selected, we can perform the same comparisons using a nested design with: \"nested_int,Ctrl,groupA\" for interaction, ",
      "\"nested_effect,Ctrl,groupA\" or \"nested_effect,Ctrl,groupB\" for a group. ")),
  optparse::make_option(c("-s", "--library_sizes"), type="character",
    default=NULL, help="Path to file containing library sizes. If missing,
      certain DROP QC plots will not be drawn, and sample ranks will be defined
      by total counts.")
 )
opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))
#############################################################################
#DRAFT INPUT/OUTPUT
############################################################################
dir.create(opt$output_files)
write.table(cbind(opt), file=file.path(opt$output_files, 
  "opt_input_values.txt"), sep="\t", col.names =FALSE, quote = FALSE)

#############################################################################
#CHECKINGS
############################################################################

# Don't check if we have given an exernal DEA file
if (is.null(opt$external_DEA_file)) {
  # Check inputs not allowed values
  if (is.null(opt$input_file)){
    stop(cat(paste0("No file with RNA-seq counts is provided.",
      "\nPlease use -i to submit file")))
  }
# Check either C and T columns or target file.
  if( (is.null(opt$Treatment_columns) | is.null(opt$Control_columns)) & 
       is.null(opt$target_file)) {
    stop(cat(paste0("You must include either the names of the control and",
      " treatment columns or a target file with a treat column.")))
  }
  # In the case of -C/-T AND -t target - give a warning
  if( (!is.null(opt$Treatment_columns) | !is.null(opt$Control_columns)) &
       !is.null(opt$target_file)) {
    warning(paste0("You have included at least one -C/-T option as well as a",
      " -t option for a target file. The target file will take precedence for",
      " assigning samples labels as treatment or control."))
  }
}

#############################################################################
#EXPRESSION ANALYSIS
############################################################################
#
if(! is.null(c(opt$target_file, opt$Control_columns, opt$Treatment_columns))) {
  target <- target_generation(from_file=opt$target_file, 
      ctrl_samples=opt$Control_columns, treat_samples=opt$Treatment_columns)
} else {
  target <- NULL
}
if(! is.null(opt$input_file)) {
  raw_count_table <- read.table(opt$input_file, 
    header=TRUE, row.names=1, sep="\t")
} else {
  raw_count_table <- NULL
}
external_DEA_data <- NULL
if (grepl("F", opt$modules)) { 
  # Open the external data file for pre-analysed deg data
  external_DEA_data <- read.table(opt$external_DEA_file,
   header=TRUE, sep="\t", row.names=1)
}
library_sizes <- opt$library_sizes
if(! is.null(library_sizes)) {
  library_sizes <- read.table(library_sizes, header=TRUE)
}


final_results <- main_degenes_Hunter(
  target=target,
  raw=raw_count_table,
  count_var_quantile=opt$count_var_quantile,
  external_DEA_data=external_DEA_data,
  output_files=opt$output_files,
  reads=opt$reads,
  pseudocounts=opt$pseudocounts,
  minlibraries=opt$minlibraries,
  filter_type=opt$filter_type,
  p_val_cutoff=opt$p_val_cutoff,
  lfc=opt$lfc,
  modules=opt$modules,
  minpack_common=opt$minpack_common,
  model_variables=opt$model_variables,
  numerics_as_factors=opt$numerics_as_factors,
  string_factors=opt$string_factors,
  numeric_factors=opt$numeric_factors,
  WGCNA_memory=opt$WGCNA_memory,
  WGCNA_norm_method=opt$WGCNA_norm_method,
  WGCNA_deepsplit=opt$WGCNA_deepsplit,
  WGCNA_min_genes_cluster=opt$WGCNA_min_genes_cluster,
  WGCNA_detectcutHeight=opt$WGCNA_detectcutHeight,
  WGCNA_mergecutHeight=opt$WGCNA_mergecutHeight,
  WGCNA_all=opt$WGCNA_all,
  WGCNA_blockwiseNetworkType=opt$WGCNA_blockwiseNetworkType,
  WGCNA_blockwiseTOMType=opt$WGCNA_blockwiseTOMType,
  WGCNA_minCoreKME=opt$WGCNA_minCoreKME,
  WGCNA_minCoreKMESize=opt$WGCNA_minCoreKMESize,
  WGCNA_minKMEtoStay = opt$WGCNA_minKMEtoStay,
  WGCNA_corType = opt$WGCNA_corType,
  multifactorial = opt$multifactorial,
  library_sizes=library_sizes
)

#############################################################################
#WRITE OUTPUT
############################################################################


  write_expression_data(final_results, opt$output_files, template_folder, opt)

  write_expression_report(final_results, opt$output_files, template_folder, opt)

#' Calculates accuracy from a measures dataframe
#' @param df dataframe with measures 
#' @return calculated metric
#' @keywords metrics
#' @export
#' @examples
#' df <- data.frame(TP = 4, FP = 3, TN = 2, FN = 1)
#' acc(df)
acc <- function(df){
    (df$TP + df$TN) / (df$TP + df$TN + df$FP + df$FN)
}

#' Calculates Positive Predictive Value from a measures dataframe
#' @param df dataframe with measures 
#' @return calculated metric
#' @keywords metrics
#' @export
#' @examples
#' df <- data.frame(TP = 4, FP = 3, TN = 2, FN = 1)
#' df$Precision <- ppv(df)
ppv <- function(df){
    df$TP / (df$TP + df$FP)
}

#' Calculates Recall from a measures dataframe
#' @param df dataframe with measures 
#' @return calculated metric
#' @keywords metrics
#' @export
#' @examples
#' df <- data.frame(TP = 4, FP = 3, TN = 2, FN = 1)
#' df$Recall <- recall(df)
recall <- function(df){
    df$TP / (df$TP + df$FN)
}

#' Calculates specificity from a measures dataframe
#' @param df dataframe with measures 
#' @return calculated metric
#' @keywords metrics
#' @export
#' @examples
#' df <- data.frame(TP = 4, FP = 3, TN = 2, FN = 1)
#' spc(df)
spc <- function(df){
    df$TN / (df$TN + df$FP)
}

#' Calculates F measure from a measures dataframe
#' @param df dataframe with measures 
#' @return calculated metric
#' @keywords metrics
#' @export
#' @examples
#' df <- data.frame(TP = 4, FP = 3, TN = 2, FN = 1)
#' df$ppv <- ppv(df)
#' df$recall <- recall(df)
#' fmeasure(df)
fmeasure <- function(df){
    2 * (df$ppv * df$recall) / (df$ppv + df$recall)
}


#' @importFrom MKinfer boot.t.test
prediction_dist_pval <- function(db_distribution) {
  dist_pval <- data.frame()
  
  for (strategy in unique(db_distribution$strategy)) {
    true_dis <- db_distribution[db_distribution$strategy == strategy & 
        db_distribution$step == "predicted", "score"]
    rand_dis <- db_distribution[db_distribution$strategy == strategy & 
        db_distribution$step == "predicted_random", "score"] 
    if(length(true_dis) > 0 && sum(true_dis) > 0){
      res <- MKinfer::boot.t.test(true_dis, 
        y = rand_dis, alternative = "greater" )
      res <- res$boot.p.value
    } else {
      res <- 1
    }
    tem_pval <- data.frame(strategy = strategy, dist_pvalue = res)
    dist_pval <- rbind(dist_pval, tem_pval)
   }
  return(dist_pval)
}

#' @importFrom stats sd 
permutations_stats <- function(
permutations = 50,
db_gs, 
background, 
sample_size){
  random_dist <- c()
  for(i in seq_len(permutations)){
    sampled_bckg <- rand_sample_bool(background, 
        sample_size = sample_size)
    random_dist <- c(random_dist, sum(sampled_bckg &  
                                      db_gs))
  }
  return(data.frame(sdev = stats::sd(random_dist),
                    mean = mean(random_dist)))
}  

#Calc contingency table from 2 boolean vectors
calc_ct <- function(experiment, gold_standard){
  TP <- sum(experiment & gold_standard)
  FP <- sum(experiment & !gold_standard)
  TN <- sum(!experiment & !gold_standard)
  FN <- sum(!experiment & gold_standard)

  cont_table <- data.frame(TP = TP, 
                        FN = FN, 
                        FP = FP, 
                        TN = TN)

  return(cont_table)
}

LRplus_sub <- function(df) {
  (df$TP / (df$TP + df$FP)) / ( df$FN / ( df$FN + df$TN ))
}

LRplus_test <- function(df) {
  ( df$TP / ( df$TP + df$FN ) ) /  ( df$FP / ( df$FP + df$TN ))
}

odds_ratio <- function(df){
  (df$TP/df$FP)/(df$FN / df$TN)
}

#' @importFrom stats pchisq
vectorial_fisher_method <- function(pval_table){
    log_pval <- log(pval_table) # Log all final p-values
    log_pval[is.na(log_pval)] <- 0 # any NAs made 0s -> not used combined score 
    if (is.vector(log_pval)){
      log_pval <- t(matrix(log_pval))
      log_pval[log_pval == -Inf] <- 10e-200
    } else {
       log_pval <- apply(as.matrix(log_pval), 2, 
         function (pval_column) { # When p values of 0 are given
                                  #- these become -Inf when logged:   
                                  # This code turn -Inf values 
                                  #- to smallest possible values
           min_col_val <- sort(unique(pval_column))[2]
           pval_column[pval_column == -Inf] <- min_col_val
           return(pval_column)
       })
    }
    xi_squared <-  -2 * rowSums(log_pval)
    degrees_freedom <- 2 * ncol(log_pval)
    combined_FDR <- stats::pchisq(xi_squared, degrees_freedom, 
                                lower.tail = FALSE)
    return(combined_FDR)
}


#' Function to calculate full set of measures from a confusion data frame
#' @param df confusion dataframe 
#' @return a data frame with several measures
#' @importFrom data.table setnames
#' @export
#' @examples
#' df <- data.frame(TP = 4, FP = 3, TN = 2, FN = 1)
#' stats <- get_stats(df)
get_stats <- function(df){
  df$acc <- acc(df)
  df$spc <- spc(df)
  df$ppv <- ppv(df)
  df$recall <- recall(df)
  df$fmeasure <- fmeasure(df)
  
  data.table::setnames(
     df,
     c("acc","ppv",      "recall","spc",         "fmeasure"),
     c("ACC","Precision","Recall","Specificity" ,"FMeasure"))

  return(df)
}

#' Obtain full set of measures from a confusion matrix
#' @param cm confusion matrix 
#' @return a data frame with several measures
#' @export
#' @examples
#' cm <- matrix(c(4,3,2,1), ncol = 2)
#' rownames(cm) <- c(TRUE,FALSE)
#' colnames(cm) <- c(TRUE,FALSE)
#' stats <- get_stats_from_cm(cm)
get_stats_from_cm <- function(cm){
  # >>>>>>>[ Real , Pred ]
  TP <- extract_from_cm(cm,"TRUE","TRUE")
  FN <- extract_from_cm(cm,"TRUE","FALSE")
  TN <- extract_from_cm(cm,"FALSE","FALSE")
  FP <- extract_from_cm(cm,"FALSE","TRUE")
  df <- data.frame(TP = TP, FP = FP, TN = TN, FN = FN)
  df <- get_stats(df)
  return(df)
}

#' Extract value from confusion matrix
#' @param cm confusion matrix
#' @param real slot name
#' @param pred slot name
#' @param default to return if value is not available
#' @return value required or default if it is not available
extract_from_cm <- function(cm,real,pred,default = 0){
  value = tryCatch(
    {
      return(cm[real,pred])
    },
    error = function(cond){
      return(0)
    })
  return(value)
}


#' @importFrom stats phyper 
v.fisher.test <- function(df){
  stats::phyper(
    q = df$TP -1, 
    m = df$TP+ df$FN, 
    n = rowSums(df[,c("TP","FP","TN","FN")])-df$TP-df$FN,
    k = df$TP+df$FP, 
    lower.tail = FALSE)
}

#' @importFrom data.table setnames
v_get_stats <- function(
df, 
selected_stats = c("acc", "ppv", "recall", "spc", "fmeasure", 
                 "LRplus_sub", "LRplus_test", "v.fisher.test", "odds_ratio"),
readable_names = TRUE

){
  stats_names <- data.frame(orig = c("acc", "ppv", "recall", "spc", "fmeasure", 
                                 "LRplus_sub", "LRplus_test", "v.fisher.test", 
                                 "odds_ratio"),
                            renamed = c("Accuracy", "Precision", "Recall", 
                                 "Specificity", "Fmax", "LRplus_sub", 
                                 "LRplus_test", "Pvalue", "Odds_ratio"))
  stats_names <- stats_names[stats_names$orig %in% selected_stats, ]
  for (stat in stats_names$orig){
    df[,stat] <- get(stat)(df)
  }
  if (readable_names) {
    data.table::setnames(df, stats_names$orig,stats_names$renamed)
  }
  return(df)
}

#' @importFrom stats ecdf
calc_quantile <- function(value, distribution){
    Fn <-  stats::ecdf(distribution)
    quantile <- 1- Fn(value)
    return(quantile)
}

# Takes boolean vectors of same length and numeric vector of weights 
# false_weights, penalization for values without weights 
# > 0.5 is reward and > 0.5 penalization
weighted_cont_table <- function(
 experiment,
 gold_standard, 
 weights, 
 background_weights = NULL,
 false_weights = 1
){
   if (is.null(background_weights)){
     background_weights <- weights
   }
   weights <- calc_quantile(weights, background_weights)
   weights <- 1 - weights 
   weights[is.na(weights)] <- false_weights
   weights <- rep(1, length(weights))
   TP <- sum(weights[experiment & gold_standard])
   FP <- sum(weights[experiment & !gold_standard])
   TN <- sum(weights[!experiment & !gold_standard])
   FN <- sum(weights[!experiment & gold_standard])
   cont_table <- data.frame(TP = TP, 
                            FN = FN, 
                            FP = FP, 
                            TN = TN)
   return(cont_table)
}


filter_and_integrate_OR <- function(cont_table, p.adjust.method = "BH" ,p_thr = 0.05){
  integrated_stats <- data.frame()
  miRNA_ct <- data.frame()
  for (strat in unique(cont_table$strategy)) { 
    strat_cont_table <- cont_table[cont_table$strategy == strat,]
    for (corr_thr in unique(cont_table$corr_cutoff)) {
      for (group_db in unique(cont_table$db_group)) { 

          miRNA_group <- strat_cont_table[strat_cont_table$db_group == group_db &
                                          strat_cont_table$corr_cutoff == corr_thr,] 
          miRNA_group_rel <- miRNA_group[!(miRNA_group$TP == 0 & 
                                          miRNA_group$FN == 0),]                   
          if (!is.na(p.adjust.method)){

          miRNA_group_rel$p.adjust <- stats::p.adjust(miRNA_group_rel$Pvalue, 
                                                method = p.adjust.method,
                                                n = nrow(miRNA_group_rel))
          } else {
             miRNA_group_rel$p.adjust <- rep(0, nrow(miRNA_group_rel))
          }
          miRNA_group_fil <- miRNA_group_rel[miRNA_group_rel$p.adjust <= p_thr,]
          miRNA_ct <- rbind(miRNA_ct, miRNA_group_rel)
          if (nrow(miRNA_group_fil) == 0) next 

          strat_stats <- data.frame(
             strategy = strat,
             db_group = group_db,
             corr_cutoff = corr_thr,
             median_OR = stats::median(miRNA_group_fil$Odds_ratio),
             coverage = nrow(miRNA_group_fil) / nrow(miRNA_group_rel),
             coverage_text = paste0(nrow(miRNA_group_fil),"/",nrow(miRNA_group_rel))
          )
          # str(unique(miRNA_group_rel$db_group))
          integrated_stats <- rbind(integrated_stats, strat_stats)
        }
     }
  }
  return(list(integrated_stats = integrated_stats, miRNA_ct = miRNA_ct))
}

#' Perform enrichment analysis on multiple lists of genes
#' @param input_file input file path. File must contain 1 cluster per line clusterA ..tab.. gene1,gene2
#' @param gene_attribute_file file to map genes to attributes e.g. fold change
#' @param org_db org_db object
#' @param gene_keytype ID used in the input file
#' @param temp_file enrichments will be read from this file if it exists, written to it if it doesnt
#' @param force if true, to run enrichments even if the temp_file exists
#' @param all_funsys list of GOs/dbs to use for enrichment analysis
#' @param task_size number of elements per packages used
#' @param workers cores to be used if parallel features are going to be used
#' @param current_organism_info organism info from text table,
#' @param pvalcutoff adjusted p-value threshold to use for clusterProfiler,
#' @param qvalcutoff q-value threshold to use for clusterProfiler,
#' @param all_custom_gmt list of custom annotation sets
#' @param kegg_data_file if KEGG included in enrich_dbs path needs
#' @param universe background of genes to use as universe
#' @export
main_clusters_to_enrichment <- function(
  input_file=NULL,
  gene_attribute_file = NULL,
  org_db = NULL,
  gene_keytype ="ENTREZID",
  temp_file = "enr_tmp.RData",
  force = FALSE,
  all_funsys = c("BP","MF","CC"),
  task_size = 1,
  current_organism_info = NULL,
  workers = 1,
  pvalcutoff = 0.1,
  qvalcutoff = 0.2,
  all_custom_gmt = NULL,
  kegg_data_file = NULL,
  universe = NULL){

  gene_attributes <- NULL
  gene_attribute_name <- NULL
  if (!is.null(gene_attribute_file)){
    gene_attributes_object <- obtain_gene_attributes(gene_attribute_file, org_db, gene_keytype)
    gene_attributes <- gene_attributes_object[["gene_attributes"]]
    gene_attribute_name <-  gene_attributes_object[["gene_attribute_name"]]
  }

  if (!file.exists(temp_file) || force) {
    cluster_genes <- read.table(input_file, header=FALSE, sep="\t")
    cluster_genes_list <- strsplit(cluster_genes[,2], ",")
    names(cluster_genes_list) <- cluster_genes[,1]
    if(gene_keytype != "ENTREZID") {
      cluster_genes_list <- lapply(cluster_genes_list, function(x){
          translate_ids_orgdb(ids=x, input_id = gene_keytype,
            org_db = org_db, just_output_ids=TRUE)}) 
    }
    names(cluster_genes_list) <- cluster_genes[,1]

    enrichments_ORA <- multienricher_ora(all_funsys =  all_funsys, 
                                  genes_list =  cluster_genes_list, 
                                  task_size = task_size, 
                                  organism_info= current_organism_info,
                                  workers = workers, 
                                  pvalueCutoff =  pvalcutoff, 
                                  qvalueCutoff = qvalcutoff,
                                  custom_sets = all_custom_gmt,
                                  kegg_file = kegg_data_file,
                                  universe = universe)


    save(enrichments_ORA, file = temp_file)

  } else {
    load(temp_file)
  }
  return(list(enrichments_ORA=enrichments_ORA, 
    gene_attributes=gene_attributes, 
    gene_attribute_name=gene_attribute_name))
}

###############################################################################
########################## FUNCTIONAL ANALYSIS LIBRARY ########################
###############################################################################


#' Scale a matrix using minimum-maximum method
#' @param data_matrix to be scaled
#' @param norm_by_col boolean flag: if true scaling will be performed 
#' by columns intead of by rows. Default: FALSE
#' @importFrom matrixStats rowRanges rowDiffs
#' @keywords method
#' @return scaled matrix
scale_data_matrix <- function(data_matrix, norm_by_col = FALSE) {
  if ( norm_by_col == TRUE) {
    data_matrix <- t(data_matrix)
  } 
  dm_min_max <- matrixStats::rowRanges(data_matrix, na.rm = TRUE)
  dm_diffs <- as.vector(matrixStats::rowDiffs(dm_min_max))
  dm_mins <- dm_min_max[,1]
  #when all values are the same in the row, all values are turned to 0.
  dm_diffs[dm_diffs == 0] <- 1 
  #main scaling function
  scaled_counts <- (data_matrix - dm_mins) / dm_diffs
  if (norm_by_col == TRUE) {
    scaled_counts <- t(scaled_counts)
  } 
  return(scaled_counts)
}




#' Load a GMT format file and return a dataframe in correct format
#' @param gmt_file file to be loaded
#' @return GMT loaded info
#' @keywords file
#' @export
#' @examples
#' gmt_file <- system.file("extData", 
#' "toy_categories_1.gmt", package = "ExpHunterSuite")
#' load_and_parse_gmt(gmt_file)
load_and_parse_gmt <- function(gmt_file) {
    # Load file
    gmt <- readLines(con = gmt_file)
    gmt_list <- strsplit(gmt, "\t")
    parsed_gmt <- do.call(rbind, lapply(gmt_list, function(category) {
          category_name <- category[1]
          genes <- category[3:length(category)]
          parsedTerms <- data.frame(Term = category_name, 
                                    Gene= genes, 
                                    stringsAsFactors = FALSE)
          return(parsedTerms)
    }))
    return(parsed_gmt)
}

#' Table with information abaut all organism available
#' @param file to be loaded. If none given, internal organism table loaded
#' @return organism table
#' @keywords method
#' @export
#' @importFrom utils read.table
#' @examples
#' ot <- get_organism_table()
get_organism_table <- function(file = NULL){
  if(is.null(file)) {
    file <- system.file("external_data", "organism_table.txt", 
                          package="ExpHunterSuite")
  }
  return(utils::read.table(file, 
                           header = TRUE, 
                           row.names=1, 
                           sep="\t", 
                           stringsAsFactors = FALSE, 
                           fill = NA))
}

add_translated_gene_ids <- function(DEGH_results, 
                                    input_ids, 
                                    input_gene_id, 
                                    gene_translation_tables) {
        input_to_entrezgene <- gene_translation_tables[["input_to_entrezgene"]]    
        input_to_symbol <- gene_translation_tables[["input_to_symbol"]]    

        if(!is.null(input_to_symbol)) {
          DEGH_results <- data.frame(SYMBOL = input_to_symbol[
            match(input_ids, input_to_symbol[[input_gene_id]]), "SYMBOL"], 
            DEGH_results)
         }
         DEGH_results <- data.frame(ENTREZID = input_to_entrezgene[
           match(input_ids, input_to_entrezgene[[input_gene_id]]), "ENTREZID"], 
           DEGH_results)
         DEGH_results <- data.frame(input_IDs=input_ids, DEGH_results)
}

get_sig_genes <- function(DEGH_results) {
    prev_genes <- DEGH_results[DEGH_results$genes_tag == "PREVALENT_DEG" &
                               !is.na(DEGH_results$ENTREZID), "ENTREZID"]
                               "%>%" <- magrittr::"%>%"
    ## TODO => ESTARIA BIEN REFLEJAR ESTA INFORMACION EN EL REPORT
    union_DEGs_df <- subset(DEGH_results, genes_tag %in% c("POSSIBLE_DEG",
                         "PREVALENT_DEG"))
    union_DEGs <- union_DEGs_df[!is.na(union_DEGs_df$input_IDs), 
                                 "input_IDs"] %>% unique
    return(list(prev_genes=prev_genes, union_DEGs_df=union_DEGs_df,
        union_DEGs=union_DEGs))
}

get_gene_lists <- function(DEGH_results, fc_colname) {
    geneList <- DEGH_results[!is.na(DEGH_results$ENTREZID),  fc_colname]
    names(geneList) <- DEGH_results[!is.na(DEGH_results$ENTREZID), "ENTREZID"]
    geneList <- sort(geneList, decreasing = TRUE)
    return(geneList)
}

get_sig_genes_cl <- function(DEGH_results) {
    cls <- unique(DEGH_results$Cluster_ID)
    # DELETE GREY MODULE
    if (any(c(0,"grey") %in% cls)) {
        cls <- cls[!cls %in% c(0,"grey")]
    } else {
        warning("Module Zero/Grey not found")
    }

    clgenes <- lapply(cls,function(cl) { # Find
        unique(DEGH_results$ENTREZID[which(DEGH_results$Cluster_ID == 
                                              cl)])
    }) 
    names(clgenes) <- cls
    return(clgenes)
}

get_gene_lists_cl <- function(DEGH_results, fc_colname) {
    DEGH_res_list <- split(DEGH_results, DEGH_results$Cluster_ID)
    lapply(DEGH_res_list, function(x) get_gene_lists(x, fc_colname))
}

#' Obtain an org_db object from a current_organism_info, obtained from
#' subsetting the results of get_organism_table()
#' @param current_organism_info row from output of get_organism_table() 
#' @export
#' @keywords org_db
#' @return org_db 
get_org_db <- function(current_organism_info) {
  org_db <- current_organism_info$Bioconductor_DB[1]
  org_db <- eval(parse(text = paste0(org_db,"::",org_db)))
  return(org_db)
}

check_id_valid_orgdb <- function(gene_id, 
                                 id_type="input", 
                                 organism_info, 
                                 outcome_action="stop") {
  org_db <- get_org_db(organism_info)
  if(id_type == "input") possible_ids <- AnnotationDbi::keytypes(org_db)
  else possible_ids <- AnnotationDbi::columns(org_db)

  if(! gene_id %in% possible_ids) {
    if(outcome_action=="stop") {
      stop(paste(c("gene id must be one of the following:", possible_ids), 
                                                              collapse=" "))
    } else if(outcome_action=="warn") {
      warning(paste(c("gene id must be one of the following:", possible_ids), 
                                                                collapse=" "))
      return(FALSE)
    }
  }
  return(TRUE)
}

#' Translates a given gene ID using a dictionary. Note: one unknown ID can
#' corresponds to many known ids. 
#' @param ids_to_translate set of IDs to be translated
#' @param annot_table dictionary to translate IDs
#' @keywords translate
#' @return translated IDs or NA if it's not possible to translate
translate_from_table <- function(ids_to_translate, annot_table){ 
  translated_ids <- unlist(lapply(ids_to_translate, function(id){
    indx <- which(annot_table[,2] == id)
    if(length(indx) == 0){
      return(NA)
    } else{
      return(annot_table[indx[1],1])
    }
  }))
  return(translated_ids)
}

get_translation_tables_orgdb <- function(
  input_gene_id, 
  input_ids, 
  current_organism_info) {
  org_db <- get_org_db(current_organism_info)
  if(input_gene_id == "ENTREZID") {
    input_to_entrezgene <- data.frame(input=input_ids, 
                                      ENTREZID=input_ids)
  } else {
    # Check input gene ID valid
    check_id_valid_orgdb(gene_id=input_gene_id, 
                         id_type="input", 
                         organism_info=current_organism_info, 
                         outcome_action="stop")

        input_to_entrezgene <- translate_ids_orgdb(ids=input_ids, 
        input_id=input_gene_id, org_db=org_db) 
  }
  symbol_output_available <- check_id_valid_orgdb(
                                        gene_id=input_gene_id, 
                                        id_type="output", 
                                        organism_info=current_organism_info, 
                                        outcome_action="warning")
            
  if(symbol_output_available == TRUE) {
    if(input_gene_id == "SYMBOL") {
      input_to_symbol <- data.frame(input=input_ids, 
                                      SYMBOL=input_ids)
    } else {
      input_to_symbol <- translate_ids_orgdb(ids=input_ids, 
      input_id=input_gene_id, output_id="SYMBOL", org_db=org_db)
    }
  } else {
    input_to_symbol <- NULL
  }
  return(list(input_to_entrezgene = input_to_entrezgene, 
              input_to_symbol = input_to_symbol))
}

translate_ids_orgdb <- function(
ids, 
input_id, 
output_id="ENTREZID", 
org_db=org_db, 
just_output_ids=FALSE){
  
  possible_ids <- AnnotationDbi::columns(org_db)
  if(! input_id %in% possible_ids) 
    stop(paste(c("gene keytype must be one of the following:", possible_ids),
                                                               collapse=" "))
    ids <- tryCatch(
      ids <- AnnotationDbi::select(org_db, keys=ids, column=output_id, 
                                  keytype=input_id),
      error=function(cond){
            ids <- NULL
        }
    )
    ids <- ids[!is.na(ids[,output_id]),]
    if(just_output_ids == TRUE) {
      return(unique(ids[,output_id]))
    } else {
      return(ids)
    }
}

translate_gmt <- function(gmt, gene_keytype, org_db){
  splitted_gmt <- split(gmt$Gene, gmt$Term)
  tr_splitted_gmt  <- lapply(splitted_gmt, function(x) {
                      tr_table <- translate_ids_orgdb(ids=x, 
                                            input_id=gene_keytype,
                                            org_db = org_db)
                      return(unique(tr_table[,2])) })
  translated_gmt <- lapply(tr_splitted_gmt, as.data.frame)
  translated_gmt <- as.data.frame(data.table::rbindlist(translated_gmt , 
    use.names = TRUE, idcol = TRUE))
  names(translated_gmt) <- c("Term","Gene")
  return(translated_gmt)
}

#' Perform topGO enrichment analysis of a list of genes
#' @param all_funsys vector of funsys to use (e.g. MF, Reatcome)
#' @param genes_list vector of genes or list of vectors, to be enriched
#' @param universe background of genes to use as universe
#' @param organism_info from the annotation table: infor on db names etc
#' @param gene_id what identifier do the genes use - must be entrez for 
#' @param algorithm alrgorithm for decting enriched functions - see topGO docs
#' @param statistic method for detecting enriched functions
#' @param nodeSize related to the creation of the GOdata object
#' @param workers for parallelization
#' @param task_size for parallelization
#' @export
#' @importClassesFrom topGO topGOdata
#' @importFrom topGO annFUN
multienricher_topGO <- function(all_funsys, genes_list, universe=NULL, 
  organism_info, gene_id="entrez", algorithm = "classic", statistic = "fisher", 
  nodeSize = 5, task_size=1, workers=1) {

  unlisted_input_flag <- FALSE
  if(! is.list(genes_list)) {
    unlisted_input_flag <- TRUE
    genes_list <- list(genes_list)
  }

 org_db <- organism_info$Bioconductor_DB[1]
 enrichments_topGO <- vector("list", length(all_funsys))
 names(enrichments_topGO) <- all_funsys
 for(funsys in all_funsys) {
  if (funsys %in% c("CC","BP","MF")){
    if(is.null(universe)) {
      go_to_genes <- topGO::annFUN.org(funsys, mapping = org_db, ID = gene_id)
      universe <- unique(unlist(go_to_genes))
    }

    geneList <- factor(as.integer(universe %in% genes_list[[1]]))
    names(geneList) <- universe
     

    # Create environment variables used for initialization of the topGOdata obj
    topGO::groupGOTerms()
    GOdata <- new("topGOdata",
                 ontology = funsys,
                 allGenes = geneList,
                 nodeSize = nodeSize,
                 mapping = org_db,
                 annotationFun = topGO::annFUN.org,
                 ID = gene_id)
    }

    enriched_cats <- parallel_list(genes_list, function(l_genes) {
      geneList <- factor(as.integer(universe %in% l_genes))
      names(geneList) <- universe
      l_GOdata <- topGO::updateGenes(object = GOdata, geneList = geneList)
      resultFis <- topGO::runTest(l_GOdata, algorithm = algorithm, 
      statistic = statistic)
    }, workers = workers, task_size = task_size )
    
    if(unlisted_input_flag) enriched_cats <- enriched_cats[[1]]
    enrichments_topGO[[funsys]] <- enriched_cats
  }
  return(enrichments_topGO)
}

#' Perform gsea enrichment analysis of a list of genes
#' @export
#' @param all_funsys vector of funsys to use (e.g. MF, Reatcome)
#' @param genes_list vector of genes or list of vectors, to be enriched
#' @param organism_info from the annotation table: infor on db names etc
#' @param org_db org db file (optional) - normally obtained from organism_info
#' @param workers for parallelization
#' @param task_size for parallelization
#' @param pvalueCutoff p-value finding enriched categories
#' @param pAdjustMethod adjustment method for p-values
#' @param kegg_file must be provided if kegg annotation required
#' @param custom_sets custom set object (processed by load_and_parse_gmt)
#' @param ... other arguments passed to the enrichment function
multienricher_gsea <- function(all_funsys=NULL, genes_list, organism_info, 
  org_db = NULL, task_size=1, workers=1, pvalueCutoff = 0.05, 
  pAdjustMethod = "BH", kegg_file=NULL, custom_sets=NULL, ...){

  unlisted_input_flag <- FALSE
  if(! is.list(genes_list)) {
    unlisted_input_flag <- TRUE
    genes_list <- list(genes_list)
  }


  common_params <- list(pvalueCutoff = pvalueCutoff, 
    pAdjustMethod = pAdjustMethod, ...)

  if(! is.null(custom_sets)) {
    if(is.null(names(custom_sets))) 
      stop("Custom sets enrichment object must be a named list")
    all_funsys <- c(all_funsys, names(custom_sets))
  }

  enrichments_gsea <- vector("list", length(all_funsys))
  names(enrichments_gsea) <- all_funsys
  org_db <- get_org_db(organism_info)
  for(funsys in all_funsys) {
    if (funsys %in% c("CC","BP","MF")){

      enrf <- prepare_enrichment_GO(enrichment_type="gsea", subont = funsys, 
        org_db = org_db)
      specific_params <- list(OrgDb = org_db, ont = funsys)
    } else  if (funsys == "Reactome"){
      enrf <- prepare_enrichment_Reactome(enrichment_type="gsea", 
                                          reactome_id = 
                                          organism_info$Reactome_ID[1])
            specific_params <- list(organism = organism_info$Reactome_ID[1])

    } else  if (funsys == "KEGG"){
      enrf <- prepare_enrichment_KEGG(enrichment_type="gsea", 
        kegg_file = kegg_file)
      specific_params <- list(organism = organism_info$KeggCode[1])

    } else if (funsys %in% names(custom_sets)) {
      enrf <- clusterProfiler::GSEA
      specific_params <- list(TERM2GENE = custom_sets[[funsys]])
    } else {
      stop("funsys", funsys, "not recognized")
    }

    #enriched_cats <- parallel_list(genes_list, function(l_genes){
    enriched_cats <- lapply(genes_list, function(l_genes){
       params_genes <- c(specific_params, common_params, list(gene = l_genes))
       enriched_cats <- do.call("enrf", params_genes)
    })
    #},workers= 1, task_size = 1 )

    # enriched_cats[sapply(enriched_cats, is.null)] <- data.frame()
    # enriched_cats <- lapply(enriched_cats, function(x) { 
    #    DOSE::setReadable(x, OrgDb = org_db, 
    #    keyType="ENTREZID")
    # })

    if(unlisted_input_flag) enriched_cats <- enriched_cats[[1]]
    enrichments_gsea[[funsys]] <- enriched_cats
  }

  return(enrichments_gsea)
}

#' @importFrom clusterProfiler enrichGO gseGO
prepare_enrichment_GO <- function(enrichment_type, subont, org_db) {
  if(enrichment_type == "ora")   enrf <- clusterProfiler::enrichGO
  if(enrichment_type == "gsea") enrf <- clusterProfiler::gseGO

  get_enr_data <- get("get_GO_data", envir = asNamespace("clusterProfiler"), 
    inherits = FALSE)      
  pattern_to_remove  <- "GO_DATA *<-"
  ENRICH_DATA <- get_enr_data(org_db, subont, "ENTREZID")
  ltorem <- grep(pattern_to_remove, body(enrf))
  body(enrf)[[ltorem]] <- substitute(GO_DATA <- ENRICH_DATA)
  return(enrf)
}

#' @importFrom ReactomePA enrichPathway gsePathway
prepare_enrichment_Reactome <- function(enrichment_type, reactome_id) {
  if(enrichment_type == "ora") enrf <- ReactomePA::enrichPathway
  if(enrichment_type == "gsea") enrf <- ReactomePA::gsePathway

  get_enr_data <- get("get_Reactome_DATA", envir = asNamespace("ReactomePA"), 
    inherits = FALSE)
  ENRICH_DATA <- get_enr_data(reactome_id) 
  pattern_to_remove <- "Reactome_DATA *<-"
  ltorem <- grep(pattern_to_remove, body(enrf))
  body(enrf)[[ltorem]] <- substitute(Reactome_DATA <- ENRICH_DATA)
  return(enrf)
}

#' @importFrom clusterProfiler gseKEGG enrichKEGG
prepare_enrichment_KEGG <- function(enrichment_type, kegg_file) {
############# DEPRECATED FUNCTION

  if(is.null(kegg_file) || ! file.exists(kegg_file) ) 
                stop("kegg_file not found or not provided. 
  It can be downloaded using download_latest_kegg_db()")

  if(enrichment_type == "ora") enrf <- clusterProfiler::enrichKEGG

  if(enrichment_type == "gsea") enrf <- clusterProfiler::gseKEGG

  ENRICH_DATA <- readRDS(kegg_file)
  pattern_to_remove <- "KEGG_DATA *<-"
  ltorem <- grep(pattern_to_remove, body(enrf))
  body(enrf)[[ltorem]] <- substitute(KEGG_DATA <- ENRICH_DATA)
  return(enrf)
}




enrichKEGG_user_data <- function (
  gene, 
  organism = "hsa", 
  keyType = "kegg", 
  pvalueCutoff = 0.05,
  pAdjustMethod = "BH", 
  universe, minGSSize = 10, 
  maxGSSize = 500,
  qvalueCutoff = 0.2, 
  user_data, ...)
{
    res <- clusterProfiler:::enricher_internal(gene, 
      pvalueCutoff = pvalueCutoff,
      pAdjustMethod = pAdjustMethod, 
      universe = universe, 
      minGSSize = minGSSize,
      maxGSSize = maxGSSize, 
      qvalueCutoff = qvalueCutoff, 
      USER_DATA = user_data,
      ...)
    if (is.null(res))
        return(res)
    res@ontology <- "KEGG"
    res@organism <- organism
    res@keytype <- keyType
    return(res)
}


#' Perform ORA enrichment analysis of a list of genes
#' @param all_funsys vector of funsys to use (e.g. MF, Reatcome)
#' @param genes_list vector of genes or list of vectors, to be enriched
#' @param universe background of genes to use as universe
#' @param organism_info from the annotation table: infor on db names etc
#' @param org_db org db file (optional) - normally obtained from organism_info
#' @param workers for parallelization
#' @param task_size for parallelization
#' @param pvalueCutoff p-value finding enriched categories
#' @param qvalueCutoff q-value finding enriched categories
#' @param pAdjustMethod adjustment method for p-values
#' @param kegg_file must be provided if kegg annotation required
#' @param custom_sets custom set object (processed by load_and_parse_gmt)
#' @param readable Whether output should include gene symbols
#' @param return_all Whether to remove list items with no enrichment
#' @param ... other arguments passed to the enrichment function
#' @importFrom DOSE setReadable
#' @export
multienricher_ora <- function(all_funsys=NULL, genes_list, universe=NULL, 
  organism_info, org_db = NULL, task_size=1, workers=1, pvalueCutoff = 0.05, 
  qvalueCutoff = 0.2, pAdjustMethod = "BH", kegg_file=NULL, 
  custom_sets=NULL, readable=TRUE, return_all=FALSE, ...){
  unlisted_input_flag <- FALSE
  if(! is.list(genes_list)) {
    unlisted_input_flag <- TRUE
    genes_list <- list(genes_list)
  }
  if( row.names(organism_info) != "Human" && "DO" %in% all_funsys) {
    warning("Cannot run DO with non-human organim, will be skipped")
    all_funsys <- all_funsys[! all_funsys %in% "DO"]
  }
  common_params <- list(universe = universe, pvalueCutoff = pvalueCutoff, 
    qvalueCutoff = qvalueCutoff, 
    pAdjustMethod = pAdjustMethod, ...)

  if(! is.null(custom_sets)) {
    if(is.null(names(custom_sets))) 
                stop("Custom sets enrichment object must be a named list")
    all_funsys <- c(all_funsys, names(custom_sets))
  }
  if (is.null(org_db)) {
    org_db <- get_org_db(organism_info)
  }
  enrichments_ORA <- vector("list", length(all_funsys))
  names(enrichments_ORA) <- all_funsys
  for(funsys in all_funsys) {
    specific_params <- NULL
    if (funsys %in% c("CC","BP","MF","ALL")){
      enrf <- prepare_enrichment_GO(enrichment_type="ora", subont = funsys, 
        org_db = org_db)
      specific_params <- list(OrgDb = org_db, ont = funsys)

    } else  if (funsys == "Reactome"){
     
      enrf <- prepare_enrichment_Reactome(enrichment_type="ora", 
        reactome_id = organism_info$Reactome_ID[1])
      specific_params <- list(organism = organism_info$Reactome_ID[1])

    } else if (funsys == "KEGG"){
      specific_params <- list(organism = organism_info$KeggCode[1])
      if (!is.null(kegg_file)){ 
        enrf <- enrichKEGG_user_data 
        ENRICH_DATA <- readRDS(kegg_file)
        specific_params<- c(specific_params, list(user_data = ENRICH_DATA))
      } else {
          stop("kegg_file not found or not provided. 
          It can be downloaded using download_latest_kegg_db()")
      }      
    } else if (funsys == "DO"){
      # No need for prepare enrichment function - hack not necessary
      enrf <- DOSE::enrichDO
    } else if (funsys == "DGN"){
      enrf <- DOSE::enrichDGN
    } else if (funsys == "DGNv"){
      enrf <- DOSE::enrichDGNv
    } else if (funsys == "NCG"){
      enrf <- DOSE::enrichNCG 
    } else if (funsys %in% names(custom_sets)) {
      enrf <- clusterProfiler::enricher
      specific_params <- list(TERM2GENE = custom_sets[[funsys]])
    } else {
      stop("funsys", funsys, "not recognized")
    }
    temp_workers <- workers
    if(funsys == "DO") workers <- 1 #DOSE not working in parallel, but its quick

    enriched_cats <- parallel_list(genes_list, function(l_genes){
        params_genes <- c(specific_params, common_params, list(gene = l_genes))
        enr <- do.call("enrf", params_genes)
     } , workers= workers, task_size = task_size
    )
    workers <- temp_workers
    if(return_all == FALSE) enriched_cats[sapply(enriched_cats,is.null)] <- NULL

    if(readable == TRUE) {
      enriched_cats <- lapply(enriched_cats, function(x) { 
        if(! is.null(x)) return(
          DOSE::setReadable(x, OrgDb = org_db, 
          keyType="ENTREZID")
          )
        else return(data.frame())
      })
    }
    # Remove species name from reactome enrichment
    if(funsys == "Reactome") { 
      enriched_cats <- lapply(enriched_cats, function(enr) {
        if(nrow(enr) > 0)
        enr@result[,"Description"] <- gsub("^.+\\r: ", "", 
                                                    enr@result[,"Description"])
        return(enr)
      })
    }
    if(unlisted_input_flag) enriched_cats <- enriched_cats[[1]]    
    enrichments_ORA[[funsys]] <- enriched_cats
  }
  return(enrichments_ORA)
}

add_term_sim_ora <- function(deg_enr_ora) {
  enr_with_termsim <- list()
  for(funsys in names(deg_enr_ora)) {
    enrichment <- deg_enr_ora[[funsys]]
    if(is.list(enrichment) && length(enrichment) > 1) {
      enr_with_termsim[[funsys]] <- sapply(enrichment, function(enr) {
        if(nrow(enr) > 0) {
          return(trycatch_pairwise_termsim(enr))
        } else {
          return(enr)
        }
      })
    } else {
      if(nrow(enrichment) > 0) {
        enr_with_termsim[[funsys]] <- trycatch_pairwise_termsim(enrichment)
      } else {
        enr_with_termsim[[funsys]] <- enrichment
      }
    }
  }
  return(enr_with_termsim)
}

#' if pairwise_termsim throws an error, remove dup cat descriptions
#' Only seems to be a problem with Reactome - to study further
#' Further investigation re: number of cats also required
#' @param enr enrichment object to be studied
#' @return enrichment object after add termsim info
#' @importFrom enrichplot pairwise_termsim
trycatch_pairwise_termsim <- function(enr){ 
  enr <- tryCatch(
  {
    enr <-enrichplot::pairwise_termsim(enr)
  },
    error = function(cond){
    message("ERROR ADDING TERM SIMILARITY TO ENRICHMENT OBJECT")
    message(cond)
    message("ATTEMPTING TO FIX BY REMOVING DUPLICATED ID DESCRIPTIONS")
    ccr <- enr@compareClusterResult
    unique_desc_id <- unique(ccr[c("ID","Description")])
    IDs_with_dupl_desc <- unique_desc_id$ID[
                                         duplicated(unique_desc_id$Description)]
    enr@compareClusterResult <- ccr[! ccr$ID %in% IDs_with_dupl_desc, ]

    enrichplot::pairwise_termsim(enr)
  })
  return(enr)
}


merge_clusters <- function(results_list) {
  merged_clusters <- lapply(results_list, clusterProfiler::merge_result)
}

filter_cluster_enrichment <- function(compareCluster, filter_list){
    compareCluster@compareClusterResult <- compareCluster@compareClusterResult[
              compareCluster@compareClusterResult$Cluster %in% filter_list,]
    return(compareCluster)
}

hamming_binary <- function(X, Y = NULL) {
    if (is.null(Y)) {
        D <- t(1 - X) %*% X
        D + t(D)
    } else {
        t(1 - X) %*% Y + t(X) %*% (1 - Y)
    }
} # from https://johanndejong.wordpress.com/
  #2015/10/02/faster-hamming-distance-in-r-2/

summarize_merged_ora <- function(ORA_merged, sim_thr=0.7, 
  common_name = "ancestor", pthreshold=0.1) {
  if(! any(c("BP","MF","CC") %in% names(ORA_merged))) {
    stop("Summarize only works with GO categories. Skipping")
    return(NULL)
  }
  ORA_merged <- ORA_merged[names(ORA_merged) %in% c("BP","MF","CC")]

  sum_enrichments <- summarize_categories(ORA_merged, sim_thr = sim_thr, 
    common_name = common_name)

  summ_ora_to_plot <- list()
  for (funsys in names(ORA_merged)) {
    sum_table <- sum_enrichments[[funsys]]
    sum_table <- (sum_table > pthreshold) + 0
    sum_table <- sum_table[rownames(sum_table) != "to_remove",]
    summ_enr_table <- sum_table
    rownames(summ_enr_table) <- get_GOid_term(rownames(summ_enr_table))

    summ_enr_clean <-  clean_parentals_in_matrix(sum_table, funsys)
    rownames(summ_enr_clean) <- get_GOid_term(rownames(summ_enr_clean))
    summ_enr_clean <- summ_enr_clean[rowSums(summ_enr_clean < pthreshold) != 0,]
   
    full_enr_table <- cluster_enr_to_matrix(
                                      ORA_merged[[funsys]]@compareClusterResult)
    full_enr_table <- (full_enr_table > pthreshold) + 0
    summ_ora_to_plot[[funsys]] <- list(summ_enr_table=summ_enr_table, 
      summ_enr_clean=summ_enr_clean, full_enr_table=full_enr_table)
  }

  return(summ_ora_to_plot)
}

summarize_categories <- function(all_enrichments, sim_thr = 0.7, 
  common_name = "significant"){
   enrichment_summary <- list()
       # save(list = ls(all.names = TRUE), file = "summarize_environment.RData")
  clusterized_terms <- clusterize_terms(all_enrichments, threshold = sim_thr, 
    common_name = common_name)

  for (funsys in names(clusterized_terms)){
    enrichments_cl <- all_enrichments[[funsys]]
    combined_enrichments <- combine_terms_by_cluster(
      enrichments_cl@compareClusterResult, clusterized_terms[[funsys]])
    combined_enrichments_tmp <-  reshape2::acast(combined_enrichments, 
      term_cluster~cluster, value.var="p.adjust", fill = 1)
    combined_enrichments <- apply(combined_enrichments_tmp, 2, FUN=as.numeric)
    dimnames(combined_enrichments) <- dimnames(combined_enrichments_tmp)
    enrichment_summary[[funsys]] <- combined_enrichments
  }
  return(enrichment_summary)
}

#' @importFrom stats as.dist
#' @importFrom Matrix forceSymmetric
#' @importFrom fastcluster hclust
#' @importFrom GO.db GOBPPARENTS GOBPANCESTOR GOMFPARENTS GOMFANCESTOR GOCCPARENTS GOCCANCESTOR
clusterize_terms <- function(all_enrichments, threshold = 0.7, 
  common_name = "significant"){
    # save(list = ls(all.names = TRUE), file = "clusterize_environment.RData")
  clusterized_terms <- list()
  for (funsys in names(all_enrichments)){
    if (funsys=="BP"){
      GO_parents <- GO.db::GOBPPARENTS
      GO_ancestor <- GO.db::GOBPANCESTOR
    } else if (funsys=="MF"){
      GO_parents <-  GO.db::GOMFPARENTS
      GO_ancestor <-  GO.db::GOMFANCESTOR
    } else if (funsys=="CC"){
      GO_parents <- GO.db::GOCCPARENTS
      GO_ancestor <- GO.db::GOCCANCESTOR
    }

    GO_ancestor <- as.list(GO_ancestor)
    GO_parents <- as.list(GO_parents)
    for (ancestor in names(GO_ancestor)){
      GO_ancestor[[ancestor]] <- c(GO_ancestor[[ancestor]], ancestor)
    }
#    print("time calc_all_paths")
#     print(system.time(
    paths <- calc_all_paths(ids = unique(names(GO_parents)), 
      GO_parents = GO_parents)
#    ))
    levels <- unlist(lapply(paths, function(id){min(lengths(id))}))

    enrichments_cl <- all_enrichments[[funsys]]
    term_sim <- enrichments_cl@termsim
    term_sim <- Matrix::forceSymmetric(term_sim, uplo="U")
    term_sim[is.na(term_sim)] <- 0
    term_dis <- stats::as.dist(1 - term_sim)
    term_clust <- fastcluster::hclust(term_dis, method = "average")
    threshold = 1 - threshold
    clust_term <- stats::cutree(term_clust, h = threshold)

    cluster_names <- lapply(clust_term, function(cluster) {
       terms_in_cl <- names(clust_term)[clust_term == cluster]
       cluster_enrichments <- enrichments_cl@compareClusterResult
       cluster_enrichments <- cluster_enrichments[
         cluster_enrichments$Description %in% terms_in_cl,]
       if (common_name == "ancestor") {
          main_name <- get_common_ancestor(unique(cluster_enrichments$ID), 
            GO_ancestor = GO_ancestor, levels = levels)
       } else if (common_name == "significant"){
          main_name <- cluster_enrichments[which.min(
            cluster_enrichments$p.adjust), "Description"]
       } 
       return(main_name)
    })
    clust_term <- unlist(cluster_names)

    clusterized_terms[[funsys]] <- clust_term
  }
  return(clusterized_terms)
}




combine_terms_by_cluster <- function(clusters_enr, term_clustering) {
  combined_terms <- data.frame()
  for (cluster in unique(clusters_enr$Cluster)) {
    clusters_enr$Description <- as.character(clusters_enr$Description)
    cluster_terms <- clusters_enr[clusters_enr$Cluster == cluster, 
                                   "Description"]
    term_clusters <- unique(term_clustering[names(term_clustering) 
      %in% cluster_terms])
    for (term_cluster in term_clusters) {
      clustered_terms <- names(term_clustering)[term_clustering == term_cluster]
      combined_terms <- rbind(combined_terms, 
        c(cluster, term_cluster, min(clusters_enr[clusters_enr$Description %in% 
                                    clustered_terms, "p.adjust"])))
    }
  }
  colnames(combined_terms) <- c("cluster", "term_cluster", "p.adjust")
  return(combined_terms)
}


get_common_ancestor <- function(terms, GO_ancestor, levels, 
  common_ancestor_method = "lowest"){
  terms_ancestors <- GO_ancestor[terms]
  common_ancestors <- Reduce(intersect, terms_ancestors)
  common_ancestors <- levels[names(levels) %in% common_ancestors]
  common_ancestor <- NULL
  if (common_ancestor_method == "lowest") {
      common_ancestors <- sort(common_ancestors,  decreasing = TRUE)
  } else if (common_ancestor_method == "highest") {
      common_ancestors <- sort(common_ancestors,  decreasing = FALSE)
  }

  common_ancestor <- common_ancestors[1]

  if (common_ancestor > 2) {
      #get_GOid_term(names(common_ancestor))
      common_ancestor <- names(common_ancestor) 

  } else {
      common_ancestor <- "to_remove"
  }
  return(common_ancestor)
}

get_all_parentals <- function(term, GO_parents, relative_level = 0){
  all_parents <- GO_parents[[term]]
  isa_parents <- all_parents[names(all_parents) == "isa"]
  term_level <- data.frame(term = term, 
                               relative_level = relative_level)
  if (any(isa_parents %in% "all")) {
    return(term_level)
  } else {
    all_branches <- term_level
    for (parent in isa_parents) {
      parental_branches <- get_all_parentals(term = parent, 
                                             GO_parents = GO_parents, 
                                             relative_level = (
                                              relative_level + 1))
      all_branches <- rbind(all_branches, parental_branches)
    }
    return(all_branches)
  }
}


#' @importFrom GO.db GOTERM
get_GOid_term <- function(GOid, output = "term"){
 term <- AnnotationDbi::Term(GO.db::GOTERM[GOid])
 names(term) <- NULL
 return(term)
}


cluster_enr_to_matrix <- function(compareClusterResult){
   compareClusterResult <- compareClusterResult[,c("Cluster", "Description", 
    "p.adjust")]
    compareClusterResult_tmp <-  reshape2::acast(compareClusterResult, 
      Description~Cluster, value.var="p.adjust", fill = 1)
    compareClusterResult <- apply(compareClusterResult_tmp, 2, FUN=as.numeric)
    dimnames(compareClusterResult) <- dimnames(compareClusterResult_tmp)
    return(compareClusterResult_tmp)
}

calc_all_paths <- function(ids, GO_parents){
    env <- environment()
    paths <- list()
    calc_path(ids = ids, GO_parents = GO_parents, env = env)
    return(paths)
}

calc_path <- function(ids, GO_parents, env){
  for (id in ids) {
    if (is.null(env$paths[[id]])) {
      env$paths[[id]] <- list()
      direct_ancestors <- GO_parents[[id]]
      direct_ancestors <- direct_ancestors[names(direct_ancestors) == "isa"]
      if (length(direct_ancestors) == 0) {
        env$paths[[id]] <- c(env$paths[[id]], list(c(id)))
      } else {
        for (anc in direct_ancestors) {
          calc_path(ids = anc, GO_parents = GO_parents,env = env)
          env$paths[[id]] <- c(env$paths[[id]], 
                               lapply(env$paths[[anc]], function(path){
                                c(id,unlist(path))}))
        }
      }
    }
  }
}

vectdist <- function(vectA, vectB){
  # VectA and B must have same length. Exception not handled
  return(sqrt(sum((vectA - vectB)^2)))
}

toDistances <- function(vectors_matrix, rows = TRUE){
  if(!rows){
    vectors_matrix = t(vectors_matrix)
  }
  # Calc similitudes of rows
  numItems = nrow(vectors_matrix)
  Mdist = matrix(Inf,nrow = numItems, ncol = numItems)
  invisible(lapply(seq(numItems), function(i){
    if(i != numItems){
      invisible(lapply(seq(i+1, numItems), function(j){
        v = vectdist(vectors_matrix[i,],vectors_matrix[j,])
        Mdist[i,j] <<- v
        Mdist[j,i] <<- v
      }))
    }
    Mdist[i,i] <<- 0
  }))
  return(Mdist)
}

#' @importFrom GO.db GOBPANCESTOR GOMFANCESTOR GOCCANCESTOR
clean_parentals_in_matrix <- function(enrichment_mx, subont){
  if (subont=="BP"){
    GO_ancestors <- GO.db::GOBPANCESTOR
  } else if (subont=="MF"){
    GO_ancestors <-  GO.db::GOMFANCESTOR
  } else if (subont=="CC"){
    GO_ancestors <- GO.db::GOCCANCESTOR
  }
  for (cluster in colnames(enrichment_mx)) {

    enriched_cats <- rownames(enrichment_mx)
    for (category_ref in enriched_cats){
       parental_terms <- enriched_cats %in% GO_ancestors[[category_ref]]
       if (any(parental_terms)) {
          enrichment_mx[parental_terms, cluster] <- 1
       }
    }
  }
  return(enrichment_mx)
}


filter_top_categories <- function(enrichments_ORA_merged, top_c = 50){
  # save(enrichments_ORA_merged, file="enrichments_ORA_merged.RData")
  if(! is.null(top_c)) {
    for (funsys in names(enrichments_ORA_merged)){
      filtered_enrichments <- 
        enrichments_ORA_merged[[funsys]]@compareClusterResult
      if (nrow(filtered_enrichments) == 0) next 
      filtered_enrichments <- filtered_enrichments[order(
        filtered_enrichments$p.adjust, decreasing = FALSE), ]
      filtered_enrichments <- Reduce(rbind,by(
        filtered_enrichments,filtered_enrichments["Cluster"], head, n = top_c))
      filtered_terms <- unique(filtered_enrichments$Description)
      enrichments_ORA_merged[[funsys]]@compareClusterResult <- 
        filtered_enrichments
      enrichments_ORA_merged[[funsys]]@termsim <- 
        enrichments_ORA_merged[[funsys]]@termsim[filtered_terms,filtered_terms]
    }
  }
  return(enrichments_ORA_merged)
}


process_cp_list <- function(enrichments_ORA, simplify_results, 
  clean_parentals){
  enrichments_ORA_tr <- list()
  for (funsys in names(enrichments_ORA)){
    enr_obj <- clusterProfiler::merge_result(enrichments_ORA[[funsys]])
    if(nrow(enr_obj@compareClusterResult) > 0){
      if (funsys %in% c("MF", "CC", "BP") && clean_parentals){
        enr_obj@fun <- "enrichGO"
        enr_obj <- clean_all_parentals(enr_obj, subont = funsys) 
      } 
      if (funsys %in% c("MF", "CC", "BP") && simplify_results){
        enr_obj@fun <- "enrichGO"
        enr_obj <- clusterProfiler::simplify(enr_obj) 
      } 
      enr_obj <- trycatch_pairwise_termsim(enr_obj)
    }                              
    enrichments_ORA_tr[[funsys]] <- enr_obj 
  }
  return(enrichments_ORA_tr)
}

#' @importFrom GO.db GOOBSOLETE
clean_GO_obsolete <- function(enr_obj) {
  obsolete_terms <- names(as.list(GO.db::GOOBSOLETE))

  if (is(enr_obj, "compareClusterResult")){
    enriched_table <- enr_obj@compareClusterResult
    enriched_table <- enriched_table[!enriched_table$ID %in% obsolete_terms,]
    enr_obj@compareClusterResult <- enriched_table
  } else if (is(enr_obj, "enrichResult")) {
    enriched_table <- enr_obj@result
    enriched_table <- enriched_table[!enriched_table$ID %in% obsolete_terms,]
    enr_obj@result <- enriched_table
  } 
  return(enr_obj)
}

#' @importFrom GO.db GOBPANCESTOR GOMFANCESTOR GOCCANCESTOR
clean_all_parentals <- function(enr_obj, subont){
   ##ADD control for enrichresults or comparecluster
  if (subont=="BP"){
    GO_ancestors <- GO.db::GOBPANCESTOR
  } else if (subont=="MF"){
    GO_ancestors <-  GO.db::GOMFANCESTOR
  } else if (subont=="CC"){
    GO_ancestors <- GO.db::GOCCANCESTOR
  }
  GO_ancestors <- as.list(GO_ancestors)
  GO_ancestors <- GO_ancestors[!is.na(GO_ancestors)]
  enr_obj <- clean_GO_obsolete(enr_obj)
  enrich_obj <- enr_obj@compareClusterResult
  pre_hamming_m <- matrix(0, nrow = length(unique(enrich_obj$Cluster)), 
                              ncol = length(unique(enrich_obj$ID)), 
                            dimnames= list(unique(enrich_obj$Cluster), 
                                          unique(enrich_obj$ID)))

  for(pair in seq(nrow(enrich_obj))){
    pair_text <- enrich_obj[pair, c("Cluster","ID")]
    pair_text$Cluster <- as.character(pair_text$Cluster)
    pre_hamming_m[pair_text[1,1], pair_text[1,2]] <- 1
  }

  hamming_matrix <- hamming_binary(pre_hamming_m)
  hamming_0 <- as.data.frame(as.table(hamming_matrix)) 
  hamming_0 <- hamming_0[hamming_0[,3] == 0,c(1,2)]
  hamming_0 <- hamming_0[hamming_0[,1] != hamming_0[,2], ]
  hamming_0[] <- lapply(hamming_0, "as.character")
  terms_to_discard <- c()

  hamming_DT <- data.table::setDT(hamming_0, key="Var1")
  to_remove_lapply <- lapply(unique(hamming_DT$Var1), function(id) {
    ancs <- GO_ancestors[[id]]
    id_pairs <- hamming_DT[hamming_DT$Var1 == id,]$Var2
    ancs[ancs %in% id_pairs]
  })
  to_remove_lapply <- unique(unlist(to_remove_lapply))
  enr_obj@compareClusterResult <- 
    enrich_obj[!enrich_obj$ID %in% to_remove_lapply,]
  return(enr_obj)
}

#' @importFrom GO.db GOTERM
get_GOid_term <- function(GOid, output = "term"){
 term <- AnnotationDbi::Term(GO.db::GOTERM[GOid])
 names(term) <- NULL
 return(term)
}


cluster_enr_to_matrix <- function(compareClusterResult){
   compareClusterResult <- compareClusterResult[,c("Cluster", "Description", 
    "p.adjust")]
    compareClusterResult_tmp <-  reshape2::acast(compareClusterResult, 
      Description~Cluster, value.var="p.adjust", fill = 1)
    compareClusterResult <- apply(compareClusterResult_tmp, 2, FUN=as.numeric)
    dimnames(compareClusterResult) <- dimnames(compareClusterResult_tmp)
    return(compareClusterResult_tmp)
}

calc_all_paths <- function(ids, GO_parents){
    env <- environment()
    paths <- list()
    calc_path(ids = ids, GO_parents = GO_parents, env = env)
    return(paths)
}

calc_path <- function(ids, GO_parents, env){
  for (id in ids) {
    if (is.null(env$paths[[id]])) {
      env$paths[[id]] <- list()
      direct_ancestors <- GO_parents[[id]]
      direct_ancestors <- direct_ancestors[names(direct_ancestors) == "isa"]
      if (length(direct_ancestors) == 0) {
        env$paths[[id]] <- c(env$paths[[id]], list(c(id)))
      } else {
        for (anc in direct_ancestors) {
          calc_path(ids = anc, GO_parents = GO_parents,env = env)
          env$paths[[id]] <- c(env$paths[[id]], 
                               lapply(env$paths[[anc]], function(path){
                                c(id,unlist(path))}))
        }
      }
    }
  }
}

vectdist <- function(vectA, vectB){
  # VectA and B must have same length. Exception not handled
  return(sqrt(sum((vectA - vectB)^2)))
}

toDistances <- function(vectors_matrix, rows = TRUE){
  if(!rows){
    vectors_matrix = t(vectors_matrix)
  }
  # Calc similitudes of rows
  numItems = nrow(vectors_matrix)
  Mdist = matrix(Inf,nrow = numItems, ncol = numItems)
  invisible(lapply(seq(numItems), function(i){
    if(i != numItems){
      invisible(lapply(seq(i+1, numItems), function(j){
        v = vectdist(vectors_matrix[i,],vectors_matrix[j,])
        Mdist[i,j] <<- v
        Mdist[j,i] <<- v
      }))
    }
    Mdist[i,i] <<- 0
  }))
  return(Mdist)
}

#' @importFrom GO.db GOBPANCESTOR GOMFANCESTOR GOCCANCESTOR
clean_parentals_in_matrix <- function(enrichment_mx, subont){
  if (subont=="BP"){
    GO_ancestors <- GO.db::GOBPANCESTOR
  } else if (subont=="MF"){
    GO_ancestors <-  GO.db::GOMFANCESTOR
  } else if (subont=="CC"){
    GO_ancestors <- GO.db::GOCCANCESTOR
  }
  for (cluster in colnames(enrichment_mx)) {

    enriched_cats <- rownames(enrichment_mx)
    for (category_ref in enriched_cats){
       parental_terms <- enriched_cats %in% GO_ancestors[[category_ref]]

       if (any(parental_terms)) {

          enrichment_mx[parental_terms, cluster] <- 1
       }

    }
  }

return(enrichment_mx)
}


obtain_gene_attributes <- function(gene_attribute_file, org_db, keytype) {
  gene_attributes <- read.table(gene_attribute_file, header=TRUE)
  gene_attribute_name <- colnames(gene_attributes)[3]
  ids <- translate_ids_orgdb(ids=gene_attributes[,2], input_id = keytype,
           org_db = org_db)
  ids <- unique(ids)
  gene_attributes <- merge(gene_attributes, ids, by.x = "geneid", 
    by.y = keytype)
  gene_attributes <- gene_attributes[!is.na(gene_attributes$ENTREZID),]
  gene_attributes <- gene_attributes[,2:4]
  gene_attr_list <- split(gene_attributes, gene_attributes$cluster)
  gene_attr_list <-lapply(gene_attr_list, function(gene_attr){
    gene_map <- gene_attr[, gene_attribute_name]
    names(gene_map) <- gene_attr$ENTREZID
    return(gene_map)
  })
  return(list(gene_attribute_name = gene_attribute_name, 
    gene_attributes = gene_attr_list))
}

#' @importFrom clusterProfiler simplify
parse_results_for_report <- function(enrichments, simplify_results = FALSE){
  enrichments_for_reports <- list()
  for (funsys in names(enrichments)){
    for (cluster in names(enrichments[[funsys]])){
      if (is.null(enrichments_for_reports[[cluster]]))
      enrichments_for_reports[[cluster]] <- list()
      enr <- enrichments[[funsys]][[cluster]]
      if (funsys %in% c("CC","MF","BP")){
        if (simplify_results) 
          enr <- clusterProfiler::simplify(enr) 
      }   
      if (length(enr$Description) > 2 ) 
      enr <- trycatch_pairwise_termsim(enr)
      enrichments_for_reports[[cluster]][[funsys]] <- enr 
    }
  }
  return(enrichments_for_reports)
}



#' @importFrom ggridges geom_density_ridges
#' @importFrom ggplot2 ggplot aes theme_classic
enrich_density <- function(enrich_result, 
                           attributes, 
                           showCategory = 30) {

  terms_attr <- get_attr_by_terms(enrich_result, attributes)
  enrich_result <- enrichplot:::fortify.internal(
                                  enrich_result,
                                  showCategory = showCategory,
                                  by = "p.adjust",
                                  )

  enriched_dist <- merge(terms_attr, enrich_result, by ="ID", all.y = TRUE)
  plot <- ggplot2::ggplot(enriched_dist, ggplot2::aes(x = attribute, 
                                              y = Description, 
                                              fill = p.adjust)) + 
  ggridges::geom_density_ridges(jittered_points = TRUE, 
                                position = "raincloud", 
                                alpha = 0.7, 
                                scale = 0.9, 
                                quantile_lines = TRUE) + 
  ggplot2::theme_classic()

  return(plot)
}

get_attr_by_terms <-function(enrich_result, attributes){
  term_list <- split(enrich_result@result, enrich_result@result$ID)
  gene2Symbol <- enrich_result@gene2Symbol
  attr_list <- lapply(term_list, get_term_attr, 
                      attributes = attributes, 
                      gene2Symbol = gene2Symbol)
 
  term_attr <- data.table::rbindlist(attr_list, 
                                     use.names = TRUE, 
                                     idcol = "ID")
  return(as.data.frame(term_attr))
}

get_term_attr <- function(term_data, attributes, gene2Symbol) {
  g_symbols <- unlist(stringr::str_split(term_data$geneID,"/"))
  g_id <- names(gene2Symbol)[match(g_symbols, gene2Symbol)]
  g_attr <- attributes[match(g_id, names(attributes))]
  names(g_attr) <- NULL
  return(data.frame(attribute = g_attr))
}

test_that("Multienricher gsea gives same results as running packets separately with universe", {

  funsys <- "BP"
  org_db <- "org.Mm.eg.db"
  gene_id <- "entrez"

 # entrez_to_enrich <- c("13853", "16819", "12266", "497097", "19062")
 # entrez_to_enrich_kegg <- c("16176", "18049", "18211", "193034", "277328")

  genes <- c("11465", "11472", "12154", "12335", "12372", 
    "12373")

  universe <- c("15118", "16873", "12483", "15171", "67490", "18787", "11911", 
    "258390", "109019", "381484", "14377", "56706", "231474", "54427", 
    "192176", "70427", "20293", "13559", "66508", "14804", "19224", 
    "381290", "12909", "54633", "619846", "22234", "22160", "12464", 
    "101497", "16193", "258952", "503491", "208043", "12034", "629378", 
    "20355", "19106", "73679", "19824", "18596", "230857", "84505", 
    "14173", "11813", "19229", "13555", "68268", "229521", "12558", 
    "16869")

  universe <- unique(c(genes, universe))

  organisms_table <- get_organism_table()
  current_organism_info <- subset(organisms_table, 
                          rownames(organisms_table) == "Mouse")

# start_time <- proc.time()
  new_res_tg <- ExpHunterSuite::multienricher_topGO(all_funsys=funsys, universe=universe,
    genes_list=list(genes, genes),
    organism_info=current_organism_info)
# end_time <- proc.time()
# pack_time_taken <- end_time - start_time
# cat(pack_time_taken, file="~/new_topgo_time_taken_5.txt")

  library(topGO)
  funsys <- "BP"
  org_db <- "org.Mm.eg.db"
  gene_id <- "entrez"

  #go_to_genes <- topGO::annFUN.org(funsys, mapping = org_db, ID = gene_id)

  geneList <- factor(as.integer(universe %in% genes))
  names(geneList) <- universe

  GOdata <- new("topGOdata",
                   ontology = funsys,
                   allGenes = geneList,
                   nodeSize = 5,
                   mapping = org_db,
                   annotationFun = annFUN.org,
                   ID = gene_id)

  resultFis <- topGO::runTest(GOdata, algorithm = "classic", statistic = "fisher")


  testthat::expect_identical(object=resultFis, expected=new_res_tg[["BP"]][[2]])


})

  test_that("Multienricher gives same results as running packets separately", {

  entrez_to_enrich_GO_react <- c("13853", "16819", "12266", "497097", "19062")
  entrez_to_enrich_kegg <- c("16176", "18049", "18211", "193034", "277328")

  packet_res_GO <- clusterProfiler::enrichGO(gene=entrez_to_enrich_GO_react, 
    OrgDb= "org.Mm.eg.db", 
    pAdjustMethod = "BH",
    ont = "BP",
    pvalueCutoff  = 0.05, qvalueCutoff = 0.2, 
    readable = FALSE)

  packet_res_reactome <- ReactomePA::enrichPathway(gene=entrez_to_enrich_GO_react, 
    organism      = "mouse",
    pvalueCutoff  = 0.05,
    pAdjustMethod = "BH",
    qvalueCutoff  = 0.2,
    readable = FALSE)

  packet_res_GO <- DOSE::setReadable(packet_res_GO, OrgDb = "org.Mm.eg.db", 
        keyType="ENTREZID")

  packet_res_reactome <- DOSE::setReadable(packet_res_reactome, OrgDb = "org.Mm.eg.db", 
        keyType="ENTREZID")

  organisms_table <- get_organism_table()
  current_organism_info <- subset(organisms_table, 
                          rownames(organisms_table) == "Mouse")
 
#  new_res_all <- ExpHunterSuite::multienricher_ora(all_funsys=c("BP", "Reactome", "KEGG"),
  new_res_all <- ExpHunterSuite::multienricher_ora(all_funsys=c("BP", "Reactome"),
    genes_list=list(entrez_to_enrich_GO_react, entrez_to_enrich_kegg), 
    organism_info = current_organism_info,
#    kegg_file=kegg_file_mouse,
    pvalueCutoff = 0.05, qvalueCutoff = 0.2
  )


  testthat::expect_identical(object=packet_res_GO, expected=new_res_all[["BP"]][[1]])
})

test_that("Multienricher gives same results as running packets separately using universe", {

  organisms_table <- get_organism_table()
  current_organism_info <- subset(organisms_table, 
                           rownames(organisms_table) == "Mouse")

  genes <- c("11465", "11472", "12154", "12335", "12372", 
    "12373")

  universe <- c("15118", "16873", "12483", "15171", "67490", "18787", "11911", 
    "258390", "109019", "381484", "14377", "56706", "231474", "54427", 
    "192176", "70427", "20293", "13559", "66508", "14804", "19224", 
    "381290", "12909", "54633", "619846", "22234", "22160", "12464", 
    "101497", "16193", "258952", "503491", "208043", "12034", "629378", 
    "20355", "19106", "73679", "19824", "18596", "230857", "84505", 
    "14173", "11813", "19229", "13555", "68268", "229521", "12558", 
    "16869")
 
#  new_res_all <- ExpHunterSuite::multienricher_ora(all_funsys=c("BP", "Reactome", "KEGG"),
  new_res_all <- ExpHunterSuite::multienricher_ora(all_funsys=c("BP", "Reactome"),
    universe=c(genes,universe),
    genes_list=list(genes), 
    organism_info = current_organism_info,
#    kegg_file=kegg_file_mouse, 
    minGSSize=5,
    pAdjustMethod = "BH",
    pvalueCutoff = 0.05, qvalueCutoff = 0.2, readable = TRUE
  )

  packet_res_GO <- clusterProfiler::enrichGO(gene=genes,
    universe=c(genes,universe),
    OrgDb= "org.Mm.eg.db", minGSSize=5,
    pAdjustMethod = "BH",
    ont = "BP",
    pvalueCutoff  = 0.05, qvalueCutoff = 0.2, 
    readable = TRUE)

  packet_res_GO <- DOSE::setReadable(packet_res_GO, OrgDb = "org.Mm.eg.db", 
        keyType="ENTREZID")


  packet_res_reactome <- ReactomePA::enrichPathway(gene=genes, 
    universe=c(genes,universe),
    organism      = "mouse", minGSSize=5,
    pvalueCutoff  = 0.05,
    pAdjustMethod = "BH",
    qvalueCutoff  = 0.2,
    readable = TRUE)
  packet_res_reactome <- DOSE::setReadable(packet_res_reactome, OrgDb = "org.Mm.eg.db", 
        keyType="ENTREZID")

  testthat::expect_identical(object=packet_res_GO, expected=new_res_all[["BP"]][[1]])

})

test_that("Run c2e main", {
  organisms_table <- get_organism_table()
  current_organism_info <- organisms_table[rownames(organisms_table) %in% "Mouse",]
  org_db <- get_org_db(current_organism_info)

  input_file <- system.file("extData", "cluster_genes.txt", package = "ExpHunterSuite")
  enr_lists <- main_clusters_to_enrichment(input_file, org_db=org_db, 
    current_organism_info=current_organism_info, gene_keytype="ENSEMBL")


   testthat::expect_equal(
    as.data.frame(enr_lists$enrichments_ORA$MF$two)[1, "ID"], 
    "GO:0008097")
})

test_that("Multienricher custom category gives same results as old and packet implementations", {

	genes <- c("GeneB", "GeneC", "GeneE","GeneF")

  gmt_file_1 <- system.file("extData", "toy_categories_1.gmt", package = "ExpHunterSuite")
  gmt_file_2 <- system.file("extData", "toy_categories_2.gmt", package = "ExpHunterSuite")

  custom_term2gene_1 <- ExpHunterSuite::load_and_parse_gmt(gmt_file_1)
  custom_term2gene_2 <- ExpHunterSuite::load_and_parse_gmt(gmt_file_2)

  organisms_table <- get_organism_table()
  current_organism_info <- subset(organisms_table, 
                        rownames(organisms_table) == "Mouse")

  packet_res_custom_1 <- clusterProfiler::enricher(genes, 
                            pvalueCutoff = 0.2,
                            pAdjustMethod = "BH",
                   		      qvalueCutoff  = 0.2,
                            minGSSize = 3,
                            TERM2GENE = custom_term2gene_1)

  # deliberately chosen to return no enrichment
  packet_res_custom_2 <- clusterProfiler::enricher(genes, 
                            pvalueCutoff = 0.2,
                            pAdjustMethod = "BH",
                            qvalueCutoff  = 0.2,
                            minGSSize = 3,
                            TERM2GENE = custom_term2gene_2)

 res_all <- ExpHunterSuite::multienricher_ora( 
    genes_list=genes, minGSSize = 3, organism_info = current_organism_info,
  	custom_sets = list(custom_term2gene_1 = custom_term2gene_1, custom_term2gene_2 = custom_term2gene_2),
    qvalueCutoff  = 0.2, pvalueCutoff = 0.2, readable=FALSE, return_all=TRUE)

  testthat::expect_equal(packet_res_custom_1, res_all[["custom_term2gene_1"]])
  testthat::expect_equal(packet_res_custom_2, res_all[["custom_term2gene_2"]])

})

test_that("new main functional enrichment with translation", {

  annot_file <- system.file("extData", "ensembl_symbol_annot_external.txt", 
                           package="ExpHunterSuite")
  annot_table <-  read.table(annot_file, header=FALSE, row.names=NULL, 
        sep="\t", stringsAsFactors = FALSE, quote = "")

  ext_data_file <- system.file("extData", "table_of_counts_external.txt", 
                          package="ExpHunterSuite")
  ext_data <- read.table(ext_data_file, 
                    header=TRUE, row.names=1, sep="\t")
  degh_out <- main_degenes_Hunter(external_DEA_data=ext_data,
                                  modules="F",
                                  lfc=4)


  # save(list = ls(all.names = TRUE), file = "~/environment_test10.RData")

  fh_out_new <- main_functional_hunter( #Perform enrichment analysis
    hunter_results = degh_out,
    model_organism = 'Zebrafish',
    annot_table = annot_table,
    organisms_table = get_organism_table(),
    input_gene_id = "ENSEMBL",
    #func_annot_db = "gR",
    #GO_subont = "BMC",
    #analysis_type = "o", #g
    enrich_dbs = c("BP", "MF", "Reactome"),
    enrich_methods = c("ORA"),
    custom = NULL,
    pthreshold = 0.1,
    qthreshold = 0.2,
    cores = 1,
    task_size = 1,
    output_files = "results",
    fc_colname = "mean_logFCs"
  )

  testthat::expect_equal(nrow(fh_out_new$ORA$BP), 51)
})

test_that("Load and translate GMT file", {


  organisms_table <- get_organism_table()
  current_organism_info <- organisms_table[rownames(organisms_table) %in% "Mouse",]
  org_db <- get_org_db(current_organism_info)

  custom_file <- system.file("extData", "cytokine.gmt", package = "ExpHunterSuite")
  custom_gmt <- ExpHunterSuite::load_and_parse_gmt(custom_file)
  tr_gmt <- translate_gmt(custom_gmt, "SYMBOL", org_db)
})

# test_that("custom functional enrichment function works ora", {

#   precomp_degh_res_file <- system.file("extData", "testdata", "precomp_expression_results.RData", 
#                            package="ExpHunterSuite")
#   load(precomp_degh_res_file)

#   gmt_file_1 <- system.file("extData", "toy_categories_test.gmt", package = "ExpHunterSuite")

#   custom_term2gene_1 <- ExpHunterSuite::load_and_parse_gmt(gmt_file_1)
#   custom_term2gene_2 <- ExpHunterSuite::load_and_parse_gmt(gmt_file_1)


#   custom_gmts <- list(gmt_file_1 = custom_term2gene_1, gmt_file_2=custom_term2gene_2)
#   print(precomp_degh_out$DE_all_genes)  
   

#   fh_out_new <- main_functional_hunter( #Perform enrichment analysis
#     hunter_results = precomp_degh_out,
#     model_organism = 'Mouse',
#     annot_table = NULL,
#     organisms_table = get_organism_table(),
#     input_gene_id = "ENSEMBL",
#     #func_annot_db = "gR",
#     #GO_subont = "C",
#     #analysis_type = "o", #g
#     enrich_dbs = c("MF", "Reactome"),
#     #enrich_methods = c("ora", "gsea", "topGO")
#     enrich_methods = "ORA",
#     custom = custom_gmts,
#     save_query = FALSE,
#     pthreshold = 0.9,
#     qthreshold = 0.9,
#     cores = 1,
#     task_size = 1,
#     output_files = "results",
#     fc_colname = "mean_logFCs"
#   )



#   fh_out <- functional_hunter( #Perform enrichment analysis
#     hunter_results = precomp_degh_out,
#     model_organism = 'Mouse',
#     annot_table = NULL,
#     organisms_table = get_organism_table(),
#     input_gene_id = "E",
#     func_annot_db = "gR",
#     GO_subont = "M",
#     custom = custom_gmts,
#     analysis_type = "o", #g
#     remote = "",
#     save_query = FALSE,
#     pthreshold = 0.9,
#     qthreshold = 0.9,
#     cores = 1,
#     task_size = 1,
#     output_files = "results",
#     fc_colname = "mean_logFCs"
#   )

#   save(list = ls(all.names = TRUE), file = "~/environment_test11.RData")
#   testthat::expect_equal(fh_out_new$CUSTOM$gmt_file_1, fh_out$CUSTOM$gmt_file_1)

# })


# test_that("Custom functional enrichment function works ora with cluster", {

#   precomp_degh_res_file <- system.file("extData", "testdata", "precomp_expression_results.RData", 
#                            package="ExpHunterSuite")
#   load(precomp_degh_res_file)

#   precomp_degh_out$DE_all_genes$genes_tag[1:8] <- "PREVALENT_DEG"

#   precomp_degh_out$DE_all_genes <- cbind(precomp_degh_out$DE_all_genes, 
#     Cluster_ID = c(1,1,1,1, 2,0,2,2, sample(0:2, 8, replace=TRUE)))
   
#   gmt_file_1 <- system.file("extData", "toy_categories_test.gmt", package = "ExpHunterSuite")

#   custom_term2gene_1 <- ExpHunterSuite::load_and_parse_gmt(gmt_file_1)
#   custom_term2gene_2 <- ExpHunterSuite::load_and_parse_gmt(gmt_file_1)
#   custom_gmts <- list(gmt_file_1 = custom_term2gene_1, gmt_file_2=custom_term2gene_2)
   
#   fh_out_new <- main_functional_hunter( #Perform enrichment analysis
#     hunter_results = precomp_degh_out,
#     model_organism = 'Mouse',
#     annot_table = NULL,
#     organisms_table = get_organism_table(),
#     input_gene_id = "ENSEMBL",
#     #func_annot_db = "gR",
#     #GO_subont = "C",
#     #analysis_type = "o", #g
#     enrich_dbs = "NULL",
#     #enrich_methods = c("ora", "gsea", "topGO")
#     enrich_methods = "ORA",
#     custom = custom_gmts,
#     pthreshold = 0.9,
#     qthreshold = 0.2,
#     cores = 1,
#     task_size = 1,
#     output_files = "results",
#     fc_colname = "mean_logFCs"
#   )
#   testthat::expect_equal(data.frame(fh_out_new$ORA$gmt_file_1)$ID, "CAT_1")
# })

test_that("main functional enrichment function works GO and Reactome - mouse", {

  precomp_degh_res_file <- system.file("extData", "testdata", "precomp_expression_results.RData", 
                           package="ExpHunterSuite")
  load(precomp_degh_res_file)

  organisms_table_file <- system.file("external_data", "organism_table.txt", 
                          package="ExpHunterSuite")
  organisms_table <- get_organism_table(organisms_table_file)

  fh_ext_out <- main_functional_hunter( #Perform enrichment analysis
         precomp_degh_out,
         'Mouse', # Use specified organism database 
         organisms_table = organisms_table,
         enrich_dbs = c("MF", "BP","Reactome"), # Enrichment analysis for GO, KEGG and Reactome
         enrich_methods = "ORA",
  )
  expect_equal(nrow(fh_ext_out$ORA$Reactome), 12)
})

################### FUNCTIONS

#' @importFrom utils read.table
#' @importFrom stringr str_remove
load_DEGH_information <- function(execution_path){ 
  DEGH_exec <- list()
  DEGH_exec[['DH_results']] <- utils::read.table(
                                     file.path(execution_path, 
                                     "Common_results/hunter_results_table.txt"),
                                     header=TRUE, row.names=1, sep="\t")
  DEGH_exec[['DH_results']]$gene_name <- rownames(DEGH_exec[['DH_results']])
  rownames(DEGH_exec[['DH_results']]) <- NULL
  DEGH_exec$DH_results <- DEGH_exec$DH_results[
                                 DEGH_exec$DH_results$gene_name != "id",]
  DEGH_exec[['Eigengene']] <- as.matrix(utils::read.table(
                                  file.path(execution_path, 
                                  "Results_WGCNA/eigen_values_per_samples.txt"),
                                  header=TRUE, row.names=1, sep="\t"))
  colnames(DEGH_exec[["Eigengene"]]) <- stringr::str_remove(
                                             colnames(DEGH_exec[["Eigengene"]]),
                                             "ME")
  DEGH_exec[['normalized_counts']] <- as.matrix(utils::read.table(
                                 file.path(execution_path, 
                                 "Results_DESeq2/Normalized_counts_DESeq2.txt"),
                                 header=TRUE, row.names=1, sep="\t"))
  DEGH_exec[['normalized_counts']] <- t(DEGH_exec[['normalized_counts']]) 
  DEGH_exec[['hub_1']] <- get_hub_genes_by_MM(DEGH_exec[['normalized_counts']], 
                                              DEGH_exec[['DH_results']])
  return(DEGH_exec)
}


filter_DEGH_data <- function(DGH_data, MM_cutoff, tag_filter){

    DGH_results <- DGH_data$DH_results 
    if (tag_filter == "prevalent") {
        tags_to_filter <- c("PREVALENT_DEG")
    } else if (tag_filter %in% c("putative", "possible")){
        tags_to_filter <- c("PREVALENT_DEG", "POSSIBLE_DEG")
    }
    
    all_degs <- DGH_results$genes_tag %in% tags_to_filter

    DGH_data$Eigengene_0 <- DGH_data$Eigengene[,"0", drop = FALSE]
    if (tag_filter == "putative") {
        modules_with_DEGS <- unique(DGH_results[all_degs & 
            DGH_results$Cluster_MM >= MM_cutoff, "Cluster_ID"])
        #Removing module 0
        modules_with_DEGS <- modules_with_DEGS[modules_with_DEGS != 0]
        DGH_data$Eigengene <- DGH_data$Eigengene[,
               colnames(DGH_data$Eigengene) %in% modules_with_DEGS]
        DGH_data$hub_1 <- DGH_data$hub_1[,
               colnames(DGH_data$hub_1) %in% modules_with_DEGS]
        
        candidate_not_deg <- DGH_results$Cluster_ID %in% modules_with_DEGS &
                                           DGH_results$genes_tag == "NOT_DEG" &
                                           DGH_results$Cluster_MM >= MM_cutoff
        candidate_not_deg <- DGH_results[candidate_not_deg, "gene_name"]
        relevant_genes <- c(candidate_not_deg, DGH_results[all_degs, 
                                                            "gene_name"]) 

    }else {
        relevant_genes <- DGH_results[all_degs, "gene_name"] 
    }

    DGH_data$DH_results$relevant_genes <- DGH_results$gene_name %in% 
                                             relevant_genes
    DGH_data$normalized_counts <- DGH_data$normalized_counts[, 
                    colnames(DGH_data$normalized_counts) %in% relevant_genes ]
                    
    return(DGH_data)
}

#' @importFrom data.table as.data.table 
load_and_parse_multimir <- function(
    multimir_table = NULL, 
    multimir_path = NULL,
    selected_predicted_databases = c("diana_microt", "elmmo", "microcosm", 
                        "miranda","mirdb", "pictar", "pita", "targetscan"),
    database_to_filter = NULL,
    filter_db_theshold = 0
    ) {
    multimir_summary <- NULL
    if (is.null(multimir_table) && file.exists(multimir_path)) {
        load(multimir_path)
        multimir_table <- multimir_summary 
    } 
    if (is.null(multimir_table)){
        stop("You have to give a parsed multiMiR data frame or an RData path")
    }
    multimir_table <- as.data.frame(multimir_table)
    dbs <- list(
    "validated_c" = c("mirecords", "mirtarbase", "tarbase"),
    "predicted_c" = selected_predicted_databases
    )
    multimir_table[["validated_c"]] <- rowSums(multimir_table[,
        dbs[["validated_c"]]], na.rm = TRUE)
    multimir_table[["predicted_c"]] <- rowSums(multimir_table[,
        dbs[["predicted_c"]]] > 0, na.rm = TRUE)
    gc()
    if ( ! is.null(database_to_filter) && filter_db_theshold > 0) {
      multimir_table <-  multimir_table[
      rowSums(!is.na(multimir_table[,database_to_filter])
      ) > filter_db_theshold | multimir_table$validated_c > 0,]
    }
    raw_databases_scores <- lapply(selected_predicted_databases, 
      function(database){
          database_scores <- multimir_table[,database]
          database_scores <- database_scores[!is.na(database_scores)]
      })
    names(raw_databases_scores) <- selected_predicted_databases
    return(list(raw_databases_scores = raw_databases_scores,
    multimir_table = data.table::as.data.table(multimir_table)))
}

parse_strategies <- function(strategies){
    strat_equivalence <- list(
    "d" = "normalized_counts",
    "E" = "Eigengene",
    "h" = "hub_1")
    parsed_strategies <- c()
    for (strategy in strategies) {
      splitted_strategy <- unlist(strsplit(strategy, ""))
      RNA_profile <- strat_equivalence[[splitted_strategy[1]]]
      miRNA_profile <- strat_equivalence[[splitted_strategy[2]]]
      parsed_strategies <- c(parsed_strategies, paste0(RNA_profile, 
          "_RNA_vs_miRNA_", miRNA_profile))
    }
    return(parsed_strategies)
}


#' @importFrom utils read.table
load_selected_targets <- function(selected_targets_file){
    selected_targets <- NULL
    if (!is.null(selected_targets_file))
        selected_targets <- utils::read.table(selected_targets_file)[,1] 
    return(selected_targets)
}

#' @importFrom data.table as.data.table
#' @importFrom tidyr unite
#' @importFrom plyr rbind.fill
perform_all_strategies <- function(
strat_names, 
RNAseq, 
miRNAseq, 
all_pairs,
tag_filter,
selected_predicted_databases,
raw_databases_scores,
MM_cutoff = 0.7,
permutations = 10, 
sample_proportion = 0.01,
selected_targets = c(),
corr_type,
corr_coef,
compare_pred_scores = FALSE,
eval_method = "aggregated"
){
 ######### PREPARE PAIRS SCAFOLD
 if (is.null(RNAseq)) {
    if(!is.null(selected_targets)){
        strat_names <- c("selected_targets_RNA_vs_miRNA_DEMs")
        message(paste0("RNAseq folder has been not set, only ",
 "selected_targets_RNA_vs_miRNA_DEMs strategy will be performed"))
    } else {
        strat_names <- c("all_posible_targets_RNA_vs_miRNA_DEMs")
        message(paste0("RNAseq folder has been not set, only ",
        " strategy will be performed"))
        selected_targets <- unique(all_pairs$RNAseq)
    }
} 
 strategies <- list()
 tag_filter <- unlist(strsplit(tag_filter, ","))
 if (length(tag_filter) == 1)
 tag_filter <- c(tag_filter,tag_filter)
 ###### FILTER DGH DATA
 RNAseq <- filter_DEGH_data(RNAseq, MM_cutoff, tag_filter[1])
 miRNAseq <- filter_DEGH_data(miRNAseq, MM_cutoff, tag_filter[2])
 message("Data has been filtered")
 gc()
 std_positions <- tidyr::unite(all_pairs, "pairs", RNAseq:miRNAseq, sep = "_")
 std_positions <- std_positions$pairs
 

 ###### BUILD EMPTY TABLES
 #Vector with strategies without significant results
 unsig_strategies <- c() 
 #Dataframe with all contingency tables
 cont_tables <- data.frame()  
 #Datafarme with stats of strategy pairs and randoms 
 miRNA_cont_tables <- data.frame()
 filters_summary <- data.frame()
 #Dataframe with stats from prediction scores comparison
 score_comp<- data.frame()
 #Dataframe with weighted fisher pvalues
 p_fisher <- data.frame()

 ####PERFORM STRATEGIES
 for(strategy in strat_names){ 
   message(paste0("\n", strategy, " strategy is been performed"))
    message("Computing stats by group")
    strategy_data <- perform_correlations(strategy = strategy, 
                    RNAseq = RNAseq, miRNAseq = miRNAseq, 
                    std_positions = std_positions, corr_coef = corr_coef,
                    corr_type  = corr_type, selected_targets = selected_targets)



   strategy_data <- data.table::as.data.table(strategy_data)
   
   # if (sum(strategy_data$correlated_pairs) == 0){
   #     unsig_strategies <- c(strategy, unsig_strategies)
   #     next
   # }
   strategies[[strategy]] <- strategy_data
  
   # all_pairs[,strategy] <- FALSE
   # all_pairs[strategy_data$pair_n, strategy] <- strategy_data$correlated_pairs
   
   all_pairs[strategy_data$pair_n, 
              paste0(strategy,"_correlation")] <- strategy_data$correlation
   all_pairs[strategy_data$pair_n, 
              paste0(strategy,"_pval")] <- strategy_data$pval
  
 }

 all_pairs <- data.table::as.data.table(all_pairs)
 # if (all(names(strategies) %in% unsig_strategies)){
 #  stop("There is not significant pairs for any strategy selected.")
 # } 

  # all_cor_dist <- parse_correlations(strategies)
  # sig_pairs <- get_sig_pairs(strategies, all_pairs)
 miRNA_cor_results <- list(strategies = strategies,
    # unsig_strategies =  unsig_strategies, 
   all_pairs = all_pairs 
   #all_cor_dist = all_cor_dist,   sig_pairs = sig_pairs
   )
 return(miRNA_cor_results)
}





get_degs_sets <- function(DH_results_table){

     degs <- list()
     degs[["pos"]] <- DH_results_table[DH_results_table$genes_tag %in%
               c("PREVALENT_DEG", "POSSIBLE_DEG") & 
                DH_results_table$mean_logFCs > 0, "gene_name"]
     degs[["neg"]] <- DH_results_table[DH_results_table$genes_tag %in%
               c("PREVALENT_DEG", "POSSIBLE_DEG") &
                DH_results_table$mean_logFCs < 0, "gene_name"]  
    return(degs)
}


prepare_for_stats <- function(
all_pairs, 
strategy, 
corr_cutoff, 
p_val_cutoff,
db_groups = c("predicted", "validated", "multimir"),
sig_pairs = FALSE,
crossval = FALSE, 
test_sample = 0.25,
corr_type = "lower"
){
 cont_tables <- data.frame()
 miRNA_cont_tables <- data.frame()
 # message("Computing contingency tables")

    strat_data <- all_pairs[,c("miRNAseq", "predicted", "validated", "multimir")]
    
    if (!sig_pairs){
        if (corr_type == "lower"){

        strat_data$sig_pairs <- all_pairs[,paste0(strategy,
                                            "_correlation")] <= corr_cutoff &
                           all_pairs[,paste0(strategy,
                                            "_pval")] <= p_val_cutoff &
                           all_pairs$known_miRNA
        } else if (corr_type == "higher"){
            strat_data$sig_pairs <- all_pairs[,paste0(strategy,
                                            "_correlation")] >= corr_cutoff &
                           all_pairs[,paste0(strategy,
                                            "_pval")] <= p_val_cutoff &
                           all_pairs$known_miRNA
        }

    } else {
        strat_data$sig_pairs <- all_pairs[,strategy]
    }

    strat_data$sig_pairs[is.na(strat_data$sig_pairs)] <- FALSE

    if (sum(strat_data$sig_pairs) == 0) return(NULL)
 
    strat_data <- data.table::as.data.table(strat_data)

    all_cts <- get_ct_by_group(strat_data = strat_data, 
                              db_groups = db_groups,
                              crossval = crossval, test_sample = test_sample)
    strat_miRNA_ct <-  all_cts$strat_miRNA_cts 
    strat_ct <-  all_cts$strat_cts
       
       if (nrow(strat_miRNA_ct) > 0) {
          strat_miRNA_ct$strategy <- strategy
          strat_miRNA_ct$corr_cutoff <- corr_cutoff
       }

       if (nrow(strat_miRNA_ct) > 0) {
          strat_ct$strategy <- strategy
          strat_ct$corr_cutoff <- corr_cutoff
      }

      return(list( strat_miRNA_ct = strat_miRNA_ct,
        strat_ct = strat_ct
        ))
}




get_cor_dist <- function(all_pairs, strategies){
    corr_dist <- data.frame()
    for (strategy in strategies){
        strat_corr <- data.frame(
        correlation = all_pairs[,paste0(strategy, "_correlation")],
        pval = all_pairs[,paste0(strategy, "_pval")],
        strategy = strategy)

        strat_corr <- strat_corr[!is.na(strat_corr$correlation),]   
        corr_dist <- rbind(corr_dist, strat_corr)
    }
    return(corr_dist)
}

#' @importFrom data.table as.data.table
perform_correlations <- function(
      strategy = "normalized_counts_RNA_vs_miRNA_normalized_counts", 
      RNAseq, miRNAseq, std_positions = NULL, corr_coef,
      corr_type, selected_targets = c()){ #correct_positions 
    #is a mirna_RNA pairs vector
    # the parsed strategy name text is used to subset RNAseq/miRNAseq obects
    
    if (strategy == "DEGs_RNA_vs_miRNA_DEMs_opp"){
      #   q()   
      
       DEGs <- get_degs_sets(RNAseq$DH_results)
       DEMs <- get_degs_sets(miRNAseq$DH_results)
        
       all_pairs <- permute_pairs(DEGs[["pos"]], DEMs[["neg"]], 
                                    corr_type = corr_type)
       all_pairs <- rbind(all_pairs, permute_pairs(DEGs[["neg"]], DEMs[["pos"]], 
                                                    corr_type = corr_type))

    } else if ( strategy == "DEGs_RNA_vs_miRNA_DEMs_sim" ){
       DEGs <- get_degs_sets(RNAseq$DH_results)
       DEMs <- get_degs_sets(miRNAseq$DH_results)
       
       all_pairs <- permute_pairs(DEGs[["pos"]], DEMs[["pos"]],
                                     corr_type = corr_type)
       all_pairs <- rbind(all_pairs, permute_pairs(DEGs[["neg"]], DEMs[["neg"]], 
                                                        corr_type = corr_type))
    
    } else if ( strategy %in% c("selected_targets_RNA_vs_miRNA_DEMs",
                                "all_posible_targets_RNA_vs_miRNA_DEMs")) {
        DEMS <- miRNAseq$DH_results[miRNAseq$DH_results$genes_tag %in% 
               c("PREVALENT_DEG", "POSSIBLE_DEG"), "gene_name"]    
        all_pairs <- permute_pairs(selected_targets, DEMS, corr_type = corr_type)
    } else {
        strat_description <- unlist(strsplit(strategy, "_RNA_vs_miRNA_"))
        RNA_profiles <- as.matrix(RNAseq[[strat_description[1]]])
        miRNA_profiles <- as.matrix(miRNAseq[[strat_description[2]]])
        if (ncol(RNA_profiles) == 0 || ncol(miRNA_profiles) == 0) {
          return(data.frame(NULL))
        }
        all_pairs <- correlate_profiles(RNA_profiles, miRNA_profiles, 
                                        corr_coef= corr_coef)
        if (strat_description[1] != "normalized_counts"){
            if (strat_description[1] == "Eigengene_0") {
                RNAseq$DH_results$relevant_genes <- TRUE
            }
            all_pairs <- expand_module(all_pairs = all_pairs, tag = "RNAseq", 
                DH_results = 
                RNAseq$DH_results[RNAseq$DH_results$relevant_genes, ])
        }
        if (strat_description[2] != "normalized_counts"){
            if (strat_description[2] == "Eigengene_0") {
                miRNAseq$DH_results$relevant_genes <- TRUE
            }
            all_pairs <- expand_module(all_pairs = all_pairs, tag = "miRNAseq", 
                DH_results = 
                miRNAseq$DH_results[miRNAseq$DH_results$relevant_genes, ])
        }
    }
    all_pairs <- data.table::as.data.table(all_pairs)

    # if(corr_type == "higher"){
    #         all_pairs$correlated_pairs <- all_pairs$correlation >= cor_cutoff & 
    #                                  all_pairs$pval < pval_cutoff
    # } else if ( corr_type == "lower"){
    #         all_pairs$correlated_pairs <- all_pairs$correlation <= cor_cutoff & 
    #                                 all_pairs$pval < pval_cutoff
    # } else {
    #     stop("Non valid --corr_type arguments")
    # }
    
     # Add extra column to indicate number of pair in std_positions
    all_pairs_str <- all_pairs[,c("RNAseq", "miRNAseq")]
    all_pairs_str <- tidyr::unite(all_pairs_str, "pairs", RNAseq:miRNAseq, 
                               sep = "_")
    all_pairs_str <- all_pairs_str$pairs  
    all_pairs$pair_n <- match(all_pairs_str, std_positions) 
    all_pairs[,c("RNAseq", "miRNAseq")] <- NULL

    return(all_pairs)
}


permute_selected_targets <- function(miRNAseq, selected_targets,corr_type){
    DEMS <- miRNAseq[miRNAseq$genes_tag %in% 
     c("PREVALENT_DEG", "POSSIBLE_DEG"), "gene_name"]
     all_pairs <- expand.grid(selected_targets, DEMS, 
                               stringsAsFactors= FALSE, 
                               KEEP.OUT.ATTRS = FALSE)
    colnames(all_pairs) <- c("RNAseq", "miRNAseq")
    if (corr_type == "higher"){
       all_pairs$correlation <- 1
    } else if (corr_type == "lower"){
       all_pairs$correlation <- -1
    }
    all_pairs$pval <- 0
    return(all_pairs)
}


permute_pairs <- function(RNAseq, miRNAseq, corr_type){
    all_pairs <- expand.grid(RNAseq, miRNAseq, 
                               stringsAsFactors= FALSE, 
                               KEEP.OUT.ATTRS = FALSE)
    colnames(all_pairs) <- c("RNAseq", "miRNAseq")
    if (corr_type == "higher"){
       all_pairs$correlation <- 1
    } else if (corr_type == "lower"){
       all_pairs$correlation <- -1
    }
    all_pairs$pval <- 0

    return(all_pairs)
}

#' @importFrom WGCNA corPvalueStudent
#' @importFrom stats cor
correlate_profiles <- function(RNA_profiles, miRNA_profiles,corr_coef) {

    if (corr_coef == "bicor") {
        correlations <- WGCNA::bicor(RNA_profiles, miRNA_profiles, 
                                maxPOutliers = round(ncol(RNA_profiles)/5, 0))    
    } else if (corr_coef %in% c("pearson", "spearman", "kendall")){

        correlations <- WGCNA::cor(RNA_profiles, miRNA_profiles, 
            method = corr_coef)
    }
    
    nSamples <- nrow(RNA_profiles)
    cor_pval <- as.data.frame(WGCNA::corPvalueStudent(
        as.matrix(correlations), nSamples))

    names_matrix <- as.matrix(correlations)  
    all_pairs <- as.data.frame(as.table(correlations)) 
    colnames(all_pairs) <- c("RNAseq", "miRNAseq", "correlation")
    
    all_pairs$correlation <- round(all_pairs$correlation, 2)

    all_pairs$pval <- as.data.frame(as.table(as.matrix(cor_pval)))$Freq

    all_pairs$miRNAseq <- as.character(all_pairs$miRNAseq)
    all_pairs$RNAseq <- as.character(all_pairs$RNAseq)
    return(all_pairs)
}



#' @importFrom dplyr desc between row_number filter arrange group_by
#' @importFrom rlang .data
get_hub_genes_by_MM <- function(normalized_counts, hunter_results, top = 1){
    #maybe we can change this function by  splitting all DGH by modules
    "%>%" <- magrittr::"%>%"
    hub_genes <- hunter_results %>% 
    dplyr::filter(.data$Cluster_MM_pval <= 0.05) %>% 
    dplyr::arrange(dplyr::desc(.data$Cluster_MM)) %>% 
    dplyr::group_by(.data$Cluster_ID) %>% 
    dplyr::filter(dplyr::between(dplyr::row_number(), 1, top))
    hub_genes_profile <- as.matrix(
      normalized_counts[,colnames(normalized_counts) %in% hub_genes$gene_name])
    
    colnames(hub_genes_profile) <- as.data.frame(hub_genes)[match(
                                            colnames(hub_genes_profile), 
                                            hub_genes$gene_name), "Cluster_ID"]
    return(hub_genes_profile)
}


#' @importFrom data.table as.data.table merge.data.table  
#' @importFrom rlang .data
expand_module <- function(all_pairs, tag, DH_results){
    mod_tag <- paste0(tag, "_mod")
    names(all_pairs)[names(all_pairs)== tag] <- mod_tag

    partial_expanded <- DH_results[,c("Cluster_ID", "gene_name")]
    partial_expanded <- data.table::as.data.table(partial_expanded)
    colnames(partial_expanded) <- c("module", tag)
    partial_expanded$module <- as.character(partial_expanded$module)
    all_pairs <- data.table::as.data.table(all_pairs)
    all_pairs <- data.table::merge.data.table(x = all_pairs, 
        y = partial_expanded, by.x = mod_tag, 
        by.y = "module", allow.cartesian  = TRUE)
    all_pairs[,mod_tag] <- NULL
    return(all_pairs)
}

#' @importFrom data.table as.data.table merge.data.table
add_multimir_info <- function(all_pairs, multiMiR = NULL) {
    all_pairs <- data.table::as.data.table(all_pairs)
    
    if (!is.null(multiMiR)){
     multiMiR <- data.table::as.data.table(multiMiR)
     all_pairs <- data.table::merge.data.table(all_pairs, 
        multiMiR, by.x = c("RNAseq","miRNAseq"), 
        by.y =c("target_ensembl","mature_mirna_acc"), all.x = TRUE, 
        suffixes = c("", ""), no.dups = TRUE)
    }
    all_pairs <- as.data.frame(all_pairs)
    for (pred_db in c("predicted_c", "validated_c")){

       all_pairs[is.na(all_pairs[,pred_db]), pred_db] <- 0

    }
    for (pred_db in c("mirecords", "mirtarbase", "tarbase")) {
       all_pairs[is.na(all_pairs[,pred_db]), pred_db] <- FALSE
    }


    all_pairs$predicted <- all_pairs$predicted_c > 0
    all_pairs$validated <- all_pairs$validated_c > 0
    all_pairs$pred_and_val <- all_pairs$predicted & all_pairs$validated
    all_pairs$multimir <- all_pairs$predicted | all_pairs$validated
    gc()
    all_pairs <- data.table::as.data.table(all_pairs)
 
    return(all_pairs)
}


#' @importFrom data.table as.data.table merge.data.table
get_entrez_symbol_translations <- function(ensembl_ids, organism_info){
    org_db <- get_org_db(organism_info)
    input_to_entrezgene <- translate_ids_orgdb(ids = ensembl_ids, 
                    input_id="ENSEMBL", output_id = "ENTREZID", org_db=org_db)
    # Fix names
    colnames(input_to_entrezgene) <- c("ensembl_gene_id", "entrezgene")     
    
    input_to_symbol <- translate_ids_orgdb(input_id="ENTREZID", 
                    output_id = "SYMBOL", org_db=org_db,
                    ids = unique(input_to_entrezgene$entrezgene))
    colnames(input_to_symbol) <- c("entrezgene", "Symbol")

    input_to_entrezgene <- data.table::as.data.table(input_to_entrezgene)
    input_to_symbol <- data.table::as.data.table(input_to_symbol)
    gene_id_translation <- data.table::merge.data.table(input_to_entrezgene, 
                             input_to_symbol, by = "entrezgene", all.x = TRUE)

    return(gene_id_translation)
}

score_comparison <- function(databases, all_pairs, sample_size, strategy){
  db_comp <- data.frame()
  all_pairs <- data.table::as.data.table(all_pairs)
  for (database in databases) {
    bckg_scores <- all_pairs[, get(database)]
    bckg_scores <- bckg_scores[!is.na(bckg_scores)]
    strat_scores <- all_pairs[get(strategy), get(database)] #Revise that
    strat_scores <- strat_scores[!is.na(strat_scores)]

    # sampled_bckg <- sample(bckg_scores, 
    #         size = 100, replace = FALSE)
    res <- data.frame(p.value = 1, boot.p.value = 1)
    if (length(strat_scores) > 1) {
      # res <- stats::t.test(x = strat_scores, y = bckg_scores, 
        res <- boot.t.test.2(x = strat_scores, y = bckg_scores, 
          alternative = "greater", 
          vectorial = FALSE, 
          R = 1000,
          chunk_size = 100)
    } 
    actual_db_comp <- data.frame(
                        database = database,
                        strategy = strategy,
                        boot.p.value = res$boot.p.value,
                        p.value = res$p.value)
    if (actual_db_comp$boot.p.value == 0){
      if(actual_db_comp$p.value == 0){
        actual_db_comp[,c("boot.p.value", "p.value")] <- 1e-16
      } else {
        actual_db_comp$boot.p.value <- actual_db_comp$p.value
      }
    }
    db_comp <- rbind(db_comp,actual_db_comp) 
  }
  db_comp <- rbind(
              db_comp, 
              data.frame(
                database = "comb_pval", strategy = strategy,
                boot.p.value = vectorial_fisher_method(db_comp$boot.p.value),
                p.value = vectorial_fisher_method(db_comp$p.value)
                ))
  return(db_comp)
}

get_miRNA_ct <- function(strat_data, db_group, crossval = FALSE, test_sample = 0.25, bootstrap = 40){
    db_gs <- strat_data[,get(db_group)]
    experiment_pairs <- strat_data[,sig_pairs]

    if (sum(experiment_pairs) == 0) return(NULL)

    if (crossval) {
        strat_group_ct <- data.frame()
        seed <- 1234
        for (bt_it in seq(1, bootstrap)){
                set.seed(seed + bt_it)
                db_gs_trues <- which(db_gs)
                db_gs_to_falsify <- sample(db_gs_trues, ceiling(test_sample*length(db_gs_trues)))

                db_gs[db_gs_to_falsify] <- FALSE
                              
                train_ct <- calc_ct(experiment = experiment_pairs, gold_standard = db_gs)
                train_ct$crossval_it <- bt_it
                train_ct$crossval_status <- "train"

                strat_group_ct <- rbind(strat_group_ct, train_ct)
        }

    } else {
     strat_group_ct <- calc_ct(experiment = experiment_pairs, 
                              gold_standard = db_gs)
    
    }




    strat_group_ct$db_group <- db_group
    return(strat_group_ct)
}


#' @importFrom plyr rbind.fill
#' @importFrom data.table setnames
get_ct_by_group <- function(
strat_data, 
db_groups = c("predicted", "validated", "multimir"),
crossval = FALSE, test_sample = 0.25
){
 strat_cts <- data.frame() #contingency table
 strat_miRNA_cts <- data.frame() #contingency table for every miRNA

 for (db_group in db_groups){

    strat_group_ct <- get_miRNA_ct(strat_data, db_group)
    strat_cts <- rbind(strat_cts, strat_group_ct)
   all_miRNA_ct <- lapply(split(strat_data, strat_data$miRNAseq), 
                            get_miRNA_ct, db_group = db_group, crossval = crossval,
                            test_sample = test_sample)
    strat_miRNA_cts <- rbind(strat_miRNA_cts, 
                              data.table::rbindlist(all_miRNA_ct, 
                                                    use.names = TRUE, 
                                                    idcol = "miRNA"))
 }
 return(list(strat_cts= strat_cts, 
    strat_miRNA_cts = strat_miRNA_cts))
}


prepare_all_pairs <- function(
RNAseq, 
miRNAseq, 
multimir,
selected_targets,
organism){
 
 

 if (is.null(RNAseq)){
    if (!is.null(selected_targets)) {
        putative_targets <- selected_targets
    } else {
        putative_targets <- unique(multimir[multimir$mature_mirna_acc  %in% 
                    miRNAseq$DH_results$gene_name, "target_ensembl"])[[1]]
    }
 } else {
    putative_targets <- colnames(RNAseq$normalized_counts)
 }
 all_pairs <- expand.grid(
                putative_targets, 
                colnames(miRNAseq$normalized_counts), 
                stringsAsFactors = TRUE)
 names(all_pairs) <- c("RNAseq", "miRNAseq")
 all_pairs <- add_multimir_info(all_pairs = all_pairs, 
                                multiMiR = multimir)

 
 all_pairs$known_miRNA <- grepl("MIMAT", all_pairs$miRNAseq)
 
 #possible_positives: Pairs with RNA and miRNA included in multiMiR. 
 possible_positives <- all_pairs$known_miRNA & 
                                 all_pairs$miRNAseq %in% 
                                  multimir$mature_mirna_acc &
                                 all_pairs$RNAseq %in% 
                                   multimir$target_ensembl 

 all_pairs$predicted <- all_pairs$predicted & possible_positives
 all_pairs$validated <- all_pairs$validated & possible_positives
 all_pairs$multimir <- all_pairs$predicted | all_pairs$validated

 return(all_pairs)
}

#' @importFrom miRBaseVersions.db miRBaseVersions.db
#' @importFrom AnnotationDbi select
translate_all_id <- function(
miRNA_IDs, 
RNA_IDs, 
organism_info, 
translate_ensembl){
 mirna_names <- unique(miRNA_IDs)
 mirna_names <- mirna_names[grepl("MIMAT", mirna_names)]
 mirna_names <-  AnnotationDbi::select(
                            miRBaseVersions.db::miRBaseVersions.db, 
                            keys = mirna_names, keytype =  "VW-MIMAT-22.0",
                            columns = c("ACCESSION", "NAME"))
 gene_id_translation <- NULL
 if (! organism_info$Bioconductor_VarName_SYMBOL[1] == "" && 
     translate_ensembl) {
     gene_id_translation <- get_entrez_symbol_translations(
         ensembl_ids = unique(RNA_IDs), 
         organism_info = organism_info)
 } else {
    message("RNA IDs were not translated")
 }
 return(list(mirna_names = mirna_names, 
    gene_id_translation= gene_id_translation))
}

#' @importFrom miRBaseVersions.db miRBaseVersions.db
#' @importFrom AnnotationDbi select
translate_miRNA_ids <- function(miRNA_IDs, miRBaseVersion = "VW-MIMAT-22.0") {
    miRNA_tr_table <- AnnotationDbi::select(
                            miRBaseVersions.db::miRBaseVersions.db, 
                            keys = miRNA_IDs, keytype = miRBaseVersion,
                            columns = c("ACCESSION", "NAME"))
    translated_miRNA <- miRNA_tr_table[match(miRNA_IDs, 
                                        miRNA_tr_table$ACCESSION) ,"NAME"]
    return(translated_miRNA)
}

#' @importFrom data.table rbindlist
parse_correlations <- function(strategies){
 all_cor_dist <- strategies
 all_cor_dist["DEGs_DEMs_permutated"] <- NULL
 all_cor_dist <- lapply(all_cor_dist, function(strategy_data){
    dist <- strategy_data[,c("correlation", "pval", "correlated_pairs")]
    return(dist)
 })
 all_cor_dist <- as.data.frame(data.table::rbindlist(all_cor_dist, 
                                                    use.names=TRUE, 
                                                    idcol= "strategy"))
 return(all_cor_dist)
}
 
get_sig_pairs <- function(strategies, all_pairs){
  groups_db <- c("all","multimir", "predicted", "validated", "pred_and_val")
  all_overlap <- list()
  strat_names <- names(strategies)
  strat_names <- strat_names[strat_names %in% colnames(all_pairs)]
  for (group_db in groups_db){
    group_matrix <- matrix(
      data = NA, nrow = length(strat_names), ncol = length(strat_names),
      dimnames = list(strat_names, strat_names))
    for(xstrategy in strat_names){
      for(ystrategy in strat_names){
        group <- rep(TRUE, nrow(all_pairs))
        if (group_db != "all"){
            group <- all_pairs[, get(group_db)]
        }
        group_matrix[xstrategy, ystrategy] <- sum(
          all_pairs[,get(xstrategy)] & all_pairs[,get(ystrategy)] & group)
      }
    }
    all_overlap[[group_db]] <- group_matrix
  }
  names(all_overlap) <- groups_db
  return(all_overlap)
}


calc_pred_ctables <- function(
 all_pairs,
 strategy,
 selected_predicted_databases)
{
 cont_tables <- data.frame()
 for (database in selected_predicted_databases){
   c_table <- calc_ct(experiment = all_pairs[,get(strategy)],
                       gold_standard = !is.na(all_pairs[,get(database)]))
   c_table$database <- database
   cont_tables <- rbind(cont_tables, c_table)
 }
 return(cont_tables)
}





select_best_strategy <- function(miRNA_cont_tables){
    #taken from https://stackoverflow.com/questions/24558328/select-the-row-with-the-maximum-value-in-each-group
    max_mIRNA_ID <- with(miRNA_cont_tables, ave(Odds_ratio, miRNA, FUN=function(x) seq_along(x) == which.max(x))) == 1
    miRNA_cont_tables_max <- miRNA_cont_tables[max_mIRNA_ID,]
    return(miRNA_cont_tables_max)
}


set_strats_readable <- function(strategies){
dictionary <- list(
    "Eigengene" = "E",
    "Eigengene_0" = "E0",
    "hub_1" = "H",
    "normalized_counts" = "C"
    )
  translated_strategies <- c()
  # str(dictionary)
  for (strategy_name in strategies){

    strategies <- unlist(strsplit(strategy_name, "_RNA_vs_miRNA_"))
    summ_name <- c()
    for (strategy in strategies) {
        if (!is.null(dictionary[[strategy]])){
            summ_name <- c(summ_name, dictionary[[strategy]])
        } else {
            if(stringr::str_detect(strategy, "_opp")) strategy <- gsub("_opp", " (opp)", strategy)
            if(stringr::str_detect(strategy, "_sim")) strategy <- gsub("_sim", " (sim)", strategy)
            summ_name <- c(summ_name, strategy)
        }
    }
    translated_str <- paste0(summ_name, c("g ","m"), collapse = "")
    translated_strategies <- c(translated_strategies, translated_str)
  }
  return(translated_strategies)
}



 get_optimal_pairs <- function(all_pairs, miRNA_cont_tables, pval_thr, corr_type){
    all_pairs <- as.data.frame(all_pairs)
   selected_miRNA_strat <- select_best_strategy(miRNA_cont_tables)
   miRNA_strat_pairs <- rep(FALSE, nrow(all_pairs))
   for(selected_miRNA in selected_miRNA_strat$miRNA) {
         best_miRNA_strat <- selected_miRNA_strat[selected_miRNA_strat$miRNA == selected_miRNA,]
      strat_corr <- paste0(best_miRNA_strat$strategy, "_correlation")
      strat_pval <- paste0(best_miRNA_strat$strategy, "_pval")
      if (corr_type == "lower"){
             miRNA_best_pairs  <- all_pairs$miRNAseq == selected_miRNA &
                        all_pairs[,strat_corr] <= best_miRNA_strat$corr_cutoff &
                        all_pairs[,strat_pval] <= pval_thr
      } else if (corr_type == "higher"){
                 miRNA_best_pairs  <- all_pairs$miRNAseq == selected_miRNA &
                        all_pairs[,strat_corr] >= best_miRNA_strat$corr_cutoff &
                        all_pairs[,strat_pval] <= pval_thr
      }
      miRNA_strat_pairs <- miRNA_strat_pairs | miRNA_best_pairs 
   }
   return(miRNA_strat_pairs)
 }



 add_attrib_to_pairs <- function(pairs, RNAseq = NULL, miRNAseq = NULL, translate_targets = FALSE){
    fields_to_extract <- c("miRNA", "miRNAseq", "RNAseq", "validated_c","predicted_c")
    if ("miRNA_loci" %in% colnames(pairs))
        fields_to_extract <- c(fields_to_extract, "miRNA_loci")  
 
       output_pairs <- pairs[,fields_to_extract]

    if (!is.null(RNAseq)) {
        RNA_data <- RNAseq$DH_results[,c("gene_name","mean_logFCs","genes_tag")]
        colnames(RNA_data) <- c("RNAseq", "Target_log2FC", "DEG_tag_target")
        output_pairs <- merge(output_pairs, RNA_data, by = "RNAseq", all.x = TRUE)
    }
    if (!is.null(miRNAseq)) {
        miRNA_data <- miRNAseq$DH_results[,c("gene_name","mean_logFCs","genes_tag")]
        colnames(miRNA_data) <- c("miRNAseq", "miRNA_log2FC", "DEM_tag_miRNA")
        output_pairs <- merge(output_pairs, miRNA_data, by = "miRNAseq", all.x = TRUE)
    }
    colnames(output_pairs)[match(c("miRNAseq", "RNAseq", "validated_c","predicted_c"),
                                   colnames(output_pairs))] <- 
            c("miRNA_ID", "Target_ID","Validated_DB_count", "Predicted_DB_count")

    output_pairs$All_DB_count <- output_pairs$Validated_DB_count + output_pairs$Predicted_DB_count


    if ("miRNA_loci" %in% colnames(output_pairs))
    colnames(output_pairs)[colnames(output_pairs) == "miRNA_loci"] <- "miRNA_source_gene"
    
    return(output_pairs)
}

#' @importFrom stats complete.cases pt qt quantile setNames
boot.t.test.2 <- function(
x, 
y = NULL, 
alternative = c("two.sided", "less", "greater"), 
mu = 0, 
paired = FALSE, 
var.equal = FALSE, 
conf.level = 0.95, 
R = 9999, 
symmetric = FALSE, 
vectorial = TRUE, 
chunk_size = 2, ...){
  alternative <- match.arg(alternative)
  if(!missing(mu) && (length(mu) != 1 || is.na(mu))) 
    stop("'mu' must be a single number")
  if(!missing(conf.level) && (length(conf.level) != 1 || 
                                !is.finite(conf.level) || 
                               conf.level < 0 || conf.level > 1)) 
    stop("'conf.level' must be a single number between 0 and 1")
  if(!is.null(y)){
    dname <- paste(deparse(substitute(x)), "and", deparse(substitute(y)))
    if (paired) 
      xok <- yok <- stats::complete.cases(x, y)
    else{
      yok <- !is.na(y)
      xok <- !is.na(x)
    }
    y <- y[yok]
  }else{
    dname <- deparse(substitute(x))
    if (paired) 
      stop("'y' is missing for paired test")
    xok <- !is.na(x)
    yok <- NULL
  }
  x <- x[xok]
  if(paired){
    x <- x - y
    y <- NULL
  }
  nx <- length(x)
  mx <- mean(x)
  vx <- var(x)
  if (is.null(y)) {
    if (nx < 2) 
      stop("not enough 'x' observations")
    df <- nx - 1
    stderr <- sqrt(vx/nx)
    if (stderr < 10 * .Machine$double.eps * abs(mx)) 
      stop("data are essentially constant")
    tstat <- (mx - mu)/stderr

    if (paired) {
      method <- "Bootstrapped Paired t-test" 
    } else {
      method <- "Bootstrapped One Sample t-test"
    }

    if (paired){
      mean_name <- "mean of the differences" 
    } else{
      mean_name <-"mean of x"
    }
    estimate <- stats::setNames(mx,mean_name)
    x.cent <- x - mx
    X <- matrix(sample(x.cent, size = nx*R, replace = TRUE), nrow = R)
    MX <- rowMeans(X)
    VX <- rowSums((X-MX)^2)/(nx-1)
    STDERR <- sqrt(VX/nx)
    TSTAT <- MX/STDERR
    EFF <- MX+mx
  }else{
    ny <- length(y)
    if(nx < 1 || (!var.equal && nx < 2)) 
      stop("not enough 'x' observations")
    if(ny < 1 || (!var.equal && ny < 2)) 
      stop("not enough 'y' observations")
    if(var.equal && nx + ny < 3) 
      stop("not enough observations")
    my <- mean(y)
    vy <- var(y)

    method <- paste("Bootstrapped", paste(if (!var.equal) "Welch", 
      "Two Sample t-test"))
    estimate <- c(mx, my)
    names(estimate) <- c("mean of x", "mean of y")
    if(var.equal){
      df <- nx + ny - 2
      v <- 0
      if (nx > 1) 
        v <- v + (nx - 1) * vx
      if (ny > 1) 
        v <- v + (ny - 1) * vy
      v <- v/df
      stderr <- sqrt(v * (1/nx + 1/ny))
      z <- c(x, y)
      Z <- matrix(sample(z, size = (nx+ny)*R, replace = TRUE), nrow = R)
      X <- Z[,seq(nx)]
      Y <- Z[,(nx+1):(nx+ny)]
      MX <- rowMeans(X)
      MY <- rowMeans(Y)
      V <- (rowSums((X-MX)^2) + rowSums((Y-MY)^2))/df
      STDERR <- sqrt(V*(1/nx + 1/ny))
      EFF <- (MX+mx) - (MY+my)
    }else{
      stderrx <- sqrt(vx/nx)
      stderry <- sqrt(vy/ny)
      stderr <- sqrt(stderrx^2 + stderry^2)
      df <- stderr^4/(stderrx^4/(nx - 1) + stderry^4/(ny - 1))
      z <- c(x, y)
      mz <- mean(z)
      x.cent <- x - mx + mz
      y.cent <- y - my + mz

      if (vectorial) {
         X <- matrix(sample(x.cent, size = nx*R, replace = TRUE), nrow = R)
         Y <- matrix(sample(y.cent, size = ny*R, replace = TRUE), nrow = R)
         MX <- rowMeans(X)
         MY <- rowMeans(Y)
         VX <- rowSums((X-MX)^2)/(nx-1)
         VY <- rowSums((Y-MY)^2)/(ny-1)

      } else  {
          iterations <- round(R / chunk_size)
          modulus <- R%%chunk_size
          X_dat <- data.table::rbindlist(
                    lapply(seq(iterations), function(i){
                        perm_mean_sd(x.cent, sample_size = chunk_size)}))
          MX <- X_dat$M
          VX <- X_dat$V
          X_dat <- NULL
          Y_dat <- data.table::rbindlist(
                    lapply(seq(iterations), function(i){
                        perm_mean_sd(y.cent, sample_size = chunk_size)}))
          MY <- Y_dat$M
          VY <- Y_dat$V
          Y_dat <- NULL
      }
      STDERR <- sqrt(VX/nx + VY/ny)
      EFF <- (MX+mx) - (MY+my)
    }
    if (stderr < 10 * .Machine$double.eps * max(abs(mx), abs(my))) 
      stop("data are essentially constant")
    tstat <- (mx - my - mu)/stderr
    TSTAT <- (MX - MY)/STDERR
  }
  if (alternative == "less") {
    pval <- stats::pt(tstat, df)
    boot.pval <- mean(TSTAT < tstat)
    cint <- c(-Inf, tstat + stats::qt(conf.level, df))
    boot.cint <- c(-Inf, stats::quantile(EFF, conf.level))
  }else if(alternative == "greater") {
    boot.pval <- mean(TSTAT > tstat)
    pval <- stats::pt(tstat, df, lower.tail = FALSE)
    cint <- c(tstat - stats::qt(conf.level, df), Inf)
    boot.cint <- c(stats::quantile(EFF, 1-conf.level), Inf)
  }else{
    pval <- 2 * stats::pt(-abs(tstat), df)
    if(symmetric)
      boot.pval <- mean(abs(TSTAT) > abs(tstat))
    else
      boot.pval <- 2*min(mean(TSTAT <= tstat), mean(TSTAT > tstat))
    alpha <- 1 - conf.level
    cint <- stats::qt(1 - alpha/2, df)
    cint <- tstat + c(-cint, cint)
    boot.cint <- stats::quantile(EFF, c(alpha/2, 1-alpha/2))
  }
  cint <- mu + cint * stderr
  names(tstat) <- "t"
  names(df) <- "df"
  names(mu) <- if (paired || !is.null(y)) "difference in means" else "mean"
  attr(cint, "conf.level") <- conf.level
  attr(boot.cint, "conf.level") <- conf.level
  rval <- list(statistic = tstat, parameter = df, p.value = pval, 
               boot.p.value = boot.pval,
               conf.int = cint, boot.conf.int = boot.cint,
               estimate = estimate, null.value = mu, 
               stderr = stderr, alternative = alternative, method = method, 
               data.name = dname)
  class(rval) <- c("boot.htest", "htest")
  rval
}
perm_mean_sd <- function(all_values, sample_size){
         X_length <- length(all_values)
         X <- matrix(sample(all_values, size = X_length*sample_size, 
          replace = TRUE), nrow = sample_size)
         MX <- rowMeans(X)
         VX <- rowSums((X-MX)^2)/(X_length-1)
         data.frame(M = MX,V = VX)
      }

#' @importFrom utils read.table 
#' @importFrom rmarkdown render
coRmiT <- function(
RNAseq_folder,
miRNAseq_folder,
output_files,
strat_names,
sample_proportion, 
organism,
multimir_db,
p_val_cutoff,
corr_cutoffs,
permutations, 
MM_cutoff,
report, #this option can be set to default value
translation_file,
databases,
translate_ensembl = FALSE,
filter_db_theshold = 0,
database_to_filter = NULL,
mc_cores = 1,
tag_filter,
corr_type,
corr_coef,
f_p_val,
selected_targets_file,
eval_method = "aggregated",  
template_folder = file.path(find.package('ExpHunterSuite'), "templates"),
organism_table_path = file.path(find.package('ExpHunterSuite'), "inst", 
    "external_data", "organism_table.txt"),
compare_pred_scores = FALSE
){


#corr_cutoffs <- gsub("'","", corr_cutoffs)
corr_cutoffs<- eval(parse(text=paste('c(', gsub("'","", corr_cutoffs), ')')))
#corr_cutoffs <- as.numeric(unlist(strsplit(corr_cutoffs, ",")))



 #create output folder
 "%>%" <- magrittr::"%>%"
 pred_dbs <- c("diana_microt", "elmmo", "microcosm",
   "miranda","mirdb", "pictar", "pita", "targetscan")
  
 #parse strategies and add default strategies
 strat_names <- c(
    "DEGs_RNA_vs_miRNA_DEMs_opp",
    "DEGs_RNA_vs_miRNA_DEMs_sim",
    "normalized_counts_RNA_vs_miRNA_normalized_counts", 
    "Eigengene_0_RNA_vs_miRNA_normalized_counts", 
    "normalized_counts_RNA_vs_miRNA_Eigengene_0", 
     parse_strategies(strat_names))
  
 # Prepare for RNA ID translation
 organism_info <- utils::read.table(organism_table_path, 
    header = TRUE, row.names=1, sep="\t", stringsAsFactors = FALSE, 
    fill = NA) 
 organism_info <- organism_info[organism_info$KeggCode %in% organism,]
 #Prepare multiMiR
 selected_predicted_databases <- unlist(strsplit(databases, ","))
 selected_predicted_databases <- pred_dbs[pred_dbs %in% unlist(
                                                selected_predicted_databases)]
 if (!is.null(database_to_filter)){
    database_to_filter <- unlist(strsplit(database_to_filter, ","))
 }
 multimir_info <- load_and_parse_multimir(multimir_path = multimir_db, 
    selected_predicted_databases = selected_predicted_databases, 
    filter_db_theshold= filter_db_theshold, 
     database_to_filter = database_to_filter  
    )
 multimir <- multimir_info[["multimir_table"]]
 raw_databases_scores <- multimir_info[["raw_databases_scores"]]
 
 #Load and prepare DGH data
 RNAseq <- list()

 if (RNAseq_folder != "") {
   RNAseq <- load_DEGH_information(RNAseq_folder) 
 } else {
   RNAseq <- NULL
 } 

 selected_targets <- load_selected_targets(selected_targets_file)
 
 miRNAseq <- load_DEGH_information(miRNAseq_folder) 
 
 all_pairs <- prepare_all_pairs(RNAseq = RNAseq, miRNAseq = miRNAseq, 
    multimir = multimir, selected_targets = selected_targets, 
    organism = organism)
 multimir <- NULL


 # Perform strategies
 miRNA_cor_results <- perform_all_strategies(strat_names = strat_names, 
    RNAseq = RNAseq, miRNAseq = miRNAseq, MM_cutoff = MM_cutoff,
    permutations = permutations, all_pairs = all_pairs, 
    selected_predicted_databases = selected_predicted_databases, 
    tag_filter = tag_filter, sample_proportion = sample_proportion, 
    raw_databases_scores = raw_databases_scores, corr_type = corr_type, 
    corr_coef = corr_coef, selected_targets = selected_targets, 
    compare_pred_scores = compare_pred_scores, eval_method = eval_method)


#remove eval_method
miRNA_cont_tables <- data.frame()
cont_tables <- data.frame()
all_pairs <- as.data.frame(miRNA_cor_results$all_pairs)
message("Preparing data for stats computing")
 for (corr_cutoff in corr_cutoffs){
 #mkdir folder
  message(paste0("Computing ", corr_cutoff, " threshold"))
   for (strategy in strat_names){
       
        contingency_tables <- prepare_for_stats(all_pairs, strategy, corr_cutoff, p_val_cutoff, corr_type = corr_type)
        if (is.null(contingency_tables)) next
          miRNA_cont_tables <- rbind(miRNA_cont_tables, contingency_tables$strat_miRNA_ct)
          cont_tables <- rbind(cont_tables, contingency_tables$strat_ct)
   }
   message("test")
 
 } 
 contingency_tables <- list(miRNA_cont_tables = miRNA_cont_tables,
                              cont_tables = cont_tables) 
 miRNA_cor_results <- c(miRNA_cor_results, 
   contingency_tables)


 miRNA_cor_results$cont_tables <- v_get_stats(miRNA_cor_results$cont_tables)
 miRNA_cor_results$miRNA_cont_tables <- 
                              v_get_stats(miRNA_cor_results$miRNA_cont_tables, 
                              selected_stats = c("v.fisher.test","odds_ratio"))
 miRNA_cont_tables <- miRNA_cor_results$miRNA_cont_tables
 integrated_stats <- filter_and_integrate_OR(miRNA_cont_tables,
                                             p.adjust.method = "BH",
                                             p_thr = f_p_val)
   
 integrated_OR <- integrated_stats[["integrated_stats"]]
 miRNA_cont_tables <- integrated_stats[["miRNA_ct"]]
 

 miRNA_cont_tables <- miRNA_cont_tables[!grepl("Eigengene_0", miRNA_cont_tables$strategy) & 
                                          !grepl("sim", miRNA_cont_tables$strategy) &
                                          !grepl("opp", miRNA_cont_tables$strategy) &
                                          miRNA_cont_tables$Odds_ratio >0.0001 &
                                          miRNA_cont_tables$Odds_ratio < 10000 &
                                          miRNA_cont_tables$Pvalue <= f_p_val,]


 if(nrow(miRNA_cont_tables) == 0 || nrow(integrated_OR) == 0){
    stop(paste0("ERROR: Any miRNA has significant overlapping with databases at any ",
                "given strategy/correlation threshold. Please try to modify parametres,"))
 }

 miRNA_cor_results$miRNA_cont_tables <-  miRNA_cont_tables                                
 miRNA_cor_results$cont_tables <- merge(miRNA_cor_results$cont_tables, 
                                          integrated_OR, 
                                          by=c("strategy","db_group", "corr_cutoff"), 
                                          all.x = TRUE)

  integrated_strat <- get_optimal_pairs(miRNA_cor_results$all_pairs, 
                                         miRNA_cor_results$miRNA_cont_tables,
                                             p_val_cutoff,
                                             corr_type = corr_type
                                             )
 all_pairs$integrated_strat <- integrated_strat
 

 integrated_ct <- prepare_for_stats(all_pairs, "integrated_strat", 0, p_val_cutoff, 
                                                sig_pairs = TRUE)

integrated_ct$cont_tables <- v_get_stats(integrated_ct$strat_ct)

 integrated_ct$miRNA_cont_tables <- 
                              v_get_stats(integrated_ct$strat_miRNA_ct, 
                              selected_stats = c("v.fisher.test","odds_ratio"))
 #integrated_ct <- integrated_ct$miRNA_cont_tables
 integrated_stats <- filter_and_integrate_OR(integrated_ct$miRNA_cont_tables,
                                            p.adjust.method = "BH",
                                            p_thr = f_p_val)
   
 integrated_OR <- integrated_stats[["integrated_stats"]]
 int_miRNA_cont_tables <- integrated_stats[["miRNA_ct"]]

 miRNA_cor_results$int_miRNA_cont_tables <- rbind(miRNA_cor_results$miRNA_cont_tables,
                                                   integrated_stats[["miRNA_ct"]])


 integrated_ct$cont_tables <- merge(integrated_ct$cont_tables, 
                                          integrated_OR, 
                                          by=c("strategy","db_group", "corr_cutoff"), 
                                          all.x = TRUE)
 miRNA_cor_results$int_cont_tables <- rbind(miRNA_cor_results$cont_tables,
                                                   integrated_ct$cont_tables)


 ## add crossvalidation

 miRNA_cont_tables$strategy <- set_strats_readable(miRNA_cont_tables$strategy)
 all_cor_dist <- get_cor_dist(all_pairs, strat_names)
 ################# GET IDS TRANSLATIONS
 RNA_IDs <- selected_targets
 if (!is.null(RNAseq_folder)){
   RNA_IDs <- all_pairs$RNAseq
 }

 translated_ids <- translate_all_id(
    miRNA_IDs = all_pairs$miRNAseq, 
    RNA_IDs = RNA_IDs, 
    organism_info = organism_info, translate_ensembl = translate_ensembl)
    
 miRNA_cor_results$integrated_pairs <-  all_pairs[
  all_pairs$integrated_strat &
  !is.na(all_pairs$integrated_strat),]  


 miRNA_cor_results$all_pairs <- all_pairs

 miRNA_cor_results <- c(
    miRNA_cor_results,
    translated_ids,
    list(
     miRNAseq = miRNAseq,
     RNAseq = RNAseq,
     selected_predicted_databases = selected_predicted_databases,
     all_cor_dist = all_cor_dist
 ))
 
 return(miRNA_cor_results)
}

get_TP <- function(cont_table) {
    cont_table["TRUE","TRUE"]
}

get_FP  <- function(cont_table) {
    cont_table["TRUE","FALSE"]
}

get_TN <- function(cont_table) {
    cont_table["FALSE","FALSE"]
}

get_FN <- function(cont_table) {
    cont_table["FALSE","TRUE"]
}

# JRP fr future: More than happy to change the input to the contingence table 
# directly. Or we can use method dispatch to call different methods depending 
# on the class of the input.
# For now it's like this to avoid people making the table the wrong way round 
# - they have to give a test_res and ground truth results as it stands.
calc_prec <- function(test_res_logical, ground_truth_logical) {
    conf_mat <- table(test_res_logical, ground_truth_logical)
    TP <- get_TP(conf_mat)
    FP <- get_FP(conf_mat)
    TP / (TP + FP)
}

calc_rec <- function(test_res_logical, ground_truth_logical) {
    conf_mat <- table(test_res_logical, ground_truth_logical)
    TP <- get_TP(conf_mat)
    FN <- get_FN(conf_mat)
    TP / (TP + FN)
}

calc_spec <- function(test_res_logical, ground_truth_logical) {
    conf_mat <- table(test_res_logical, ground_truth_logical)
    TN <- get_TN(conf_mat)
    FP <- get_FP(conf_mat)
    TN / (TN + FP)
}

calc_acc<- function(test_res_logical, ground_truth_logical) {
    conf_mat <- table(test_res_logical, ground_truth_logical)
    TP <- get_TP(conf_mat)
    TN <- get_TN(conf_mat)
    (TP + TN) / sum(conf_mat)
}

calc_f1 <- function(test_res_logical, ground_truth_logical) {
    precision_val <- calc_prec(test_res_logical, ground_truth_logical)
    recall_val <- calc_rec(test_res_logical, ground_truth_logical)
    2 * ((precision_val * recall_val) / (precision_val + recall_val))
}

#' @importFrom DESeq2 DESeqDataSetFromMatrix rlog
#' @importFrom WGCNA allowWGCNAThreads
#' @importFrom diffcoexp diffcoexp
#' @importFrom utils write.table
#' @importFrom SummarizedExperiment assay
analysis_diffcoexp <- function(data, path, target) {
    dds <- DESeq2::DESeqDataSetFromMatrix(countData = data, 
                                          colData = target["treat"], 
                                          design = ~ treat)
    rld <- DESeq2::rlog(dds, blind=FALSE)
    mat_counts <- SummarizedExperiment::assay(rld)
    treat <- mat_counts[,target$treat=="Treat"]
    control <- mat_counts[,target$treat=="Ctrl"]

    WGCNA::allowWGCNAThreads()
    res <- diffcoexp::diffcoexp(exprs.1 = control,
                                exprs.2 = treat, 
                                r.method = "spearman", 
                                q.dcgth=1)

    utils::write.table(res$DCLs, file=file.path(path, "DCLs.txt"), sep="\t", 
                       quote=FALSE, row.names=FALSE)
    utils::write.table(res$DCGs, file=file.path(path, "DCGs.txt"), sep="\t", 
                       quote=FALSE, row.names=FALSE)
    return(res)
}


build_design_for_WGCNA <- function(target, 
                                   string_factors=NULL, 
                                   numeric_factors=NULL){
# FOR WGCNA: Check that the appropriate factor columns can be found in the 
# target file and makes a data frame with the specified factor
    if(!is.null(string_factors)){ 

        target_string_factors <- target[,colnames(target) %in% string_factors,
                                                    drop = FALSE]
        target_string_factors <- data.frame(sapply(target_string_factors, 
                                                   as.factor), 
                                            stringsAsFactors=TRUE)
        return(target_string_factors)
    }

    if(!is.null(numeric_factors)){
        if(sum(nchar(numeric_factors) == 0)) return("") 

        target_numeric_factors <- target[,colnames(target) %in% 
                                              numeric_factors,
                                              drop = FALSE]
        return(target_numeric_factors)
    }
    
    return(NULL)
}



perform_WGCNA_combinations <- function(WGCNA_all=FALSE, 
                    WGCNA_input, 
                    index_treatmn_cols, 
                    index_control_cols, 
                    path, 
                    target_numeric_factors, 
                    target_string_factors, 
                    WGCNA_memory, 
                    WGCNA_deepsplit, 
                    WGCNA_detectcutHeight, 
                    WGCNA_mergecutHeight, 
                    WGCNA_min_genes_cluster, 
                    WGCNA_blockwiseNetworkType, 
                    WGCNA_blockwiseTOMType, 
                    WGCNA_minCoreKME, 
                    WGCNA_minCoreKMESize, 
                    WGCNA_minKMEtoStay,
                    corType){
    results <- list()
    if(WGCNA_all == TRUE) { #TODO => Este bloque de código es repetitivo. 
        WGCNA_input_treatment <- WGCNA_input[, index_treatmn_cols]
        WGCNA_input_control <- WGCNA_input[, index_control_cols]
        WGCNA_treatment_path <- file.path(path, "Treatment_only_data")
        dir.create(WGCNA_treatment_path)
        cat('Performing WGCNA correlation analysis for treated samples\n')
        results[['WGCNA_treatment']] <- analysis_WGCNA(
                       data=WGCNA_input_treatment,
                       path=WGCNA_treatment_path,
                       target_numeric_factors=target_numeric_factors,
                       target_string_factors=target_string_factors,
                       WGCNA_memory=WGCNA_memory,
                       WGCNA_deepsplit=WGCNA_deepsplit,
                       WGCNA_detectcutHeight=WGCNA_detectcutHeight,
                       WGCNA_mergecutHeight=WGCNA_mergecutHeight,
                       WGCNA_min_genes_cluster=WGCNA_min_genes_cluster,
                       cor_only=TRUE, 
                       blockwiseNetworkType = WGCNA_blockwiseNetworkType, 
                       blockwiseTOMType = WGCNA_blockwiseTOMType,
                       WGCNA_minCoreKME = WGCNA_minCoreKME,
                       WGCNA_minCoreKMESize = WGCNA_minCoreKMESize,
                       WGCNA_minKMEtoStay = WGCNA_minKMEtoStay,
                       corType = corType
        )

        WGCNA_control_path <- file.path(path, "Control_only_data")
        dir.create(WGCNA_control_path)

        cat('Performing WGCNA correlation analysis for control samples\n')
        results[['WGCNA_control']] <- analysis_WGCNA(data=WGCNA_input_control,
                       path=WGCNA_control_path,
                       target_numeric_factors=target_numeric_factors,
                       target_string_factors=target_string_factors,
                       WGCNA_memory=WGCNA_memory,
                       WGCNA_deepsplit=WGCNA_deepsplit,
                       WGCNA_detectcutHeight=WGCNA_detectcutHeight,
                       WGCNA_mergecutHeight=WGCNA_mergecutHeight,
                       WGCNA_min_genes_cluster=WGCNA_min_genes_cluster,
                       cor_only=TRUE, 
                       blockwiseNetworkType = WGCNA_blockwiseNetworkType, 
                       blockwiseTOMType = WGCNA_blockwiseTOMType,
                       WGCNA_minCoreKME = WGCNA_minCoreKME,
                       WGCNA_minCoreKMESize = WGCNA_minCoreKMESize,
                       WGCNA_minKMEtoStay = WGCNA_minKMEtoStay,
                       corType = corType)
    }
        

    cat('Performing WGCNA correlation analysis for all samples\n')
    results[['WGCNA_all']] <- analysis_WGCNA(data=WGCNA_input,
                    path=path,
                    target_numeric_factors=target_numeric_factors,
                    target_string_factors=target_string_factors,
                    WGCNA_memory=WGCNA_memory,
                    WGCNA_deepsplit=WGCNA_deepsplit,
                    WGCNA_detectcutHeight=WGCNA_detectcutHeight,
                    WGCNA_mergecutHeight=WGCNA_mergecutHeight,
                    WGCNA_min_genes_cluster=WGCNA_min_genes_cluster,
                    cor_only=FALSE, 
                    blockwiseNetworkType = WGCNA_blockwiseNetworkType, 
                    blockwiseTOMType = WGCNA_blockwiseTOMType,
                    WGCNA_minCoreKME = WGCNA_minCoreKME,
                    WGCNA_minCoreKMESize = WGCNA_minCoreKMESize,
                    WGCNA_minKMEtoStay = WGCNA_minKMEtoStay,
                    corType = corType)        
    return(results)
}





#' @importFrom utils assignInNamespace write.table
#' @importFrom WGCNA pickSoftThreshold blockwiseModules labels2colors 
#' plotDendroAndColors moduleEigengenes corPvalueStudent 
#' binarizeCategoricalVariable orderMEs
#' @importFrom grDevices pdf dev.control recordPlot dev.off
#' @importFrom graphics par text abline
#' @importFrom stats cor
#' @importFrom ggplot2 ggplot scale_x_continuous scale_y_continuous 
#' geom_count aes_string
analysis_WGCNA <- function(data,
                          path, 
                          target_numeric_factors, 
                          target_string_factors, 
                          WGCNA_memory, WGCNA_deepsplit, 
                          WGCNA_detectcutHeight, 
                          WGCNA_mergecutHeight, 
                          WGCNA_min_genes_cluster, 
                          cor_only, 
                          blockwiseNetworkType, 
                          blockwiseTOMType, 
                          WGCNA_minCoreKME, 
                          WGCNA_minCoreKMESize, 
                          WGCNA_minKMEtoStay,
                          corType) {

    if(is.null(WGCNA_minCoreKMESize)){
        WGCNA_minCoreKMESize <- WGCNA_min_genes_cluster/3
    } 

    data <- t(data)
    nSamples <- nrow(data)

    ####################################################################
    ## THRESHOLDING - EFFECTS OF BETA ON TOPOLOGY AND AUTO SELECTION
    ####################################################################
    utils::assignInNamespace(x="..minNSamples", 
                            value=3, #Overwrite harcoded limit from 4 to 3
                            ns="WGCNA")
    powers <- c(seq(10), seq(from = 12, to=30, by=2))

    bicor <- WGCNA::bicor

    corFnc <- cor
    if(corType == "bicor")
        corFnc <- bicor

    sft <- WGCNA::pickSoftThreshold(data, powerVector = powers, verbose = 5, corFnc = corFnc)

    # Calculate Power automatically
    sft_mfs_r2 <- -sign(sft$fitIndices[,3])*sft$fitIndices[,2]
    min_pow_ind <- which(sft_mfs_r2 > 0.9)[1] # First time the value passes 0.9

    in_advance <- 2
    if(is.na(min_pow_ind)) {
        # Plan B to calculate power
        for(i in seq(length(sft_mfs_r2)-in_advance)) {
            vals <- sft_mfs_r2[i:(i+in_advance)]
            diffs <- abs(diff(vals))
            # Condition to meet
            if(sum(diffs < 0.01) == in_advance & sft_mfs_r2[i] > 0.8) {
              min_pow_ind <- i
                 break
            }
        }
    }

    if(is.na(min_pow_ind)) {
        warning(paste0("Could not obtain a valid power (beta) value for WGCNA",
            " so the default of 30 will be used - proceed with caution"))
        # assumes 30 will be the largest testable power
        min_pow_ind <- length(sft_mfs_r2) 
    }
    pow <- sft$fitIndices[min_pow_ind, "Power"]
    # cor <- WGCNA::cor # TO CORRECT FUNCTION OVERRIDE DUE TO OTHER PACKAGES

    grDevices::pdf(file.path(path, "thresholding.pdf"))
        grDevices::dev.control(displaylist="enable")
        graphics::par(mfrow = c(1,2));
        cex1 = 0.9;
    # Scale-free topology fit index as function of the soft-thresholding power
        plot(sft$fitIndices[,1], -sign(sft$fitIndices[,3])*sft$fitIndices[,2],
             xlab="Soft Threshold (power)",
             ylab="Scale Free Topology Model Fit,signed R^2",type="n",
             main = paste("Scale independence"));
        graphics::text(sft$fitIndices[,1], 
                      -sign(sft$fitIndices[,3])*sft$fitIndices[,2],
                      labels=powers,cex=cex1,col="red");
        # this line corresponds to using an R^2 cut-off of h
        graphics::abline(h=0.90, col="red")
        graphics::abline(h=0.80, col="red", lty="dashed")
        graphics::abline(v=pow, col="black", lty="dotted")

        # Mean connectivity as a function of the soft-thresholding power
        plot(sft$fitIndices[,1], sft$fitIndices[,5],
             xlab="Soft Threshold (power)",ylab="Mean Connectivity", type="n",
             main = paste("Mean connectivity"))
        graphics::text(sft$fitIndices[,1], sft$fitIndices[,5], 
                       labels=powers, cex=cex1,col="red")
        graphics::abline(v=pow, col="black", lty="dotted")

        power_threshold_effects <- grDevices::recordPlot()
    grDevices::dev.off()

    ####################################################################
    ## CLUSTER SAMPLES TO GENERATE MODULES
    ####################################################################

    # Have to improve this as generating fork.
    tom_file_base <- "generatedTOM"
    if(length(list.files(path=path, 
                         pattern=paste0(tom_file_base, ".*\\.RData"))) > 0) {
        loadTOM_TF <- TRUE
        saveTOM_TF <- FALSE
    } else {
        loadTOM_TF <- FALSE
        saveTOM_TF <- TRUE
    }

    cor <- WGCNA::cor # This issue has been reported to the WGCNA author
    net <- WGCNA::blockwiseModules(data, power = pow,
        # Increase to memory limit in order to obtain more realistic results
                            maxBlockSize = WGCNA_memory, 
                            minModuleSize = WGCNA_min_genes_cluster,
                            deepSplit = WGCNA_deepsplit, 
                            detectCutHeight = WGCNA_detectcutHeight,
                            reassignThreshold = 0, 
                            mergeCutHeight = WGCNA_mergecutHeight,
                            numericLabels = TRUE, 
                            pamRespectsDendro = FALSE,
                            loadTOM = loadTOM_TF,    
                            saveTOM = saveTOM_TF,
                            saveTOMFileBase = file.path(path, tom_file_base), 
                            verbose = 5, 
                            networkType = blockwiseNetworkType,
                            TOMType = blockwiseTOMType,
                            minCoreKME = WGCNA_minCoreKME,
                            minCoreKMESize = WGCNA_minCoreKMESize, 
                            minKMEtoStay = WGCNA_minKMEtoStay,
                            corType = corType)
    cor<-stats::cor

    moduleColors = WGCNA::labels2colors(net$colors)

    # Plot the dendrogram and the module colors underneath
    grDevices::pdf(file.path(path, 'wcgnaModules.pdf'))
        WGCNA::plotDendroAndColors(net$dendrograms[[1]],
                    moduleColors[net$blockGenes[[1]]],
                    "Module colors",
                    dendroLabels = FALSE, hang = 0.03,
                    addGuide = TRUE, guideHang = 0.05)
    grDevices::dev.off()

    MEs <- WGCNA::moduleEigengenes(data, net$colors)$eigengenes

    gene_module_cor <- as.data.frame(WGCNA::cor(data, MEs, use = "p"));
    gene_module_cor_p <- as.data.frame(WGCNA::corPvalueStudent(
                                        as.matrix(gene_module_cor), nSamples));
    colnames(gene_module_cor_p) = colnames(gene_module_cor) <- gsub("ME", 
                                      "Cluster_", colnames(gene_module_cor_p) )

    utils::write.table(gene_module_cor, 
                       file=file.path(path, "gene_MM.txt"), 
                       sep="\t", quote=FALSE)
    utils::write.table(gene_module_cor_p, 
                       file=file.path(path, "gene_MM_p_val.txt"), 
                       sep="\t", quote=FALSE)
    utils::write.table(MEs, 
                      file=file.path(path, "eigen_values_per_samples.txt"), 
                      sep="\t", quote=FALSE)

    if(cor_only == TRUE) {
        return("cor_only")
    }

    ####################################################################
    ## PROCESS TRAIT DATA
    ####################################################################

    trait <- data.frame(row.names=rownames(data))

    if(is.data.frame(target_numeric_factors)) {
        trait <- data.frame(trait, target_numeric_factors)
    }
    if(is.data.frame(target_string_factors)) {
    # Code to convert string factors to numeric (1 vs. 0 for each category).
    # NOTE YOU NEED A MINIMUM COUNT OF 2 FOR EACH CATEGORY IN THE FACTORS
        binarized_string_factors <- lapply(names(target_string_factors), 
            function(factor_name) {
                string_factor <- target_string_factors[[factor_name]]
                binarized_table <- WGCNA::binarizeCategoricalVariable(
                                  string_factor,
                                minCount = 1,
                                includePairwise = FALSE,
                                includeLevelVsAll = TRUE)
                colnames(binarized_table) <- paste(factor_name, 
                                               levels(string_factor),sep="_")
                return(binarized_table)
        })
        trait <- data.frame(trait, binarized_string_factors)
    }

    ###################################################################
    ## CLUSTER DATA
    ###################################################################

    ####################################################################
    ## CLUSTER MODULES WITH TRAITS AND PRODUCE HEATMAP
    ####################################################################

    MEs_colors <- MEs <- WGCNA::moduleEigengenes(data, net$colors)$eigengenes
    ME_numeric <- as.numeric(gsub("ME", "", colnames(MEs)))
    colnames(MEs_colors) <- paste0("ME", WGCNA::labels2colors(ME_numeric))

    moduleTraitCor = WGCNA::cor(MEs_colors, trait, use = "p")
    moduleTraitPvalue = WGCNA::corPvalueStudent(moduleTraitCor, nSamples)
 
    textMatrix =  paste(signif(moduleTraitCor, 2), "\n(",
                        signif(moduleTraitPvalue, 1), ")", sep = "")
    dim(textMatrix) = dim(moduleTraitCor)
    colnames(moduleTraitCor) <- names(trait)
    rownames(moduleTraitCor) <- names(MEs)
    moduleTraitCor_df <- as.data.frame(as.table(moduleTraitCor))
    colnames(textMatrix) <- names(trait)
    rownames(textMatrix) <- names(MEs)
    textMatrix_df <- as.data.frame(as.table(textMatrix))
    colnames(moduleTraitCor_df) <- c("Module","Trait","Correlation")
    moduleTraitCor_df$Text_correlation <- textMatrix_df[["Freq"]]
    ####################################################################
    # Produce trait AND module correlations and p-values
    ####################################################################
    ME_numeric_traits <- WGCNA::orderMEs(cbind(MEs, trait))

    trait_and_module_cor <- WGCNA::cor(ME_numeric_traits, use = "p")
    trait_and_module_cor_p <- WGCNA::corPvalueStudent(trait_and_module_cor, 
                                                      nSamples)
    trait_module_cor_val <- as.data.frame(as.table(trait_and_module_cor))
    trait_module_p_val <- as.data.frame(as.table(trait_and_module_cor_p))

    trait_module_all_vals <- data.frame(trait_module_cor_val, 
                                        trait_module_p_val[,3])
    names(trait_module_all_vals) <- c("A", "B", "correlation", "p-value")
    utils::write.table(trait_module_all_vals, 
                       file=file.path(path, "trait_and_module_all_vals.txt"), 
                       sep="\t", quote=FALSE, row.names=FALSE)

    ####################################################################
    ## Produce Tables
    ####################################################################
    # Report tables: 
    # Genes per module 
    gene_module_cor <- as.data.frame(WGCNA::cor(data, MEs, use = "p"))
    gene_module_cor_p <- as.data.frame(WGCNA::corPvalueStudent(
                                        as.matrix(gene_module_cor), nSamples))
    colnames(gene_module_cor_p) = colnames(gene_module_cor) <- gsub("ME", 
                                       "Cluster_", colnames(gene_module_cor_p))
    # Genes per trait
    gene_trait_cor <- as.data.frame(WGCNA::cor(data, trait, use = "p"));
    gene_trait_cor_p <- as.data.frame(WGCNA::corPvalueStudent(
                                          as.matrix(gene_trait_cor), nSamples))
    # Module per trait 
    # (also produced above for the plot - should give smae results.)
    module_trait_cor = WGCNA::cor(MEs, trait, use = "p")
    module_trait_cor_p <- WGCNA::corPvalueStudent(module_trait_cor, nSamples)
    row.names(module_trait_cor) = row.names(module_trait_cor_p) <- gsub("ME", 
                                     "Cluster_", row.names(module_trait_cor_p))

    utils::write.table(trait, 
       file=file.path(path, "sample_trait.txt"), 
       sep="\t", quote=FALSE)
    utils::write.table(gene_trait_cor, 
           file=file.path(path, "gene_trait.txt"), sep="\t", quote=FALSE)
    utils::write.table(gene_trait_cor_p, 
           file=file.path(path, "gene_trait_p_val.txt"), sep="\t", quote=FALSE)
    utils::write.table(module_trait_cor, 
           file=file.path(path, "module_trait.txt"), sep="\t", quote=FALSE)
    utils::write.table(module_trait_cor_p, 
           file=file.path(path, "module_trait_p_val.txt"), 
           sep="\t", quote=FALSE)

    cluster_ID<- net$colors
    Cluster_MM <- unlist(lapply(names(cluster_ID), function(x){
        gene_module_cor[x, paste0("Cluster_", cluster_ID[x])]
    }))
    Cluster_MM_pval <- unlist(lapply(names(cluster_ID), function(x){
        # Get the MM value
        gene_module_cor_p[x, paste0("Cluster_", cluster_ID[x])] 
    }))

    # Plot cluster ID vs. ID of cluster with lowest MM p-value for each gene
    MM_Cluster_ID <- apply(gene_module_cor_p, 
                           1, 
                           function(x) which(x == min(x))) - 1 
                           # Cluster ID with minimum MM p value
    cluster_vs_MM <- ggplot2::ggplot(as.data.frame(cbind(cluster_ID, 
                                                         MM_Cluster_ID)), 
                              ggplot2::aes_string(x = "cluster_ID" ,
                                                  y = "MM_Cluster_ID")) +
                              ggplot2::geom_count() + 
                              ggplot2::scale_x_continuous("Cluster ID", 
                                labels = function(x) paste0("Cluster_", x)) + 
                   ggplot2::scale_y_continuous("Cluster with highest MM value", 
                                labels = function(x) paste0("Cluster_", x))

    # NEW - get the p value for the cluster id, not the lowest p-value

    return(list(
          gene_cluster_info = data.frame(ENSEMBL_ID = names(cluster_ID), 
                                         Cluster_ID = as.numeric(cluster_ID),
                                         Cluster_MM = Cluster_MM,
                                         Cluster_MM_pval = Cluster_MM_pval),
          package_objects=list(gene_module_cor = gene_module_cor, 
                               gene_module_cor_p = gene_module_cor_p,
                               gene_trait_cor = gene_trait_cor, 
                               gene_trait_cor_p = gene_trait_cor_p, 
                               module_trait_cor = module_trait_cor, 
                               module_trait_cor_p = module_trait_cor_p),
          plot_objects=list(sorted_colours = WGCNA::labels2colors(ME_numeric),
                            trait_vs_module = moduleTraitCor_df,
                            trait_and_module = ME_numeric_traits,
                            power_threshold_effects = power_threshold_effects,
                            cluster_vs_MM = cluster_vs_MM)
            )
    )
}

corM2igraph <- function(corM, cor_abs_thr = 0.75){
  cor_df <- data.frame(row=rownames(corM)[row(corM)[upper.tri(corM)]], 
           col=colnames(corM)[col(corM)[upper.tri(corM)]], 
           corr=corM[upper.tri(corM)])
  cor_df$corr_type <- ifelse(c(cor_df$corr) > 0, "corr","anticorr")
  cor_df$corr <- abs(cor_df$corr)
  cor_df_fil <- cor_df[cor_df$corr >= cor_abs_thr,]
  nodes <- data.frame(nodes = unique(c(cor_df_fil$row, cor_df_fil$col)))
  return(list(nodes = nodes, edges = cor_df_fil))
}
    
#' @importFrom stats cor
analysis_diff_correlation <- function(all_genes_stats, data, control, treat, PCIT_filter=FALSE){
    # DE and PIF calculation, see "Microarray data processing, normalization and differential expression" in https://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1000382
    # DE => Diff exp, PIF =>Phenotypic Impact Factor
    nonZeroVal <- 10^-5
    if(sum(data==0) > 0) data[data == 0] <- replicate(sum(data == 0), jitter(nonZeroVal))
    if(sum(control==0) > 0) control[control == 0] <- replicate(sum(control == 0), jitter(nonZeroVal))
    if(sum(treat==0) > 0) treat[treat == 0] <- replicate(sum(treat == 0), jitter(nonZeroVal))
    metrics <- all_genes_stats['logFC_DESeq2']
    names(metrics) <- 'de'
    data <- log(data, 2)
    control <- log(control, 2)
    treat <- log(treat, 2)

    average <- apply(data, 1, mean) # Ai in https://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1000382
    treat_average <- apply(treat, 1, mean) # Ai in https://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1000382
    control_average <- apply(control, 1, mean) # Ai in https://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1000382
    metrics$average <- average
    metrics$treat_average <- treat_average
    metrics$control_average <- control_average
    metrics$pif <- average * metrics$de

    if(!PCIT_filter) {
        control <- stats::cor(t(control)) # Compute correlations
        treat <- stats::cor(t(treat)) # Compute correlations
    }else{
        # JRP Removed CeTF as it depended on GenomeTools which has been deprecated
        # control <- CeTF::PCIT(control, tolType = "mean")$adj_sig
        # treat <- CeTF::PCIT(treat, tolType = "mean")$adj_sig
    }

    #diferential_wiring: https://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1000382
    dw <- control - treat
    prevalent_gene_stats <- all_genes_stats[all_genes_stats$genes_tag == 'PREVALENT_DEG', ]
    if(nrow(prevalent_gene_stats) == 0){
        message('Using POSSIBLE_DEG instead PREVALENT_DEG')
        prevalent_gene_stats <- all_genes_stats[all_genes_stats$genes_tag == 'POSSIBLE_DEG', ]
    }
    metrics$rif1 <- scale(get_rif(1, metrics, prevalent_gene_stats, dw, control, treat)) # Regulatory Impact Factor (RIF):  https://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1000382
    metrics$rif2 <- scale(get_rif(2, metrics, prevalent_gene_stats, dw, control, treat)) # Regulatory Impact Factor (RIF):  https://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1000382
    #Differential hubbing (differential conectivity): https://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1000382
    metrics$ctrl_cn <- apply(control,1,function(x) length(which(abs(x) > 0.8)))
    metrics$treat_cn <- apply(treat,1,function(x) length(which(abs(x) > 0.8)))
    metrics$diff_cn <- metrics$ctrl_cn - metrics$treat_cn
    # As Horvath (WGCNA): DiffK (https://www.ncbi.nlm.nih.gov/pmc/articles/PMC1998880/pdf/335_2007_Article_9043.pdf)
    k_control <- metrics$ctrl_cn/max(metrics$ctrl_cn)
    k_treat <- metrics$treat_cn/max(metrics$treat_cn)
    metrics$diffK <- k_control - k_treat
    return(metrics)
}

get_rif <- function(rif_version, metrics, deg_stats, dw, control_r, treat_r){
    genes <- row.names(metrics)
    rif <- NULL
    if(rif_version == 1){
        rif <- unlist(lapply(genes, compute_rif1, data=metrics, deg_stats=deg_stats, dw=dw))
    }else if(rif_version == 2){
        rif <- unlist(lapply(genes, compute_rif2, data=metrics, deg_stats=deg_stats, control_r=control_r, treat_r=treat_r))
    }
    return(rif)
}

# Regulatory Impact Factor (RIF):  https://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1000382
compute_rif1 <- function(g, data, deg_stats, dw){ #eq 4
    #adapted: It will be computed for all genes and not only TF. Also, all genes DW will be take into account instead of only significant DE genes
    # When implemented significant RNAseq DEgenes change to DE only.
    # Also, implement eq 5 and perform the mean between two eqs. 
    # A posterior paper, saids that the two measures are very different.
    deg_pif <- data[row.names(deg_stats),"pif" ]
    rif <- sum(deg_pif * dw[row.names(deg_stats), g]^2)/length(deg_pif)
    return(rif)
}

compute_rif2 <- function(g, data, deg_stats, control_r, treat_r){ #eq 5
    #adapted: It will be computed for all genes and not only TF. Also, all genes DW will be take into account instead of only significant DE genes
    # When implemented significant RNAseq DEgenes change to DE only.
    # Also, implement eq 5 and perform the mean between two eqs. 
    # A posterior paper, saids that the two measures are very different.
    rif_treat <- (data[row.names(deg_stats),"treat_average" ] * treat_r[row.names(deg_stats), g])^2
    rif_control <- (data[row.names(deg_stats),"control_average" ] * control_r[row.names(deg_stats), g])^2
    rif <- sum(rif_control - rif_treat)/length(rif_treat)
    return(rif)
}

#! /usr/bin/env Rscript
#############################################
############## FUNCTIONAL HUNTER ###########
#############################################

# this is wrapped in a tryCatch. The first expression works when source
# executes, the
# second expression works when R CMD does it.
full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile),  
               error=function(e) # works when using R CMD
              normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                commandArgs())], '='))[2]))
main_path_script <- dirname(full.fpath)


root_path <- file.path(main_path_script, '..', '..')
  # Load custom libraries
custom_libraries <- c('functional_analysis_library.R', 
                      'plotting_functions.R',
                      'general_functions.R')
for (lib in custom_libraries){
  source(file.path(root_path, 'R', lib))
}
template_folder <- file.path(root_path, 'inst', 'templates')


#Loading libraries  
# suppressPackageStartupMessages(require(knitr))


#############################################
### MAIN 
#############################################

# Parse command line
#------------------------------------------------

option_list <- list(
  optparse::make_option(c("-i", "--input_hunter_folder"), type="character",
    help="DEgenes Hunter's differential expression analysis output folder"), 
  optparse::make_option(c("-o", "--output_files"), type="character", 
    default="results", help="Output path. Default=%default")
)
opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))

results_path <- opt$output_files

############ CREATE FOLDERS #########3

# source(file.path(main_path_script, '../../R', 'functional_analysis_library.R'))
# source(file.path(main_path_script, '../../R', 'plotting_functions.R'))

check_and_create_dir(opt$output_files)
#############################################
### LOAD AND PARSE 
#############################################

DEGH_results <- read.table(file.path(opt$input_hunter_folder, "Common_results", 
    "hunter_results_table.txt"), header=TRUE, row.names=1, sep="\t", 
stringsAsFactors = FALSE)
aux <- which(DEGH_results$genes_tag == "FILTERED_OUT") 
if(length(aux) > 0){
    DEGH_results <- DEGH_results[-aux,]
}


#############################################
### PREPARE AND TRANSFORM DATA
#############################################

####
# LOAD NORMALIZED COUNTS
####
    norm_counts <- as.matrix(read.table(file.path(opt$input_hunter_folder, 
        "Results_DESeq2", "Normalized_counts_DESeq2.txt"), header=TRUE, 
        row.names=1, sep="\t", stringsAsFactors = FALSE))
    scaled_counts <- scale_data_matrix(data_matrix = as.matrix(norm_counts))
    scaled_counts_table <- as.data.frame(as.table(scaled_counts))
    colnames(scaled_counts_table) <- c("Gene","Sample","Count")
        
    ####
    # LOAD WGCNA clusters representative profiles with samples
    cl_eigvalues <- as.matrix(read.table(file.path(opt$input_hunter_folder, 
        "Results_WGCNA", "eigen_values_per_samples.txt"), header=TRUE, 
        row.names=1, sep="\t", stringsAsFactors = FALSE))
    cl_eigvalues <- as.data.frame(as.table(cl_eigvalues),
        stringsAsFactors = FALSE)
    colnames(cl_eigvalues) <- c("Sample","Cluster_ID","Count") 
    cl_eigvalues_gnorm <- cl_eigvalues
    cl_eigvalues_gnorm$Count <- (cl_eigvalues_gnorm$Count + 1) / 2 
    
    ####
    # LOAD WGCNA - PVal (Cluster - Trait)
    wgcna_pval_cl_trait <- as.matrix(read.table(file.path(
        opt$input_hunter_folder, "Results_WGCNA", "module_trait_p_val.txt"), 
        header=TRUE, row.names=1, sep="\t", stringsAsFactors = FALSE))
    wgcna_corr_cl_trait <- as.matrix(read.table(file.path(
        opt$input_hunter_folder, "Results_WGCNA", "module_trait.txt"), 
        header=TRUE, row.names=1, sep="\t", stringsAsFactors = FALSE))
    
    ####
    # LOAD WGCNA - Correlation (Sample - Trait)
    wgcna_count_sample_trait <- as.matrix(read.table(file.path(
        opt$input_hunter_folder, "Results_WGCNA", "sample_trait.txt"), 
        header=TRUE, row.names=1, sep="\t", stringsAsFactors = FALSE))
    wgcna_count_sample_trait <- scale_data_matrix(wgcna_count_sample_trait, 
        norm_by_col = TRUE)


# Obtain clusters
cls <- unique(DEGH_results$Cluster_ID)
if(any(c(0,"grey") %in% cls)){
  
  #  cls <- cls[!cls %in% c(0,"grey")]
}else{
    warning("Cluster Zero/Grey not found")
}
clgenes <- lapply(cls,function(cl){unique(rownames(DEGH_results[
        which(DEGH_results$Cluster_ID == cl),]))}) # Find
names(clgenes) <- cls


############################################################
##                    GENERATE REPORT                     ##
############################################################

DEGH_results_orig <- DEGH_results # Necessary to reuse the initial details properly
for (cl in cls){
    # Take output name
    aux <- paste0("cl_func_",cl,".html")
    outf_cls_i <- file.path(results_path, aux)
    # Generate report
    DEGH_results <- DEGH_results_orig[which(DEGH_results_orig$Cluster_ID == cl), ]
    rmarkdown::render(file.path(template_folder, 
        'cl_func_report.Rmd'), output_file = outf_cls_i, 
        intermediates_dir = results_path)
}

#!/usr/bin/env Rscript


################### OPTIONS
option_list <- list(
    optparse::make_option(c("-r", "--RNAseq_folder"), type="character", 
        default="",
        help="ExpHunterSuite RNAseq execution folder"),
    optparse::make_option(c("-m", "--miRNAseq_folder"), type="character", 
        default=NULL,
        help="ExpHunterSuite miRNAseq execution folder"),
    optparse::make_option(c("--selected_targets"), type="character", 
        default=NULL,
        help="One column file indicating external selected targets"),
    optparse::make_option(c("-o", "--output_files"), type="character",
     default=".", 
        help = "Output folder"),
    optparse::make_option(c("-S", "--strategies"), type="character", 
        default="EE,Eh,Ed,hd,hE",
        help = "EE = Anticorrelation between RNAseq modules Eigengene and miRNAseq modules Eigengene, 
                Eh = Anticorrelation between RNAseq modules Eigengene and miRNAseq hub genes, 
                Ed = Anticorrelation between RNAseq modules Eigengene and miRNAseq DEG expression profiles, 
                hd = Anticorrelation between RNAseq hub genes and miRNAseq DEG expression profiles, 
                hE = Anticorrelation between RNAseq hub genes and miRNAseq modules Eigengene. ",
            "Default : %default"),
    optparse::make_option(c("-d", "--databases"), type="character", 
        default="targetscan,mirdb,diana_microt,elmmo,microcosm,miranda,pictar,pita",
        help = "Set prediction databases included on multiMiR to use as gold standard."),
     optparse::make_option(c("--add_databases"), type="character", 
        default=NA,
        help = "Comma sepparated files, containing 2 columns Genes\tmiRNA including additional databases. Default=%default."),
    optparse::make_option("--tag_filter target_tag,miRNA_tag", type="character", 
        default="putative,putative",
        help = "Set filter type for RNAseq and miRNAseq input data by comma separated string. 
                Available options are: 'prevalent' to use only PREVALENT_DEG. 
                'all_possible' to use all PREVALENT_DEG and POSIBLE_DEG. 
                'putative' to use PREVALENT_DEG, POSIBLE_DEG and miRNA or RNAs that correlates with their expression profile."),
    optparse::make_option(c("-C","--corr_coef"), type="character", 
        default="pearson",
        help = "Correlation method: chose between pearson, spearman, kendall and bicor"),
    optparse::make_option(c("--organism"), type ="character", default="hsa",
        help= "Reference organism to use. Available 'hsa' for human and 'mmu' for mouse."),
    optparse::make_option(c("-M", "--multimir_db"), type ="character", 
        default=NULL,
        help= "Indicate .RData file with parsed multiMiR data."),
    optparse::make_option(c("-p", "--p_val_cutoff"), type="double", 
        default=0.05,
        help="Correlation P value threshold. Default=%default"),
    optparse::make_option("--corr_type TYPE", type="character", 
        default="lower",
        help = "Set if correlations are [lower] or [higher] than the --p_val_cutoff. Default=%default"),
    optparse::make_option(c("-c", "--corr_cutoffs"), type="character", 
        default="-0.95,-0.9,-0.85,-0.8,-0.75,-0.7,-0.65,-0.6",
        help="Correlation threshold . Default=%default"),
     optparse::make_option(c("-F", "--f_p_val"), type="double", 
        default=0.05,
        help="Fisher exact test P value threshold . Default=%default"),
    optparse::make_option(c("-s", "--sample_proportion"), type="double", 
        default=0.01,
        help="Score distribution sample proportion. Default=%default"),
    optparse::make_option(c("-P", "--permutations"), type="double", 
        default=50,
        help="Permutations of random tests. Default=%default"),
    optparse::make_option(c("-u", "--filter_db_theshold"), type="double", 
        help="Minimun predicted databases that must support a pair. Default=%default",
        default=0),
    optparse::make_option(c("-f", "--database_to_filter"), type="character", 
        default="targetscan,mirdb,diana_microt,elmmo,microcosm,miranda,pictar,pita",
        help="Predicted databases that must support a pair on --filter_db_threshold. Default=%default"),
    optparse::make_option(c("--module_membership_cutoff"), type="double", 
        default=0.7,
        help="This script reject genes with lower module membership to their modules. Default=%default"),
    optparse::make_option(c("-R", "--report"), ,type = "character", 
        default="miRNA_RNA_comparison.html",
        help="Name of the html file. Default : %default"),
    optparse::make_option(c("-t", "--translation_file"), type = "character", 
        default = NULL,
        help = 'Two columns (\t) file with miRNA names translation: 
                Same IDs as input on first column, and miRBase ID on second column (MIMAT000000)'),
    optparse::make_option(c("-T", "--translate_ensembl"), type="logical", 
        default=FALSE, action = "store_true",
        help = 'Translate mRNA Ensembl ID to ENTREZ ID and GENESYMBOL and include the translation in output.'),
     optparse::make_option(c("--save_temp"), type="logical", 
        default=FALSE, action = "store_true",
        help = 'Activate to store results in temporal file'),
    optparse::make_option(c("--compare_pred_scores"), type="logical", 
        default=FALSE, action = "store_true",
        help = 'Compute predition scores comparison. Default=%default'),
    optparse::make_option(c("--eval_method"), type="character", 
        default="aggregated",
        help = "Select evaluation method by computing Fisher's exact test P value and odds ratio: 
                'aggregated' metrics and filtering is applied to all pairs. 
                            Strategies with Pvalue under threshold will be discarded. 
                'specific' metrics and filtering is applied to pairs of every miRNA. 
                            miRNAs with Pvalue under threshold will be discarded.
                             Strategies without significant miRNAs will be discarded
                Default=%default"),
    optparse::make_option(c("--genomic_ranges"), type="character", 
        default=NULL,
        help = "Path to miRNA genomic ranges Default=%default"),
    optparse::make_option(c("--genome_ref"), type="character", 
        default="hg38",
        help = "Names of the genome version Default=%default"),
    optparse::make_option(c("--mapping_output"), type="character", 
        default="Target_log2FC",
        help = "Select the output column to show in functional report: Predicted_DB_count, Validated_DB_count, Correlation, Target_log2FC, miRNA_log2FC. Default=%default"),
    optparse::make_option(c("--output_pairs"), type="character", 
        default="All",
        help = "Select Filters for output pairs. validated to write only validated pairs and multimir to write pairs ")

)
opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))

################### INITIALIZE
options(scipen = 0.001, 
        digits = 3)

if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){ 

    full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile),  
                   error=function(e) # works when using R CMD
                  normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                    commandArgs())], '='))[2]))
    main_path_script <- dirname(full.fpath)
    root_path <- file.path(main_path_script, '..', '..')
    custom_libraries <- c("plotting_functions.R", "write_report.R", 
        "general_functions.R",  "statistics_functions.R", 
        "miRNA_RNA_functions.R", "functional_analysis_library.R", 
        "main_cormit.R", "mkinfer_modified.R")
    for (lib in custom_libraries){
        source(file.path(root_path, 'R', lib))
      }
    template_folder <- file.path(root_path, 'inst', 'templates')
    organism_table_path <- file.path(root_path,"inst","external_data", 
        "organism_table.txt")

} else {
    require('ExpHunterSuite')
    root_path <- find.package('ExpHunterSuite')
    template_folder <- file.path(root_path, 'templates')
    organism_table_path <- file.path(root_path, "inst","external_data",
        "organism_table.txt")
}
 
 #### NOTAS PARA EL SCRIPT
#
######################
## ADD POSSITIVE CORRELATIONS
#####################
## ADD OR HEATMAP FOR MIRNA/STRATEGY/CORRELATION CUTOFF



check_and_create_dir(opt$output_files)

temp_file <- file.path(normalizePath(opt$output_files), "../temp.RData")
exec_cormit <- !(opt$save_temp && file.exists(temp_file)) 
if (exec_cormit){
    miRNA_cor_results <-  coRmiT( 
        RNAseq_folder=opt$RNAseq_folder,
        miRNAseq_folder=opt$miRNAseq_folder,
        output_files=opt$output_files,
        strat_names=unlist(strsplit(opt$strategies, ",")),
        organism=opt$organism,
        multimir_db=opt$multimir_db,
        sample_proportion = opt$sample_proportion,
        p_val_cutoff=opt$p_val_cutoff,
        corr_cutoffs=opt$corr_cutoffs,
        MM_cutoff=opt$module_membership_cutoff,
        permutations = opt$permutations,
        report=opt$report,
        databases = opt$databases,
        translate_ensembl = opt$translate_ensembl,  
        mc_cores = opt$mc_cores,
        tag_filter = opt$tag_filter,
        f_p_val= opt$f_p_val,
        filter_db_theshold = opt$filter_db_theshold,
        database_to_filter = opt$database_to_filter,
        translation_file = opt$translation_file,
        organism_table_path = organism_table_path, 
        corr_type = opt$corr_type,
        corr_coef = opt$corr_coef,
        selected_targets_file = opt$selected_targets,
        template_folder = template_folder,
        compare_pred_scores = opt$compare_pred_scores,
        eval_method = opt$eval_method,
        add_databases_files = opt$add_databases)
    miRNA_cor_results$all_pairs$RNAseq_mod <- miRNA_cor_results$RNAseq$DH_results[match(miRNA_cor_results$all_pairs$RNAseq, miRNA_cor_results$RNAseq$DH_results$gene_name),"Cluster_ID"]

    miRNA_cor_results$weighted_c_table <- NULL

    if (opt$save_temp)
        save(miRNA_cor_results, file = temp_file)

} else {
    load(temp_file)
}

write.table(miRNA_cor_results$cont_tables, 
    file.path(opt$output_files, "strategies_stats.txt"), 
    quote=FALSE, sep="\t",  row.names=FALSE)

miRNA_cor_results <- c(miRNA_cor_results, 
    list(report_name =opt$report,
         template_folder =template_folder,
         output_files =normalizePath(opt$output_files),
         #p_val_cutoff =opt$p_val_cutoff,
         #corr_cutoff =opt$corr_cutoff,
        # eval_method = opt$eval_method,
         #sample_proportion =opt$sample_proportion,
         genomic_ranges = opt$genomic_ranges,
         genome_ref = opt$genome_ref,
         mapping_output = opt$mapping_output,
         output_pairs = opt$output_pairs))
do.call("write_global_cormit", miRNA_cor_results)

#do.call("write_miRNA_cor_report", miRNA_cor_results)

#' Hunter installation Function
#'
#' This function allows you to install scripts.
#' @param to Path to folder in which copy Hunter scripts
#' @keywords installation
#' @return void
#' @export
#' @examples
#' install_DEgenes_hunter("./")
install_DEgenes_hunter <- function(to){
    scripts_path <- file.path(find.package('ExpHunterSuite'), 'scripts')
    hunter_scripts <- list.files(scripts_path, pattern = ".R", 
                                 recursive = FALSE, include.dirs = FALSE)
    for(hunter_script in hunter_scripts){
        file.copy(file.path(scripts_path, hunter_script), to, overwrite = TRUE)
    }
}

#! /usr/bin/env Rscript
#############################################
############## FUNCTIONAL HUNTER ###########
#############################################
if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){
    full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile),  
               error=function(e) # works when using R CMD
              normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                commandArgs())], '='))[2]))
    main_path_script <- dirname(full.fpath)
    root_path <- file.path(main_path_script, '..', '..')

    custom_libraries <- c('general_functions.R', 
        'functional_analysis_library.R', 'plotting_functions.R', 
        'main_functional_hunter.R', "io_handling.R", "plotting_functions.R", 
        "write_report.R")
    for (lib in custom_libraries){
        source(file.path(root_path, 'R', lib))
    }
    organisms_table_file <- file.path(root_path, "inst", "external_data", 
        "organism_table.txt")
    template_folder <- file.path(root_path, 'inst', 'templates')

}else{
    require('ExpHunterSuite')
    root_path <- find.package('ExpHunterSuite')
    template_folder <- file.path(root_path, 'templates')
    organisms_table_file <- file.path(root_path, "external_data", 
        "organism_table.txt")
}
 
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
##                                                                      ##
##                            INITIALIZE                                ##                                                     
##                                                                      ##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##

option_list <- list(
  optparse::make_option(c("-i", "--input_hunter_folder"), type="character",
    help="DEgenes Hunter differential expression analysis output folder"), 
  optparse::make_option(c("-m", "--model_organism"), type="character",
    help=paste0("Species to use for functional enrichment. You can see all",
        " available species running functional_Hunter.R -L")),
  optparse::make_option(c("-L", "--List_organisms"), action="store_true", 
    type="logical", default=FALSE, 
    help="Print all organisms available and ends the program"),
  optparse::make_option(c("-a", "--annot_file"), type="character", 
    default = NULL,
    help=paste0("If the species used does not exist in organism_table.txt,",
        " please add a two-column file mapping orthologue ENSEMBL gene ids",
        " from a model organism (first column) to the original IDs",
        " (second column).")),
  optparse::make_option(c("-t", "--input_gene_id"), type="character", 
    default="E",
    help=paste0("Input gene IDs. Available IDs are: ENSEMBL (E), entrezgene",
        " (e), TAIR/Arabidopsis (T), Gene Names (G), Gene Symbol (e.g. HGNC for human). [Default:%default]")),          
  optparse::make_option(c("-f", "--func_annot_db"), type="character", 
    default="gKR",
    help=paste0("Functional annotation database and enrichment method(s) to",
        " use (topGO: G = GO | clusterProfiler: K = KEGG, g = GO, R = ",
        "Reactome). D = Disease Ontology (DO), d = DGN. [Default=%default]")),
  optparse::make_option(c("-k", "--kegg_data_file"), ,type = "character", default=NULL,
    help=paste0("KEGG database file. Can download with download_latest_kegg_db().",
        "If not required but not provided, it will be downloaded to working directory")), 
  optparse::make_option(c("-G", "--GO_subont"), type="character",
    default=c("BMC"),
    help=paste0("GO sub-ontologies to use for functional analysis ",
        "(M = Molecular Function, B = Biological Process, C = Celular",
        " Component). Default=%default")), # Not Checked
  optparse::make_option(c("-C", "--custom"), ,type = "character", default=NULL,
    help=paste0("Files with custom functional annotation database ",
        "(in GMT format) separated by commas (,)")),
  optparse::make_option(c("--gmt_id"), type="character", default="ENTREZID",
    help="What identifier is being used for the genes in the custom gmt file. Default=%default"),
  optparse::make_option(c("-A", "--analysis_type"), type="character", 
    default="o",
    help=paste0("Analysis performance (g = Gene Set Enrichment Analysis,",
    " o = Over Representation Analysis). Default=%default")), # Not Checked
   optparse::make_option(c("-r", "--remote"), ,type = "character", default="",
    help=paste0("Flags to activate remote query from enrichments and Genes",
        " translation. Use (b) to launch biomaRt translation; (k) to use Kegg",
        " remote data base")),
  optparse::make_option(c("-q", "--save_query"), type="logical", 
    action = "store_true", default=FALSE,
    help="Flag to save biomaRt query."), 
  optparse::make_option(c("-P", "--pthreshold"), type="double", default=0.1,
    help="Enrichment p-value threshold. [Default = %default]"),
  optparse::make_option(c("-Q", "--qthreshold"), type="double", default=0.2,
    help="Enrichment q-value threshold. [Default = %default]"),
  optparse::make_option(c("-c", "--cores"), ,type = "numeric", default=1,
    help=paste0("Cores to be used to parallelize clusters enrichments.",
        " Default : %default")),
  optparse::make_option(c("-s", "--task_size"), ,type = "numeric", default=10,
    help=paste0("Number of items to be processed in each parallel task.",
        " Default : %default")),  
  optparse::make_option(c("-o", "--output_files"), type="character", 
    default="results",
    help="Output path. Default=%default"),
  optparse::make_option(c("-R", "--report_modes"), type="character", 
    default="fci",
    help="HTML report modes. 'f' for functional_report, 'c' for cluster_main_report and 'i' for individual module report. Default=%default"),
  optparse::make_option(c("--corr_threshold"), type = "double", default = 0.8,
     help = "Clusters with abslute correlation higher than this theshold are combined in clusters_main_report.html. Default: %default"), 
  optparse::make_option(c("-u", "--universe"), type="character", default=NULL, 
    help="Background genes for enrichment. Default all. Alternative = expressed"),
  optparse::make_option("--clean_parentals", type="logical", default=FALSE, 
    action = "store_true", help="Clean parentals GO terms that appears on the same clusters as child."),
  optparse::make_option("--simplify", type="logical", default=FALSE, 
    action = "store_true", help="Apply simplify function from cluster profiler to enrichment."),
  optparse::make_option("--showCategories", type="integer", default=30, 
    help="Number of top categories to show on clusterProfiler dotplot, cnet and emaplot"),
  optparse::make_option("--max_genes_plot", type="integer", default=200, 
    help="Number of genes to show on clusterProfiler cnet"),
  optparse::make_option(c("-T", "--top_categories"), type="integer", default=50,
    help="Number of top categories for each cluster. Default=%default"),
  optparse::make_option(c("-S", "--sim_thr"), type="double", default=NULL,
    help="Similarity cutoff for grouping categories in Summary mode. Default=%default"),
  optparse::make_option("--group_results", type="logical", default=FALSE, 
    action = "store_true", help="Functions are grouped in most frequent words in emaplots."),
   optparse::make_option("--summary_common_name", type="character", default="ancestor", 
                        help="Name of the term groups. 'significant' to use the most significant term of each group. 'ancestor' to use the common ancestor of the group")
)
opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))

# NB remote and save_query don't do anything - need to fix remote so it can toggle Biomart

# Special IDs
fc_colname <- "mean_logFCs"

organisms_table <- get_organism_table(organisms_table_file)
if(opt$List_organisms || ! opt$model_organism %in% rownames(organisms_table)){
    print(as.character(rownames(organisms_table)))
    stop('Check this list and choose one model species.')
}

hunter_results <- load_hunter_folder(opt$input_hunter_folder)
if(is.null(opt$annot_file)) {
    annot_table <- NULL
} else {
    annot_table <- read.table(opt$annot_file, header=FALSE, row.names=NULL, 
        sep="\t", stringsAsFactors = FALSE, quote = "")
}

current_organism_info <- organisms_table[rownames(organisms_table) %in% opt$model_organism,]
org_db <- get_org_db(current_organism_info)
all_custom_gmt <- NULL
if (!is.null(opt$custom)) {
    custom_files <- unlist(strsplit(opt$custom, ","))
    all_custom_gmt <- lapply(custom_files, load_and_parse_gmt)
    names(all_custom_gmt) <- custom_files
    names(all_custom_gmt) <- basename(names(all_custom_gmt))

  if(opt$gmt_id != "ENTREZID") {
    all_custom_gmt <- lapply(all_custom_gmt, function(gmt){
        tr_gmt <- translate_gmt(gmt, opt$gmt_id, org_db)
        return(tr_gmt)
    })
  }
}

if(opt$input_gene_id == "e") input_gene_id <- "ENTREZID"
if(opt$input_gene_id == "E") input_gene_id <- "ENSEMBL"
if(opt$input_gene_id == "T") input_gene_id <- "TAIR"
if(opt$input_gene_id == "G") input_gene_id <- "GENENAME"
if(opt$input_gene_id == "S") input_gene_id <- "SYMBOL"

# Simplest option just to grow the vectors, given complexity of input arguments & interplay
enrich_dbs <- vector()
if(grepl("R", opt$func_annot_db)) enrich_dbs = c(enrich_dbs, "Reactome")
if(grepl("K", opt$func_annot_db)) enrich_dbs = c(enrich_dbs, "KEGG")
if(grepl("G", opt$func_annot_db) || grepl("g", opt$func_annot_db)) {
    if(grepl("B", opt$GO_subont)) enrich_dbs = c(enrich_dbs, "BP")
    if(grepl("C", opt$GO_subont)) enrich_dbs = c(enrich_dbs, "CC")
    if(grepl("M", opt$GO_subont)) enrich_dbs = c(enrich_dbs, "MF")
}
if(grepl("D", opt$func_annot_db)) enrich_dbs = c(enrich_dbs, "DO")
if(grepl("d", opt$func_annot_db)) enrich_dbs = c(enrich_dbs, "DGN")
enrich_methods <- vector()
if(grepl("G", opt$func_annot_db)) enrich_methods <- c(enrich_methods, "topGO")
if(grepl("o", opt$analysis_type)) enrich_methods <- c(enrich_methods, "ORA")
if(grepl("g", opt$analysis_type)) enrich_methods <- c(enrich_methods, "GSEA")

kegg_data_file <- opt$kegg_data_file
if("KEGG" %in% enrich_dbs) {
    kegg_data_file <- get_kegg_db_path(opt$kegg_data_file, current_organism_info=current_organism_info)
    if(! file.exists(kegg_data_file)) stop(paste("KEGG file:", kegg_data_file, "not found"))
}

clusters_flag <- grepl("c", opt$report_modes) || grepl("i", opt$report_modes)


func_results <- main_functional_hunter(
       hunter_results = hunter_results,
       model_organism = opt$model_organism,
       annot_table = annot_table,
       input_gene_id = input_gene_id,
       custom = all_custom_gmt,
       enrich_dbs = enrich_dbs,
       kegg_data_file = kegg_data_file,
       enrich_methods = enrich_methods,
       annotation_source = "orgdb", # Other option Biomart, to be added
       pthreshold = opt$pthreshold,
       qthreshold = opt$qthreshold,
       cores = opt$cores,
       task_size = opt$task_size,
       output_files = opt$output_files,
       organisms_table = organisms_table,
       fc_colname = fc_colname,
       universe = opt$universe,
       clean_parentals = opt$clean_parentals,
       simplify = opt$simplify,
       top_categories = opt$top_categories,
       sim_thr = opt$sim_thr,
       summary_common_name = opt$summary_common_name,
       clusters_flag = clusters_flag
)

write_enrich_files(func_results, opt$output_files)

write_functional_report(hunter_results = hunter_results, 
                            func_results = func_results, 
                            output_files = opt$output_files,
                            organisms_table = organisms_table,
                            template_folder = template_folder,
                            cores =  opt$cores,
                            task_size = opt$task_size,
                            report = opt$report_modes,
                            showCategories = opt$showCategories,
                            max_genes = opt$max_genes_plot,
                            group_results = opt$group_results,
                            corr_threshold =opt$corr_threshold,
                            pvalcutoff = opt$pthreshold
)

#! /usr/bin/env Rscript

#' @author Fernando Moreno Jabato <jabato(at)uma(dot)es>
#' @importFrom naivebayes naive_bayes

#############################################
### CONFIGURE 
#############################################

options(warn=1)

option_list <- list(
  optparse::make_option(c("-t", "--train"), type="character", default = NULL,
    help="[USE ONE] Train file with column 'Prediction'"),
  optparse::make_option(c("-f", "--folder"), type="character", default = NULL,
    help="[USE ONE] Folder with synthetic data and DEG analysis result"),
  optparse::make_option(c("-p", "--path"), type="character", default = NULL,
    help="[USE ONE] Path with several synthetic folders"), 
  optparse::make_option(c("-m", "--model"), type="character", default = NULL,
    help="[USE ONE] Model RData file. Avoid training process"), 
  optparse::make_option(c("-F", "--formula"), type="character", default = "PL",
    help=paste0("String which indicates if formula must include pvalue columns",
      " (P), logFC columns (L) or both (PL). Default: both")),
  optparse::make_option(c("-c", "--change"), type="character", default = "",
    help=paste0("String which indicates if specific columns must be mutated ",
      "applying: -log to pvalue columns (P), ABS to logFC columns (L) or both",
      " (PL).")),
  optparse::make_option(c("-T", "--test"), type="character", default = NULL,
    help=paste0("[OPTIONAL] Test dataset. If include 'Prediction' column stats",
                " will be calculated")),
  optparse::make_option(c("-s", "--save_session"), type="logical",
    default = FALSE, action = "store_true",
    help="Flag to activa SAVE SESSION mode"),
  optparse::make_option(c("-v", "--verbose"), type="logical",
    default = FALSE, action = "store_true",
    help="Activate verbose mode"),
  optparse::make_option(c("-o", "--outfile"), type="character",
    help=paste0("Output file basenames"))
)

opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))

if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){
  # Obtain this script directory
  full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile),  
                 error=function(e) # works when using R CMD
                normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                  commandArgs())], '='))[2]))
  main_path_script <- dirname(full.fpath)
  root_path <- file.path(main_path_script, '..', '..')
  # Load custom libraries
  custom_libraries <- c('simulate_treatment_control.R')
  for (lib in custom_libraries){
    source(file.path(root_path, 'R', lib))
  }
}else{
  require('ExpHunterSuite')
  root_path <- find.package('ExpHunterSuite')
}


#############################################
### FUNCTIONS 
#############################################
transformPvals <- function(df){
  pvalCols <- which(grepl("pvalue",colnames(df)))
  invisible(lapply(pvalCols,function(i){
    df[,i] <<- -log(df[,i])
  }))
  return(df)
}

transformLogFCs <- function(df){
  logfcCols <- setdiff(which(grepl("logFC",colnames(train))),
                       which(colnames(train) == "mean_logFCs"))
  invisible(lapply(logfcCols,function(i){
    df[,i] <<- abs(df[,i])
  }))
  return(df)
}

transformSet <- function(df,tr){
  if(grepl("P",tr)){
    df <- transformPvals(df)
  }
  if(grepl("L",tr)){
    df <- transformLogFCs(df)
  }
  return(df)
}


#############################################
### LOAD 
#############################################


# Load train set
if(!is.null(opt$train)){
  if(opt$verbose) message("Loading TRAINING dataset from file")
  train <- read.table(file = opt$train, sep = "\t", header = TRUE)
}else if(!is.null(opt$folder)){
  if(opt$verbose) message("Loading TRAINING dataset from folder")
  train <- transformSet(load_synth_dataset(opt$folder),opt$change)

}else if(!is.null(opt$path)){
  if(opt$verbose) message("Loading TRAINING dataset from paths set")
  target_folder = tryCatch(
    {
      normalizePath(opt$path)
    },
    warning=function(w){
      return(file.path(getwd(),opt$path))
    }
  )
  train <- as.data.frame(do.call(rbind,lapply(tail(list.dirs(target_folder, 
                                                        full.names = TRUE),-1), 
                                              function(folder){
    message(paste0("\t",folder))
    return(transformSet(load_synth_dataset(folder),opt$change))
  })))
}else if(!is.null(opt$model)){
  if(opt$verbose) message("Loading already trained model")
  load(opt$model)
}else{
  stop("No training set given. Finishing ...")
}

# Verbose point
if(opt$verbose) message("Loading TESTING dataset")

# Load test set
testLoaded <- FALSE
if(!is.null(opt$test)){
  test <- read.table(file = opt$test, sep = "\t", header = TRUE)
  test <- transformSet(test,opt$change)
  testLoaded <- TRUE
}

#############################################
### TRAIN AND TEST 
#############################################

if(is.null(opt$model)){
  # Verbose point
  if(opt$verbose) message("Creating formula")

  # Prepare formula
  target_cols <- c()
  if(grepl("L",opt$formula)){
    target_cols <- c(target_cols, setdiff(which(grepl("logFC",colnames(train))),
                                      which(colnames(train) == "mean_logFCs")))
  }
  if(grepl("P",opt$formula)){
    target_cols <- c(target_cols, which(grepl("pvalue",colnames(train))))
  }

  formula_t <-paste0("Prediction ~ ",paste(unique(colnames(train)[target_cols]),
                    collapse = " + "))
  formula_t <- as.formula(formula_t)

  # Verbose point
  if(opt$verbose) message("Training model")
  # Train NB model
  model <- naivebayes::naive_bayes(formula_t, data = train)
}

if(testLoaded){
  # Verbose point
  if(opt$verbose) message("Predicting over TESTING set")

  # Predict over dataset
  prediction <- naivebayes:::predict.naive_bayes(model,test)
  if("Prediction" %in% colnames(test)){
    tab <- table(test$Prediction, prediction, dnn = c(TRUE,FALSE))
    prediction_stats <- get_stats_from_cm(tab) 
  }  
}

#############################################
### EXPORT 
#############################################

# Verbose point
if(opt$verbose) message("Exporting results")

# Export model
if(is.null(opt$model)){
  save(model,file = paste(opt$outfile,"model.RData",sep = "_"))
}
# Export session
if(opt$save_session){
  save.image(paste(opt$outfile,"session.RData",sep = "_"))
}
if(testLoaded){
  # Export prediction
  write.table(cbind(rownames(test),as.character(prediction)),
    paste(opt$outfile,"pred",sep = "_"), 
    col.names = FALSE, row.names = FALSE, quote = FALSE)
  if("Prediction" %in% colnames(test)){
    # Export stats over dataset
    write.table(prediction_stats,paste(opt$outfile,"stats",sep = "_"),
      row.names = FALSE, quote = FALSE)
  }  
}

#' Function to generate targets file
#' Load target file if it exists, otherwise use the -C and -T flags. 
#' Note target takes precedence over target.
#' @param from_file input targets file
#' @param ctrl_samples control samples
#' @param treat_samples target samples
#' @return targets info loaded
#' @keywords design
#' @export
#' @importFrom utils read.table
#' @examples
#' # To generate target or load from file. 
#' # Example shows generation based on names of treament/control samples.
#' # These should match the column names in the table of counts.
#' case_sample_names <- paste("case", seq(5), sep="_")
#' control_sample_names <- paste("control", seq(5), sep="_")
#' target <- target_generation(ctrl_samples=control_sample_names, 
#'                             treat_samples=case_sample_names)
target_generation <- function(from_file=NULL, 
                              ctrl_samples=NULL, 
                              treat_samples=NULL){
    target <- NULL
    if(! is.null(from_file)) {
      target <- utils::read.table(from_file, 
                                    header=TRUE, 
                                    sep="\t", 
                                    check.names = FALSE)
      # Check there is a column named treat
      if(! "treat" %in% colnames(target)) {
        stop(cat("No column named treat in the target file.\nPlease resubmit"))
      }
    } else {
      index_control_cols <- unlist(strsplit(ctrl_samples, ",")) 
      index_treatmn_cols <- unlist(strsplit(treat_samples, ","))
      # Create the target data frame needed in calls to the DE detection methods
      target <- data.frame(sample=c(index_control_cols, index_treatmn_cols), 
        treat=c(rep("Ctrl", length(index_control_cols)), 
                rep("Treat", length(index_treatmn_cols))))
    }    
    return(target)
}    


#' Function to write expression package results.
#' Write to disk the results of each diff expression package
#' @param df_list dataframes list 
#' @param prefix prefix concatenated to output file name
#' @param root path where write
#' @return void
#' @keywords output
#' @export
#' @importFrom utils write.table
#' @examples
#' # Typically used with the @all_counts_for_plotting slot of the main output
#' deg_res <- list(aa=data.frame(a=seq(5),b=seq(5)), 
#'                 bb=data.frame(a=seq(6,10),b=seq(6,10)))
#' write_df_list_as_tables(deg_res, prefix="test", root=tempdir())
write_df_list_as_tables <- function(df_list, prefix, root=getwd()) {
  for(pack in names(df_list)) {
      folder <- file.path(root, paste0("Results_", pack))
      if(!dir.exists(folder)) dir.create(folder)
      utils::write.table(df_list[[pack]],
      file=file.path(folder, paste0(prefix, pack, '.txt')), 
      quote=FALSE, col.names = NA, sep="\t")
  }
}

#' @importFrom utils tail
load_hunter_folder <- function(path = NULL){
    packages_results <- list.dirs(path,recursive=FALSE)
    packages_results <- packages_results[grepl("Results_",packages_results)]
    packages_results <- packages_results[!grepl("WGCNA$",packages_results)]

    dgh_exp_results <- list()
    dgh_exp_results[["DE_all_genes"]] <- utils::read.table(
        file.path(path,"Common_results","hunter_results_table.txt"))

    dgh_exp_results[["sample_groups"]] <- utils::read.table(
        file.path(path,"control_treatment.txt"))

    WGCNA_res <- load_WGCNA_results(path,dgh_exp_results[["DE_all_genes"]])
    if(length(WGCNA_res) > 0) dgh_exp_results[["WGCNA_all"]] <- WGCNA_res

    dgh_exp_results[["all_data_normalized"]] <- lapply(packages_results, function(pack_path) {
        #pack_name <- utils::tail(unlist(strsplit(pack_path,"_")),1)
        exp_matrix_path <- grep("Norm", dir(file.path(pack_path)), value=TRUE)
        exp_matrix <- utils::read.table(file.path(pack_path, exp_matrix_path))
        return(exp_matrix)
    })
    names(dgh_exp_results[["all_data_normalized"]]) <- sapply(strsplit(packages_results,"_"), utils::tail, 1)
    return(dgh_exp_results)
}

#' Loads stored WGCNA results in DEgenes Hunter expression analysis package 
#' folder and returns in compact object
#' @param path of expression analysis results (Hunter folder)
#' @param main_deg_table dataframe with main DEG analysis results
#' @return WGCNA results loaded
#' @keywords input
#' @importFrom utils read.table
load_WGCNA_results <- function(path, main_deg_table){
    info <- list()
    if(dir.exists(file.path(path,"Results_WGCNA"))){
        # Package objects
        package_objects <- list()
        package_objects[["gene_module_cor"]] <- utils::read.table(
            file.path(path,"Results_WGCNA","gene_MM.txt"))
        package_objects[["gene_module_cor_p"]] <- utils::read.table(
            file.path(path,"Results_WGCNA","gene_MM_p_val.txt"))
        package_objects[["gene_trait_cor"]] <- utils::read.table(
            file.path(path,"Results_WGCNA","gene_trait.txt"))
        package_objects[["gene_trait_cor_p"]] <- utils::read.table(
            file.path(path,"Results_WGCNA","gene_trait_p_val.txt"))
        package_objects[["module_trait_cor"]] <- utils::read.table(
            file.path(path,"Results_WGCNA","module_trait.txt"))
        package_objects[["module_trait_cor_p"]] <- utils::read.table(
           file.path(path,"Results_WGCNA","module_trait_p_val.txt"))
        # gene_cluster_info
        gene_cluster_info <- cbind(rownames(main_deg_table), 
            main_deg_table[,c("Cluster_ID", "Cluster_MM", "Cluster_MM_pval")])
        colnames(gene_cluster_info) <- c("ENSEMBL_ID", "Cluster_ID", 
                                         "Cluster_MM", "Cluster_MM_pval")
        gene_cluster_info <- gene_cluster_info[rownames(
                                        package_objects[["gene_module_cor"]]),]
        # plot_objects
        plot_objects <- list()

        sample_module <- utils::read.table(
            file.path(path,"Results_WGCNA","eigen_values_per_samples.txt"))
        sample_trait <- utils::read.table(
            file.path(path,"Results_WGCNA","sample_trait.txt"))
        plot_objects[["trait_and_module"]] <- cbind(sample_module,sample_trait)

        # Store into info
        info$package_objects <- package_objects
        info$gene_cluster_info <- gene_cluster_info
        info$plot_objects <- plot_objects
    }

    return(info)
}

#' get path for KEGG db for downloading/usage
#' @param kegg_data_file file path. if null, inst/kegg_data_files will be used
#' @param current_organism_info information for the organism, including KEGG code
get_kegg_db_path <- function(kegg_data_file, current_organism_info){
  if(is.null(kegg_data_file)) {
    kegg_code <- current_organism_info$KeggCode[1]
    kegg_data_file <- paste0(kegg_code, "_KEGG.rds")

    if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ) {
      # Root path must be defined outside function i.e. in script that calls it
      kegg_data_file <- file.path(root_path, 'inst', 'kegg_data_files', kegg_data_file)
    }
    else { 
      kegg_data_file <- system.file("kegg_data_files", kegg_data_file, package="ExpHunterSuite")
    }
  }
  return(kegg_data_file)
}

#' download kegg db for a given organism
#' @param current_organism_info organism info for which to download the file
#' @param file where to save the file
download_latest_kegg_db <- function(current_organism_info, file) {
  organism <- current_organism_info$KeggCode[1]
  prepare_KEGG <- get("prepare_KEGG", envir=asNamespace("clusterProfiler"), inherits = FALSE)
  ENRICH_DATA <- prepare_KEGG(organism, "KEGG", "kegg")
  saveRDS(ENRICH_DATA, file=file)
}

write_table_ehs <- function(x, file) {
    utils::write.table(x=x, file=file, quote=FALSE, col.names=TRUE, 
        row.names = FALSE, sep="\t")
}

write_enrich_tables <- function(func_res_tables, method_type, output_files){
    for(res in names(func_res_tables)) {
        filename <- file.path(output_files, 
                paste(res, method_type, "results", sep="_"))
        res_to_print <- func_res_tables[[res]]
        write_table_ehs(res_to_print, file=filename)
    }
}
write_enrich_clusters <- function(func_clusters, output_path) {
    lapply(names(func_clusters), function(funsys) {
      func_res <- func_clusters[[funsys]]
      merged_func_res <- clusterProfiler::merge_result(func_res)
      write_table_ehs(merged_func_res,
        file=file.path(output_path, paste0("enr_cls_", funsys, ".csv")))
      # write_enrich_tables(func_res, 
      #   paste0(funsys, "_cluster"), output_path)
    })
}

#' Write enrichment files related to functional_hunter results list
#' @param func_results functional enrichment results
#' @param output_path output folder path
#' @return void
#' @keywords export
#' @export
#' @importFrom utils write.table
#' @examples
#' # Use a @functional_hunter or @multienricher result object to 
#' # create real files
#' write_enrich_files(list(),"./")
write_enrich_files <- function(func_results, output_path=getwd()) {
    if(!dir.exists(output_path)) dir.create(output_path)
    final_params <- func_results$final_main_params[! names(func_results$final_main_params) %in%
        c("hunter_results", "organisms_table", "annot_table", "custom")]
    write_table_ehs(data.frame(A = names(final_params), 
                               B = sapply(final_params, paste, collapse=" ")), 
                       file = file.path(output_path,"functional_opt.txt"))

    if("ORA" %in% names(func_results)) {
        write_enrich_tables(func_results$ORA, "ORA", output_path)
    }
    if("GSEA" %in% names(func_results)) {
        write_enrich_tables(func_results$GSEA, "GSEA", output_path)
    }
    if("DEGH_results_annot" %in% names(func_results)) 
        write_table_ehs(func_results$DEGH_results_annot, 
                           file=file.path(output_path, 
                                          "hunter_results_table_annotated.txt"))
    if("WGCNA_ORA" %in% names(func_results)) {
        write_enrich_clusters(func_results$WGCNA_ORA_expanded, output_path)
        # func_res_cls <- lapply(func_results$WGCNA_ORA, as.data.frame)
        # write_enrich_tables(func_res_cls, "cls_ORA", output_path)

        # for(cl in unique(func_results$DEGH_results_annot$Cluster_ID)) {
        #     DEGH_res_cl <- func_results$DEGH_results_annot[func_results$DEGH_results_annot$Cluster_ID == cl,]              
        #     write_table_ehs(DEGH_res_cl[DEGH_res_cl$genes_tag == "PREVALENT_DEG", 
        #                         c("Symbol", "entrezgene", "mean_logFCs", "combined_FDR")],
        #                     file=file.path(output_files, paste0("cluster_", cl, "_DEGs.txt")))
        # }
    }
}



#' Loads Hunter DEG analysis table and simulated TRUE DEG vector. Concat in
#' training/testing ML data frame format and returns it.
#' @param folder where files are placed
#' @param hunter_table_file DEG analys result table file name
#' @param prediction_file_regex regex to find TRUE DEG file generated
#' @return a data frame with DEG result and real predictions column
#' @export
#' @examples
#' # Used to load a synth + DE analysis result folder
#' ht <- load_synth_dataset() 
#' # Will return an empty object, you need a correct folder
load_synth_dataset <- function(
    folder = NULL,
    hunter_table_file = "hunter_results_table.txt",
    prediction_file_regex = "_predv"){
    # Check
    if(is.null(folder)){
      warning("Folder has not been specified. Returning NULL")
      return(NULL)
    }
    # Load hunter table
    ht <- read.table(file = file.path(folder,hunter_table_file), sep = "\t",
                     header = TRUE)
    # Load prediction
    prediction_file <- list.files(folder)
    prediction_file <- prediction_file[which(grepl(prediction_file_regex,
                                                   prediction_file))]
    if(length(prediction_file) > 1) prediction_file <- prediction_file[1]
    true_preds <- read.table(file = file.path(folder,prediction_file),
                             sep = "\t", header = TRUE)
    # Concat
    ht$Prediction <- unlist(lapply(seq(nrow(ht)),function(i){
        return(true_preds[which(true_preds[,1] == rownames(ht)[i])[1],2])
    }))
    # Return
    return(ht)
}

test_that("main expression function works", {
  toc_file <- system.file("extData", "table_of_counts_min.txt", 
  	                      package="ExpHunterSuite")
  toc <- read.table(toc_file, 
                    header=TRUE, row.names=1, sep="\t")
  
  target_file <- system.file("extData", "target.txt", 
  	                         package="ExpHunterSuite")

  target <- target_generation(from_file=target_file)

  degh_out <- main_degenes_Hunter(raw=toc, 
                                  target=target,
                                  modules="D",
                                  minpack_common=1)
  testthat::expect_equal(row.names(degh_out$DE_all_genes)[1:3], c("ENSMUSG00000055493", "ENSMUSG00000026822", "ENSMUSG00000024164"))
  testthat::expect_equal(degh_out$DE_all_genes[1:3, "FDR_DESeq2"],  c(1.105994e-186, 3.362235e-44, 6.545641e-29))

  write_expression_report(exp_results=degh_out)
})

test_that("main expression function works with 0 DEGs", {
  toc_file <- system.file("extData", "table_of_counts_no_changes.txt", 
  	                      package="ExpHunterSuite")
  toc <- read.table(toc_file, 
                    header=TRUE, row.names=1, sep="\t")

  target_file <- system.file("extData", "target.txt", 
  	                         package="ExpHunterSuite")

  target <- target_generation(from_file=target_file)

  degh_out <- main_degenes_Hunter(raw=toc, 
                                  target=target,
                                  modules="D",
                                  minpack_common=1)

 testthat::expect_equal(row.names(degh_out$DE_all_genes)[1:3], 
  c("ENSMUSG00000051951", "ENSMUSG00000102348", "ENSMUSG00000103201"))

})

#!/usr/bin/env Rscript

options(warn=1)
if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){
  # Obtain this script directory
  full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile), 
                 error=function(e) # works when using R CMD
                normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                  commandArgs())], '='))[2]))
  main_path_script <- dirname(full.fpath)
  root_path <- file.path(main_path_script, '..', '..')
  # Load custom libraries
  custom_libraries <- c('io_handling.R', "functional_analysis_library.R")
  for (lib in custom_libraries){
    source(file.path(root_path, 'R', lib))
  }
  template_folder <- file.path(root_path, 'inst', 'templates')
  organisms_table_file <- file.path(root_path, "inst", "external_data", 
        "organism_table.txt")
} else {
  require('ExpHunterSuite')
  root_path <- find.package('ExpHunterSuite')
  template_folder <- file.path(root_path, 'templates')
  organisms_table_file <- file.path(root_path, "external_data", 
        "organism_table.txt")
}

########################## OPTIONS
option_list <- list(
	  optparse::make_option(c("-K", "--kegg_data_file"), ,type = "character", default=NULL,
                        help=paste0("path to download KEGG data file. If unspecified, defaults to inst/kegg_data_files")), 
      optparse::make_option(c("-O", "--model_organism"), type="character", default="Human", 
                        help="Model organism.")
)
opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))


kegg_files_path <- file.path(root_path, 'inst', 'kegg_data_files')
if(! file.exists(kegg_files_path)) dir.create(kegg_files_path)

organisms_table <- get_organism_table(organisms_table_file)
current_organism_info <- organisms_table[rownames(organisms_table) %in% opt$model_organism,]

kegg_data_file <- get_kegg_db_path(opt$kegg_data_file, current_organism_info=current_organism_info)

download_latest_kegg_db(current_organism_info, kegg_data_file)

#' Perform Functional analysis with different 
#' enrichment corpora.
#' @param hunter_results output of main_degenes_hunter - list with DEG analysis results
#' @param model_organism organisms which genes are being studied
#' @param annot_table (OPTIONAL) annotation table to translate gene IDs
#' @param organisms_table (OPTIONAL) configuration table for given organism.
#'  Use see get_organism_table()
#' @param input_gene_id (OPTIONAL) type og gene IDs given. Allowed: ENSEMBL (E)
#' , entrezgene (e), TAIR/Arabidopsis (T), Gene Names (G).
#' @param custom (OPTIONAL) list of dataframes with GMT corpus
#' @param enrich_dbs annotation dbs to be used: BP,MF,CC,Reactome,KEGG
#' - MF and Reactome default
#' @param kegg_data_file if KEGG included in enrich_dbs path needs
#' @param enrich_methods enrichment methods to be used: ORA or GSEA
#' @param annotation_source annote with orgdbs or Biomart? Currently on orgdbs
#' @param pthreshold pvalue threshold
#' @param qthreshold qvalue threshold
#' @param cores cores to be used if parallel features are going to be used.
#'  Default: 1
#' @param task_size number of elements per packages used
#' @param output_files output folder
#' @param fc_colname main logFC colname (into hunter_results dataframe)
#' @param universe whether to use all genes as the background, or expressed only
#' @param simplify simplify the merged enrichments
#' @param clean_parentals clean parental terms in merged enrichment
#' @param top_categories numbers of categories from each cluster to use for merge
#' @param sim_thr value to use when combining similar categories in summary mode
#' @param summary_common_name 'significant' to use the most significant term to label each summarized group
#' @param clusters_flag execute clusters enrichments
#' @return functional result object with enrichments performed
#' @keywords method
#' @export
#' @importFrom magrittr %>%
#' @importFrom clusterProfiler merge_result
#' @examples
#' # Load DE analysis result
#' data(degh_output)
#' func_results <- main_functional_hunter(hunter_results = degh_output,
#' model_organism = "Mouse", enrich_dbs = "MF")
main_functional_hunter <- function(
    hunter_results,
    model_organism,
    annot_table = NULL,
    organisms_table = get_organism_table(),
    input_gene_id = "ENSEMBL", # Common examples: ENSEMBL, ENTREZID etc.
    custom = NULL,
    enrich_dbs = c("MF", "Reactome"),
    kegg_data_file = "",
    enrich_methods = "ORA",
    annotation_source = "orgdb", # Other option Biomart, to be added
    pthreshold = 0.1,
    qthreshold = 0.2,
    cores = 1,
    task_size = 1,
    output_files = "results",
    fc_colname = "mean_logFCs",
    universe = NULL,
    clean_parentals = FALSE,
    simplify = FALSE,
    top_categories = 50,
    sim_thr = NULL,
    summary_common_name = "ancestor",
    clusters_flag = FALSE
    ){

    ############################################################
    ##                                                        ##
    ##                INITIAL SETUP                           ## 
    ##                                                        ##
    ############################################################

    # Get initial parameters
    final_main_params <- as.list(environment(), all=TRUE)
    func_results <- list()
    func_results[["final_main_params"]] <- final_main_params

    # Check organism selected and obtain the organisms table
    if (any(is.null(model_organism), 
            !model_organism %in% rownames(organisms_table))) {
        stop(paste0('Model organism does not appear in organism table. 
            Currently available organisms are:', rownames(organisms_table)))
    } else {
        current_organism_info <- subset(organisms_table, 
                rownames(organisms_table) %in% model_organism)  
    }

    # Get sample class info - JRP - to remove when updating expression hunter
    case_treat <- ifelse(hunter_results$sample_groups$class == "C", 
        "Control", "Treatment")
    sample_classes <- paste0("* [", case_treat, "] ", 
        hunter_results$sample_groups$name)

    # Load DEGenesHunter results
    DEGH_results <- hunter_results$DE_all_genes
    DEGH_results <- DEGH_results[DEGH_results$genes_tag != "FILTERED_OUT", ]

    clusters_flag <- "Cluster_ID" %in% colnames(DEGH_results) && clusters_flag
    ##############################################################
    ##                                                          ##
    ##  LOAD COMPLEMENTARY FILES AND ADD ENTREZ AND SYMBOL IDs  ## 
    ##                                                          ##
    ##############################################################

    if(!is.null(annot_table)){
        input_IDs <- translate_from_table(rownames(DEGH_results), annot_table)
    } else {
        input_IDs <- rownames(DEGH_results)
    }

    if(annotation_source == "biomart") {
        stop("biomart functionality for ID translation not implemented")
    } else if (annotation_source == "orgdb") {
        gene_translation_tables <- get_translation_tables_orgdb(
            input_gene_id=input_gene_id, 
            input_ids=input_IDs, 
            current_organism_info=current_organism_info)
    }

    DEGH_results <- add_translated_gene_ids(DEGH_results, input_IDs, 
        input_gene_id, gene_translation_tables)

    ############################################################
    ##                                                        ##
    ##              GET SIG GENES AND GENE LISTS              ## 
    ##                                                        ##
    ############################################################

    prev_genes <- get_sig_genes(DEGH_results)[["prev_genes"]]
    geneList <- get_gene_lists(DEGH_results, fc_colname)
    
    if(clusters_flag) {

        cl_genes <- get_sig_genes_cl(DEGH_results)
        cl_geneList <- get_gene_lists_cl(DEGH_results, fc_colname)
        cl_geneList <- cl_geneList[! names(cl_geneList) == 0]
    }

    if(! is.null(universe)) {
        if(universe == "expressed") {
            universe <- DEGH_results$ENTREZID[DEGH_results$genes_tag != "FILTERED_OUT"]
        }
    }

    ############################################################
    ##                                                        ##
    ##                     PERFORM ENRICHMENTS                ## 
    ##                                                        ##
    ############################################################

    if("GSEA" %in% enrich_methods){
        deg_enr_gsea <- multienricher_gsea(all_funsys=enrich_dbs, 
            genes_list=geneList, 
            organism_info = current_organism_info, kegg_file = kegg_data_file,
            pvalueCutoff = pthreshold)

        deg_enr_gsea <- add_term_sim_ora(deg_enr_gsea)
        func_results$GSEA <- deg_enr_gsea
        if (clusters_flag) message("GSEA not performed on clusters as non-sensical")
    }

    if("ORA" %in% enrich_methods){
        deg_enr_ora  <- multienricher_ora(all_funsys = enrich_dbs, 
            genes_list = prev_genes, organism_info = current_organism_info, 
            pvalueCutoff = pthreshold, qvalueCutoff = qthreshold, 
            custom_sets = custom, kegg_file = kegg_data_file,
            return_all = TRUE, universe = universe)
        func_results$ORA  <- add_term_sim_ora(deg_enr_ora)

        if(clusters_flag){
            clusters_enr_ora <- multienricher_ora(all_funsys=enrich_dbs, 
                genes_list=cl_genes, organism_info = current_organism_info, 
                pvalueCutoff = pthreshold, qvalueCutoff = qthreshold, 
                custom_sets=custom, kegg_file = kegg_data_file, workers=cores, 
                task_size=task_size, return_all = TRUE, universe=universe)

            clusters_enr_ora_merged<- process_cp_list(clusters_enr_ora, simplify, clean_parentals)
            func_results$WGCNA_ORA <- clusters_enr_ora_merged
            func_results$WGCNA_ORA_expanded <- add_term_sim_ora(clusters_enr_ora)

            # TO improve and add to reports
            if(! is.null(sim_thr))
              func_results$summarized_ora <- summarize_merged_ora(clusters_enr_ora_merged, 
                  sim_thr, summary_common_name, pthreshold)
        }
    }

    ############################################################
    ##                                                        ##
    ##                     EXPORT DATA                        ## 
    ##                                                        ##
    ############################################################

    # Reorder rows by combined FDR
    DEGH_results <- DEGH_results[order(DEGH_results[,"combined_FDR"]),] 
    func_results$DEGH_results_annot <- DEGH_results
    
    return(func_results)
}

test_that("main functional enrichment function works Reactome", {

  precomp_degh_res_file <- system.file("extData", "testdata", "precomp_expression_results.RData", 
                           package="ExpHunterSuite")
  load(precomp_degh_res_file)
  organisms_table_file <- system.file("external_data", "organism_table.txt", 
                          package="ExpHunterSuite")
  fh_out <- main_functional_hunter( #Perform enrichment analysis
         precomp_degh_out,
         'Mouse', # Use specified organism database 
         enrich_dbs = c("MF", "BP","Reactome"), # Enrichment analysis for GO, KEGG and Reactome
         enrich_methods = "ORA",
  )

  testthat::expect_equal(nrow(as.data.frame(fh_out$ORA$MF)), 22)
  testthat::expect_equal(as.data.frame(fh_out$ORA$BP)[1:2,"ID"], c("GO:0043277", "GO:0006911"))
})

#! /usr/bin/env Rscript

#' @author Fernando Moreno Jabato <jabato(at)uma(dot)es>

#############################################
### CONFIGURE 
#############################################

options(warn=1)
if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){
  # Loading libraries
  # suppressPackageStartupMessages(require(optparse)) 
  # suppressPackageStartupMessages(require(TCC))

  # Obtain this script directory
  full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile),  
                 error=function(e) # works when using R CMD
                normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                  commandArgs())], '='))[2]))
  main_path_script <- dirname(full.fpath)
  root_path <- file.path(main_path_script, '..', '..')
  # Load custom libraries
  custom_libraries <- c('simulate_treatment_control.R')
  for (lib in custom_libraries){
    source(file.path(root_path, 'R', lib))
  }
}else{
  require('ExpHunterSuite')
  root_path <- find.package('ExpHunterSuite')
}




#Loading libraries

option_list <- list(
  optparse::make_option(c("-r", "--replicates"), type="integer", default=3,
    help="Number of replicates for control/treatment group. Default: %default"),
  optparse::make_option(c("-n", "--ngenes"), type="integer", default=20000,
    help="Number of genes. Default : %default"),
  optparse::make_option(c("-d", "--DEGs_proportion"), type="double", 
    default=0.2,
    help=paste0("Proportion of differentially expressed genes (DEGs).",
      " Default: %default")),
  optparse::make_option(c("-f", "--FC_min"), type="double", default=1.3,
    help="Minimum Fold Change value. Default : %default"),
  optparse::make_option(c("-F", "--FC_max"), type="double", default=3.0,
    help="Maximum Fold Change value. Default : %default"),
  optparse::make_option(c("-T", "--P_up"), type="double", default=1,
    help=paste0("Proportion of DEGs that will be up-regulated in treatment",
      " group. Rest will be down-regulated. Default : %default")),
  optparse::make_option(c("-i", "--inputfile"), type="character", 
    default = NULL,
    help="[OPTIONAL] Input genes count matrix to be used for simulation"),
  optparse::make_option(c("-g", "--group"), type="character", 
    default = NULL,
    help=paste0("Columns from input file to be used as same group of ",
      "replicates. Specify indices separated by commas. Note: start at 1")),
  optparse::make_option(c("-o", "--outfile"), type="character",
    help=paste0("Output file basenames. A *_scount and *_predv files will",
      " be generated"))
)

opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))


#############################################
### LOAD & PREPARE 
#############################################

degsynth(
  inputfile = opt$inputfile,
  outfile = opt$outfile,
  replicates = opt$replicates,
  ngenes = opt$ngenes,
  DEGs_proportion = opt$DEGs_proportion,
  FC_min = opt$FC_min,
  FC_max = opt$FC_max,
  P_up = opt$P_up,
  group = opt$group)

#' Funtion used to scale a vector of numeric values from 
#' (x_min,x_max) range to 
#' (y_min,y_max) range.
#'     f : R -> R
#'      (x_min, x_max) -> (y_min, y_max)
#'                         vect - x_min
#'  f(vect,y_min,y_max) =  ------------- x (y_max - y_min) + y_min
#'                         x_max - x_min
#'
#' @param vect vector to be transformated
#' @param nmin new minimum
#' @param nmax new maximum
#' @return transformated vector to new range
scale_range <- function(vect,nmin,nmax){
    ((vect - min(vect))/(max(vect)-min(vect)))*(nmax-nmin)+nmin
}


#' Custom function to generate exponentially degradated foldchange range
#' @param means means vector
#' @param fcmin minimum foldchange
#' @param fcmax maximum foldchange
#' @param meanlog param of rlnorm function
#' @param sdlog param of rlnorm function
#' @return a vector of foldchanges to be applied
#' @importFrom stats ecdf rlnorm
fcfunc <- function(means,fcmin=1.4, fcmax=3, meanlog = 1, sdlog = 0.8){
    # Generate exponential distributio 
    xx <- stats::rlnorm(length(means), meanlog = meanlog, sdlog = sdlog)
    xx <- scale_range(xx,fcmin, fcmax)
    ecdffun <- stats::ecdf(xx)
    xx <- data.frame(FC = xx, Quant = unlist(lapply(xx,ecdffun)))
    xx <- xx[order(xx$Quant),]
    xx$Quant <- xx$Quant[seq(from = nrow(xx), to = 1)] # Apply inverse 
    # Prepare quantiles of observed
    ecdffun <- stats::ecdf(means)
    means <- data.frame(X = means, Quant = unlist(lapply(means,ecdffun)))
    # Merge values
    if(all(means$B %in% xx$B)){
        means <- merge(means,xx,by.y = "Quant",sort = FALSE) 
    }else{
        stop("Close search not implemented yet")
    }
    return(means)
}

#' Main function to generate synthetic data using an specific exponential 
#' distribution for logFC
#' @param outfile output file
#' @param inputfile input file
#' @param replicates number of replicates
#' @param ngenes number of genes
#' @param DEGs_proportion numeric (0,1) proportion of DEG genes to be simulated
#' @param FC_min minimum Fold-Change
#' @param FC_max maximum Fold-Change
#' @param P_up proportion og up-regulated genes
#' @param group optional group identifiers seprated by commas
#' @keywords synthetic
#' @export
#' @return synthetic data generated
#' @importFrom utils read.table write.table
#' @examples
#' synthetic_dataset <- degsynth()
#' # Returns simulated count dataset and indication of which are DE/not DE
degsynth <- function(
    outfile = NULL,
    inputfile = NULL,
    replicates = 3,
    ngenes = 20000,
    DEGs_proportion = 0.2,
    FC_min = 1.3,
    FC_max = 3.0,
    P_up = 1,
    group = NULL
    ){

    # Load count table if proceed
    if(is.null(inputfile)){
        bcount <- NULL
        group <- NULL
    }else{
        bcount <- utils::read.table(file = inputfile, row.names = 1, 
            header = TRUE, sep = "\t")
        if(is.null(group)){
            group <- NULL
        }else{
            group <- as.numeric(unlist(strsplit(group,",")))
        }
    }

    # Prepare DEGs proportion
    posdeg <- DEGs_proportion * P_up
    negdeg <- DEGs_proportion - posdeg


    #############################################
    ### SIMULATE 
    #############################################
    simul <- STC(Ngene      = ngenes,
                 DEG.foldchange = function(means){fcfunc(means,
                                                          fcmin = FC_min,
                                                          fcmax = FC_max)}, 
                 replicates = replicates, 
                 bcount     = bcount, 
                 group      = group, 
                 PDEG       = c(posdeg,negdeg))
    prediction_vector <- simul$trueDEG
    prediction_vector <- replace(prediction_vector, 
        prediction_vector != 0, "TRUE")
    prediction_vector <- replace(prediction_vector, 
        prediction_vector == 0, "FALSE")
    prediction_vector <- as.data.frame(prediction_vector)
    prediction_vector <- cbind(rownames(prediction_vector),prediction_vector)
    colnames(prediction_vector) <- c("Gene","Prediction")




    #############################################
    ### EXPORT OR RETURN
    #############################################
    if(is.null(outfile)) {
        return(list(simul_count=simul$count, 
                    prediction_vector=prediction_vector))
    } else {
        utils::write.table(simul$count, file=paste0(outfile,"_scount"), 
            quote = FALSE, col.names = TRUE, sep = "\t")
        utils::write.table(prediction_vector, file=paste0(outfile,"_predv"), 
            quote = FALSE, col.names = TRUE, sep = "\t", row.names = FALSE)
    }
}

###############################################################################
############### Plot functions
###############################################################################
#' @importFrom knitr knit knit_expand
#' @importFrom stats runif
resize <- function(g, fig_height=5, fig_width=12) {
  g_deparsed <- paste0(deparse(function() {g}), collapse = '')
  sub_chunk <- paste0("\n```{r sub_chunk_", floor(stats::runif(1) * 10000), 
    ", fig.height=", fig_height, ", fig.width=", fig_width, 
    ", echo=FALSE}", "\n(", g_deparsed, ")()\n```\n\n\n")
  cat(knitr::knit(text = knitr::knit_expand(text = sub_chunk), quiet = TRUE))
}

#' @importFrom ggplot2 fortify
#' @importFrom DT datatable
plot_enrResult_DT <- function(ER){
  toshow <- ggplot2::fortify(ER)[,!colnames(ER@compareClusterResult) %in% 
                                c("pvalue","qvalue")]
  toshow$Cluster <- gsub("[\n,\t,\r]{0,1}\\(.*\\)","",toshow$Cluster)
  DT::datatable(toshow, filter = 'top', rownames = FALSE, 
                      extensions = c('Buttons','ColReorder'),
                      options = list(
                        colReorder = TRUE,
                        dom = 'lftBip',
                          buttons = c('copy', 'csv', 'excel')
  ))
}


#' @importFrom stats runif
rechunk <- function(code, counter = NULL,chunk_options = "") {

   if (chunk_options != ""){
    chunk_options <- paste0(", ", chunk_options)
   } 
   if (!is.null(counter)){
    counter <- counter + 1
   } else {
    counter <- floor(stats::runif(1) * 10000)
   }
  if (!is.character(code)){
   code_deparsed <- paste0(deparse(function() {code}), collapse = '')
   code_deparsed <- paste0("(", code_deparsed, ")()")
  } else {
    code_deparsed <- code
  }
   sub_chunk <- paste0("\n```{r sub_chunk_", counter, chunk_options, 
   "}", "\n", code_deparsed, "\n```\n\n\n")

   cat(knitr::knit(text = knitr::knit_expand(text = sub_chunk), quiet = TRUE))
   return(counter)
}



#' @importFrom knitr knit knit_expand
#' @importFrom stats runif
plot_in_div <- function(g, fig_height=7, fig_width=12, 
## height and width in inches
  cex = 1, #size multiplier
  max_size = 50, # max size for plots in inches
  min_size = 10, 
  counter = NULL) {
  cat('\n<div class="plot_real_size">\n')
  fig_height <- fig_height * cex
  fig_width <- fig_width * cex

  if(fig_height > max_size){
    fig_height <- max_size
    }else if(fig_height < min_size){
    fig_height <- min_size
  }
  if(fig_width > max_size){
    fig_width <- max_size
    }else if(fig_width < min_size){
    fig_width <- min_size  
  }
  g_deparsed <- paste0(deparse(function() {g}), collapse = '')
  # set.seed(Sys.time())
  if (!is.null(counter)){
    chunk_name <- paste0("sub_chunk_", counter)
    counter <- counter + 1
  } else {
    chunk_name <- paste0("sub_chunk_", floor(stats::runif(1) * 10000))
  }
  sub_chunk <- paste0("\n```{r ", chunk_name, ", fig.height=", 
    fig_height, ", fig.width=", fig_width, ", echo=FALSE}", 
    "\n(", g_deparsed, ")()\n```\n\n\n") 
    # sub_chunk_", floor(runif(1) * 1000000), "
  cat(knitr::knit(text = knitr::knit_expand(text = sub_chunk), quiet = TRUE))
  cat('\n</div>\n')
  return(counter)
}
 
get_plot_df <- function(enrich_obj, showCategory = 30) {
  extract_geneSets <- get_unexported_function("enrichplot", "extract_geneSets")
  list2df <- get_unexported_function("enrichplot", "list2df")
  update_n <- get_unexported_function("enrichplot", "update_n")
  geneSets <- extract_geneSets(enrich_obj, update_n(enrich_obj, showCategory))
  geneSets <- list2df(geneSets)
  return(geneSets)
}

check_categories <- function(enrichplot_obj, min_categories = 1) {
  aux <- enrichplot_obj@result
  categories_count <- length(aux$Description[aux$p.adjust <= 
                                            enrichplot_obj@pvalueCutoff])  
  check <- categories_count > min_categories 
  return(check)
}

get_clusters_count <- function(results_WGCNA){
  cl_count <- length(unique(
                    results_WGCNA[['gene_cluster_info']][['Cluster_ID']]))
  return(cl_count)
}

get_features_count <- function(results_WGCNA){
  trait_count <- length(colnames(
          results_WGCNA[['package_objects']][['module_trait_cor']]))
  return(trait_count)
}

get_clusters <- function(compareCluster_obj){ # TODO: what's happening here?
    all_clusters <- compareCluster_obj@compareClusterResult$Cluster
    return(unique(all_clusters))
}

get_categories <- function(enrich_obj, showCategory = 30){
  if(is.null(enrich_obj)) return(NULL)
  enrich_obj_class <- class(enrich_obj)[1]
  if(enrich_obj_class == "enrichResult"){
      categories <- get_plot_df(enrich_obj, showCategory)$categoryID
    }else if(enrich_obj_class == "compareClusterResult"){
      categories <- enrich_obj@compareClusterResult$Description
    }else{
      warning("Not valid enrich object")
      return(NULL)
    }
    return(unique(categories))
}

get_genes <- function(enrich_obj, showCategory = 30){
  genes <- get_plot_df(enrich_obj, showCategory)$Gene
  return(unique(genes))
}

force_max_genes_enrobj <- function(enrich_obj, maxGenes = 200, showCategory = 30) {
  plot_df <- get_plot_df(enrich_obj, showCategory)
  for(i in seq_len(nrow(plot_df))) {
    sub_df <- plot_df[1:i,]
    unique_genes <- unique(sub_df$Gene)
    if(length(unique_genes) == maxGenes) {
      cat_ids <- plot_df[1:i, "categoryID"]
      return(enrich_obj[enrich_obj$Description %in% cat_ids, asis=TRUE])
    }
  }
}

calc_width_clusters <- function(elements, multiplier = 0.3){
 width <- elements * multiplier
 return(width)
}

calc_height_clusters <- function(elements, multiplier = 0.3){
  height <- elements * multiplier
  return(height)
}

calc_width <- function(enrich_obj, 
  showCategory = 30, 
  category_character_size = 0.035, 
  character_width = 3, 
  legend_size = 1){
  enrich_obj_class <- class(enrich_obj)[1]
  if(enrich_obj_class %in% c("enrichResult", "gseaResult")){
    longer_x_element <- length(get_genes(enrich_obj, showCategory))
  }else if(enrich_obj_class == "compareClusterResult"){
      longer_x_element <- length(get_clusters(enrich_obj))
    }else{
      warning("Not valid enrich object")
  }

  width_size <- (legend_size + (category_character_size * 
    max(nchar(as.character(get_categories(enrich_obj, showCategory))))) + 
    (character_width * longer_x_element))

  if(width_size > 100){
    width_size <- 50
  }
  return(width_size)
}

calc_height <- function(enrich_obj, 
  showCategory = NA, 
  min_size = 7, 
  gene_character_size = 3, 
  category_name_size = 0.1){
  enrich_obj_class <- class(enrich_obj)[1]
  if(enrich_obj_class  %in% c("enrichResult", "gseaResult")){
    plot_area_elements <- max(nchar(as.character(get_genes(enrich_obj, 
      showCategory = showCategory))))
  }else if(enrich_obj_class == "compareClusterResult"){
      plot_area_elements <- 2
    }else{
      warning("Not valid enrich object")
  }
  if(!is.na(showCategory)){
    categories <- showCategory
  } else {
    categories <- length(get_categories(enrich_obj))

  }
    height_size <- (gene_character_size* plot_area_elements) + 
        (category_name_size * categories)
  if(height_size < min_size){
    height_size <- min_size
  } else if(height_size > 100){
    height_size <- 50
  }

  return(height_size)
}

set_default_width <- function(enrich_obj, 
  default = 12, 
  showCategory = 30, 
  threshold = 30, 
  character_size = 0.04){
    longer_category <- max(nchar(as.character(get_categories(enrich_obj, 
                                                           showCategory))))
    if(longer_category > threshold){
        default_width <- default + character_size * longer_category
    }else{
        default_width <- default
    }
    return(default_width)
}
# Prepare resize function

#' @importFrom ggplot2 ggplot aes_string geom_tile
clusters_heatplot <- function(compare_cluster_obj){
  pp <- ggplot2::ggplot(compare_cluster_obj, ggplot2::aes_string(x = "Cluster", 
        y = "Description", fill = "p.adjust")) + 
  ggplot2::geom_tile() 
}


#' @importFrom ggplot2 theme_light theme element_text element_blank
set_standard_size <- function(pp){
  pp <- pp + ggplot2::theme_light() + 
        ggplot2::theme(axis.text.y = ggplot2::element_text(size = 10, 
          face = 'bold'), axis.title = ggplot2::element_blank()) 
  return(pp)
}

#' @importFrom ggplot2 ggplot aes_string geom_tile theme_minimal theme 
#' element_blank element_text scale_fill_gradient2 geom_text
gg_heatmap <- function(data_table, 
  input = "", 
  traspose = FALSE, 
  y_axis= "y_axis",
  x_axis = "x_axis", 
  fill = "Freq",
  text_plot = NULL,
  labs = TRUE,
  x_angle = 25,
  text_size = 2.5,
  text_colour = "black",
#  dendro = NULL,
  col = c("#0000D5","#FFFFFF","#D50000"),
  na_col = "grey50"){

    if(traspose){
      data_table <- t(data_table)
    }
    if(input == "matrix"){
      data_table <- as.data.frame(as.table(as.matrix(data_table)))
      colnames(data_table) <- c(y_axis, x_axis, fill)
    }

    pp <- ggplot2::ggplot(data_table, ggplot2::aes_(x = as.name(x_axis), 
          y = as.name(y_axis), fill = as.name(fill))) +
    ggplot2::geom_tile(show.legend = TRUE) +
    ggplot2::theme_minimal() +
    ggplot2::theme(panel.grid.major = ggplot2::element_blank())+
    ggplot2::theme(axis.text.x = ggplot2::element_text(angle = x_angle, 
        face = "bold", hjust = 1),
      axis.text.y = ggplot2::element_text(face = "bold")) +
    ggplot2::scale_fill_gradient2(
      low = col[1],
      high = col[3],
      mid= col[2],
      na.value = na_col,
      guide = "colourbar",
      aesthetics = "fill"
    )
    if(!labs){
      pp <- pp + ggplot2::theme(
        axis.title.x = ggplot2::element_blank(),
        axis.title.y = ggplot2::element_blank())
    }
    if(!is.null(text_plot)){
      pp <- pp + ggplot2::geom_text(ggplot2::aes_(label=as.name(text_plot)), 
        colour = text_colour, size = text_size) 
    }
    return(pp)
}

#' @importFrom ggplot2 ggplot_gtable ggplot_build
extract_legend <- function(a.gplot){ 
# https://stackoverflow.com/questions/13649473/(...)
# (...)add-a-common-legend-for-combined-ggplots  
  tmp <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(a.gplot))
  leg <- which(unlist(lapply(tmp$grobs, function(x) x$name) == "guide-box"))
  legend <- tmp$grobs[[leg]]
  return(legend)
}


make_top_n_expression_table <- function(count_data, n=5) {
  top_n_index <- order(rowSums(count_data), decreasing=TRUE)[1:n]
  sample_totals <- colSums(count_data)
  top_n_count <- count_data[top_n_index, ]
  top_n_perc <- apply(top_n_count, 1, function(x) { 
    round(x / sample_totals * 100, 3)
  })
  knitr::kable(t(top_n_perc))
}



#' Function to generate scatter plot for each gene in a hunter table
#' and show logFC per package
#' taking into account the variablity between them
#' @param ht : hunter table dataframe
#' @param var_filter : variability threshold to show gene into this
#' graph (low variability will be removed)
#' @param title : plot title
#' @param top : plots only the top N items with more variance.
#' NULL will not filter
#' @param alpha : transparency of dots
#' @return plot ready to be rendered
#' @author Fernando Moreno Jabato <jabato(at)uma(dot)com>
#' @importFrom ggplot2 ggplot aes_string geom_point scale_shape_manual ylim 
#' xlab ggtitle theme theme_classic element_blank
#' @importFrom stats var
ht2logFCPlot <- function(ht,
  var_filter = 0.001, 
  title = "Filtered logFC", 
  top = 50, 
  alpha = 0.5){

  logFC_columns <- grep("logFC_", colnames(ht), value=TRUE)
  logFCs_tab <- data.frame(gene_name=row.names(ht), ht[,logFC_columns])
  logFCs_tab <- logFCs_tab[! apply(logFCs_tab, 1, anyNA), ]
  logFCs_tab$var <- apply(logFCs_tab[,-1], 1, var)
  variable_genes <- logFCs_tab[logFCs_tab$var > var_filter, "gene_name"]
  logFCs_tab <- logFCs_tab[variable_genes, ]
  logFCs_tab <- logFCs_tab[order(logFCs_tab$var, decreasing=TRUE), ]
  if(nrow(logFCs_tab) > top) logFCs_tab <- logFCs_tab[seq(top), ]  
  logFCs_tab$gene_name <- factor(logFCs_tab$gene_name, 
    levels = logFCs_tab$gene_name[order(logFCs_tab$var, decreasing=TRUE)])

  if( length(variable_genes) == 0) return(paste("No genes have variance greater than", 
                                                var_filter))

  plot_tab <- reshape2::melt(logFCs_tab, id="gene_name", measure.vars=logFC_columns)
  names(plot_tab) <- c("Gene", "Package", "logFC")
  plot_tab$Package <- gsub("logFC_", "", plot_tab$Package)

  pp <- ggplot2::ggplot(plot_tab, ggplot2::aes_string(x = "Gene", 
      y = "logFC", colour = "Package", shape = "Package")) + 
        ggplot2::geom_point(alpha = alpha) 
  pp <- pp + 
    ggplot2::xlab(paste0("Gene (",100 - round(length(variable_genes)/length(
      unique(rownames(ht))),4)*100,
    "% of genes have lower variability than the threshold)")) + 
    ggplot2::ggtitle(title) + 
    ggplot2::theme_classic() + 
    ggplot2::theme(text = ggplot2::element_text(size = 14),axis.text.x=ggplot2::element_blank(),
        axis.ticks.x=ggplot2::element_blank()) +
    ggplot2::geom_hline(yintercept = 0,linetype="dashed", color = "#636363")
  return(pp)
}


#' @importFrom ggplot2 aes_string aes ggplot geom_point scale_colour_gradientn guide_colorbar guides
#' @importFrom ggrepel geom_text_repel 
plot_odds_ratio <- function(cont_tables, OR_col = "Odds_ratio", pval_col = "Pvalue", y_col = "strategy", text_col = "TP") {
  cont_tables <- as.data.frame(cont_tables)
  idx <- order(cont_tables[,OR_col], decreasing = TRUE)
  cont_tables[,y_col] <- factor(cont_tables[,y_col],
                            levels=rev(unique(cont_tables[,y_col][idx])))
 

  OR <- ggplot2::ggplot(cont_tables, ggplot2::aes_string(x = OR_col, y = y_col)) +
  ggplot2::geom_point(ggplot2::aes_string(color = pval_col, size = "TP")) + 
  ggrepel::geom_text_repel(ggplot2::aes(label = as.character(cont_tables[,text_col]))) +
  ggplot2::scale_colour_gradientn(
    colours = c("red", "white", "blue"),
    values = c(0, 0.05, 1),
    name = pval_col, 
    guide=ggplot2::guide_colorbar(reverse=TRUE))+
  ggplot2::guides(size = FALSE)
  return(OR)
}

get_pie_clusters <- function(enrich_DF, enrichplot){
  showed_categories <- enrichplot$data$name
  all_categories <- enrich_DF@compareClusterResult$Description
  plotted_clusters <- enrich_DF@compareClusterResult[all_categories %in% showed_categories,"Cluster"]
  return(unique(plotted_clusters))
}

#' @importFrom enrichplot cnetplot
clnetplot <- function(compareCluster, ...) {
  mod_enrich_obj <- compareCluster
  mod_enrich_obj@compareClusterResult$geneID <- as.character(mod_enrich_obj@compareClusterResult$Cluster)
  clnet_plot <- enrichplot::cnetplot(mod_enrich_obj, ...) 
  return(clnet_plot)
}



parse_dim_table <- function(dimension, dim_data, tag = "##"){
  pca_dim_data <- dim_data[[dimension]]

  cat(paste0(tag, " Valiable and factors inpection for ", gsub("Dim.", "dimension ", dimension), "\n"))
  quanti_data <- pca_dim_data$quanti
  cat(paste0(tag,"# **Quantitative** Variables and Dimension Relationship\n"))
  cat("This table shows the relationship between the dimension and the numerical variables. 
    It includes the Correlation coefficient as a association metric and P value as significance mentric. \n")
  quanti_table <- DT::datatable(quanti_data, filter = 'top', rownames = TRUE, extensions = c('Buttons','ColReorder'),
    options = list(paging = TRUE,
                   colReorder = TRUE,
                   dom = 'lftBip',
                   buttons = c('copy', 'csv', 'excel')))
  invisible(rechunk(quanti_table, chunk_options = "echo=FALSE, results = 'asis', warning = FALSE, message = FALSE"))

  cat(paste0(tag,"# **Qualitative** Variables and Dimension Relationship\n"))
  cat("This table shows the relationship between the dimension and the categorical variables. 
       It includes the P value as significance mentric and R2 as a association metric.
       R2 represents the proportion (in ranges from 0 to 1) of the total variability in the categorical variable that is accounted for by the PCA dimension.
       A higher R2 value indicates a better fit or a stronger association between the variable and the dimension. \n")
   quali_table <- DT::datatable(pca_dim_data$quali, filter = 'top', rownames = TRUE, extensions = c('Buttons','ColReorder'),
    options = list(paging = TRUE,
                   colReorder = TRUE,
                   dom = 'lftBip',
                   buttons = c('copy', 'csv', 'excel')))
  invisible(rechunk(quali_table, chunk_options = "echo=FALSE, results = 'asis', warning = FALSE, message = FALSE"))

  cat(paste0(tag,"# **Qualitative** Factors and Dimension Relationship\n"))
   quali_factors_table <- DT::datatable(pca_dim_data$category, filter = 'top', rownames = TRUE, extensions = c('Buttons','ColReorder'),
    options = list(paging = TRUE,
                   colReorder = TRUE,
                   dom = 'lftBip',
                   buttons = c('copy', 'csv', 'excel')))
  invisible(rechunk(quali_factors_table, chunk_options = "echo=FALSE, results = 'asis', warning = FALSE, message = FALSE"))


}

#' Main DEgenes Hunter Function
#'
#' This function allows you to perform the DEG analysis with different 
#' algorithms.
#' @param raw Dataframe with raw counts per sample and feature
#' @param target set with control and treatment information
#' @param external_DEA_data external DEA set 
#' @param count_var_quantile quantile variance threshold
#' @param output_files otput files path
#' @param reads set of reads
#' @param minlibraries minimum of libraries to use a set of data
#' @param filter_type filter type to ba applied. Allowed: "separate" or "global"
#' @param p_val_cutoff p-value threshold
#' @param lfc minimum logFold Change
#' @param modules modules to be executed. Allowed: DESeq2 (D), NOISeq (N), 
#' Limma (L), EdgeR (E), WGCNA (W)
#' @param minpack_common minimum of pack that must be significant to tag 
#' a gene as significant
#' @param model_variables custom model
#' @param pseudocounts boolean, activate if the input contains pseudocounts
#' @param numerics_as_factors transform numeric values to factors. Default: TRUE
#' @param string_factors string factors for WGCNA
#' @param numeric_factors numeric factors for WGCNA
#' @param WGCNA_memory see WGCNA package
#' @param WGCNA_norm_method see WGCNA package
#' @param WGCNA_deepsplit see WGCNA package
#' @param WGCNA_min_genes_cluster see WGCNA package
#' @param WGCNA_detectcutHeight see WGCNA package
#' @param WGCNA_mergecutHeight see WGCNA package
#' @param WGCNA_all see WGCNA package
#' @param WGCNA_blockwiseNetworkType see WGCNA package
#' @param WGCNA_blockwiseTOMType see WGCNA package
#' @param WGCNA_minCoreKME see WGCNA package
#' @param WGCNA_minCoreKMESize see WGCNA package
#' @param WGCNA_minKMEtoStay see WGCNA package
#' @param WGCNA_corType see WGCNA package
#' @param multifactorial specify interaction/effect when multifactorial design
#' @param library_sizes NULL or a dataframe with sample names and library sizes
#' @return expression analysis result object with studies performed
#' @keywords method
#' @importFrom rlang .data
#' @export
#' @examples
#' data(toc)
#' data(target)
#' degh_out <- main_degenes_Hunter(raw=toc, target=target, modules="D")

main_degenes_Hunter <- function(
    raw = NULL,
    pseudocounts = FALSE,
    target = NULL,
    count_var_quantile = 0,
    external_DEA_data = NULL,
    output_files = getwd(),
    reads = 2,
    minlibraries = 2,
    filter_type = "separate",
    p_val_cutoff = 0.05,
    lfc = 1,
    modules ="DELNW",
    minpack_common = 4,
    model_variables = "",
    numerics_as_factors = TRUE,
    string_factors = NULL,
    numeric_factors = NULL,
    WGCNA_memory = 5000,
    WGCNA_norm_method = "DESeq2",
    WGCNA_deepsplit = 2,
    WGCNA_min_genes_cluster = 20,
    WGCNA_detectcutHeight = 0.995,
    WGCNA_mergecutHeight = 0.25,
    WGCNA_all = FALSE,
    WGCNA_blockwiseNetworkType = "signed",
    WGCNA_blockwiseTOMType = "signed",
    WGCNA_minCoreKME = 0.5,
    WGCNA_minCoreKMESize = NULL,
    WGCNA_minKMEtoStay = 0.2,
    WGCNA_corType = "pearson",
    multifactorial = "",
    library_sizes = NULL
  ){
    modified_input_args <- check_input_main_degenes_Hunter(raw, 
      minlibraries, pseudocounts,reads, external_DEA_data, modules, model_variables,
      active_modules, WGCNA_all, minpack_common, target, 
      string_factors, numeric_factors, multifactorial)
    modules <- modified_input_args[['modules']]
    active_modules <- modified_input_args[['active_modules']]
    minpack_common <- modified_input_args[['minpack_common']]
    raw <- modified_input_args[['raw']]
    reads <- modified_input_args[['reads']]
    target <- modified_input_args[['target']]
    final_main_params <- list(
      'minpack_common' = minpack_common,
      'p_val_cutoff' = p_val_cutoff,
      'lfc' = lfc,
      'modules' = modules,
      'active_modules' = active_modules
    )

    ############################################################
    ##                         I/O DATA                       ##
    ############################################################


    # Infer replicates and group index from target
    index_control_cols <- as.character(target$sample[target$treat == "Ctrl"])
    index_treatmn_cols <- as.character(target$sample[target$treat == "Treat"])
    replicatesC <- length(index_control_cols)
    replicatesT <- length(index_treatmn_cols)
    design_vector <- c(rep("C", replicatesC), rep("T", replicatesT))
    sample_groups <- data.frame(class = design_vector, 
                                name = c(index_control_cols, 
                                         index_treatmn_cols))
    # Check if there are enough replicates for specified method
    if((replicatesC < 2) | (replicatesT < 2)) 
       stop(paste0('At least two replicates per class (i.e. treatment and',
                   ' control) are required\n'))

    
    numeric_factors <- split_str(numeric_factors, ",")
    if (sum(numeric_factors == "") >= 1) numeric_factors <- NULL #esto esta para controlar que no haya elementos vacios

    string_factors <- split_str(string_factors, ",")

    string_factors <-  c("treat", string_factors)

    
    if(!is.null(target) & grepl("W", modules)) {
      target_numeric_factors <- build_design_for_WGCNA(target, 
           numeric_factors=numeric_factors)
      target_string_factors <- build_design_for_WGCNA(target, 
          string_factors=string_factors)
    }

    if(numerics_as_factors == TRUE) { 
      # Now coerce the targets to factors for the multifactorial analysis
       target <-  data.frame(sapply(target, as.factor), 
         stringsAsFactors=TRUE)

    }
    model_formula_text <- prepare_model_text(model_variables, 
                                             multifactorial)
   
    if(grepl(":(interaction|effect)", multifactorial)){
      target <- prepare_target_for_multifactorial(target, multifactorial)
    }
    # Prepare count table for analysis
    #Indexing selected columns from input count dataframe   
    raw <- raw[c(index_control_cols,index_treatmn_cols)]
    raw[is.na(raw)] <- 0 # Substitute NA values

    filtered_data <- filter_count(reads, minlibraries, raw, filter_type, 
                               index_control_cols, index_treatmn_cols, target)
    raw_filter <- filtered_data[["raw"]]
    cpm_table <- filtered_data[["cpm_table"]]

    var_filter <- filter_by_variance(raw_filter, 
                                     q_filter = count_var_quantile, 
                                     target = target)
    default_norm <- list(default = 
                      as.data.frame(var_filter[["deseq2_normalized_counts"]]))
    raw_filter <- var_filter[["fil_count_mtrx"]]

    

    #computing PCA for all_genes
    full_pca <- compute_pca(pca_data = default_norm$default,
                            target = target,
                            string_factors = string_factors, 
                            numeric_factors = numeric_factors)
    PCA_res <- list("all_genes" = full_pca)
    ############################################################
    ##             PERFORM EXPRESION ANALYSIS                 ##
    ############################################################
    dir.create(output_files)


    exp_results <- perform_expression_analysis(modules, replicatesC, 
                     replicatesT, raw_filter, p_val_cutoff, target, 
                     model_formula_text, external_DEA_data, 
                     multifactorial)
    exp_results[["all_data_normalized"]] <- c(default_norm,
                                              exp_results[["all_data_normalized"]])

    #################################################################
    ##                       CORRELATION ANALYSIS                   ##
    ##################################################################

    combinations_WGCNA <- NULL
    if(grepl("W", modules)) { # CASE W: WGCNA
      cat('Correlation analysis is performed with WGCNA\n')
      path <- file.path(output_files, "Results_WGCNA")
      dir.create(path)
      all_data_normalized <- exp_results[['all_data_normalized']]
      if(WGCNA_norm_method %in% names(all_data_normalized)) {
        WGCNA_input <- all_data_normalized[[WGCNA_norm_method]]
      } else if(WGCNA_norm_method== "none") {
        WGCNA_input <- raw_filter
      } else {
        warning(paste0("To run WGCNA, you must also run the method chosen for",
          " normalization in the --modules flag, or specify none"))
      }

      combinations_WGCNA <- perform_WGCNA_combinations(WGCNA_all, WGCNA_input, 
        index_treatmn_cols, index_control_cols, path, target_numeric_factors, 
        target_string_factors, WGCNA_memory, WGCNA_deepsplit, 
        WGCNA_detectcutHeight, WGCNA_mergecutHeight, WGCNA_min_genes_cluster, 
        WGCNA_blockwiseNetworkType, WGCNA_blockwiseTOMType, WGCNA_minCoreKME, 
        WGCNA_minCoreKMESize, WGCNA_minKMEtoStay, corType = WGCNA_corType)

      if(length(combinations_WGCNA[['WGCNA_all']]) == 1) {
        warning("Something went wrong with WGCNA on the full dataset")
        modules <- gsub("W", "", modules)
      }
    }

    if(grepl("X", modules)) { # CASE X: diffcoexp
      cat('Correlation analysis is performed with diffcoexp\n')
      path <- file.path(output_files, "Results_diffcoexp")
      dir.create(path)

      results_diffcoexp <- analysis_diffcoexp(data = raw_filter,
                                           path = path,
                                           target = target)
    }


    #################################################################
    ##                    BUILD MAIN RESULT TABLE                  ##
    #################################################################
    mean_cpm <- rowMeans(raw_filter)
    DE_all_genes <- NULL
    DEG_pca <- NULL
    if (any(grepl("[DENL]",modules))){
      DE_all_genes <- unite_DEG_pack_results(exp_results, p_val_cutoff, 
                                             lfc, minpack_common)
      DE_all_genes <- merge(DE_all_genes, mean_cpm, by=0, sort=FALSE)
      names(DE_all_genes)[names(DE_all_genes) == "y"] <- "mean_expression_cpm"
      DE_all_genes <- transform(DE_all_genes, row.names=Row.names, Row.names=NULL)
      # Add the filtered genes back
      DE_all_genes <- add_filtered_genes(DE_all_genes, raw)


      #computing PCA for PREVALENT DEG

      prevalent_degs <- rownames(DE_all_genes[DE_all_genes$genes_tag == "PREVALENT_DEG",])
      if (length(prevalent_degs) > 1) {
        pca_deg_data <- default_norm$default
        pca_deg_data <- pca_deg_data[rownames(pca_deg_data) %in% prevalent_degs,]
        DEG_pca <- compute_pca(pca_data = pca_deg_data,
                              target = target,
                              string_factors = string_factors, 
                              numeric_factors = numeric_factors)
      }
    }
    PCA_res[["DEGs"]] <- DEG_pca

    if(grepl("W", modules)) { # Check WGCNA was run and returned proper results
      DE_all_genes <- merge(by.x=0, by.y="ENSEMBL_ID", x= DE_all_genes, 
          y =combinations_WGCNA[['WGCNA_all']][['gene_cluster_info']])
      rownames(DE_all_genes) <- DE_all_genes$Row.names
      DE_all_genes$Row.names <- NULL
    }

    correlation_metrics <- NULL
    if(grepl("P", modules)) { # CASE P: PCIT, TODO: RESTORE FUNCTION, PEDRO 
      # TODO : This is not working, variables "DESeq2_counts" 
      # are not being generated inside this function
      all_data_normalized <- exp_results[['all_data_normalized']]$DESeq2
      raw <- raw[c(index_control_cols,index_treatmn_cols)]
      correlation_metrics <- analysis_diff_correlation(
        DE_all_genes, 
        all_data_normalized, 
        all_data_normalized[index_control_cols], 
        all_data_normalized[index_treatmn_cols], 
        PCIT_filter=FALSE) # The analysis is very very slow, so we disable it
      DE_all_genes <- transform(merge(DE_all_genes, correlation_metrics, by.x=0, by.y=0), row.names=Row.names, Row.names=NULL)
    }

    if(grepl("X", modules)) { #results_diffcoexp
      aux <- row.names(DE_all_genes)
      DE_all_genes$DCG <- aux %in% 
         results_diffcoexp$DCGs$Gene[results_diffcoexp$DCGs$q < 1]
      DE_all_genes$DCL <- aux %in% results_diffcoexp$DCLs$Gene.1 | 
         aux %in% results_diffcoexp$DCLs$Gene.2
    }

   


    coverage_df <- get_counts(cnts_mtx=raw, library_sizes=library_sizes)
    mean_counts_df <- get_mean_counts(cnts_mtx=raw, cpm_table=cpm_table, reads=reads, minlibraries=minlibraries)
    exp_genes_df <- get_gene_stats(cpm_table=cpm_table, reads=reads)    

 

    final_results <- list()
    final_results[['cpm_table']] <- cpm_table
    final_results[['raw_filter']] <- raw_filter
    final_results[['sample_groups']] <- sample_groups
    final_results[['DE_all_genes']] <- DE_all_genes
    final_results[['index_control_cols']] <- index_control_cols
    final_results[['index_treatmn_cols']] <- index_treatmn_cols
    final_results[['design_vector']] <- design_vector
    final_results[['replicatesC']] <- replicatesC
    final_results[['replicatesT']] <- replicatesT
    final_results[['final_main_params']] <- final_main_params
    final_results[["var_filter"]] <- var_filter
    final_results[["coverage_df"]] <- coverage_df
    final_results[["mean_counts_df"]] <- mean_counts_df
    final_results[["exp_genes_df"]] <- exp_genes_df
    final_results[["target"]] <- target 
    final_results[["numeric_factors"]] <- numeric_factors
    final_results[["string_factors"]] <- string_factors
    final_results[["PCA_res"]] <- PCA_res
    if(!is.null(combinations_WGCNA)){
      final_results <- c(final_results, combinations_WGCNA)
    }
    final_results <- c(final_results, exp_results)
    return(final_results)


}


check_input_main_degenes_Hunter <- function(raw, 
                                            minlibraries, 
                                            pseudocounts = FALSE,
                                            reads, 
                                            external_DEA_data, 
                                            modules, 
                                            model_variables, 
                                            active_modules, 
                                            WGCNA_all, 
                                            minpack_common, 
                                            target, 
                                            string_factors, 
                                            numeric_factors,
                                            multifactorial){

    if (pseudocounts) {
      raw <- round(raw)
    }
    if (minlibraries < 1){
      stop(cat(paste0("Minimum library number to check minimum read counts",
        " cannot be less than 1.\nIf you want to avoid filtering, set",
        " --reads to 0.")))
    }

    if (reads == 0){
      cat(paste0("Minimum reads option is set to zero. ",
        "Raw count table will not be filtered."))
    }

    if (!is.null(external_DEA_data)) {
      message("External DEA dataframe given.")
      if(is.null(target)){
        warning(paste0("External DEA dataframe given but no target",
          " - creating a dummy one"))
        target <- data.frame(samples=c("case_dummy_1", "case_dummy_2", 
                                       "control_dummy_1", "control_dummy2"), 
                             treat=c("Treat","Treat", "Ctrl", "Ctrl"))
      }
      if(is.null(raw)) {
        warning(paste0("External DEA dataframe given but no count data",
          " - creating a dummy one"))
        raw <- as.data.frame(matrix(sample(1:100, nrow(external_DEA_data)*nrow(target),replace=TRUE), 
                                    nrow=nrow(external_DEA_data), 
                                    ncol=nrow(target), 
                                  dimnames = list(row.names(external_DEA_data), 
                                                  target$sample)))
        reads <- 0
      }
    }

    if (model_variables != "" & grepl("N", modules) & nchar(modules) == 1) {
      stop(cat(paste0("You cannot run an experimental design that uses the",
        " model_variables option with NOISeq only.")))
    }
    if (model_variables != "" & grepl("N", modules) & nchar(modules) > 1) {
      warning(paste0("NOISeq will not be run as you have an experimental",
        " design that uses the model_variables option."))
      modules <- gsub("N", "", modules)
    }
    if(model_variables != "" & multifactorial != ""){
      stop("Cannot provide both a --multifactorial design and additional vars",
        "via --model_variables")
    }

    if(grepl('P', modules) && !grepl('D', modules)){
      modules <- paste0(modules, 'D')
    }

    active_modules <- nchar(modules)
    user_modules <- unlist(strsplit(modules, '')) # TODO. Maybe use this variable for checks perfomed before
    active_modules <- active_modules - sum(grepl("[WPX]", user_modules)) # Remove from module count modules that are not involved in differential expresion
    if(sum(target$treat == "Ctrl") < 3 | sum(target$treat == "Treat") < 3)
          active_modules <- active_modules - sum(grepl("[LN]", user_modules))
    if(minpack_common > active_modules){
      minpack_common <- active_modules
      warning(paste0("The number of active modules is lower than the thresold",
        " for tag PREVALENT DEG. The thresold is set to the",
        " number of active modules."))
    }


    # If no -t check there is no -v value
    if( is.null(target) & model_variables != "") {
      stop(cat(paste0("You should not include a -v value if you do not",
        " have a target table file.")))
    }

    # If factors are specified but WGCNA not selected, throw a warning.
   
    return(list(modules=modules, 
                active_modules=active_modules, 
                minpack_common=minpack_common, 
                raw=raw, reads=reads, target=target))
}

#' @importFrom edgeR cpm
filter_count <- function(reads, 
                         minlibraries, 
                         raw, 
                         filter_type, 
                         index_control_cols, 
                         index_treatmn_cols,
                         target){
    # Prepare filtered set
    cpm_table <- edgeR::cpm(raw)
    if(reads != 0){
      if (filter_type == "separate") {
        # genes with cpm greater than --reads value for 
        # at least --minlibrariess samples for either case or control samples
        to_keep_control <- rowSums(cpm_table[, index_control_cols] > 
                                reads) >= minlibraries
        to_keep_treatment <- rowSums(cpm_table[, index_treatmn_cols] >
                                  reads) >= minlibraries
        # Keep if at least minlibraries in either of them
        keep_cpm <- to_keep_control | to_keep_treatment 
        raw <- raw[keep_cpm,] # Filter out count data frame
      } else if (filter_type == "global") {
        # genes with cpm greater than --reads value for
        # at least --minlibrariess samples
        keep_cpm <- rowSums(cpm_table >= reads) >= minlibraries 
        raw <- raw[keep_cpm,] # Filter out count data frame
      } else if (grepl("^combined", filter_type) == TRUE) {
        split_filter <- strsplit(filter_type, ":")[[1]]
        filt_factors <- strsplit(split_filter[2], ",")[[1]]
        if(! all(filt_factors %in% colnames(target)))
          stop("Filter factors not found in target")
        combs_list <- split(target, target[,filt_factors])
        samples_per_combo <- lapply(combs_list, function(x) as.vector(x$sample))
        cpm_tab_per_combo <- sapply(samples_per_combo, function(x) cpm_table[, x])
        combos_passing_filter <- sapply(cpm_tab_per_combo, 
          function(x) rowSums(x >= reads) >= minlibraries)
        keep_cpm <- apply(combos_passing_filter, 1, any)
        raw <- raw[keep_cpm,] # Filter out count data frame
      } else {
        warning("Unrecognized minimum read filter type. No filter will be used")
        raw <- raw
      }
    }
    return(list(raw=raw, cpm_table=cpm_table))
}

#' @importFrom matrixStats rowVars
#' @importFrom stats quantile formula
#' @importFrom DESeq2 DESeqDataSetFromMatrix DESeq counts
filter_by_variance <- function(count_matrix, q_filter, target){

  dds <- DESeq2::DESeqDataSetFromMatrix(countData = count_matrix,
                                  colData = target,
                                  design = stats::formula("~ treat"))
  dds <- DESeq2::DESeq(dds)
  normalized_counts <- DESeq2::counts(dds, normalized=TRUE)
  variances <- matrixStats::rowVars(normalized_counts)
  threshold <- stats::quantile(variances, q_filter)
  fil_count_mtrx <- count_matrix[variances >= threshold,]
  return(list(fil_count_mtrx = fil_count_mtrx, 
    variance_dis = variances, 
     thr = q_filter,
     deseq2_normalized_counts = normalized_counts))
}

prepare_model_text <- function(model_variables, 
                               multifactorial){
    # Prepare model text
    if(model_variables != "") {
      model_variables_unlist <- unlist(strsplit(model_variables, ","))
      model_formula_text <- paste("~", paste(model_variables_unlist, 
                                  "+", collapse=" "), "treat")
    } else if (multifactorial != "") {
      mf_text <- split_mf_text(multifactorial)
      mf_factor_A <- mf_text[[1]]
      mf_factor_B <- mf_text[[2]]
      model_formula_text <- paste0("~ ", mf_factor_B, " + ", 
                         mf_factor_A, " + ", mf_factor_B,":", mf_factor_A)
    } else {
      model_formula_text <- "~ treat"
    }
    cat("Model for gene expression analysis is:", model_formula_text, "\n")
    return(model_formula_text)
}

split_mf_text <- function(multifactorial) {
      factors_contrast <- strsplit(multifactorial,":")[[1]]
      factors <- strsplit(factors_contrast[1], ",")[[1]]
      mf_factorA <- factors[1]
      mf_factorB <- factors[2]
      contrast_varA_varB <- strsplit(factors_contrast[2],",")[[1]]
      mf_contrast <- contrast_varA_varB[1]
      mf_varA <- contrast_varA_varB[2]
      mf_varB <- contrast_varA_varB[3]
      return(list(mf_factorA=mf_factorA, mf_factorB=mf_factorB, 
        mf_contrast=mf_contrast, mf_varA=mf_varA, mf_varB=mf_varB))
}

prepare_target_for_multifactorial <- function(target, multifactorial) {
  mf_text <- split_mf_text(multifactorial)
  target[, mf_text[["mf_factorA"]]] <- stats::relevel(
                                             target[,mf_text[["mf_factorA"]]],
                                             mf_text[["mf_varA"]])
  target[, mf_text[["mf_factorB"]]] <- stats::relevel(
                                             target[,mf_text[["mf_factorB"]]],
                                             mf_text[["mf_varB"]])
  return(target)
}

get_counts <- function(cnts_mtx, library_sizes)
{
    if (!is.null(library_sizes)){

        total_counts <- library_sizes[, c("sample","initial_total_sequences")]
        # Total reads might have been counted without taking into account ExpHunterSuite blacklist,
        # which would lead to errors. This next line removes blacklisted samples from total reads table.
        # Also, we have no guarantee they are sorted the same way, so we do it ourselves.
        total_counts <- total_counts[match(colnames(cnts_mtx), total_counts$sample), ]
        # sample_ID column no longer needed
        total_counts <- total_counts$initial_total_sequences
        gene_counts <- colSums(cnts_mtx)
        counted_frac <- gene_counts/total_counts

    } else {
        total_counts <- colSums(cnts_mtx)
        counted_frac <- NULL
    }

    coverage_df <- data.frame(sample_ID = colnames(cnts_mtx),
                                total_counts = total_counts)
    coverage_df$counted_frac <- counted_frac

    coverage_df <- coverage_df[order(coverage_df$total_counts),]
    coverage_df$sample_rank <- seq(1, nrow(coverage_df))

    return(coverage_df)
}

get_mean_counts <- function(cnts_mtx, cpm_table, reads, minlibraries)
{
    passed_filter <- rowSums(cpm_table >= reads) >= minlibraries
    min_1_read <- rowSums(cnts_mtx >= 1) >= minlibraries
    min_10_reads <- rowSums(cnts_mtx >= 10) >= minlibraries

    # Create a vector with the strictest filter passed
    all <- data.frame(counts=rowMeans(cnts_mtx),
                      filter=rep("all", nrow(cnts_mtx)))
    min_1_read <- data.frame(counts=rowMeans(cnts_mtx[min_1_read,]),
                              filter=rep("min_1_read", nrow(cnts_mtx[min_1_read,]))) 
    min_10_reads <- data.frame(counts=rowMeans(cnts_mtx[min_10_reads,]),
                              filter=rep("min_10_reads", nrow(cnts_mtx[min_10_reads,]))) 
    passed_filter <- data.frame(counts=rowMeans(cnts_mtx[passed_filter,]),
                                filter=rep("passed_filter", nrow(cnts_mtx[passed_filter,])))

    res <- rbind(all,min_1_read,min_10_reads,passed_filter)
    return(res)
}

get_gene_stats <- function(cpm_table, reads)
{
    cutoff_passed_matrix <- cpm_table >= reads
    cutoff_passed_matrix <- cutoff_passed_matrix[rowSums(cutoff_passed_matrix) > 0, ]
    exp_genes_df <- data.frame(sample_ID = colnames(cutoff_passed_matrix),
                                expressed_genes = colSums(cutoff_passed_matrix))
    exp_genes_df <- exp_genes_df[order(exp_genes_df$expressed_genes),]
    exp_genes_df$count_rank <- seq(1,nrow(exp_genes_df))
    cutoff_passed_matrix <- cutoff_passed_matrix[, exp_genes_df$sample_ID]
    union_vec = inters_vec <- cutoff_passed_matrix[,1]
    for (sample in 1:ncol(cutoff_passed_matrix))
    {
        union_vec <- union_vec | cutoff_passed_matrix[,sample]
        inters_vec <- inters_vec & cutoff_passed_matrix[,sample]
        exp_genes_df$union_expressed_genes[sample] <- sum(union_vec)
        exp_genes_df$inters_expressed_genes[sample] <- sum(inters_vec)
    }
    return(exp_genes_df)
}

#!/usr/bin/env Rscript

options(warn=1)
if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){
  # Obtain this script directory
  full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile), 
                 error=function(e) # works when using R CMD
                normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                  commandArgs())], '='))[2]))
  main_path_script <- dirname(full.fpath)
  root_path <- file.path(main_path_script, '..', '..')
  # Load custom libraries
  custom_libraries <- c('general_functions.R', "io_handling.R",
    'functional_analysis_library.R', 'plotting_functions.R', "main_clusters_to_enrichment.R",
    "write_report.R")
  for (lib in custom_libraries){
    source(file.path(root_path, 'R', lib))
  }
  template_folder <- file.path(root_path, 'inst', 'templates')
  organisms_table_file <- file.path(root_path, "inst", "external_data", 
        "organism_table.txt")
} else {
  require('ExpHunterSuite')
  root_path <- find.package('ExpHunterSuite')
  template_folder <- file.path(root_path, 'templates')
  organisms_table_file <- file.path(root_path, "external_data", 
        "organism_table.txt")
}

########################## OPTIONS
option_list <- list(
  optparse::make_option(c("-i", "--input_file"), type="character", default=NULL,
                        help="2 columns - cluster and comma separated gene ids"),
  optparse::make_option(c("-w", "--workers"), type="integer", default=1,
                        help="number of processes for parallel execution. Default=%default"),
  optparse::make_option(c("-p", "--pvalcutoff"), type="double", default=0.1,
                        help="Cutoff for P value and adjusted P value for enrichments. Default=%default"),
  optparse::make_option(c("-q", "--qvalcutoff"), type="double", default=0.2,
                        help="Cutoff for Q value for enrichments. Default=%default"),
  optparse::make_option(c("-t", "--task_size"), type="integer", default=1,
                        help="number of clusters per task. Default=%default"),
  optparse::make_option(c("-F", "--force"), type="logical", default=FALSE, 
                        action = "store_true", help="Ignore temporal files"),
  optparse::make_option(c("-f", "--funsys"), type="character", default="BP,MF,CC,KEGG,Reactome", 
                        help="Funsys to execute: MF => GO Molecular Function, BP => GO Biological Process, CC => GO Celular Component, 
                        KEGG => KEGG, Reactome => Reactome, DO => Disease Ontology, DGN => DisGeNET. Default=%default"),
  optparse::make_option(c("-K", "--kegg_data_file"), ,type = "character", default=NULL,
                        help=paste0("KEGG database file. Can download with download_latest_kegg_db().",
                          "If required but not provided, it will be downloaded to the current working directory")), 
  optparse::make_option(c("--custom"), type="character", default=NULL,
                        help="Comma separated path of custom enrichment sets."),
  optparse::make_option(c("--showCategories"), type="integer", default=30, 
                        help="Number of top categories to show on clusterProfiler dotplot and emaplot."),
  optparse::make_option(c("-c", "--clean_parentals"), type="logical", default=FALSE, 
                        action = "store_true", help="Clean parentals GO terms that appears on the same clusters than child."),
  optparse::make_option(c("-s", "--simplify"), type="logical", default=FALSE, 
                        action = "store_true", help="Apply simplify function from cluster profiler to enrichment."),
  optparse::make_option(c("-O", "--model_organism"), type="character", default="Human", 
                        help="Model organism. Human or Mouse"),
  optparse::make_option(c("-M", "--mode"), type="character", default="PR", 
                        help="String indicating report modes to execute. R: Generate report for each cluster and all clusters combined. P: Plots (dotplot and emaplot) combining all cluster enrichments. S: Summary mode (sumamrized categories heatmap)"),
  optparse::make_option(c("-S", "--sim_thr"), type="double", default=0.7,
                        help="Similarity cutoff for grouping categories in Summary mode. Default=%default"),
  optparse::make_option(c("-U", "--universe"), ,type = "character", default=NULL,
                        help="One column file listing the universe genes."), 
  optparse::make_option(c("-C", "--summary_common_name"), type="character", default="ancestor", 
                        help="Name of the term groups. 'significant' to use the most significant term of each group. 'ancestor' to use the common ancestor of the group"),
  optparse::make_option(c("-T", "--top_categories"), type="integer", default=NULL,
                        help="For the merged ORA object, only consider the top n results per cluster. Default=%default"),
  optparse::make_option(c("-d", "--description_file"), type="character", default=NULL,
                        help="Markdown file describing of the enriched clusters."),
  optparse::make_option(c("-k", "--gene_keytype"), type="character", default="ENTREZID",
                        help="What identifier is being used for the genes in the clusters?. Default=%default"),
  optparse::make_option(c("--gmt_id"), type="character", default="ENTREZID",
                        help="What identifier is being used for the genes in the custom gmt file. Default=%default"),
  optparse::make_option(c("-g", "--gene_attribute_file"), type="character", default=NULL,
                        help="3 columns tabular file- Cluster - InputGeneID - NumericAttribute. Header must be indicated as cluster - geneid - [numeric_atribute]"),
  optparse::make_option(c("-G", "--group_results"), type="logical", default=FALSE, 
                        action = "store_true", help="Functions are grouped in most frequent words in emaplots."),
  optparse::make_option("--max_genes_plot", type="integer", default=200, 
                        help="Number of genes to show on clusterProfiler cnet"),
  optparse::make_option(c("-o", "--output_path"), type="character", default=NA,
                        help="Define the output path.")
)
opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))

##################################### INITIALIZE ##
library(clusterProfiler)
output_path <- opt$output_path
if (is.na(output_path)){
  output_path <- file.path(getwd(), "functional_enrichment")
}
dir.create(output_path)
output_path <- normalizePath(output_path)
temp_file <- file.path(output_path, "enr_tmp.RData")
all_funsys <- unlist(strsplit(opt$funsys, ","))
organisms_table <- get_organism_table(organisms_table_file)
current_organism_info <- organisms_table[rownames(organisms_table) %in% opt$model_organism,]
org_db <- get_org_db(current_organism_info)

#Load universe
if (is.null(opt$universe) || length(opt$universe) == 0){
  universe <- opt$universe
}else if (file.exists(opt$universe)){
  universe <- read.table(opt$universe)[,1]
  if (opt$gene_keytype != "ENTREZID"){
    universe <- translate_ids_orgdb(ids=universe, 
                                    input_id = opt$gene_keytype,
                                    output_id="ENTREZID",
                                    org_db = org_db, 
                                    just_output_ids=TRUE)
  }
}

# Load customs
all_custom_gmt <- NULL
if (!is.null(opt$custom)) {
    custom_files <- unlist(strsplit(opt$custom, ","))
    all_custom_gmt <- lapply(custom_files, load_and_parse_gmt)
    names(all_custom_gmt) <- custom_files
    names(all_custom_gmt) <- basename(names(all_custom_gmt))

  if(opt$gmt_id != "ENTREZID") {
    all_custom_gmt <- lapply(all_custom_gmt, function(gmt){
        tr_gmt <- translate_gmt(gmt, opt$gmt_id, org_db)
        return(tr_gmt)
    })
  }
}

if("KEGG" %in% all_funsys) {
  kegg_data_file <- get_kegg_db_path(opt$kegg_data_file, current_organism_info=current_organism_info)
  if(! file.exists(kegg_data_file)) stop(paste("KEGG file:", kegg_data_file, "not found"))
}

################################### MAIN ##
ce_list <- main_clusters_to_enrichment(
  input_file = opt$input_file,
  gene_attribute_file = opt$gene_attribute_file,
  org_db = org_db,
  gene_keytype = opt$gene_keytype,
  temp_file = temp_file,
  force = opt$force,
  all_funsys = all_funsys,
  task_size = opt$task_size,
  current_organism_info = current_organism_info,
  workers = opt$workers,
  pvalcutoff = opt$pvalcutoff,
  qvalcutoff = opt$qvalcutoff,
  all_custom_gmt = all_custom_gmt,
  kegg_data_file = kegg_data_file,
  universe = universe
)
enrichments_ORA <- ce_list[["enrichments_ORA"]]

write_clusters_to_enrichment(
  output_path=output_path,
  mode=opt$mode,
  enrichments_ORA=enrichments_ORA,
  task_size = opt$task_size,
  workers = opt$workers,
  template_folder = template_folder,
  top_categories = opt$top_categories,
  group_results = opt$group_results,
  n_category = opt$showCategories,
  sim_thr = opt$sim_thr, 
  summary_common_name = opt$summary_common_name, 
  pvalcutoff = opt$pvalcutoff,
  gene_attributes = ce_list[["gene_attributes"]],
  gene_attribute_name = ce_list[["gene_attribute_name"]], 
  max_genes = opt$max_genes_plot,
  simplify = opt$simplify,
  clean_parentals = opt$clean_parentals
)

## ----DEA_one_pack, echo=TRUE, results="hide", message=FALSE, warning=FALSE----
library(ExpHunterSuite)
data(toc)
data(target)
degh_out_one_pack <- main_degenes_Hunter(raw=toc, 
                                         target=target,
                                         modules="D") # D for DESeq2

## ----input_data, echo=TRUE, results="as.is", message=FALSE, warning=FALSE-----
head(toc)
head(target)

## ----input_files, echo=TRUE, results="as.is"----------------------------------
system.file("extData", "table_of_counts.txt", package = "ExpHunterSuite")
system.file("extData", "target.txt", package = "ExpHunterSuite")

## ----DEA_multi_pack, echo=TRUE, results="hide", message=FALSE, warning=FALSE----
degh_out_multi_pack <- main_degenes_Hunter(raw=toc, 
                                    target=target, 
                                    modules="DEL") # D:DESeq2 E:EdgeR, L:limma

## ----standard_DEA_results, echo=TRUE------------------------------------------
head(degh_out_multi_pack$DE_all_genes)

## ----standard_DEA_model_target, echo=TRUE, results="as.is"--------------------
target_multi <- data.frame(target,
  age_group = c("adult", "child", "adult", "child", "adult", "adult", "child"))
target_multi

## ----standard_DEA_model_execute, results="hide", message=FALSE, warning=FALSE----
degh_out_model <- main_degenes_Hunter(raw=toc, target=target_multi, 
                                      modules="D", model_variables="age_group")

## ----standard_CEA, results="hide", message=FALSE, warning=FALSE---------------
degh_out_coexp <- main_degenes_Hunter(raw=toc, target=target_multi, 
                                      modules="DW", string_factors="age_group")

## ----standard_FA, results="as.is",  eval=FALSE--------------------------------
# 
# fh_out_one_pack <- main_functional_hunter( #Perform enrichment analysis
#         degh_out_one_pack,
#         model_organism = 'Mouse', # Use specified organism database
#         enrich_dbs = c("MF", "BP", "CC", "Kegg", "Reactome"), # Enrichment analysis for GO, KEGG and Reactome
#         enrich_methods = "o" # Use overepresentation analysis only
# )
# 

## ----coexp_FA, results="as.is", eval=FALSE------------------------------------
# 
# fh_out_coexp <- main_functional_hunter( # Perform enrichment analisys
#         degh_out_coexp,
#         model_organism = 'Mouse', # Use specified organism database
#         enrich_dbs = c("MF", "BP", "CC", "Kegg", "Reactome"), # Enrichment analysis for GO, KEGG and Reactome
#         enrich_methods = "o" # Use overepresentation analysi only
# )
# 

## ----c2e, echo=TRUE,  eval=FALSE, results="hide"------------------------------
# input_file <- system.file("extData", "cluster_genes.txt", package = "ExpHunterSuite")
# print(readLines(input_file,n=2))
# 
# organisms_table <- get_organism_table()
# current_organism_info <- organisms_table[rownames(organisms_table) %in% "Mouse",]
# org_db <- get_org_db(current_organism_info)
# 
# enr_lists <- main_clusters_to_enrichment(input_file, org_db=org_db,
#     current_organism_info=current_organism_info, gene_keytype="ENSEMBL")

## ----write_reports, echo=TRUE, eval=FALSE, results="hide"---------------------
# 
# write_expression_report(exp_results=degh_out_coexp)
# write_enrich_files(func_results=fh_out_one_pack)
# write_functional_report(hunter_results=degh_out_coexp,
#                         func_results=fh_out_coexp)

################## GENERAL FUNCTIONS ########################

#' Method used to import hidden functions legally for CRAN check
#' @param pkg package
#' @param fun function to be luaded
#' @return function requried
get_unexported_function <- function(pkg, fun){
  return(get(fun, envir = asNamespace(pkg),inherits = FALSE))
} 


check_and_create_dir <- function(folder){
 folder <- file.path(folder)
 if (!dir.exists(folder)){
    dir.create(folder, recursive = TRUE)
 } 
}


handling_errors <- function(a){
  normalized_counts <- NULL
  expres_diff <- NULL
  all_genes_df <- NULL
  return(list(expres_diff, normalized_counts, all_genes_df))
}

standard_error <- function(x) {
  sd(x) / sqrt(length(x))
}

########################################################  
# Functions to generate output files
unite_DEG_pack_results <- function(exp_results, 
                                   p_val_cutoff, 
                                  lfc, 
                                  minpack_common) {
  DEG_pack_columns <- exp_results[['DEG_pack_columns']] 
  all_DE <- exp_results[['all_counts_for_plotting']] 
  all_FDR_names <- exp_results[['all_FDR_names']]
  all_LFC_names <- exp_results[['all_LFC_names']] 
  all_pvalue_names <- exp_results[['all_pvalue_names']]
  final_pvalue_names <- exp_results[['final_pvalue_names']]
  final_logFC_names <- exp_results[['final_logFC_names']]
  final_FDR_names <- exp_results[['final_FDR_names']]
  # Reorder all output by gene id so all equal
  all_DE <- lapply(all_DE, function(x) x[order(row.names(x)),])
  gene_ids <- row.names(all_DE[[1]])
  
  # Initialise output dataframe and add p/FDR/logFC 
  #values from the different packages
  all_DE_df <- data.frame(row.names=gene_ids)
  for(i in seq(length(all_DE))) {
    all_DE_df[final_logFC_names[i]] <- all_DE[[i]][, all_LFC_names[i]]
    all_DE_df[final_FDR_names[i]] <- all_DE[[i]][, all_FDR_names[i]]
    all_DE_df[final_pvalue_names[i]] <- all_DE[[i]][, all_pvalue_names[i]]
  }

  # Add TRUE/FALSE for each DEG package
  for(i in seq(length(all_DE))) {
    all_DE_df[DEG_pack_columns[i]] <- all_DE_df[, 
                              final_FDR_names[i]] < p_val_cutoff & 
                              abs(all_DE_df[, final_logFC_names[i]]) >= lfc
    all_DE_df[DEG_pack_columns[i]][
    is.na(all_DE_df[DEG_pack_columns[i]])] <- FALSE
    # Check: if no DE genes for package, give warning
    if(sum(all_DE_df[, DEG_pack_columns[i]]) == 0) 
      warning(paste("No significant", DEG_pack_columns[i], "found"))
  }
  # Get DEG_counts
  all_DE_df["DEG_counts"] <- rowSums(all_DE_df[DEG_pack_columns], na.rm = TRUE)
 
  # Calc and add combined FDR-values
  all_DE_df$combined_FDR <- vectorial_fisher_method(all_DE_df[final_FDR_names])

  # Reorder by combined FDR value
  all_DE_df <- all_DE_df[order(all_DE_df[,"combined_FDR"]), ]
  
  # Label as significant or not using combined FDR values
  all_DE_df[, "FDR_labeling"] <- ifelse(
                                   all_DE_df[, "combined_FDR"] < p_val_cutoff, 
                                   "SIGN", 
                                   "NOTSIGN")

  # Calculate average fold changes
  all_DE_df[, "mean_logFCs"] <- rowMeans(all_DE_df[final_logFC_names])

  # Add PREVALENT_DEG tag if as many as minpack_common; 
  # POSSIBLE_DEG if less but > 0; NOT_DEG if == 0
  genes_tag <- ifelse(all_DE_df[, "DEG_counts"] >= minpack_common, 
                      yes = "PREVALENT_DEG",
                      no = ifelse(all_DE_df[, "DEG_counts"] > 0, 
                                  yes = "POSSIBLE_DEG", 
                                  no = "NOT_DEG")
  )
  all_DE_df[, "genes_tag"] <- genes_tag

  return(all_DE_df)
}

add_filtered_genes <- function(all_DE_df, raw) {
  filtered_genes <- row.names(raw)[! row.names(raw) %in% row.names(all_DE_df)]
  filtered_df <- as.data.frame(matrix(NA, 
                                      ncol = ncol(all_DE_df), 
                                      nrow = length(filtered_genes)))
  colnames(filtered_df) <- colnames(all_DE_df)
  row.names(filtered_df) <- filtered_genes
  filtered_df[, "genes_tag"] <- rep("FILTERED_OUT", dim(filtered_df)[1])
  all_DE_df <- rbind(all_DE_df, filtered_df)

  return(all_DE_df)
}

debug_point <- function(file, message = "Debug point",envir = NULL){
    if(!dir.exists(dirname(file))){
      dir.create(dirname(file), recursive = TRUE)
    }
    if(is.null(envir)){
      current_envir <- environment() # current environment
      envir <- parent.env(current_envir)
    }

    envir$debug_message <- message
    on.exit(file.remove(file))
    save(list = names(envir), file = file, 
         envir = envir, precheck = FALSE)      
    on.exit()
}

#' @importFrom ggplot2 ggplot aes_string geom_bar theme element_text
#' @importFrom grDevices pdf dev.off
#' @importFrom utils write.table
save_times <- function(time_control, 
                       output="times_control.txt", 
                       plot_name = "time_control.pdf"){
 spent_times <- list()
    invisible(lapply(seq(2,length(time_control)), function(time_control_i){
      spent_times[[names(time_control)[time_control_i]]] <<- as.numeric(
                                      unlist(time_control[time_control_i]) - 
                                      unlist(time_control[time_control_i - 1]))
    }))
    spent_times_df <- do.call(rbind.data.frame, spent_times)
    colnames(spent_times_df) <- c("time")
    spent_times_df$control <- names(spent_times)
    pp <- ggplot2::ggplot(spent_times_df, ggplot2::aes_string(x = "control", 
                                                              y = "time")) +
          ggplot2::geom_bar(stat = "identity") +
          ggplot2::theme(axis.text.x=ggplot2::element_text(angle = 25, 
                                                           hjust=1))
    grDevices::pdf(file.path(dirname(output), plot_name))    
      plot(pp)
    grDevices::dev.off()
    utils::write.table(spent_times_df, 
                       file=output, 
                       quote=FALSE, 
                       row.names=FALSE, 
                       sep="\t")
}

#' Parallelize function execution in list with multiple elements
#' @param X R list
#' @param FUNC Function to execute in each element of the list
#' @param workers Number of process to use in parallel execution
#' @param task_size Number of list elements to be executed on each 
#' BiocParallel task
#' @param ... Additional arguments to BiocParallel or to FUNC
#' @keywords performance
#' @return list
#' @importFrom utils str
#' @importFrom BiocParallel MulticoreParam bptry bplapply bpok
parallel_list <- function(X, FUNC, workers=2, task_size=1, ...){
    log <- FALSE
    main_log_path <- file.path(getwd(), 'bcplogs')
    log_path <- NA_character_
    workers <- workers - 1 # Reserve one core for main execution process
    if( workers == 0) workers <- 1
    if(workers > 1){
      timestamp <- as.integer(Sys.time())
      log_path <- file.path(main_log_path, as.character(timestamp))
      if(file.exists(log_path)){
        timestamp = timestamp + 1
        log_path <- file.path(main_log_path, as.character(timestamp))
      }
      log <- TRUE
      dir.create(log_path, recursive = TRUE)
    }
    param <- BiocParallel::MulticoreParam( 
      workers, tasks = ceiling(length(X)/task_size), stop.on.error = TRUE,
      log = log, threshold = "INFO", logdir = log_path
    )
    message(paste('items:', length(X), 'task_size:', task_size))
    print(param)    
    res <- BiocParallel::bptry(
      BiocParallel::bplapply(X, FUNC, BPPARAM = param, ...)
    )
    exec_status <- BiocParallel::bpok(res)
    fails <- which( exec_status == FALSE)
    message(paste('exec_status => exec items:', 
                  length(exec_status), 
                  'fails:', 
                  length(fails)))
    if(length(fails) > 0 ){
      message(tail(attr(res[[fails[1]]], "traceback")))
      stop(paste('Parallel execution has failed at item', fails[1],
                 'and a total of', length(fails) , 'items have failed.'))
    } else {
      unlink(main_log_path, recursive = TRUE)
    }

    return(res)
}



rand_sample_bool <- function(bool, sample_size){
#takes a TRUE FALSE vector and keep only sample_size TRUEs
    sampled_bool <- rep(FALSE, length(bool)) 
    random_i <- sample(which(bool == TRUE), 
                        size = sample_size, 
                        replace = FALSE)
    sampled_bool[random_i] <- TRUE
    return(sampled_bool)
}

check_and_quit <- function(object){
  str(object)
  q()
}


#' @importFrom GenomicRanges makeGRangesFromDataFrame
#' @importFrom annotatr annotate_regions build_annotations
annotate_genomic_ranges <-function(intervals_df, genome){
 g_regs <- GenomicRanges::makeGRangesFromDataFrame(intervals_df, keep.extra.columns = TRUE)
 annots <- annotatr::build_annotations(genome = genome, annotations = paste0(genome,"_basicgenes"))
 annotated_g_regs <- annotatr::annotate_regions(regions = g_regs,annotations = annots, ignore.strand = FALSE)
 annotated_g_regs <- data.frame(annotated_g_regs)
 annotated_g_regs <- annotated_g_regs[!is.na(annotated_g_regs$annot.symbol),]
 return(annotated_g_regs)
}    

split_str <- function(string, split = NULL) {
  if (is.null(split))
    stop("split character must be specifyed")
  if (is.null(string)) return(NULL)  
  if (nchar(string) <= 1) return(NULL)  

  splitted_str <- unlist(strsplit(string, split = split))
  return(splitted_str)
}

merge_factors <- function(table, target, factors) {
  if (length(factors) == 0 ) {
    return(table)
  }  

  table <- merge(table, target[,factors, drop = FALSE], by = "row.names", all = TRUE)
    row.names(table) <-  table[,"Row.names"]
    table[,"Row.names"] <- NULL 
    return(table)
}

remove_pattern_from_df <- function(dataframe, rgx){
  df_parsed <- as.data.frame(lapply(dataframe,
                    remove_text_from_column, 
                    rgx = rgx))
  return(df_parsed)
}

remove_text_from_column <- function(column, rgx) {
  parsed_col <- gsub(rgx, "", column)
  return(parsed_col)
}

#!/usr/bin/env Rscript

suppressPackageStartupMessages(library(ggplot2)) 
suppressPackageStartupMessages(library(limma))
suppressPackageStartupMessages(library(edgeR))
suppressPackageStartupMessages(library(DESeq2)) 
suppressPackageStartupMessages(library(NOISeq))
suppressPackageStartupMessages(library(VennDiagram))
suppressPackageStartupMessages(library(gplots))
suppressPackageStartupMessages(library(optparse))
suppressPackageStartupMessages(library(stringr))
suppressPackageStartupMessages(library(plyr))
suppressPackageStartupMessages(library(knitr))
suppressPackageStartupMessages(library(FSA))
suppressPackageStartupMessages(require(rmarkdown))
suppressPackageStartupMessages(require(reshape2))
suppressPackageStartupMessages(require(PerformanceAnalytics))
suppressPackageStartupMessages(require(WGCNA))


script_path <- tryCatch(normalizePath(parent.frame(2)$ofile), 
               error=function(e) # works when using R CMD
              normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                commandArgs())], '='))[2]))




option_list <- list(
  make_option(c("-t", "--template"), type="character", default="main_report",
    help=paste0("Specify DEgenesHunter report template: 'main_report' for main",
      " DEGhunter execution, 'functional_report' for functional enrichment for",
      " differential expression packages and 'clusters_main_report' for ",
      "functional enrichments of all clusters")),
  make_option(c("-i", "--input"), type="character", default=NULL,
    help=paste0("Specify DEGhunter sesion for generate report. ADVICE: ",
      "--debug flag must be activated in previous DEGhunter execution")),
  make_option(c("-o", "--output"), type="character", default=NULL,
    help=paste0("Specify output path. Report html will be created with the",
      " name of the report template"))
  )
actual_opt <- parse_args(OptionParser(option_list=option_list))

if(actual_opt$template == "functional_report"){
  suppressPackageStartupMessages(require(topGO))
  suppressPackageStartupMessages(require(biomaRt)) 
  suppressPackageStartupMessages(require(KEGGREST))
  suppressPackageStartupMessages(require(clusterProfiler))
}


if(is.null(actual_opt$output)){
  actual_opt$output <- file.path("./", actual_opt$template)
}
if(!grepl(".html", actual_opt$output)){
  actual_opt$output <- paste0(actual_opt$output, ".html")
}

load(actual_opt$input)
source(file.path(dirname(script_path), 'lib', 'plotting_functions.R'))
source(file.path(dirname(script_path), 'lib', 'functional_analysis_library.R'))
### load enviroment

### load template
DEGhunter_templates <- file.path(dirname(script_path), "templates")


if(actual_opt$template == "functional_report"){
  results_path = dirname(actual_opt$output)
}

template <- normalizePath(file.path(DEGhunter_templates, paste0(
    actual_opt$template, ".Rmd")))
out_file <- file.path(actual_opt$output)
temp_dir <- file.path(normalizePath(dirname(actual_opt$output)), "tmp")
print(paste0("Rendering ", template, "..."))
### Render
rmarkdown::render(template,
    output_file = out_file,
    intermediates_dir = temp_dir,
    # clean = FALSE
    )

#!/usr/bin/env Rscript
################################################################################
############################### Parse pseudocounts #################################
################################################################################


############################################################
##                      SETUP PROGRAM                     ##
############################################################

options(warn=1)
if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){
  # Obtain this script directory
  full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile), 
                 error=function(e) # works when using R CMD
                normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                  commandArgs())], '='))[2]))
  main_path_script <- dirname(full.fpath)
  root_path <- file.path(main_path_script, '..', '..')
  # Load custom libraries
  custom_libraries <- c('main_parse_pseudocounts.R',
    'general_functions.R')
  for (lib in custom_libraries){
    source(file.path(root_path, 'R', lib))
  }
}else{
  require('ExpHunterSuite')
  root_path <- find.package('ExpHunterSuite')
}

################### OPTIONS
option_list <- list(
    optparse::make_option(c("-i", "--input_folder"), type="character", 
        default="",
        help="Folder that include subfolders names as sequencing samples that contains quntification files"),
    optparse::make_option(c("-m", "--mapper"), type="character", 
        help="Program that were use to quantify mappings. salmon, kallisto, rsem and stringtie are available.",
        default="salmon"),
     optparse::make_option(c("-a", "--annotation_file"), type="character", 
        help="GFF or GTF annotation file.",
        default=NULL),
    optparse::make_option(c("--output_type"), type="character", 
        help="Indicate if the output must include [G] gene counts (gene_counts.txt), [T] transcript counts (transcript_counts.txt) or both",
        default="GT"),
   optparse::make_option(c("-o", "--output_folder"), type="character",
     	default=".", 
        help = "Output folder"))
opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))

quant_data <- parse_pseudocounts(opt$input_folder, opt$mapper, opt$output_type, opt$annotation_file)

if(grepl("G", opt$output_type))
	write.table(quant_data$genes, file = file.path(opt$output_folder, "genes_counts.txt"), quote = FALSE, sep = "\t")

if(grepl("T", opt$output_type))
	write.table(quant_data$transcripts, file = file.path(opt$output_folder, "transcipts_counts.txt"), quote = FALSE, sep = "\t")

merge_dim_tables <- function(dim_data_simp){
	quali_data <- data.frame()
	quanti_data <- data.frame()
	cat_data <- data.frame()
	dim_data_simp$call <- NULL
	for(dimension in names(dim_data_simp)){
		
		dim_data <- dim_data_simp[[dimension]]
		if (!is.null(dim_data$quali)) {

			quali_data_dim <- as.data.frame(dim_data$quali)
			quali_data_dim$dimension <- dimension
			quali_data_dim$factor <- rownames(quali_data_dim)
			quali_data <- rbind(quali_data, quali_data_dim)	
        }
     if (!is.null(dim_data$quanti)){
			quanti_data_dim <-  as.data.frame(dim_data$quanti)
			quanti_data_dim$dimension <- dimension
			quanti_data_dim$factor <- rownames(quanti_data_dim)
			quanti_data <- rbind(quanti_data, quanti_data_dim)	
		}
		if (!is.null(dim_data$category)){
			cat_data_dim <-  as.data.frame(dim_data$category)
			cat_data_dim$dimension <- dimension
			cat_data_dim$factor <- rownames(cat_data_dim)
			cat_data <- rbind(cat_data, cat_data_dim)	
		}

	}
	return(list(quantitative = quanti_data,
		qualitative = quali_data,
		qual_category = cat_data))
}

#' @importFrom FactoInvestigate dimRestrict eigenRef
get_PCA_dimensions <- function(pca_obj, min_dimensions = 2) {
    ref = FactoInvestigate::eigenRef(pca_obj, time = "10s", parallel=FALSE) # to avoid use parallel computation that greedy takes all cpu cores
    rand = c(ref$inertia[1], diff(ref$inertia)) * 100
    keep_dimensions <- FactoInvestigate::dimRestrict(pca_obj, rand = rand)
    if(keep_dimensions < min_dimensions){
      keep_dimensions <- min_dimensions
      message('Significant axis are less than 2. The first two axis will be selected to continue the analysis')
    }
    return(keep_dimensions)
}

#' @importFrom FactoMineR PCA dimdesc HCPC
compute_pca <- function(pca_data, 
												target, 
												string_factors = NULL, 
												numeric_factors = NULL) {
	pca_data <- as.data.frame(t(pca_data))
	 
	std_pca <- FactoMineR::PCA(pca_data, scale.unit=TRUE, 
																				graph = FALSE)                                                     
	dim_to_keep <- get_PCA_dimensions(std_pca)

	rownames(target) <- as.character(target$sample)

	if (!is.null(numeric_factors)){
		pca_data <- merge_factors(pca_data, target, numeric_factors)

	}

	if (!is.null(string_factors)) {
	  pca_data <- merge_factors(pca_data, target, string_factors)
	}

	pca_res <- FactoMineR::PCA(pca_data,ncp = dim_to_keep, 
										scale.unit=TRUE, 
										graph = FALSE,
										quanti.sup = numeric_factors, 
										quali.sup=string_factors)	
	dim_data <- FactoMineR::dimdesc(pca_res, axes=seq(1, dim_to_keep))
  dim_data_merged <- merge_dim_tables(dim_data)

  res.hcpc <- FactoMineR::HCPC(pca_res, graph = FALSE)

	return(list(pca_data = pca_res,
							dim_to_keep = dim_to_keep,
							dim_data = dim_data,
							dim_data_merged = dim_data_merged,
							res.hcpc = res.hcpc))
}

#############################################
### QC AND BENCHMARKING FUNCTIONS 
#############################################

calculate_sensitivity <- function(True_Pos, False_Neg){
  sensitivity <- True_Pos/(True_Pos + False_Neg)
  return(sensitivity)
}

calculate_specificity <- function(True_Neg, False_Pos){
  specificity <- True_Neg/(False_Pos + True_Neg)
  return(specificity)
} 

calculate_positive_predictive_value_PPV <- function(True_Pos, False_Pos){
  positive_predictive_value_PPV <- True_Pos/(True_Pos + False_Pos)
  return(positive_predictive_value_PPV)
}

calculate_negative_predictive_value_NPV <- function(True_Neg, False_Neg){
  negative_predictive_value_NPV <- True_Neg/(False_Neg + True_Neg)
  return(negative_predictive_value_NPV)
}

calculate_accuracy <- function(True_Pos, True_Neg, False_Pos, False_Neg){
  negative_predictive_value_NPV <- True_Neg/(False_Neg + True_Neg)
  return(negative_predictive_value_NPV)
}

#' main_parse_pseudocounts
#' Functions that loads pseudommappers results and complite it into a table of counts
#' @param input_folder folder with pseudocounts files
#' @param mapper string pseudomapping program
#' @param output_type string indicate i fthe output must be 'G' genes, 'T' transcripts or both
#' @param annotation_file gtf or gff path, requested for parsin salmon results
#' @examples
#'	\dontrun{
#'		parse_pseudocounts(input_folder = "Path_to_folder",
#'							mapper = "salmon",
#'							annotation_file = "path_to_gtf")	
#'  }
#' @export
#' @keywords method
#' @importFrom GenomicFeatures makeTxDbFromGFF
#' @importFrom AnnotationDbi keys select
#' @importFrom tximport tximport
parse_pseudocounts <- function(input_folder, mapper, output_type, annotation_file){
	results <- list()
	file_type <- list("salmon" = "*.sf", 
					  "kallisto" = "*abundance.h5",
					  "stringtie" = "*t_data.ctab",
					  "rsem" = "*.genes.results.gz"
					  )


	if (mapper == "stringtie") {
		stop("StringTie input has not yet been included.")
		#https://bioconductor.org/packages/release/bioc/vignettes/tximport/inst/doc/tximport.html#StringTie
	}
	
	tx2gene <- NULL
	if (mapper == "salmon") {
		annot_txdb <- GenomicFeatures::makeTxDbFromGFF(annotation_file)
		db_keys <- AnnotationDbi::keys(annot_txdb,  keytype = "TXNAME")
		tx2gene <- AnnotationDbi::select(annot_txdb, db_keys, "GENEID", "TXNAME")
		tx2gene <- remove_pattern_from_df(tx2gene, "[.][0-9]+")
	}
	input_files <- Sys.glob(file.path(input_folder, "*", file_type[[mapper]]))
	names(input_files) <- basename(Sys.glob(file.path(input_folder, "*")))

	if (grepl("G", output_type)) {
		txi_genes <- tximport::tximport(input_files, type = mapper, tx2gene = tx2gene, ignoreTxVersion = TRUE)

		results[["genes"]] <- txi_genes$counts
	}
	if (grepl("T", output_type)) {
		txi_transcripts <- tximport::tximport(input_files, type = mapper, tx2gene = tx2gene, txOut = TRUE, ignoreTxVersion = TRUE)
		results[["transcripts"]] <- txi_transcripts$counts
	}
	return(results)
}

#' Function to Simulate Treatment-Control paired experiments for genes
#' expression count tables. Pipeline inspired on TCC simulateReadCounts function
#' @param Ngene number of genes to be included in the experiment
#' @param PDEG Percentage of Differentially Expressed Genes. If one number
#' is specified, only Up-regulated genes will be simulated. If a vector of two
#' number is given, proportions will be applied for Up and Down regualted
#' simulation respectively
#' @param DEG.foldchange foldchange to be applied. Can be a number or a
#' function which must accept only a vector and return a vector of the same
#' length with the fold-change values. Warning: NOT LOG FOLD CHANGE
#' @param replicates to be ge generated of each group (Treatment and Control)
#' @param bcount matrix of genes counts used to simulate. If NULL,
#' toc dataset will be loaded
#' @param group columns to be used as group replicates. If NULL,
#' all columns will be used.
#' @return a list with COUNT) a simulated gene counts matrix with simulated
#' genes and sample names and TRUEDEG) a vector with real up/down regulated
#' genes
#' @author Fernando Moreno Jabato <jabato(at)uma(dot)es>
#' @importFrom utils data
#' @importFrom stats median var runif rnbinom
STC <- function(Ngene = 1000, 
    PDEG = 0.2, 
    DEG.foldchange = 2, 
    replicates = 3, 
    bcount = NULL, 
    group = NULL){
    # Load & prepare base data
    if(is.null(bcount)){
        toc <- NULL
        utils::data("toc", envir = environment())
        bcount <- toc
        group <- seq(3)
    }
    if(is.null(group)){
        group <- seq(ncol(bcount))
    }
    rpm.a <- sweep(bcount[, group], 2, stats::median(
        colSums(bcount[, group]))/colSums(bcount[,group]), "*")
    rpm.a <- rpm.a[apply(rpm.a, 1, stats::var) > 0, ]
    mean.a <- apply(rpm.a, 1, mean)
    var.a <- apply(rpm.a, 1, stats::var)
    dispersion <- (var.a - mean.a)/(mean.a * mean.a)
    population <- data.frame(mean = mean.a, disp = dispersion)
    population <- population[population$disp > 0, ]
    population <- population[sample(seq(nrow(population)), 
        Ngene, replace = TRUE), ]
    # Define which genes will be DEGs
    if(is.vector(PDEG) && length(PDEG) > 1){
        ndeg_pos <- floor(Ngene*abs(PDEG[1]))
        ndeg_neg <- floor(Ngene*abs(PDEG[2]))
    }else{
        ndeg_pos <- floor(Ngene*abs(PDEG))
        ndeg_neg <- 0
    }
    ndeg <- ndeg_pos + ndeg_neg
    shuffle <- function(vector){vector[sample(seq_along(vector))]}
    DEGs <- c(rep(1.0,ndeg_pos),rep(-1.0,ndeg_neg),rep(0.0,Ngene - ndeg)) 
    DEGs <- shuffle(DEGs)
    # Check foldchange
    if(is.function(DEG.foldchange)){
        DEG.foldchange <- DEG.foldchange(population$mean)$FC
    }else{
        DEG.foldchange <- rep(DEG.foldchange,Ngene)
    }
    # Prepare foldchange
    experiments <- matrix(0,ncol = replicates * 2, nrow = Ngene)
    invisible(lapply(seq_along(DEGs),function(i){
        # Check
        if(DEGs[i] == 0){
            experiments[i,] <<- rep(1.0,ncol(experiments))
            return()
        } 
        # Generate foldchange
        experiments[i,] <<- c(rep(1.0,replicates),
                         DEGs[i] * stats::runif(replicates,
                             min = DEG.foldchange[i] - 0.2, 
                             max = DEG.foldchange[i] + 0.2))
    }))    
    # Generate
    experiments <- apply(experiments, 2, function(x, pp = population) {
        stats::rnbinom(n = Ngene, 
            mu = (abs(x)^(x/abs(x)))*pp$mean, size = 1/pp$disp)
    })
    # Change names
    colnames(experiments) <- c(paste0("Ctrl_rep",seq(replicates)),
        paste0("Treat_rep",seq(replicates)))
    rownames(experiments) <- paste0("gene_",seq(Ngene))
    names(DEGs) <- rownames(experiments)
    # END
    return(list(count = experiments, trueDEG = DEGs))
}

#' Table of counts dummy dataset for DEgeneHunter examples
#'
#' @docType data
#'
#' @usage data(toc)
#'
#' @format A truncated data.frame of aligned read counts for genes (rows) and 
#' samples (columns)
#'
#' @keywords datasets
#'
#' @examples
#' data(toc)
"toc"

#' Target data containing sample information for the toc data
#'
#' @docType data
#'
#' @usage data(target)
#'
#' @format A data.frame of sample information. Rows are samples, columns are 
#' different experimental variables/factors
#'
#' @keywords datasets
#'
#' @examples
#' data(target)
"target"

#' Output data from the main_degenes_Hunter function
#'
#' @docType data
#'
#' @usage data(degh_output)
#'
#' @format An output object from the main_degenes_Hunter function. 
#'
#' @keywords datasets
#'
#' @examples
#' data(degh_output)
"degh_output"

################################################
### FUNCTIONS: DIFFERENCIAL EXPRESSION PACKAGES
################################################
# Manager function
#-----------------------------------------------
perform_expression_analysis <- function(modules, 
                                        replicatesC, 
                                        replicatesT, 
                                        raw_filter, 
                                        p_val_cutoff, 
                                        target, 
                                        model_formula_text, 
                                        external_DEA_data=NULL,
                                        multifactorial){
    # Prepare results containers
    all_data_normalized     <- list()
    all_counts_for_plotting <- list()
    all_package_results     <- list()
    package_objects         <- list()
    all_FDR_names           <- c()
    all_LFC_names           <- c()
    all_pvalue_names        <- c()
    final_logFC_names       <- c()
    final_FDR_names         <- c()
    final_pvalue_names      <- c()
    DEG_pack_columns        <- c()

    #####
    ################## CASE D: DESeq2
    #####
    if(grepl("D",modules)){
      if(replicatesC >= 2 & replicatesT >= 2) {
        # Verbose
        cat('Gene expression analysis is performed with DESeq2.\n')
        # Calculate results
        results <- analysis_DESeq2(data   = raw_filter,
                                   p_val_cutoff = p_val_cutoff,
                                   target = target,
                                   model_formula_text = model_formula_text,
                                   multifactorial = multifactorial)
        # Store results
        all_data_normalized[['DESeq2']] <- results[[1]]
        all_counts_for_plotting[['DESeq2']] <- results[[2]]
        package_objects[['DESeq2']] <- results[[3]]

        # Result Plot Visualization
        if (!is.null(all_counts_for_plotting[['DESeq2']])){
          all_FDR_names      <- c(all_FDR_names, 'padj')
          all_LFC_names      <- c(all_LFC_names, 'log2FoldChange')
          all_pvalue_names   <- c(all_pvalue_names, 'pvalue')
          final_pvalue_names <- c(final_pvalue_names, 'pvalue_DESeq2')
          final_logFC_names  <- c(final_logFC_names, 'logFC_DESeq2')
          final_FDR_names    <- c(final_FDR_names, 'FDR_DESeq2')
          DEG_pack_columns   <- c(DEG_pack_columns, 'DESeq2_DEG')
        } 
      } else {
      warning("DESeq2 will not be performed due to too few replicates")
      }
    }

    #####
    ################## CASE E: edgeR (AT LEAST 2 REPLICLATES)
    #####
    if(grepl("E", modules)){ 
      if(replicatesC >= 2 & replicatesT >= 2) {
        # Verbose point
        cat('Gene expression analysis is performed with edgeR.\n')
        # Calculate results
        results <- analysis_edgeR(data   = raw_filter,
                                  target = target,
                                  model_formula_text = model_formula_text,
                                  multifactorial = multifactorial)
        # Store results
        all_data_normalized[['edgeR']] <- results[[1]]
        all_counts_for_plotting[['edgeR']] <- results[[2]]
        package_objects[['edgeR']] <- results[[3]]

        # Result Plot Visualization
        if (!is.null(all_counts_for_plotting[['edgeR']])){
          all_FDR_names      <- c(all_FDR_names, 'FDR')
          all_LFC_names      <- c(all_LFC_names, 'logFC')
          all_pvalue_names   <- c(all_pvalue_names, 'PValue')
          final_pvalue_names <- c(final_pvalue_names, 'pvalue_edgeR')
          final_logFC_names  <- c(final_logFC_names, 'logFC_edgeR')
          final_FDR_names    <- c(final_FDR_names, 'FDR_edgeR')
          DEG_pack_columns   <- c(DEG_pack_columns, 'edgeR_DEG')
        }
      } else {
      warning("edgeR will not be performed due to too few replicates")
      }
    }

    #####
    ################## CASE L: limma (AT LEAST 3 REPLICATES)
    #####
    if(grepl("L", modules)){ 
      if(replicatesC >= 3 & replicatesT >= 3) {
        # Verbose
        cat('Gene expression analysis is performed with limma.\n')

        # Calculate results
        results <- analysis_limma(data   = raw_filter,
                                  target = target,
                                  model_formula_text = model_formula_text,
                                  multifactorial = multifactorial)
        # Store results
        all_data_normalized[['limma']]     <- results[[1]]
        all_counts_for_plotting[['limma']] <- results[[2]]

        # Result Plot Visualization
        if (!is.null(all_counts_for_plotting[['limma']])){
          all_FDR_names      <- c(all_FDR_names, 'adj.P.Val')
          all_LFC_names      <- c(all_LFC_names, 'logFC')
          all_pvalue_names   <- c(all_pvalue_names, 'P.Value')
          final_pvalue_names <- c(final_pvalue_names, 'pvalue_limma')
          final_logFC_names  <- c(final_logFC_names, 'logFC_limma')
          final_FDR_names    <- c(final_FDR_names, 'FDR_limma')
          DEG_pack_columns   <- c(DEG_pack_columns, 'limma_DEG')
        }
      } else {
        warning("limma will not be performed due to too few replicates")
      }
    }

    #####
    ################## CASE N: NOISeq (AT LEAST 3 REPLICLATES)
    #####
    if(grepl("N", modules)){ 
        if(replicatesC >= 3 & replicatesT >= 3) {

        # Verbose
        cat(paste0("Gene expression analysis is performed with NOISeqBIO",
                   " function within NOISeq.\n"))
        # Calculate results
        results <- analysis_NOISeq(data    = raw_filter, 
                                   target = target)
        # Store results
        all_data_normalized[['NOISeq']]     <- results[[1]]
        all_counts_for_plotting[['NOISeq']] <- results[[2]]
        package_objects[['NOISeq']] <- results[[3]]

        #Result Plot Visualization
        if (!is.null(all_counts_for_plotting[['NOISeq']])){
          all_FDR_names      <- c(all_FDR_names, 'adj.p')
          all_LFC_names      <- c(all_LFC_names, 'log2FC')
          all_pvalue_names   <- c(all_pvalue_names, 'prob')
          final_pvalue_names <- c(final_pvalue_names, 'pvalue_NOISeq')
          final_logFC_names  <- c(final_logFC_names, 'logFC_NOISeq')
          final_FDR_names    <- c(final_FDR_names, 'FDR_NOISeq')
          DEG_pack_columns   <- c(DEG_pack_columns, 'NOISeq_DEG')
        }
      } else {
        warning("NOISeq will not be performed due to too few replicates")
      }
    }

    #####
    ################## CASE F: External DEG file
    #####
    if(grepl("F",modules)){
      cat('External DEA data to be processed \n')
      # Calculate results
      results <- analysis_external_DEA(raw_filter, external_DEA_data)
      # Store results
      all_data_normalized[['externalDEA']] <- results[[1]]
      all_counts_for_plotting[['externalDEA']] <- results[[2]]
      package_objects[['externalDEA']] <- results[[3]]

      # Result Plot Visualization

      if (!is.null(all_counts_for_plotting[['externalDEA']])){
        all_pvalue_names   <- c(all_pvalue_names, "P.Value")
        all_LFC_names      <- c(all_LFC_names, "logFC")
        all_FDR_names      <- c(all_FDR_names, "adj.P.Val")
        final_pvalue_names <- c(final_pvalue_names, 'pvalue_externalDEA')
        final_logFC_names  <- c(final_logFC_names, 'logFC_externalDEA')
        final_FDR_names    <- c(final_FDR_names, 'FDR_externalDEA')
        DEG_pack_columns   <- c(DEG_pack_columns, 'externalDEA_DEG')
      }

    }

    exp_results <- list()
    exp_results[['all_data_normalized']]     <- all_data_normalized
    exp_results[['all_counts_for_plotting']] <- all_counts_for_plotting
    exp_results[['all_package_results']]     <- all_package_results
    exp_results[['package_objects']]         <- package_objects
    exp_results[['all_FDR_names']]           <- all_FDR_names
    exp_results[['all_LFC_names']]           <- all_LFC_names
    exp_results[['all_pvalue_names']]        <- all_pvalue_names
    exp_results[['final_logFC_names']]       <- final_logFC_names
    exp_results[['final_FDR_names']]         <- final_FDR_names
    exp_results[['final_pvalue_names']]      <- final_pvalue_names
    exp_results[['DEG_pack_columns']]        <- DEG_pack_columns

    return(exp_results)
}


# DESeq2
#-----------------------------------------------
#' @importFrom DESeq2 DESeqDataSetFromMatrix DESeq results counts
#' @importFrom stats formula
analysis_DESeq2 <- function(data, p_val_cutoff, target, model_formula_text,
  multifactorial){
    if(grepl(":nested", multifactorial)){
      mf_text <- split_mf_text(multifactorial)
      factor_table <- table(target[, mf_text$mf_factorA])
      ordered_factors <- names(factor_table)[order(factor_table, decreasing=TRUE)]
      # Following necessary to perform design matrix trick in DESeq2 Vignette:
      # Group-specific condition effects, individuals nested within groups
      ordering_factor <- ordered(target[, mf_text$mf_factorA], 
        levels=ordered_factors)
      # Reorder table by group then by patient (paired samples, i.e. factor B)
      target <- target[order(ordering_factor,  target[, mf_text$mf_factorB]), ]
      bigger_grouping <- target[, mf_text$mf_factorA] == ordered_factors[1]
      smaller_grouping <- target[, mf_text$mf_factorA] == ordered_factors[2]
      target[smaller_grouping, mf_text$mf_factorB] <- 
                  target[bigger_grouping, mf_text$mf_factorB][seq(1, sum(smaller_grouping))]
      target <- target[order(as.integer(row.names(target))), ]

      model_formula_text <- paste0("~ ", mf_text$mf_factorA, " + ", 
                         mf_text$mf_factorA, ":", mf_text$mf_factorB, " + ", mf_text$mf_factorA,":treat")
      m1 <- model.matrix(stats::formula(model_formula_text), target)

      all.zero <- apply(m1, 2, function(x) all(x==0))
      idx <- which(all.zero);  
      m1 <- m1[,-idx]
  
      dds <- DESeq2::DESeqDataSetFromMatrix(countData = data,
            colData = target,
            design = ~1)
      dds <- DESeq2::DESeq(dds, full=m1)

    } else {
      dds <- DESeq2::DESeqDataSetFromMatrix(countData = data,
                                  colData = target,
                                  design = stats::formula(model_formula_text))
      dds <- DESeq2::DESeq(dds)
    }

    if(multifactorial != "") {
      mf_options <- get_mf_DE_options(package_name="DESeq2", package_object=dds,
                                      multifactorial=multifactorial, 
                                      target=target)
      all_DESeq2_genes <- do.call(DESeq2::results, 
                                c(object=dds, alpha = p_val_cutoff, mf_options))
    } else {
      all_DESeq2_genes <- DESeq2::results(dds, alpha = p_val_cutoff)
    }
    normalized_counts <- as.data.frame(DESeq2::counts(dds, normalized=TRUE))
    return(list(normalized_counts, 
           as.data.frame(all_DESeq2_genes), 
           list(de_deseq2=all_DESeq2_genes, DESeq2_dataset=dds)))
}

#' @importFrom edgeR DGEList calcNormFactors estimateDisp exactTest glmQLFit 
#' glmQLFTest topTags cpm
#' @importFrom stats formula model.matrix
analysis_edgeR <- function(data, target, model_formula_text, multifactorial){
    # This is the default model_formula_text if nothing else is given.
    if(model_formula_text == "~ treat") {
    # Building edgeR object
        d_edgeR <- edgeR::DGEList(counts=data, group=as.character(target$treat))
        d_edgeR <- edgeR::calcNormFactors(d_edgeR)
        d_edgeR <- edgeR::estimateDisp(d_edgeR)
        d_edgeR_DE <- edgeR::exactTest(d_edgeR, dispersion = "auto", 
                                   pair=c("Ctrl", "Treat"))
    } else {
        d_edgeR <- edgeR::DGEList(counts=data)
        model_design <- stats::model.matrix(
                            stats::formula(paste(model_formula_text)), 
                            target)
        d_edgeR <- edgeR::estimateDisp(d_edgeR, model_design)
        d_edgeR_DE <- edgeR::glmQLFit(d_edgeR, model_design)

      if(multifactorial != "") {
        mf_options <- get_mf_DE_options(package_name="edgeR", 
                                        package_object=d_edgeR_DE,
                                        multifactorial=multifactorial,
                                        target=target)
        d_edgeR_DE <- do.call(edgeR::glmQLFTest, 
                              list(glmfit=d_edgeR_DE, unlist(mf_options)))
      } else {
        d_edgeR_DE <- edgeR::glmQLFTest(d_edgeR_DE)
      }
    }
    all_genes_df <- edgeR::topTags(d_edgeR_DE, 
                                   nrow(d_edgeR), sort.by="none")$table
    dennorm <- edgeR::cpm(d_edgeR, log=TRUE)
    # Getting normalized counts
    normalized_counts <- edgeR::cpm(d_edgeR)
    # Return calculated info
    return(list(normalized_counts, all_genes_df, list(d_edgeR=d_edgeR, 
                                                    dennorm=dennorm)))
}

#' @importFrom limma voom eBayes topTable lmFit
#' @importFrom edgeR DGEList calcNormFactors
#' @importFrom stats formula model.matrix
analysis_limma <- function(data, target, model_formula_text, multifactorial){
    model_design <- stats::model.matrix(stats::formula(
                                         paste(model_formula_text)), 
                                      target)

    # Calculating differential expression
    DGE_List <- edgeR::DGEList(counts=data) # Building object (DGEList)
    # Calculation of the normalization factor
  DGE_List <- edgeR::calcNormFactors(DGE_List)  
  # Converting the read counts to log2-cpm
    log2_cpm <- limma::voom(DGE_List, model_design) 
    fit <- limma::lmFit(log2_cpm, model_design)
    fit2 <- limma::eBayes(fit)
    if(multifactorial != "") {
      mf_options <- get_mf_DE_options(package_name="limma", 
                                      package_object=model_design,
                                      multifactorial=multifactorial,
                                      target=target)
      todos_limma <- do.call(limma::topTable, list(fit=fit2, adjust.method="BH",
                                number=nrow(fit2), unlist(mf_options)))
    } else {
          todos_limma <- limma::topTable(fit2, adjust.method="BH",
                                         number=nrow(fit2), coef="treatTreat")
    }
    normalized_counts <- log2_cpm$E
  # Return calculated info
  return(list(normalized_counts, todos_limma))
}

#' @importFrom NOISeq readData noiseqbio tmm
#' @importFrom Biobase assayData
analysis_NOISeq <- function(data, target){
    group <- as.character(target$treat)
    # Experimental design
    myfactors = data.frame(Tissue = rev(group), TissueRun = rev(group))
    # Calculation of differential expression
    mydata <- NOISeq::readData(data, myfactors) # Building NOISeq Object (eset)

    all_NOISeq <- NOISeq::noiseqbio(mydata, 
                                  k = 0.5, 
                                  norm = "tmm", 
                                  factor="Tissue", 
                                  lc = 1, 
                                  r = 50, 
                                  adj = 1.5, 
                                  plot = FALSE,    
                                  a0per = 0.9, 
                                  random.seed = 12345, 
                                  filter = 1, 
                                  cv.cutoff = 500, 
                                  cpm = 1)

    normalized_counts <- NOISeq::tmm(Biobase::assayData(mydata)$exprs, 
                               long = 1000, lc = 1) # Getting normalized counts
    expres_diff_all <- as.data.frame(all_NOISeq@results)

    prob_all <- expres_diff_all$prob
    # Inverse
    prob_all <- 1 - prob_all
    expres_diff_all <- data.frame(expres_diff_all, adj.p = prob_all)

    # Return calculated info
    return(list(normalized_counts, expres_diff_all, all_NOISeq))
}

get_mf_DE_options <- function(
  package_name, 
  package_object, 
  multifactorial, 
  target) {
  mf_text <- split_mf_text(multifactorial)
  
  if(package_name == "DESeq2") {
    if(mf_text[["mf_contrast"]] == "interaction") {
        mf_opt <- list(name=DESeq2::resultsNames(package_object)[4])
    } else if (mf_text[["mf_contrast"]] == "effect") {

        mf_opt <- DESeq2::resultsNames(package_object)[3]
        mf_opt <- list(contrast=c(
          mf_text[["mf_factorA"]],
          levels(target[, mf_text[["mf_factorA"]]])[2],
           mf_text[["mf_varA"]]
        ))
    } else if (mf_text[["mf_contrast"]] == "nested_int") {
        target_factor <- unique(target[,mf_text[["mf_factorA"]]])
        mf_opt <- list(contrast=list(paste0(mf_text[["mf_factorA"]], mf_text[["mf_varB"]], ".treatTreat"), 
        paste0(mf_text[["mf_factorA"]], 
          target_factor[! target_factor %in% mf_text[["mf_varB"]]], ".treatTreat" ))
        )
    } else if (mf_text[["mf_contrast"]] == "nested_effect") {
      target_factor <- unique(target[,mf_text[["mf_factorA"]]])
      mf_opt <- list(name=paste0(mf_text[["mf_factorA"]],  mf_text[["mf_varB"]], ".treatTreat" ))
    } 
  } else if (package_name %in% c("edgeR","limma")) { 
    if(mf_text[["mf_contrast"]] == "interaction") {
      mf_opt <- list(name=colnames(package_object)[4])
    } else if (mf_text[["mf_contrast"]] == "effect") {
      mf_opt <- list(name=paste0(
          mf_text[["mf_factorA"]],
          levels(target[, mf_text[["mf_factorA"]]])[2]
      ))
    } else if (mf_text[["mf_contrast"]] == "nested") {
      stop("Nested experiment design cannot be used with package other than DESeq2")
    }
  }
  if(! exists("mf_opt")) stop("Check multifactorial arguments flag valid")
  return(mf_opt)
}

# Dummy function to convert external, already processed/analysed data into 
# an object suitable for the big table function
analysis_external_DEA <- function(raw_data, dea_data) {
    return(list(raw_data, dea_data, dea_data))
}

#!/usr/bin/env Rscript
library(dplyr)

filter_and_rank <- function(cv_cont_table){
   # cv_cont_table <- cv_cont_table[cv_cont_table$Pvalue <= 0.05,]
    if (nrow(cv_cont_table) == 0 ) return(NULL)

    cv_cont_table <- cv_cont_table[,c("miRNA","strategy", "corr_cutoff", "crossval_status", "Odds_ratio", "TP")]
     #cv_cont_table2 <- reshape(cv_cont_table, idvar = c("miRNA","strategy", "corr_cutoff"), timevar = "crossval_status", direction = "wide")
    #if ("Odds_ratio.train" %in% colnames(cv_cont_table)) return(NULL)
    # cv_cont_table <- cv_cont_table  %>% arrange(desc(Odds_ratio.test), desc(TP.test), corr_cutoff)
    # cv_cont_table$OR_test_rank <- seq(1, nrow(cv_cont_table))    
    # str(cv_cont_table)
  #  cv_cont_table$OR_test_rank <- rank(-cv_cont_table$Odds_ratio.test, ties.method ="min")
    cv_cont_table$OR_train_rank <- rank(-cv_cont_table$Odds_ratio,ties.method ="min")
    return(cv_cont_table)

}

get_test_rank <- function(cv_cont_table){
    top_rank_train_val <- min(cv_cont_table$OR_train_rank)
    top_rank_train <- cv_cont_table[cv_cont_table$OR_train_rank ==top_rank_train_val,]
    lower_rank_test <- min(top_rank_train$OR_test_rank)
   # print(top_rank_train)
    result <- data.frame(test_rank = top_rank_train[top_rank_train$OR_test_rank == lower_rank_test, c("miRNA","strategy","corr_cutoff","OR_test_rank")])
    return(result)
}


################### OPTIONS
option_list <- list(
    optparse::make_option(c("-i", "--input_cormit"), type="character", 
        default="",
        help="coRmit folder. Note that this script needs an execution of comit coRmiT with '--save_temp' option activated."),
    optparse::make_option(c("-s", "--test_size"), type="double", 
        help="Proportion of input taken as test. Default=%default",
        default=0.25),
   optparse::make_option(c("-o", "--output_files"), type="character",
     default=".", 
        help = "Output folder")

        )
opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))

################### INITIALIZE
options(scipen = 0.001,
        digits = 3)

if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){

    full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile),
                   error=function(e) # works when using R CMD
                  normalizePath(unlist(strsplit(commandArgs()[grep('^--file=',
                    commandArgs())], '='))[2]))
    main_path_script <- dirname(full.fpath)
    root_path <- file.path(main_path_script, '..', '..')
    custom_libraries <- c("plotting_functions.R", "write_report.R",
        "general_functions.R",  "statistics_functions.R",
        "miRNA_RNA_functions.R", "functional_analysis_library.R",
        "main_cormit.R", "mkinfer_modified.R")
    for (lib in custom_libraries){
        source(file.path(root_path, 'R', lib))
      }
    template_folder <- file.path(root_path, 'inst', 'templates')
    organism_table_path <- file.path(root_path,"inst","external_data",
        "organism_table.txt")

} else {
    require('ExpHunterSuite')
    root_path <- find.package('ExpHunterSuite')
    template_folder <- file.path(root_path, 'templates')
    organism_table_path <- file.path(root_path, "inst","external_data",
        "organism_table.txt")
}



cormit_path <- file.path(opt$input_cormit, "temp.RData")
load(cormit_path)

p_val_cutoff <- 0.05
# all_pairs <- miRNA_cor_results$all_pairs
# miRNA_strat_result <- miRNA_cor_results$miRNA_cont_tables
# miRNA_strat_result <-  miRNA_strat_result[miRNA_strat_result$Pvalue <= 0.05 & miRNA_strat_result$db_group == "multimir",]
# best_miRNA_strategies <- as.data.frame(select_best_strategy(miRNA_strat_result))
# strat_combinations <- as.data.frame(unique(miRNA_strat_result[, c("corr_cutoff", "strategy")]))
# miRNA_cont_tables <- data.frame()



# for (combination in seq(1,nrow(strat_combinations))) {
#     corr_cutoff <- strat_combinations[combination, "corr_cutoff"]
#     strategy <- strat_combinations[combination, "strategy"]

#     contingency_tables <- prepare_for_stats(all_pairs, strategy, corr_cutoff,
#     db_groups = "multimir", p_val_cutoff = p_val_cutoff , crossval = TRUE, test_sample = opt$test_size)
#     miRNA_cont_tables <- rbind(miRNA_cont_tables, contingency_tables[["strat_miRNA_ct"]])
# }


# message("Stats prepared")
# miRNA_cont_tables <- v_get_stats(miRNA_cont_tables, 
#                               selected_stats = c("v.fisher.test","odds_ratio"))

# miRNA_cont_tables$iterations <- paste(miRNA_cont_tables$miRNA, miRNA_cont_tables$crossval_it, sep = "_")

# splitted_cont_tables <- split(miRNA_cont_tables, miRNA_cont_tables$iterations)


# cross_rank <- lapply(splitted_cont_tables, filter_and_rank)
# full_ranking <-as.data.frame(data.table::rbindlist(cross_rank, 
#                                      use.names = TRUE, 
#                                      idcol = "iters"))
### cross_rank <- lapply(cross_rank, get_test_rank)
message("Train and test datasets ordered")
#### cross_rank <-as.data.frame(data.table::rbindlist(cross_rank, 
#                                      use.names = TRUE, 
#                                      idcol = "iters"))
#### colnames(cross_rank) <- c("iteration", "miRNA", "strategy","corr_cutoff","test_rank")
#  save(full_ranking,best_miRNA_strategies, file = "test.Rdata")
load("test.Rdata")
best_miRNA_strategies$acc_ratio <- 0
best_miRNA_strategies$total_strats <- 0

train_rank_dist <- data.frame()
for (miRNA in best_miRNA_strategies$miRNA) {
    best_miRNA_strat <- unlist(best_miRNA_strategies[best_miRNA_strategies$miRNA == miRNA,])

    best_rank_in_train <- full_ranking[full_ranking$miRNA == miRNA &
                                        full_ranking$strategy == best_miRNA_strat[7] &
                                        full_ranking$corr_cutoff == best_miRNA_strat[8],]
    train_rank_dist <- rbind(train_rank_dist, best_rank_in_train)
}

# save(train_rank_dist, file = "test_stats.RData")
# q()
message("Computing report")
dir.create(opt$output_files)
outf <- file.path(normalizePath(opt$output_files),"coRmiT_cv.html")
    rmarkdown::render(file.path(template_folder, 'cormit_cv.Rmd'),
                      output_file = outf, intermediates_dir = opt$output_files)

#!/usr/bin/env Rscript

# library(multiMiR)
# library(optparse)
library(data.table)
library(dplyr)
library(stringr)

option_list <- list(
    optparse::make_option(c("-s", "--chunk_size"), type="integer",default=100,
        help="Database chunks size"),
    optparse::make_option(c("-o", "--output"), type="character",
        help="Tabulated file with information about each sample"),
    optparse::make_option(c("-O", "--organism"), type="character",
        help="Tabulated file with information about each sample"),
    optparse::make_option(c("-c", "--cache_mode"), type="logical", 
        default=FALSE, action = "store_true",
        help=paste0("In this mode each chunk wil be launched in a diferent",
            " job. You must launch this script many times as chunks you have",
            " divided the data. You can incorporate this mode inside a loop",
            " and check if output/temp/finished exist"))
)

opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))


temp_dir <- file.path(opt$output, "temp", opt$organism)
remaining_miRNAs_file <- file.path(temp_dir, "remaining_mirnas.txt")
log_file <- file.path(opt$output, paste0(opt$organism, "_multimir_log"))
# print(opt)
# print(all_organism)
##### Prepare data
if(opt$cache_mode && file.exists(remaining_miRNAs_file)){
    remaining_mirnas <- readLines(remaining_miRNAs_file)
}else{
    miRNA_info <- multmiMiR::list_multimir("mirna")
    # list_organism <- list(rep(0, length(all_organism)))
 #     names(list_organism) <- all_organism
    allmiRNAs <- miRNA_info[miRNA_info$org == opt$organism, 
             "mature_mirna_id"] %>% unique
    remaining_mirnas <- allmiRNAs
        
    if(opt$cache_mode){
        dir.create(temp_dir, recursive=TRUE)
        writeLines(remaining_mirnas, con = remaining_miRNAs_file)
    }
}


### Download data 
org_path <- file.path(temp_dir, opt$organism)
if(!dir.exists(org_path)){
    dir.create(org_path, recursive = TRUE)
}

while(length(remaining_mirnas) > 0){
    mirna_chunk <- sample(remaining_mirnas, opt$chunk_size, 
        replace = TRUE) %>% unique
    print(str(mirna_chunk))
    multimir <- multiMiR::get_multimir(mirna = mirna_chunk, table = "all", 
        org = opt$organism, predicted.cutoff.type="n", 
        predicted.cutoff = 1000000000, predicted.site = "all", limit = NULL)
    multimir <- multimir@data
    #str(multimir)
    if(nrow(multimir) == 0) {
        no_downloaded_mirnas <- mirna_chunk
        multimir <- data.frame(mature_mirna_id = no_downloaded_mirnas)
        remaining_mirnas <- remaining_mirnas[! remaining_mirnas %in% 
             no_downloaded_mirnas]
        cat(paste0(length(no_downloaded_mirnas), " of ", length(mirna_chunk),
           " miRNAs can not been found on multiMiR. ", length(remaining_mirnas), 
           " miRNAs remaining..."), file = log_file, append=TRUE, sep = "\n")
    } else {
        downloaded_mirnas <- multimir[,"mature_mirna_id"] %>% unique
        if (length(remaining_mirnas[remaining_mirnas %in%
            downloaded_mirnas]) == 0) {
            #para cuando una query devuelve mirnas con los identificadores
            # cambiados (actualizados o obsoletos)
            remaining_mirnas <- remaining_mirnas[! remaining_mirnas %in% 
            mirna_chunk]   
        } else {
            remaining_mirnas <- remaining_mirnas[! remaining_mirnas %in% 
               downloaded_mirnas]
        }
        cat(paste0(length(downloaded_mirnas), " of ", length(mirna_chunk), 
            " miRNAs has been downloaded. ", length(remaining_mirnas), 
            " miRNAs remaining..."), file = log_file, append=TRUE, sep = "\n")
    }

    save(multimir, file = file.path(org_path, paste0("data_", 
        length(list.files(org_path, pattern = ".RData")) + 1, ".RData")))
    
    if(opt$cache_mode){
        writeLines(remaining_mirnas, con = remaining_miRNAs_file)
        stop(paste("Program stopped in a controlled way. Launch it again."))
    }
}

print("Merging results...")    
org_mirna_targets <- lapply(list.files(org_path), function(cache_file){
    load(file.path(org_path, cache_file))
    return(as.data.table(multimir))
})

org_mirna_targets <- as.data.frame(unique(rbindlist(org_mirna_targets, 
            fill = TRUE))) # This unique is recommended by the author

print("Filtering starts")

org_mirna_targets <- org_mirna_targets[,c("target_ensembl", 
    "mature_mirna_acc", "database", "score")] 
## Remove empty strings for miRNAs included in databases but with no data
org_mirna_targets <- org_mirna_targets[org_mirna_targets$target_ensembl != "" & 
           org_mirna_targets$mature_mirna_acc != "" & 
           !is.na(org_mirna_targets$database),]

databases_names <- unique(org_mirna_targets$database)
print("Paste starts")
print(databases_names)

# all_pairs <- as.data.table(org_mirna_targets[, c("target_ensembl", 
#            "mature_mirna_acc")])
# all_pairs <- unique(all_pairs)
# unique_pairs <- paste0(all_pairs$target_ensembl, "_AND_", 
#           all_pairs$mature_mirna_acc)

# print("Unique starts")
# multimir_summary <- data.table(pairs = unique_pairs)

# for (db in databases_names) { #This block is for chnagin data structure
#     print(db)
#     database_pairs <- as.data.table(org_mirna_targets[
#               org_mirna_targets$database == db, c("target_ensembl", 
#                  "mature_mirna_acc")])
#     database_pairs <- unique(database_pairs)
#     database_pairs <- paste0(database_pairs$target_ensembl, "_AND_", 
#               database_pairs$mature_mirna_acc)
#     db_pairs <- multimir_summary$pairs %in% database_pairs
#     print(str(db_pairs))
#     multimir_summary[[db]] <- db_pairs
# } 
# print(str(multimir_summary))

# new_columns <- str_split_fixed(multimir_summary$pairs, "_AND_", n = 2)
# multimir_summary$target_ensembl <- new_columns[,1]
# multimir_summary$mature_mirna_acc <- new_columns[,2]
# all_pair2 <- multimir_summary$pairs
# multimir_summary$pairs <- NULL
# multimir_summary <- as.data.frame(multimir_summary)

save(org_mirna_targets, file = file.path(opt$output, 
    paste0(opt$organism, ".RData")))
# save(multimir_summary, file = file.path(opt$output, paste0("parsed_", 
#                     opt$organism, ".RData")))
#suppressPackageStartupMessages(require(mirbase.db)) 







cat("FINISHED", file=file.path(opt$output, paste0(opt$organism, "_finished")), 
    sep="\n")
# unlink(temp_dir, recursive = TRUE, force = TRUE)

#!/usr/bin/env Rscript

option_list <- list(
	optparse::make_option(c("-i", "--input_file"), type="character", default=NULL,
                        help="DEgenesHunter results table"),
	optparse::make_option(c("-o", "--output_file"), type="character", default='./',
                        help="Define the output path. Default = './'")
)

if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){ 

    full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile),  
                   error=function(e) # works when using R CMD
                  normalizePath(unlist(strsplit(commandArgs()[grep('^--file=', 
                    commandArgs())], '='))[2]))
    main_path_script <- dirname(full.fpath)
    root_path <- file.path(main_path_script, '..', '..')
    custom_libraries <- c("miRNA_RNA_functions.R")
    for (lib in custom_libraries){
        source(file.path(root_path, 'R', lib))
      }
    template_folder <- file.path(root_path, 'inst', 'templates')
    organism_table_path <- file.path(root_path,"inst","external_data", 
        "organism_table.txt")

} else {
    require('ExpHunterSuite')
    root_path <- find.package('ExpHunterSuite')
    template_folder <- file.path(root_path, 'templates')
    organism_table_path <- file.path(root_path, "inst","external_data",
        "organism_table.txt")
}



opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))

original_table <- read.table(opt$input_file,header=TRUE, row.names=1, sep="\t")

annotated_table <- original_table

annotated_table$miRNA_name <- translate_miRNA_ids(rownames(annotated_table))

write.table(annotated_table, file.path(opt$output_file, "hunter_results_table_translated.txt"), quote=FALSE, row.names=TRUE, sep="\t")

#! /usr/bin/env Rscript

#' @author Fernando Moreno Jabato <jabato(at)uma(dot)es>

#############################################
### CONFIGURE 
#############################################

if( Sys.getenv('DEGHUNTER_MODE') == 'DEVELOPMENT' ){
    full.fpath <- tryCatch(normalizePath(parent.frame(2)$ofile), 
               error=function(e) # works when using R CMD
              normalizePath(unlist(strsplit(commandArgs()[grep('^--file=',
               commandArgs())], '='))[2]))
    main_path_script <- dirname(full.fpath)
    root_path <- file.path(main_path_script, '..', '..')

    custom_libraries <- c('statistics_functions.R', 'main_rtable2measures.R')
    for (lib in custom_libraries){
        source(file.path(root_path, 'R', lib))
    }

    #############################################
    ### LOAD LIBRARIES
    #############################################
    # suppressPackageStartupMessages(require(optparse))
    # suppressPackageStartupMessages(library(ROCR)) 
}else{
    require('ExpHunterSuite')
    root_path <- find.package('ExpHunterSuite')
}



option_list <- list(
  optparse::make_option(c("-i", "--inputfile"), type="character", 
    help="DEGenesHunter expression resulta table file"),
  optparse::make_option(c("-r", "--realprediction"), type="character",
    help="Real prediction values file"),
  optparse::make_option(c("-e", "--experiment"), type="character", 
    default = NULL,
    help="[OPTIONAL] Experiment metrics file to be included to output."),
  optparse::make_option(c("-f", "--logFC_threshold"), type="double", default=1,
    help="Log Fold Change threshold. Default : %default"),
  optparse::make_option(c("-p", "--pval_threshold"), type="double", 
    default=0.01,
    help="P-value threshold. Default : %default"),
  optparse::make_option(c("-a", "--aucfile"), type="character", default=NULL,
    help="If provided, AUC values for each method will be stored on this file"),
  optparse::make_option(c("-o", "--outfile"), type="character",
    help="Output file")
)

opt <- optparse::parse_args(optparse::OptionParser(option_list=option_list))



#############################################
### LOAD & PREPARE
#############################################

df_cuts <- rtable2measures(htfile = opt$inputfile,
                          realprediction = opt$realprediction,
                          aucfile = opt$aucfile,
                          experiment = opt$experiment,
                          logFC_threshold = opt$logFC_threshold,
                          pval_threshold = opt$pval_threshold)

# Store results table
write.table(df_cuts, file = opt$outfile, quote = FALSE, col.names = TRUE, 
    sep = "\t", row.names = FALSE)

#' Write Main DEgenes Hunter report
#'
#' This function allows you to report the DEG analysis.
#' @param exp_results DEG analysis results object
#' @param output_files output file path
#' @param template_folder RMD templates folder
#' @param opt option list
#' @return void
#' @export
#' @importFrom rmarkdown render
#' @examples
#' # Load DE analysis results
#' degh_output <- list() # data(degh_output)
#' write_expression_report(degh_output)
write_expression_report <- function(exp_results, 
    output_files=getwd(),
    template_folder = NULL, 
    opt=NULL){
    if (is.null(template_folder))
        template_folder <- file.path(find.package('ExpHunterSuite'), 'templates')
    
    if(length(exp_results) == 0){
        warning("Experiment results is not complete")
        return(NULL)
    }
    if(is.null(opt)){ opt <- exp_results[['final_main_params']]}
    final_main_params <- exp_results[['final_main_params']]
    DEG_pack_columns <- exp_results[['DEG_pack_columns']] 
    all_counts_for_plotting <- exp_results[['all_counts_for_plotting']] 
    all_FDR_names <- exp_results[['all_FDR_names']]
    all_LFC_names <- exp_results[['all_LFC_names']] 
    all_pvalue_names <- exp_results[['all_pvalue_names']]
    final_pvalue_names <- exp_results[['final_pvalue_names']]
    final_logFC_names <- exp_results[['final_logFC_names']]
    final_FDR_names <- exp_results[['final_FDR_names']]
    package_objects <- exp_results[['package_objects']]
    results_WGCNA <- exp_results[['WGCNA_all']]
    index_control_cols <- exp_results[['index_control_cols']] 
    index_treatmn_cols <- exp_results[['index_treatmn_cols']] 
    raw_filter <- exp_results[['raw_filter']] 
    design_vector <- exp_results[['design_vector']]
    all_data_normalized <- exp_results[['all_data_normalized']] 
    replicatesC <- exp_results[['replicatesC']] 
    replicatesT <- exp_results[['replicatesT']] 
    DE_all_genes <- exp_results[['DE_all_genes']]
    final_results <- exp_results[['final_results']] 
    var_filter <-  exp_results[['var_filter']] 
    cpm_table <- exp_results[['cpm_table']]
    coverage_df <- exp_results[['coverage_df']]
    mean_counts_df <- exp_results[['mean_counts_df']]
    exp_genes_df <- exp_results[['exp_genes_df']]
    numeric_factors <- exp_results[["numeric_factors"]]
    string_factors <- exp_results[["string_factors"]] 
    PCA_res <- exp_results[["PCA_res"]]  
    outf <- file.path(normalizePath(output_files),"DEG_report.html")
    rmarkdown::render(file.path(template_folder, 'main_report.Rmd'),
                      output_file = outf, intermediates_dir = output_files)
}

write_expression_data <- function(final_results, output_files, opt = NULL, template_folder){

  write.table(final_results[['raw_filter']], 
    file=file.path(output_files, "filtered_count_data.txt"), quote=FALSE, 
    col.names=NA, sep="\t")
  write.table(final_results[['sample_groups']], file=file.path(output_files, 
    "control_treatment.txt"), row.names=FALSE, quote=FALSE, sep="\t")
  write_df_list_as_tables(final_results[['all_data_normalized']], 
    prefix = 'Normalized_counts_', root = output_files)
  write_df_list_as_tables(final_results[['all_counts_for_plotting']], 
    prefix = 'allgenes_', root = output_files)
  dir.create(file.path(output_files, "Common_results"))
  write.table(final_results[['DE_all_genes']], file=file.path(output_files, 
    "Common_results", "hunter_results_table.txt"), quote=FALSE, 
  row.names=TRUE, sep="\t")

  
  write_pca_data(final_results[['PCA_res']], output_files)
}

write_pca_data <- function(PCA_res, output_files){
    pca_output <- file.path(output_files, "PCA_results")
    dir.create(pca_output)
    all_genes_pca <- PCA_res$all_genes$dim_data_merged
    
    all_genes_merged_metrics <- merge_dim_table_metrics(all_genes_pca)

    write.table(all_genes_merged_metrics, file = file.path(pca_output, "all_genes_dim_metrics.txt"),sep = "\t", quote = FALSE, row.names=FALSE)
    hcpc_table <- PCA_res$all_genes$res.hcpc$call$X
    hcpc_table$samples <- rownames(hcpc_table)
    write.table(hcpc_table, file = file.path(pca_output, "all_genes_hcpc.txt"),sep = "\t", quote = FALSE, row.names=FALSE)
    prevalent_pca <- PCA_res$DEGs$dim_data_merged
    if (!is.null(prevalent_pca)){
        hcpc_table_deg <- PCA_res$DEGs$res.hcpc$call$X
        hcpc_table_deg$samples <- rownames(hcpc_table_deg)
        write.table(hcpc_table_deg, file = file.path(pca_output, "prevalent_hcpc.txt"),sep = "\t", quote = FALSE, row.names=FALSE)
        prevalent_merged_metrics <- merge_dim_table_metrics(prevalent_pca)
        write.table(prevalent_merged_metrics, file = file.path(pca_output, "prevalent_dim_metrics.txt"),sep = "\t", quote = FALSE, row.names=FALSE)
 
    }

}

merge_dim_table_metrics <- function(merged_dim_table){

        
        if(nrow(merged_dim_table$qualitative) > 0) {
            names(merged_dim_table$qualitative)[names(merged_dim_table$qualitative) == "R2"] <- "metric"
            merged_dim_table$qualitative$metric_type <- "R2"
        } else { merged_dim_table$qualitative <- NULL}
        
        if(nrow(merged_dim_table$quantitative) > 0) {
            names(merged_dim_table$quantitative)[names(merged_dim_table$quantitative) == "correlation"] <- "metric"
            merged_dim_table$quantitative$metric_type <- "correlation"
        } else { merged_dim_table$quantitative <- NULL}

        if(nrow(merged_dim_table$qual_category) > 0) {
            names(merged_dim_table$qual_category)[names(merged_dim_table$qual_category) == "Estimate"] <- "metric"
            merged_dim_table$qual_category$metric_type <- "coord_var_barycentre"
         } else { merged_dim_table$qual_category <- NULL}

        dim_data_merged <-data.table::rbindlist(merged_dim_table, use.names = TRUE,idcol = "var_type")
        dim_data_merged <- as.data.frame(dim_data_merged)
        dim_data_merged <- dim_data_merged[,c("factor","var_type" ,"metric_type", "dimension","metric","p.value")]
        return(dim_data_merged)
} 


write_global_cormit <- function(
strategies,
cont_tables,
mirna_names,
miRNAseq,
miRNA_cont_tables,
gene_id_translation,
RNAseq,
report_name,
template_folder,
output_files,
mapping_output,
output_pairs,
all_cor_dist,
int_miRNA_cont_tables,
int_cont_tables,
integrated_pairs,
selected_predicted_databases,#
all_pairs, #
genomic_ranges,
genome_ref
){
    integrated_pairs <- as.data.frame(integrated_pairs)
    miRNA_cont_tables <- as.data.frame(miRNA_cont_tables)
    integrated_pairs <- integrated_pairs[integrated_pairs$miRNAseq %in% unique(miRNA_cont_tables[miRNA_cont_tables$db_group == "multimir", "miRNA"]),]
     miRNA_cont_tables$miRNA <- mirna_names[match(miRNA_cont_tables$miRNA, mirna_names$ACCESSION), "NAME"]   
     rmarkdown::render(
               file.path(template_folder, 'global_cormit.Rmd'), 
               output_file = file.path(output_files, report_name), 
               intermediates_dir = file.path(output_files))



    if (!is.null(genomic_ranges)){
        g_ranges <- read.table(genomic_ranges, header = FALSE)
        g_ranges <- g_ranges[,c(1,2,3,4,6)]
        colnames(g_ranges) <- c("chromosome", "start","end","miRNA","strand")
        annotated <- annotate_genomic_ranges(g_ranges, genome_ref)
        annotated <- annotated[!grepl("MIR",annotated$annot.symbol),]
        miRNA_annot <- aggregate(annot.symbol ~ miRNA, annotated, unique)
        miRNA_annot$annot.symbol <- unlist(lapply(miRNA_annot$annot.symbol,paste, collapse = ","))
        integrated_pairs$miRNA_loci <- miRNA_annot[match(integrated_pairs$miRNAseq, miRNA_annot$miRNA),"annot.symbol"]
        integrated_pairs[is.na(integrated_pairs$miRNA_loci),"miRNA_loci"] <- ""
    }    

    integrated_pairs$miRNA <- mirna_names[match(integrated_pairs$miRNAseq, mirna_names$ACCESSION), "NAME"]
    
    output_pairs_all <- add_attrib_to_pairs(integrated_pairs, RNAseq, miRNAseq)

    gene_id_translation <- as.data.frame(gene_id_translation)
    output_pairs_all$Target_SYMBOL <- gene_id_translation[match(output_pairs_all$Target_ID, gene_id_translation$ensembl_gene_id), "Symbol"]

    integrated_pairs$db_type <- ifelse(integrated_pairs$multimir, "DB","ND")
     out_pairs <- data.frame()
     genes_attr <- data.frame()
     attr <- all_pairs[,c("miRNAseq", "RNAseq", "validated_c", "predicted_c")]
     for (miRNA in unique(integrated_pairs$miRNA)){

        DB <- data.frame(miRNA = paste0(miRNA, "_DB"), 
        genes= paste(integrated_pairs$RNAseq[integrated_pairs$db_type== "DB" & integrated_pairs$miRNA == miRNA],collapse = ","))
        
        if (!output_pairs == "multimir") {

            ALL <- data.frame(miRNA = paste0(miRNA, "_ALL"), 
            genes= paste(integrated_pairs$RNAseq,collapse = ","))
            out_pairs <- rbind(out_pairs, DB, ALL)
        } else {
            out_pairs <- rbind(out_pairs, DB)
        }
     }
    write.table(out_pairs, col.names = FALSE, sep = "\t",file = file.path(output_files,"integrated_miRNA.txt"), quote = FALSE, row.names = FALSE)
    write.table(output_pairs_all, col.names = TRUE, sep = "\t",file = file.path(output_files,"target_results_table.txt"), quote = FALSE, row.names = FALSE)

}

#' @importFrom heatmaply heatmaply
write_summarize_heatmaps <- function(summarized_ORA, output_path) {
  for (funsys in names(summarized_ORA)) {
   summ_ora_funsys <- summarized_ORA[[funsys]]
   heatmaply::heatmaply(summ_ora_funsys$summ_enr_table, 
                         grid_color = "gray50",
                         seriate = "mean",
                         grid_width = 0.00001,
                         dendrogram = "both",
                         scale_fill_gradient_fun = ggplot2::scale_fill_gradient2(
                         low = "#EE8291", 
                         high = "white", 
                         midpoint = 0.5, 
                         limits = c(0, 1)),
                         file = file.path(output_path, paste0("summ_",funsys,'_heatmap.html')))
    heatmaply::heatmaply(summ_ora_funsys$summ_enr_clean, 
                         grid_color = "gray50",
                         seriate = "mean",
                         grid_width = 0.00001,
                         fontsize_row = 11,
                          fontsize_col = 13,
                         dendrogram = "both",
                         scale_fill_gradient_fun = ggplot2::scale_fill_gradient2(
                         low = "#EE8291", 
                         high = "white", 
                         midpoint = 0.5, 
                         limits = c(0, 1)),
                         file = file.path(output_path, paste0("summ_rem_parent_",funsys,'_heatmap.html')))
    heatmaply::heatmaply(summ_ora_funsys$full_enr_table, 
                        grid_color = "gray50",
                        seriate = "mean",
                        dendrogram = "both",
                        grid_width = 0.00001,
                        scale_fill_gradient_fun = ggplot2::scale_fill_gradient2(
                        low = "#EE8291", 
                        high = "white", 
                        midpoint = 0.5, 
                        limits = c(0, 1)),
                        file = file.path(output_path, paste0("full_",funsys,'_heatmap.html')))
  }
}


write_merged_cluster_report <- function(enrichments_ORA, results_path, template_folder, 
    sample_classes=NULL, DEGH_results=NULL, showCategories, group_results) {
    message("\tRendering full cluster reports")
    if(is.null(enrichments_ORA)) {
        message("No WGCNA ORA results, not printing cluster report")
    } else {
        flags_cluster <- sapply(enrichments_ORA, function(x) nrow(x@compareClusterResult)) != 0
        names(flags_cluster) <- names(enrichments_ORA)
        outf_cls <- file.path(results_path, "clusters_func_report.html")
        rmarkdown::render(file.path(template_folder, 
          'clusters_main_report.Rmd'), output_file = outf_cls, 
          intermediates_dir = results_path)
    }
}


#' Write Main clusters to enrichment output
#' This function allows you to report the Functional analysis.
#' @param output_path output folder
#' @param output_file output file name for heatmaps
#' @param mode type of output to produce - P for plots, R for reports, and S for summarized heatmaps
#' @param enrichments_ORA list of enrich results for all clusters
#' @param task_size number of elements per packages used
#' @param workers (OPTIONAL) cores for parallel features
#' @param template_folder (OPTIONAL) RMD templates folder
#' @param top_categories numbers of categories from each cluster to use for merge
#' @param group_results experimental - whether to group results in the emap plot
#' @param n_category number of categories in the figures (per cluster)
#' @param sim_thr value to use when combining similar categories in summary mode
#' @param summary_common_name 'significant' to use the most significant term to label each summarized group
#' 'ancestor' to use the common ancestor of the group"
#' @param pvalcutoff used to select terms for summarizing
#' @param gene_attributes named list of attributes e.g. FCs for emap plot coloured nodes (genes)
#' @param gene_attribute_name name for the legend in the emap plot for the nodes (genes)
#' @param max_genes maximum number of genes to plot in cnet plot
#' @param simplify Activate for process ClusterProfiler results with simplify function
#' @param clean_parentals Activate to reduce significant terms in GO by removin the parentals 
#' @return void
#' @importFrom enrichplot emapplot
#' @importFrom enrichplot dotplot
#' @importFrom ggplot2 ggsave
#' @export
write_clusters_to_enrichment <- function( 
  output_path="results",
  output_file="results",
  mode="PR",
  enrichments_ORA=NULL,
  task_size = 1,
  workers = 1,
  template_folder = file.path(find.package('ExpHunterSuite'), 'templates'),
  top_categories = NULL,
  group_results = FALSE,
  n_category = 30,
  sim_thr = 0.7, 
  summary_common_name = "ancestor", 
  pvalcutoff = 0.1, 
  gene_attributes=NULL,
  gene_attribute_name=NULL, 
  max_genes = 200,
  simplify = FALSE,
  clean_parentals = FALSE) {

  enrichments_ORA_merged <- process_cp_list(enrichments_ORA, simplify_results = simplify, 
    clean_parentals = clean_parentals)
 
  if(!is.null(top_categories))
         enrichments_ORA_merged <- filter_top_categories(enrichments_ORA_merged, top_categories)


  if (grepl("R", mode)){
      enrichments_for_reports <- parse_results_for_report(enrichments_ORA)  
      write_enrich_clusters(enrichments_ORA, output_path)
      write_func_cluster_report(enrichments_for_reports, output_path, gene_attributes, 
        workers = workers, task_size = task_size, template_folder=template_folder, gene_attribute_name=gene_attribute_name)
  }



  if (grepl("P", mode)) {
    for (funsys in names(enrichments_ORA_merged)){
      if (length(unique(enrichments_ORA_merged[[funsys]]@compareClusterResult$Description)) < 2 ) next

      if (group_results == TRUE){
        pp <- enrichplot::emapplot(enrichments_ORA_merged[[funsys]], showCategory= n_category, pie="Count", layout = "nicely", 
                    shadowtext = FALSE, node_label = "group", group_category = TRUE, 
                    nCluster = min(floor(nrow(enrichments_ORA_merged[[funsys]])/7), 20), nWords = 6, repel = TRUE)
      }else{
        pp <- enrichplot::emapplot(enrichments_ORA_merged[[funsys]], showCategory= n_category, pie="Count", layout = "nicely", 
                    shadowtext = FALSE, repel = TRUE)
      }

      ggplot2::ggsave(filename = file.path(output_path,paste0("emaplot_",funsys,".png")), pp, width = 30, height = 30, dpi = 300, units = "cm", device='png')

      pp <- enrichplot::dotplot(enrichments_ORA_merged[[funsys]], showCategory= n_category, label_format = 70)
      ggplot2::ggsave(filename = file.path(output_path,paste0("dotplot_",funsys,".png")), pp, width = 60, height = 40, dpi = 300, units = "cm", device='png')

    }
  }

  if (grepl("S", mode)) {
    summarized_merged_ora <- summarize_merged_ora(enrichments_ORA_merged, sim_thr, summary_common_name, pvalcutoff)
    write_summarize_heatmaps(summarized_merged_ora, output_path)
  }
  if(grepl("R", mode)) {
    write_merged_cluster_report(enrichments_ORA_merged, results_path=output_path, template_folder, 
            showCategories=n_category, group_results=group_results)
  }
}



#' Write Main DEgenes Hunter functional report
#' This function allows you to report the Functional analysis.
#' @param hunter_results DEG analysis results
#' @param func_results functional results
#' @param output_files output folder.
#' @param template_folder (OPTIONAL) RMD templates folder
#' @param cores (OPTIONAL) cores for parallel features
#' @param organisms_table (OPTIONAL) configuration table for given organism.
#'  Use see get_organism_table()
#' @param fc_colname (OPTIONAL) main logFC colname (into hunter_results
#'  dataframe)
#' @param task_size number of elements per packages used
#' @param report string with reports to be written. Allowed: clusters (c)
#' @param showCategories number of categories in the figures (per cluster) 
#' @param group_results experimental - whether to group results in the emap plot
#'  and functional (f). Default = "fc"
#' @param max_genes maximum number of genes to plot in cnet plot
#' @param corr_threshold minimun module eigengene-trait vector absolute Pearson R value  
#' @param pvalcutoff maximum module eigengene-trait vector correlation P value 
#' @return void
#' @importFrom rmarkdown render
#' @export
#' @examples
#' # Load func and DE results
#' data(degh_output)
#' func_results <- list() 
#' func_results <- main_functional_hunter(degh_output, "Mouse")
#' write_functional_report(degh_output, func_results)
write_functional_report <- function(
hunter_results, 
func_results, 
output_files=getwd(), 
fc_colname="mean_logFCs", 
organisms_table=NULL, 
template_folder = file.path(find.package('ExpHunterSuite'), 'templates'),
cores = 2,
task_size = 1, 
report = "fc",
showCategories = 30,
group_results = FALSE,
max_genes = 200,
corr_threshold = 0.8,
pvalcutoff = 0.05
    ){
    # TO parallelize properly
    clean_tmpfiles_mod <- function() {
      message("Calling clean_tmpfiles_mod()")
    }
    assignInNamespace("clean_tmpfiles", clean_tmpfiles_mod, ns = "rmarkdown")

 
    results_path <- normalizePath(output_files)
    model_organism <- func_results$final_main_params$model_organism

    if(is.null(organisms_table)){
        organisms_table <- get_organism_table()
    }
    current_organism_info <- subset(organisms_table, 
    rownames(organisms_table) %in% model_organism) 
    
    # Prepare the flag lists
    flags_ora <- sapply(func_results$ORA, nrow) != 0
    names(flags_ora) <- names(func_results$ORA)
    flags_gsea <- sapply(func_results$GSEA, nrow) != 0
    names(flags_gsea) <- names(func_results$GSEA)

    # JRP to clean up - should take target directly
    experiments <- hunter_results$sample_groups
    sample_classes <- apply(experiments, 1, function(x) paste0("* [", x[1],
                      "] ", x[2]))

    attr_vector <- func_results$DEGH_results_annot[
       !is.na(func_results$DEGH_results_annot$ENTREZID), fc_colname]
    names(attr_vector) <- func_results$DEGH_results_annot[
       !is.na(func_results$DEGH_results_annot$ENTREZID), "ENTREZID"]
    gene_attribute_name <- "Log2FC"

    enrichments_ORA <- func_results$WGCNA_ORA
    DEGH_results <- func_results$DEGH_results_annot
    enrichments_ORA_expanded <- func_results$WGCNA_ORA_expanded

    if(grepl("f", report)){
        message("\tRendering regular report")
        outf <- file.path(results_path, "functional_report.html")
        rmarkdown::render(file.path(template_folder, 'functional_report.Rmd'), 
            output_file = outf, intermediates_dir = results_path)  

    }
    
    
    if(!any(grepl("WGCNA", names(func_results))) && grepl("c|i", report)) {
        message("Cluster reports chosen but no cluster results available. 
                Reports wont be plotted")
    } else {
        mod_t_cor_p <- hunter_results$WGCNA_all$package_objects$module_trait_cor_p
        mod_t_cor <- hunter_results$WGCNA_all$package_objects$module_trait_cor
        corr_cl <- mod_t_cor[abs(mod_t_cor[,"treat_Ctrl"]) > corr_threshold 
                                & mod_t_cor_p[,"treat_Ctrl"] < 0.05,]
        corr_cl <- rownames(corr_cl)
        corr_cl <- gsub("Cluster_","",corr_cl)
        if (length(corr_cl) > 0) {
            enrichments_ORA <- lapply(enrichments_ORA, 
                                filter_cluster_enrichment, filter_list = corr_cl)
        } else {
            warning(paste0(c("There are not clusters with higher absolute ",
                             "correlation with treat/control hinger than ",
                             corr_threshold, ". Modify corr_threshold option. ",
                             "Reporting enrichments of all clusters...")))
        }
    
    }
    if(grepl("c", report)){
    
            write_merged_cluster_report(enrichments_ORA, results_path, 
                                template_folder, sample_classes, DEGH_results, 
                                showCategories, group_results)
            write_summarize_heatmaps(func_results$summarized_ora, results_path)
    }

   
    if(grepl("i", report)) {
        #PCC adaptar todo esto para usar write_func_cluster_report
        # JRP This will get us one day
        norm_counts <- hunter_results[["all_data_normalized"]][["DESeq2"]]
        scaled_counts <- scale_data_matrix(data_matrix = as.matrix(norm_counts))
        scaled_counts_table <- as.data.frame(as.table(scaled_counts))
        colnames(scaled_counts_table) <- c("Gene","Sample","Count")

        message("\tRendering individual cluster reports")
        if(is.null(enrichments_ORA)) {
          message("No WGCNA ORA results,not printing individual cluster report")
        } else {
        cls  <- unique(DEGH_results$Cluster_ID)
        cls <- cls[cls != 0]
        trait_module <- hunter_results$WGCNA_all$plot_objects$trait_and_module
        cl_eigvalues <- as.matrix(trait_module[,grepl("^ME",
                                            colnames(trait_module))])
        cl_eigvalues <- as.data.frame(as.table(cl_eigvalues),
          stringsAsFactors = FALSE)
        colnames(cl_eigvalues) <- c("Sample","Cluster_ID","Count")
        cl_eigvalues_gnorm <- cl_eigvalues
        cl_eigvalues_gnorm$Count <- (cl_eigvalues_gnorm$Count + 1) / 2
        pack_obj <- hunter_results$WGCNA_all$package_objects
        wgcna_pval_cl_trait <- as.matrix(pack_obj$module_trait_cor_p)
        wgcna_corr_cl_trait <- as.matrix(pack_obj$module_trait_cor)
        wgcna_count_sample_trait <- as.matrix(trait_module[, !grepl("^ME",
          colnames(trait_module))])
        wgcna_count_sample_trait <- scale_data_matrix(wgcna_count_sample_trait, 
          norm_by_col = TRUE)

        message("\tRendering specific cluster reports")
        parallel_list(cls, function(cl) {
        #lapply(cls, function(cl) {
            cl_flags_ora <- lapply(enrichments_ORA_expanded, function(x) {
              nrow(x[[which(names(x) == cl)]]) != 0 
            })
            temp_path_cl <- file.path(results_path, paste0(cl,"_temp_cl_rep"))
            outf_cls_i <- file.path(results_path, paste0("cl_func_",cl,".html"))
            DEGH_results <- DEGH_results[which(DEGH_results$Cluster_ID == cl), ]
            rmarkdown::render(file.path(template_folder, 
                   'cl_func_report.Rmd'), output_file = outf_cls_i, 
                   clean=TRUE, intermediates_dir = temp_path_cl)
        #}) 
        }, workers=cores, task_size=task_size)
        unlink(list.files(results_path, pattern="_temp_cl_rep$", 
                        full.names=TRUE), recursive=TRUE) 
      }
    }
}

#' @importFrom rmarkdown render
write_miRNA_cor_report <- function(
report_name,
template_folder, 
output_files , 
p_val_cutoff,
corr_cutoff, 
strategies, 
unsig_strategies,
cont_tables,
filters_summary,
score_comp,
all_pairs,
mirna_names,
gene_id_translation,
sample_proportion,
selected_predicted_databases,
all_cor_dist,
miRNAseq, 
miRNA_cont_tables,
eval_method,
miRNA_cont_tables_adj,
RNAseq,
sig_pairs,
raw_databases_scores,
p_fisher,
mapping_output,
output_pairs           
){
 "%>%" <- magrittr::"%>%"

 rmarkdown::render(
               file.path(template_folder, 'miRNA_RNA.Rmd'), 
               output_file = file.path(output_files, report_name), 
               intermediates_dir = file.path(output_files))
    
}

parse_strat_text <- function(strategies){
  o_text <- c()
  default_strats <-  c(
    "Eigengene_0_RNA_vs_miRNA_normalized_counts", 
    "normalized_counts_RNA_vs_miRNA_Eigengene_0", 
    "DEGs_DEMs_permutated")
  strategies <- strategies[!strategies %in% default_strats]
  dictionary <- list(
    "Eigengene" = "Eigengene profiles for coexpression modules",
    "hub_1" = "hub gene profile for coexpression modules",
    "normalized_counts" = "normalized counts"
    )
  for (strategy_name in strategies){
    strategy <- unlist(strsplit(strategy_name, "_RNA_vs_miRNA_"))
    o_text <- c(o_text,
      paste0("\t+ **", 
             strategy_name, 
             ":** correlates ", 
             dictionary[[strategy[1]]], 
             " of RNAseq data with ", 
             dictionary[[strategy[1]]],
             " of miRNAseq data."))
  }
  return(paste(o_text, collapse = "\n"))
} 


write_func_cluster_report <- function(enrichments_for_reports, output_path, 
  gene_attributes, workers, task_size, template_folder, gene_attribute_name="fold change", max_genes = 200){
  clean_tmpfiles_mod <- function() {
    message("Calling clean_tmpfiles_mod()")
  }
  assignInNamespace("clean_tmpfiles", clean_tmpfiles_mod, ns = "rmarkdown")

  parallel_list(names(enrichments_for_reports), function(cluster) {
    temp_path_cl <- file.path(output_path, paste0(cluster,"_temp"))
    func_results <- enrichments_for_reports[[cluster]]
    cl_flags_ora <- sapply(func_results, nrow) > 0
    attr_vector <- gene_attributes[[cluster]]
    outfile <- file.path(output_path, paste0(cluster, "_func_report.html"))
    test_env <- list2env(list(func_results=func_results, 
      cl_flags_ora=cl_flags_ora))
    rmarkdown::render(file.path(template_folder, 
                   'clusters_to_enrichment.Rmd'), output_file = outfile, 
               clean=TRUE, intermediates_dir = temp_path_cl, envir=test_env)
  }, workers=workers, task_size=task_size)
  # temp files not deleted properly in parallel 
  unlink(list.files(output_path, pattern="_temp$", full.names=TRUE), 
    recursive=TRUE)
}

#' Calculates emasures from result tables
#' @param htfile hunter table file
#' @param realprediction real predictions file
#' @param aucfile output AUC plot file
#' @param experiment experiment info file to be loaded
#' @param logFC_threshold logFC threshold
#' @param pval_threshold p-value threshold
#' @keywords metrics
#' @export
#' @return metrics table
#' @importFrom utils read.table write.table head
#' @importFrom ROCR prediction performance
#' @examples 
#' htfile <- system.file("extData", 
#' "hunter_results_table.txt", package = "ExpHunterSuite")
#' realprediction <- system.file("extData", 
#' "ground_truth_DE.txt", package = "ExpHunterSuite")
#' rtable2measures(htfile, realprediction)
rtable2measures <- function(
  htfile,
  realprediction,
  aucfile = NULL,
  experiment = NULL,
  logFC_threshold = 1,
  pval_threshold = 0.01
  ){

  # Load hunter table
  dgh_res_raw <- utils::read.table(file = htfile, sep = "\t", 
    header = TRUE, stringsAsFactors = FALSE, row.names = 1)

  # Load real predictions
  true_pred <- utils::read.table(file = realprediction, sep = "\t", 
    header = TRUE, stringsAsFactors = FALSE)

  # Sort
  dgh_res_raw <- dgh_res_raw[true_pred[,1],]

  # Prepare target columns
  pck_names <- c("DESeq2_DEG","edgeR_DEG","limma_DEG","NOISeq_DEG")
  pck_names <- pck_names[pck_names %in% colnames(dgh_res_raw)]
  dgh_res <- dgh_res_raw[,pck_names]
  dgh_res$Gene <- rownames(dgh_res)

  # CALCULATE AUCS
  if(!is.null(aucfile)){
    pck_fdrnames <- c("FDR_DESeq2","FDR_edgeR","FDR_limma",
      "FDR_NOISeq", "combined_FDR")
    dgh_fdr <- dgh_res_raw[, pck_fdrnames]
    dgh_fdr[is.na(dgh_fdr)] <- 1
    dgh_fdr <- 1 - dgh_fdr
    real_df <- as.data.frame(replicate(ncol(dgh_fdr), true_pred[,2]))
    pred <- ROCR::prediction(dgh_fdr, real_df)
    perf_auc <- ROCR::performance(pred, "auc")
    auc_vals <- data.frame(Method = gsub("_FDR","",gsub("FDR_","",
                                    colnames(dgh_fdr))), 
                 Set = rep("All",ncol(dgh_fdr)),
                 Measure = rep("auc", ncol(dgh_fdr)),
                 Value = unlist(perf_auc@y.values), stringsAsFactors = FALSE) 
    # Write
    utils::write.table(auc_vals, file = aucfile, sep = "\t", 
      col.names = FALSE, row.names = FALSE, quote = FALSE)  
  }



  # Calculate values for combined values
  dgh_res$Combined <- (abs(dgh_res_raw$mean_logFCs) >= logFC_threshold) &
      (dgh_res_raw$combined_FDR <= pval_threshold)


  # Combine into unique matrix
  preds_res <- merge(x = dgh_res, y = true_pred, by = "Gene")
  genes <- preds_res$Gene
  preds_res <- as.matrix(preds_res[,-1])
  preds_res[is.na(preds_res)] <- FALSE# Substitute NAs

  # Prepare matrix for vote cuts
  votes <- unlist(lapply(seq(nrow(preds_res)),function(i){
    sum(utils::head(preds_res[i,],-1)) 
  }))

  # Add cuts
  cut_names <- unlist(lapply(seq(length(pck_names)),function(pckcut){
    preds_res <<- cbind(preds_res,votes >= pckcut)
    return(paste0("Cut_",pckcut))
  }))


  colnames(preds_res) <- c(pck_names,"Combined","Prediction",cut_names)
  rownames(preds_res) <- genes

  cut_names <- c(cut_names,"Combined")

  #############################################
  ### CALC MEASURES
  #############################################

  # Calculate TP,TN,FP,FN
  df_cuts <- as.data.frame(do.call(rbind,lapply(cut_names,function(cut){
    return(data.frame(Cut = cut,
              TP = sum(preds_res[,cut] & preds_res[,"Prediction"]),
              FP = sum(preds_res[,cut] & !preds_res[,"Prediction"]),
              TN = sum(!preds_res[,cut] & !preds_res[,"Prediction"]),
              FN = sum(!preds_res[,cut] & preds_res[,"Prediction"])))
  })))

  # Calculate values for

  # Calculate measures
  df_cuts <- get_stats(df_cuts)

  #############################################
  ### EXPORT
  #############################################
  # Concat experiment info (if proceed)
  if(!is.null(experiment)){
    exp_info <- utils::read.table(file = experiment, 
      sep = "\t", header = TRUE, stringsAsFactors = FALSE)
    exp_info <- as.data.frame(do.call(cbind,lapply(seq(ncol(exp_info)),
      function(i){
        aux <- data.frame(Value = rep(exp_info[1,i],nrow(df_cuts)))
        colnames(aux) <- colnames(exp_info)[i]
        return(aux)
    })))
    df_cuts <- cbind(exp_info,df_cuts)  
  }

  return(df_cuts)

}

#! /usr/bin/env bash

. ~soft_bio_267/initializes/init_degenes_hunter
# https://laderast.github.io/2019/02/12/package-building-description-namespace/
# Rscript -e "usethis::use_package('optparse')" # To add dependencies that are not consigned by roxygen documentation
unset DEGHUNTER_MODE

current=`pwd`
cd ../../
Rscript -e "devtools::document()"
Rscript -e "devtools::install(dependencies=FALSE)"

cd $current
mkdir install_folder
Rscript -e "ExpHunterSuite::install_DEgenes_hunter('install_folder')"
install_folder/degenes_Hunter.R -h

#! /usr/bin/env bash

. ~soft_bio_267/initializes/init_degenes_hunter

export DEGHUNTER_MODE=DEVELOPMENT
export PATH=../scripts:$PATH

which degenes_Hunter.R
# grep -o '"\-\-[A-Za-z0-9_]*' options_file | sed 's/"--\([A-Za-z0-9_]*\)/\1=opt$\1,/g'
degenes_Hunter.R -h

```{r "corr_stats_config", include = FALSE}
# ###########################################
# ################# READ ME #################
# ###########################################
#
# This template assumes that the following variables exists
# => clgenes :: a named list of vectors with cluster name as IDs and genes of each clusters as vectors
# => cl :: cluster ID to be used
# => norm_counts :: a data frame with columns Gene (character), Sample (character) and Count (numeric) 
# => norm_counts_gnorm :: a data frame with columns Gene (character), Sample (character) and Count (numeric) 
# => cl_eigvalues_gnorm ::
# => wgcna_pval_cl_trait ::
# => wgcna_corr_cl_trait ::
# => wgcna_count_sample_trait ::
#
# This templates include the following chunk names:
# => <cl>_corr_stats_config
# => <cl>_explote_genes
 
# Load necessary packages
# require(ggplot2)
n_category <- 30
pval_thr_forFeatures <- 0.05

```
## **Profiles data**


```{r "explore_genes", echo = FALSE, message = FALSE, warning = FALSE, results = 'asis'}
# Load genes expressions
#cl_i_genes <- unique(rownames(DEGH_results)[which(DEGH_results$Cluster_ID == {{cl}})])
cl_i_genes <- unique(rownames(DEGH_results))
# cl_i_genes <- clgenes[[as.character({{cl}})]]
tgt_indx <- which(scaled_counts_table$Gene %in% cl_i_genes)
# Prepare regular cluster info
cl_profile <- cl_eigvalues_gnorm[cl_eigvalues_gnorm$Cluster_ID == paste0("ME", cl),]
# message(cl_profile)
# cl_profile <- cl_eigvalues_gnorm[grepl(paste(c("ME",{{cl}},"$"),collapse=""), cl_eigvalues_gnorm$Cluster_ID),]
# message(cl_profile)
# q()
# Plot
pp <- ggplot2::ggplot(mapping = ggplot2::aes(x = Sample)) + 
	  ggplot2::geom_line(data = scaled_counts_table[tgt_indx,], alpha = 0.05, mapping = ggplot2::aes(y = Count, group = Gene)) +
	  ggplot2::labs(y = "Scaled gene expression value") +
	  ggplot2::geom_line(data = cl_profile, mapping = ggplot2::aes(y = Count, group = Cluster_ID), color = "blue", linetype = "dashed", size = 0.5) +
	  ggplot2::scale_y_continuous(sec.axis = ggplot2::sec_axis(~.*2-1, name = "Eigengene Value (broken blue line)")) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 30, hjust = 1))
plot(pp)

# Tidy to expanded
tidy_cl_exp <- scaled_counts_table[tgt_indx, ]
max_cl_exp <- matrix(, length(unique(tidy_cl_exp$Gene)), length(unique(tidy_cl_exp$Sample)))
colnames(max_cl_exp) <- unique(tidy_cl_exp$Sample)
row.names(max_cl_exp) <- unique(tidy_cl_exp$Gene)
max_cl_exp[as.matrix(tidy_cl_exp[, 1:2])] <- tidy_cl_exp[, "Count"]

invisible(plot_in_div(gplots::heatmap.2(max_cl_exp, Colv=FALSE, col="bluered", 
	  key.title="Expression", keysize=1.2, density.info="none", dendrogram ="row", trace="none")))
```

```{r "wgcna_corr", echo = FALSE, results = 'asis'}
# Take pval of Cluster-Treats
clpvals <- wgcna_pval_cl_trait[paste0("Cluster_", cl),]
clcorrs <- wgcna_corr_cl_trait[paste0("Cluster_", cl),]
# Filter
clpvals <- clpvals[clpvals <= pval_thr_forFeatures]
```
```{r "wgcna_corr_pl1", echo = FALSE, results = 'asis', eval = length(clpvals) > 0}
cat(paste0("After apply a threshold of ",pval_thr_forFeatures," over cluster correlation with Traits following relationships have been signifcant observed:"))
clcorrs <- clcorrs[names(clpvals)]
# Transform
clpvals <- as.data.frame(as.table(clpvals))
colnames(clpvals) <- c("Trait","Cl_Trait_Pval")
clcorrs <- as.data.frame(as.table(clcorrs))
colnames(clcorrs) <- c("Trait","Cl_Trait_Corr")
# Merge
cltraits <- merge(x = clcorrs, y = clpvals, by = "Trait", all = TRUE)
# Plot Cluster correlations
pp <- ggplot2::ggplot(cltraits,ggplot2::aes(x = Trait, y = Cl_Trait_Corr, color = Cl_Trait_Pval)) + 
	  ggplot2::geom_point() +
	  ggplot2::geom_hline(yintercept = 0) + 
 	  ggplot2::labs(y = "Correlation between eigengene and external variable", color="Correlation\np-value") +
	  ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 30, hjust = 1)) + 
	  ggplot2::ylim(c(-1,1))
plot(pp)
```
```{r "wgcna_corr_pl2", echo = FALSE, results = 'asis', eval = length(clpvals) > 0}
# cat("\nTEXTHERE:\n")
# Prepare Sample-Trait correlations
smtraits <- as.matrix(wgcna_count_sample_trait[,as.character(cltraits$Trait)])
rownames(smtraits) <- rownames(wgcna_count_sample_trait)
colnames(smtraits) <- cltraits$Trait
smtraits <- as.data.frame(as.table(smtraits))
colnames(smtraits) <- c("Sample","Trait","Correlation")
colnames(cl_profile) <- colnames(smtraits)
cl_profile$Trait <- rep(paste0("Cluster_",cl),nrow(cl_profile))
# smtraits <- rbind(smtraits,cl_profile)

pp2 <- ggplot2::ggplot(smtraits, ggplot2::aes(x = Sample, y = Correlation, group = Trait, color = Trait)) +
	  ggplot2::geom_point(na.rm = TRUE) +
	  ggplot2::geom_line(na.rm = TRUE) +
	  ggplot2::geom_line(data = cl_profile, linetype = "dashed", size = 0.5, color = "blue") +
	  ggplot2::labs(y="Scaled external variable value", color="External Variable") +
	  ggplot2::scale_y_continuous(sec.axis = ggplot2::sec_axis(~.*2-1, name = "Eigengene value (broken blue line)")) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 30, hjust = 1))
plot(pp2)
```
```{r "wgcna_corr_plERR", echo = FALSE, results = 'asis', eval = length(clpvals) <= 0}
cat(paste0("After apply a threshold of ",pval_thr_forFeatures," over cluster correlation with Traits **any correlation have been tagged as significant**"))
```

---  
title: "ExpHunterSuite: corMiT crosvalidation report"
author: "SysBioLab"
output: 
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container {
        max-width: 90%;

    }

    .plot_real_size {
      overflow: scroll;
      max-height: 600px;
    }

    .plot_real_size img{
      max-width: none;
      max-height: none;
    }

    embed {
   
  }   
</style>
# **CoRmiT: Crossvalidation**

## Rank of the test dataset for the best strategies in train 

```{r config, echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'}

mirna_names <- translate_miRNA_ids(unique(full_ranking$miRNA))
```

```{r ranking_dist, echo = FALSE, warning =FALSE, message=FALSE, results = 'asis', eval = FALSE} 

cross_rank$significant <- ifelse(cross_rank$miRNA %in% best_miRNA_strategies$miRNA, "significant", "not significant")

cross_rank$miRNA_n <- mirna_names[match(cross_rank$miRNA, mirna_names$ACCESSION), "NAME"]   

ggplot2::ggplot(cross_rank, ggplot2::aes(x = miRNA_n, y = test_rank, fill = significant)) + 

ggplot2::geom_boxplot()+
 ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 315,hjust=0,vjust=0)) +
 ggplot2::labs(y= "Rank position of test dataset")


```

## Rank of the best strategy

Ranking values of the best strategy in corMiT but using random sections of databases

```{r ranking_dist_train, echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'} 

train_rank_dist$miRNA_n <- mirna_names[match(train_rank_dist$miRNA, mirna_names$ACCESSION), "NAME"]   


p_meds <- plyr::ddply(train_rank_dist, plyr::.(miRNA_n), summarise, med = median(OR_train_rank))

ggplot2::ggplot(train_rank_dist, ggplot2::aes(x = miRNA_n, y = OR_train_rank)) + 
ggplot2::geom_boxplot()+
ggplot2::geom_text(data = p_meds, ggplot2::aes(x = miRNA_n, y = med, label = med), 
              size = 3, vjust = -1.5)+


 ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 60,hjust=1,vjust=1, size = 18),
                 axis.text.y = ggplot2::element_text(size = 18), 
                 axis.title.y = ggplot2::element_text(size = 15, face = "bold")) +
 ggplot2::labs(y= "Ranking position in 75% of DB", x = NULL)+
 ggplot2::scale_y_continuous(limits = c(0, 25))



```

```{r ranking_tp_corr, echo = FALSE, warning =FALSE, message=FALSE, results = 'asis', eval = FALSE} 
	
	rank_tp <- cross_rank %>% group_by(miRNA) %>% summarise(median_rank = median(test_rank))
	rank_tp$TP <- best_miRNA_strategies[match(rank_tp$miRNA, best_miRNA_strategies$miRNA), "TP"]
	ggplot2::ggplot(rank_tp, ggplot2::aes(x = TP, y = median_rank)) +ggplot2::geom_point()+
	ggplot2::geom_label(label = miRNA) 
```

```{r ranking_scatter, echo = FALSE, warning =TRUE, message=TRUE, results = 'asis'} 

plot_objects <- list()
full_ranking <- full_ranking[full_ranking$miRNA %in% best_miRNA_strategies$miRNA,]
full_ranking$miRNA_n <- mirna_names[match(full_ranking$miRNA, mirna_names$ACCESSION), "NAME"] 
miRNA_cor_results$miRNA_cont_tables$miRNA_n  <- mirna_names[match(miRNA_cor_results$miRNA_cont_tables$miRNA, mirna_names$ACCESSION), "NAME"] 
train_rank_dist2 <- data.frame()
for (miRNA in unique(full_ranking$miRNA_n)) {
   miRNA_ct <- miRNA_cor_results$miRNA_cont_tables[miRNA_cor_results$miRNA_cont_tables$miRNA_n == miRNA,]
    miRNA_ct <- miRNA_ct[miRNA_ct$db_group == "multimir",]
    miRNA_ct$full_db_rank <- rank(-miRNA_ct$Odds_ratio, ties.method ="min")
    
    miRNA_rank <- full_ranking[full_ranking$miRNA_n == miRNA,]
    full_and_sample_eq <- match(paste0(miRNA_rank$strategy, miRNA_rank$corr_cutoff),
                                paste0(miRNA_ct$strategy, miRNA_ct$corr_cutoff))
    miRNA_rank$full_db_rank <- miRNA_ct$full_db_rank[full_and_sample_eq]
    miRNA_rank$Pvalue <- miRNA_ct$Pvalue[full_and_sample_eq]

    # miRNA_rank <- miRNA_rank[miRNA_rank$Pvalue <= 0.05,]
    
    top_miRNA_rank <- miRNA_rank[miRNA_rank$full_db_rank == 1 & !is.na(miRNA_rank$full_db_rank),]
   
    train_rank_dist2 <- rbind(train_rank_dist2,top_miRNA_rank)
    pp <- ggplot2::ggplot(miRNA_rank, ggplot2::aes(y = OR_train_rank, x = full_db_rank))+
    ggplot2::geom_jitter(size = 0.5,width = 0.2, height = 0.2, alpha = 0.3) + 
    ggplot2::geom_boxplot(data = top_miRNA_rank, mapping= ggplot2::aes(y = OR_train_rank, x = 0))+
    ggplot2::scale_y_reverse()+
    ggplot2::scale_x_reverse()+
    ggplot2::ggtitle(miRNA)+
    ggplot2::labs(y = NULL, x = NULL)+
    ggplot2::geom_vline(xintercept  = 1.5, linetype="dashed")
        plot_objects[[miRNA]] <- pp
}
# save(train_rank_dist, file = "/mnt/scratch/users/bio_267_uma/josecordoba/NGS_projects/LaforaRNAseq/analysis/target_wf_pearson/ctrl_vs_mut_deff/coRmiT.R_0000/test2.Rdata")
gridExtra::grid.arrange(grobs = plot_objects, ncol = 3, nrow = ceiling(length(plot_objects)/3),
  left = "Ranking position in 75% of DB", bottom = "Ranking position in 100% of DB")
```


## Rank of the best strategy 2

Ranking values of the best strategy in corMiT but using random sections of databases

```{r ranking_dist_train2, echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'} 

train_rank_dist2$miRNA_n <- mirna_names[match(train_rank_dist2$miRNA, mirna_names$ACCESSION), "NAME"]   

ggplot2::ggplot(train_rank_dist2, ggplot2::aes(x = miRNA_n, y = OR_train_rank)) + 
ggplot2::geom_boxplot()+


 ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 60,hjust=1,vjust=1, size = 18),
                 axis.text.y = ggplot2::element_text(size = 18), 
                 axis.title.y = ggplot2::element_text(size = 15, face = "bold")) +
 ggplot2::labs(y= "Ranking position in 75% of DB", x = NULL)+
 ggplot2::scale_y_continuous(limits = c(0, 25))



```

---
title: "ExpHunterSuite: Clusters Functional Report"
author: "SysBioLab"
output:
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container { 
        max-width: 90%;
    
    }

    .plot_real_size {
      overflow: scroll;
      max-height: 600px;
    }

    .plot_real_size img{
      max-width: none;
      max-height: none;
    }

    embed {
   
  }   
</style>

# **Functional Report**


```{r main_init, echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'} 
if (!is.null(opt$description)){
  cat(opt$description)
}

```

```{r render_partial, echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'} 

res <- list()
for(funsys in names(cl_flags_ora)) {
  if(cl_flags_ora[[funsys]] == TRUE) {
    func_clusters <- func_results[[funsys]]
    enrich_obj <- func_clusters
    exp_res <- knitr::knit_expand("ora_plots_all_objects.Rmd")
    res[[funsys]] <- knitr::knit(text=exp_res, quiet = TRUE)
  } else {
    res[[funsys]] <- paste0("## **",funsys," - Over Representation Analysis**\nNo enrichment found for ", funsys,"\n")
  }
}

cat(unlist(res), sep = '\n')

```

```{r diffcoexp_conditional_block_head, eval=grepl("X", opt$modules), echo=FALSE, results='asis'}
cat("### **diffcoexp Results**

The package diffcoexp looks for genes that are coexpressed with different sets genes for case and control samples respectively, i.e. differentially co-expressed

The following plot shows the ratio of differentially co-expressed links (DCLs) to the total number of co-expressed links for each gene for which at least one DCL is available:")
```

```{r diffcoexp_conditional_block_plot, eval=grepl("X", opt$modules), echo=FALSE, warning=FALSE, message=FALSE}
DCGs <- results_diffcoexp$DCGs
DCG_plot_df <-data.frame(DCG_ratio = DCGs$DCLs/DCGs$CLs, 
                mean_logFCs = DE_all_genes[DCGs$Gene,"mean_logFCs"],
                DCG_qval = DCGs$q,
                combined_FDR = DE_all_genes[DCGs$Gene,"combined_FDR"])

# Use these to vary signficance according to DCG and combiend FDR for DE
DCG_plot_df$DCG_sig <- ifelse(DCG_plot_df$DCG_qval < 0.05, "sig", "not_sig")
DCG_plot_df$DE_sig <- ifelse(DCG_plot_df$combined_FDR < 0.05, "sig", "not_sig")
ggplot2::ggplot(DCG_plot_df, ggplot2::aes(x=DCG_ratio, y=mean_logFCs, colour=DCG_sig, shape=DE_sig)) + ggplot2::geom_point()

```

---  
title: "ExpHunterSuite: All Clusters Report"
author: "SysBioLab"
output: 
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container {
        max-width: 90%;

    }

    .plot_real_size {
      overflow: scroll;
      max-height: 600px;
    }

    .plot_real_size img{
      max-width: none;
      max-height: none;
    }

    embed {
   
  }   
</style>

```{r config, include=FALSE, message=FALSE}
require(ggplot2)
require(knitr)
require(clusterProfiler)
require(enrichplot)
require(DOSE)
```

```{r child="func_initial_details.Rmd", eval=!is.null(sample_classes)}
```
```{r child="func_deg_details.Rmd", eval=!is.null(sample_classes)}
```

## **Overview of clusters**
The following table shows the numbers of prevalent vs. non-prevalent genes found in 
each cluster (first two columns), followed by the number  of significantly enriched 
categories found in each cluster for each of the annotation sources found using the ORA method (remaining columns).

```{r func_specific, echo=FALSE, results='asis', eval=!is.null(sample_classes)}

# JRP - to decide - count entrez NAs to make consistent, or remove as not used for enrichements? For now we leave
# DEGH_results_clust_stats <- DEGH_results[! is.na(DEGH_results$entrezgene) &  DEGH_results$Cluster_ID != 0,]
DEGH_results_clust_stats <- DEGH_results[DEGH_results$Cluster_ID != 0,] 
prev_cluster <- t(table(DEGH_results_clust_stats$genes_tag, DEGH_results_clust_stats$Cluster_ID))

enr_per_cluster <- sapply(func_results$WGCNA_ORA_expanded, function(enr_list) {
  sapply(enr_list, nrow)
})
# Have to ensure row names match up
cluster_info <- cbind(prev_cluster, enr_per_cluster[row.names(prev_cluster),])

DT::datatable(cluster_info, filter = 'top', rownames = FALSE, extensions = c('Buttons','ColReorder'),
                    options = list(
                      colReorder = TRUE,
                      dom = 'lftBip',
                        buttons = c('copy', 'csv', 'excel')
))

to_show <- DEGH_results[,c("Cluster_ID","SYMBOL","genes_tag",colnames(DEGH_results)[1])]
to_show$Cluster_ID <- as.character(to_show$Cluster_ID)
DT::datatable(to_show, filter = 'top', rownames = FALSE, extensions = c('Buttons', 'ColReorder'),
                    options = list(
                      colReorder = TRUE,
                      dom = 'lftBip',
                        buttons = c('copy', 'csv', 'excel')
))

cat("The following modules were run:\n")
cat(names(flags_cluster))

cat("Results were found in at least one module for the following:\n")
cat(names(flags_cluster[flags_cluster==TRUE]))

```

```{r multiple_children, echo=FALSE, results='asis'}
res <- list()
for(funsys in names(flags_cluster)) {
  if(flags_cluster[[funsys]] == TRUE) {
    enrich_obj <- enrichments_ORA[[funsys]]
    exp_res <- knitr::knit_expand("ora_plots_compareCluster.Rmd")
    res[[funsys]] <- knitr::knit(text=exp_res, quiet=TRUE)
  } else {
    res[[funsys]] <- paste0("## **No enrichment found for ", funsys, "**\n")
  }
}
cat(unlist(res), sep = '\n')

```

```{r "{{method_name}}_cor_miRNA_config", include = FALSE, warnig = FALSE}
# ###########################################
# ################# READ ME #################
# ###########################################
#
# This template assumes that the following variables exists
# => current_organism_info$Bioconductor_DB[1] :: organism database to translate genes
# => method :: method object
# => method_name :: method name
  
method <- "{{method_name}}"
outdir <- file.path(output_files, method)
check_and_create_dir(outdir)
strat_pairs <- as.data.frame(all_pairs)
strat_pairs <- strat_pairs[, "{{method_name}}"]
strat_data <- all_pairs[strat_pairs,]
strat_data$miRNA_names <- mirna_names[match(strat_data$miRNAseq, mirna_names$ACCESSION), "NAME"]
strat_data <- merge(strat_data, gene_id_translation, by.x = "RNAseq", by.y = "ensembl_gene_id", all.x = TRUE)

# q()
strat_data <- as.data.frame(strat_data)
data.table::setnames(strat_data, paste0(method, "_correlation"), "Correlation")
selected_cols <- c("miRNA_names", "miRNAseq","Symbol","RNAseq", "predicted", "validated", "multimir","pred_and_val","predicted_c", "validated_c","entrezgene", "Correlation")
strat_data <- strat_data[,selected_cols]

strat_data_table <- strat_data[,c("miRNA_names", "miRNAseq","Symbol","RNAseq", "predicted_c", "validated_c","entrezgene", "Correlation")]


DH_results_genes <- match(strat_data_table$RNAseq, RNAseq$DH_results$gene_name)
DH_results_miRNA <- match(strat_data_table$miRNAseq, miRNAseq$DH_results$gene_name)

strat_data_table$DEG_tag_target <- RNAseq$DH_results[DH_results_genes, "genes_tag"]
strat_data_table$Target_log2FC <- RNAseq$DH_results$mean_logFCs[DH_results_genes]

strat_data_table$DEM_tag_miRNA <- miRNAseq$DH_results[DH_results_miRNA, "genes_tag"]
strat_data_table$miRNA_log2FC <- miRNAseq$DH_results$mean_logFCs[DH_results_miRNA]



DH_results_miRNAs <- match(strat_data_table$miRNAseq, miRNAseq$DH_results$gene_name)
approach <- unlist(strsplit(method, "_RNA_vs_miRNA_"))
if (approach[2] != "normalized_counts") {

  strat_data_table$miRNA_mod <- miRNAseq$DH_results[DH_results_miRNAs, "Cluster_ID"]    
  strat_data_table$miRNA_MM <- miRNAseq$DH_results[DH_results_miRNAs, "Cluster_MM"]  

}
if (approach[1] != "normalized_counts") {
  
  strat_data_table$RNA_mod <- RNAseq$DH_results[DH_results_genes, "Cluster_ID"]    
  strat_data_table$RNA_MM <- RNAseq$DH_results[DH_results_genes, "Cluster_MM"]  

}

strat_data_table2 <- strat_data_table
data.table::setnames(strat_data_table, 
  c("miRNA_names", "miRNAseq", "Symbol",       "RNAseq",   "predicted_c",       "validated_c",       "entrezgene"),
  c("miRNA",       "miRNA_ID", "Symbol_target","Target_ID","Predicted_DB_count","Validated_DB_count","ENTREZGENE_target"))


```



```{r "{{method_name}}_cor_miRNA_table", echo = FALSE, message=FALSE, warning=FALSE, results = "asis"}
if (nrow(strat_data_table) > 20000){
  cat(paste0("\n\nThis strategy finds too many significant pairs. Please check ", method, "/target_results_table.txt. \n\n"))

} else {
  DT::datatable(strat_data_table, filter = 'top', rownames = FALSE, extensions = c('Buttons','ColReorder'),
        options = list(
          paging = TRUE,
          colReorder = TRUE,
          dom = 'lftBip',
          buttons = c('copy', 'csv', 'excel')
    ))
}
# strat_data <- data.frame()
```


```{r "{{method_name}}_cor_miRNA_table_error", eval = FALSE, results='asis', echo = FALSE, message=FALSE, warning=FALSE}

cat("This method was launched but no significant miRNA-target gene relations were found in multiMiR.\n")

```



```{r "{{method_name}}_dem_summary_config", echo = FALSE, message=FALSE, warning=FALSE}

all_miRNA_summary <- data.frame( stringsAsFactors = FALSE)
for (mirna_name in unique(paste0(strat_data$miRNA_names, "_",strat_data$miRNAseq))) {
   mirna_name <- unlist(strsplit(mirna_name, "_"))
   mirna <- mirna_name[1]
   mirna_id <- mirna_name[2]
   mirna_info <- strat_data[strat_data$miRNAseq == mirna_id, ]

   
   dem_summary <- data.frame(stringsAsFactors = FALSE,
               miRNA = mirna,
               miRNA_ID = mirna_id,
               Significant_targets = length(unique(mirna_info$RNAseq)),
               Targets_in_multiMiR = length(unique(mirna_info$RNAseq[mirna_info$multimir])),
               Predicted_targets = length(unique(mirna_info$RNAseq[mirna_info$predicted])),
               Validated_targets = length(unique(mirna_info$RNAseq[mirna_info$validated]))
               )
  
   all_miRNA_summary <- rbind(all_miRNA_summary, dem_summary)
}
```

```{r "{{method_name}}_dem_summary_text", echo = FALSE,  results='asis',message=FALSE, warning=FALSE}

cat("### Strategy miRNAs summary\n\n")

```

```{r "{{method_name}}_dem_summary", echo = FALSE, message=FALSE, warning=FALSE}

DT::datatable(all_miRNA_summary, filter = 'top', rownames = FALSE, extensions = c('Buttons','ColReorder'),
      options = list(          paging = TRUE,
        paging = TRUE,
        colReorder = TRUE,
        dom = 'lftBip',
        buttons = c('copy', 'csv', 'excel')
    ))

```

```{r "{{method_name}}_specific_overlapping_config", echo = FALSE, message=FALSE, warning=FALSE, eval = specific_evaluation}

specific_ct <- miRNA_cont_tables_adj[miRNA_cont_tables_adj$strategy == "{{method_name}}",]
splitted_c_tables <- split(specific_ct, specific_ct$db_group)
stats_table <- list()
for(db_group in c("predicted", "validated")){
  filtered_cont <- splitted_c_tables[[db_group]]

  cat(paste0("\n\n### Dotplot of miRNA Odds ratio and Fisher's exact test P value for ", db_group, " pairs databases\n\n\n"))

 filtered_cont$FDR <- filtered_cont$p.adjust

  overal_OR <- plot_odds_ratio(filtered_cont, 
                                OR_col = "Odds_ratio",
                                pval_col = "FDR",
                                y_col = "miRNA",
                                text_col = "FDR")
  plot(overal_OR)

  cat("\n\n")
  stats_table[[db_group]] <- filtered_cont[,c("miRNA", "Pvalue","FDR", "Odds_ratio")]
  # stats_table[[db_group]]$FDR <- filtered_cont$p.adjust

cat("\n\n")
}


```

```{r "{{method_name}}_specific_overlapping_pred_text", echo = FALSE,  results='asis', message=FALSE, warning=FALSE, eval = specific_evaluation}

  cat("\n\n### Table of miRNA Odds ratio and Fisher's exact test P value for predicted pairs databases\n\n")
```

```{r "{{method_name}}_specific_overlapping_pred_table", echo = FALSE, message=FALSE, warning=FALSE, eval = specific_evaluation}

  DT::datatable(stats_table[["predicted"]], filter = 'top', rownames = FALSE, extensions = c('Buttons','ColReorder'),
      options = list(          paging = TRUE,
        paging = TRUE,
        colReorder = TRUE,
        dom = 'lftBip',
        buttons = c('copy', 'csv', 'excel')
    ))
``` 

```{r "{{method_name}}_specific_overlapping_val_text", echo = FALSE,  results='asis', message=FALSE, warning=FALSE, eval = specific_evaluation}

  cat("\n\n### Table of miRNA Odds ratio and Fisher's exact test P value for validated pairs databases\n\n")
```

```{r "{{method_name}}_specific_overlapping_val_table", echo = FALSE, message=FALSE, warning=FALSE, eval = specific_evaluation}

  DT::datatable(stats_table[["validated"]], filter = 'top', rownames = FALSE, extensions = c('Buttons','ColReorder'),
      options = list(          paging = TRUE,
        paging = TRUE,
        colReorder = TRUE,
        dom = 'lftBip',
        buttons = c('copy', 'csv', 'excel')
    ))
``` 


```{r "{{method_name}}_dem_modules", echo = FALSE, message = FALSE, warning = FALSE, eval = approach[1] %in% c("Eigengene", "hub_1")}
"%>%" <- magrittr::"%>%"

all_miRNA_mod_summary <- strat_data_table %>% 
                            dplyr::group_by(miRNA, miRNA_ID, RNA_mod) %>% 
                            dplyr::summarise(Significant_targets = length(Target_ID), 
                                      Targets_in_multiMiR = sum(Predicted_DB_count > 0 | Validated_DB_count > 0), 
                                      Predicted_targets = sum(Predicted_DB_count > 0), 
                                      Validated_targets = sum(Validated_DB_count > 0))

all_miRNA_mod_summary <- as.data.frame(all_miRNA_mod_summary)

all_miRNA_mod_summary$Pred_percentage <- all_miRNA_mod_summary$Predicted_targets / all_miRNA_mod_summary$Significant_targets
all_miRNA_mod_summary$Val_percentage <- all_miRNA_mod_summary$Validated_targets / all_miRNA_mod_summary$Significant_targets
# save(all_miRNA_mod_summary, strat_data_table, file = "/mnt/scratch/users/bio_267_uma/josecordoba/NGS_projects/LaforaRNAseq/target_miRNA_wf/miRNA_RNAseq_analysis.R_0002/test_mod.RData")
# q()



utils::write.table(all_miRNA_mod_summary, file = file.path(outdir, "all_miRNA_mod_summary.txt"), quote=FALSE, col.names=TRUE, row.names = FALSE, sep="\t")

DT::datatable(all_miRNA_mod_summary, filter = 'top', rownames = FALSE, extensions = c('Buttons','ColReorder'),
      options = list(          paging = TRUE,
        paging = TRUE,
        colReorder = TRUE,
        dom = 'lftBip',
        buttons = c('copy', 'csv', 'excel')
    ))




```

```{r "{{method_name}}_dem_table_config", echo = FALSE, message=FALSE, warning=FALSE}

library(plyr)

save(strat_data_table2, file = file.path(outdir, "test.RData"))
if (output_pairs == "validated"){
  strat_data_table2 <- strat_data_table2[strat_data_table2$Validated_DB_count > 0,]
} else if ( output_pairs == "multimir"){
  save(strat_data_table2,file = file.path(outdir, "strat_data_table2.RData") )
    strat_data_table2 <- strat_data_table2[strat_data_table2$Validated_DB_count > 0 | strat_data_table2$Predicted_DB_count > 0,]
} 


utils::write.table(strat_data_table2, file = file.path(outdir, "target_results_table.txt"), quote=FALSE, col.names=TRUE, row.names = FALSE, sep="\t")
  

data_for_clusters_to_enrichments <-  plyr::ddply(strat_data_table2, .(miRNA), summarise, genes = list(Target_ID))
data_for_clusters_to_enrichments$genes <- unlist(lapply(data_for_clusters_to_enrichments$genes, function(gene_set){paste(gene_set, collapse = ",")}))
utils::write.table(data_for_clusters_to_enrichments, col.names = FALSE, row.names = FALSE, quote = FALSE, sep = "\t", file =  file.path(outdir, "miRNA_to_enrich.txt"))
gene_attributes <- strat_data_table2[,c("miRNA","Target_ID", mapping_output)]
colnames(gene_attributes) <- c("cluster", "geneid", mapping_output)
utils::write.table(unique(gene_attributes), file = file.path(outdir, "gene_attributes.txt"), quote=FALSE, col.names=TRUE, row.names = FALSE, sep="\t")

utils::write.table(all_miRNA_summary, file = file.path(outdir, "all_miRNA_summary.txt"), quote=FALSE, col.names=TRUE, row.names = FALSE, sep="\t")

```

---
author: "SysBioLab"
output:
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container { 
        max-width: 90%;
    
    }

    .plot_real_size {
      overflow: scroll;
      max-height: 600px;
    }

    .plot_real_size img{
      max-width: none;
      max-height: none;
    }

    embed {
   
  }   
</style>


# **CoRmiT: RNAseq - miRNAseq correlation report**

This report summarizes the results of ExpHunterSuite CoRmiT, comparing RNAseq and miRNAseq data and looking for putative miRNA-gene target interactions based on different strategies.

```{r config, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}

cols <- c("Eg Em"= "#1F77B4", "Cg Cm"="#FF7F0E", "Eg Cm"="#2CA02C", "Hg Cm"="#D62728", "Eg Hm"="#9467BD", "Hg Em"="#8C564B", "Hg Hm"="#E377C2", "Cg Hm" ="#767676", "Cg Em" ="#000000" ) #, "#7F7F7F", "#BCBD22", "#17BECF")




```

### **Correlation distributions**

Distributions of correlation values between miRNAs and target genes obtained using the different strategies.
Horizontal white lines within the distributions represent the 90 and 95 percentiles. 
The **DEGs_RNA_vs_miRNA_DEMs_opp** and **DEGs_RNA_vs_miRNA_DEMs_sim** strategy are not included.


```{r methods_comparison_violin_cor, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}
all_cor_dist_fil <- all_cor_dist[!all_cor_dist$strategy %in% c("DEGs_RNA_vs_miRNA_DEMs_opp", "DEGs_RNA_vs_miRNA_DEMs_sim") &
                                 !grepl("Eigengene_0" ,all_cor_dist$strategy),]

#all_cor_dist_fil$strategy <- set_strats_readable(all_cor_dist_fil$strategy)
pp <- ggplot2::ggplot(all_cor_dist_fil, ggplot2::aes(x = strategy, y = correlation, fill = strategy)) +
  ggplot2::geom_violin(col = "white",trim = FALSE, scale = "width",draw_quantiles = c(0.1, 0.05)) + #ggplot2::ylim(-1.000001, min + 0.00001) +
  #ggplot2::geom_hline(yintercept= , linetype = "dashed", colour = "black") +
  ggplot2::geom_boxplot(width=0.05)+ ggplot2::ylim(-1, 1) +
  ggplot2::coord_flip() +
  ggplot2::theme(axis.text.y = ggplot2::element_blank(), axis.title.y = ggplot2::element_blank()) +
  ggplot2::labs(fill = "Strategy") + 
  ggplot2::scale_fill_manual(values = cols)

pp
```

### **Correlation p-values distribution**

Distributions of correlation p-values between miRNAs and target genes obtained using the different strategies.
The dashed line represents a p-value of 0.05
The **DEGs_RNA_vs_miRNA_DEMs_opp** and **DEGs_RNA_vs_miRNA_DEMs_sim** strategy are not included.

```{r methods_comparison_violin_p, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}

pp <- ggplot2::ggplot(all_cor_dist_fil, ggplot2::aes(x = strategy, y = pval, fill = strategy)) +
  ggplot2::geom_violin(trim = FALSE, scale = "width") + 
  #ggplot2::geom_hline(yintercept= p_val_cutoff, linetype = "dashed", colour = "black") +
  ggplot2::geom_boxplot(width=0.05)+ ggplot2::ylim(-0.01, 1) +
  ggplot2::coord_flip() +
  ggplot2::theme(axis.text.y = ggplot2::element_blank(), axis.title.y = ggplot2::element_blank()) +
  ggplot2::labs(fill = "Strategy") + 
  ggplot2::scale_fill_manual(values = cols)
pp

```

### **Overal Odds Ratio**

```{r overall_OR, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}
cont_tables$strategy <- set_strats_readable(cont_tables$strategy)
cont_tables <- cont_tables[!grepl("DEG",cont_tables$strat) & !grepl("E0",cont_tables$strat),]

strat_names <- t(as.data.frame(strsplit(cont_tables$strategy, " ")))

cont_tables$RNA_strat <- strat_names[,1]
cont_tables$miRNA_strat <- strat_names[,2]


cont_tables_pred <- cont_tables[cont_tables$db_group == "multimir",]
pp <- ggplot2::ggplot(cont_tables_pred, ggplot2::aes(x=corr_cutoff, y=Odds_ratio,color = strategy)) +
            ggplot2::geom_point(ggplot2::aes( shape = RNA_strat),size = 4, stat="identity", alpha = 0.8) +
            ggplot2::geom_line(ggplot2::aes(linetype = miRNA_strat), size = 0.7, stat="identity", alpha = 0.8)+
            ggplot2::xlab("Pearson's R threshold") + ggplot2::ylab("Odds ratio") +
            ggplot2::theme_minimal()+
      ggplot2::theme(legend.position="bottom",
            legend.box = "vertical",
            axis.text = ggplot2::element_text(size = 17),
            axis.title = ggplot2::element_text(size = 20, face = "bold"),
            legend.text = ggplot2::element_text(size = 17),
            legend.title = ggplot2::element_text(size=17, face = "bold")) +
      ggplot2::guides(colour = ggplot2::guide_legend(nrow = 4))+
   #   scale_size_manual(name = "Validated pairs")+
      ggplot2::scale_color_manual(name = "Strategy",
                                  values = cols)
plot_in_div(pp,10,10) 

```

### **miRNA_strategy**

ADD TEXT

```{r miRNA_strategy, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}
 
 # miRNA_cont_tables_val <- miRNA_cont_tables[miRNA_cont_tables$db_group == "validated",]
    miRNA_cont_tables_mult <- miRNA_cont_tables[miRNA_cont_tables$db_group == "multimir",]
miRNA_cont_tables_mult$strategy <- set_strats_readable(miRNA_cont_tables_mult$strategy)
  best_strats_mult <- select_best_strategy(miRNA_cont_tables_mult)
  #best_strats_val <- select_best_strategy(miRNA_cont_tables_val)
  miRNA_cont_tables_mult$strat_corr <- paste(miRNA_cont_tables_mult$strategy , as.character(miRNA_cont_tables_mult$corr_cutoff), sep = "_")
  ggplot2::ggplot(miRNA_cont_tables_mult, ggplot2::aes(x = miRNA, y = Odds_ratio, color = strategy)) + 
  ggplot2::geom_point(size = 2)+ 
  ggplot2::geom_line(ggplot2::aes(group = strat_corr),size = 0.1) +
  ggplot2::scale_color_manual(values = cols) + 
  ggplot2::geom_point(data = best_strats_mult, ggplot2::aes(x = miRNA, y = Odds_ratio),color = "black",size = 2, pch = 21)+ 
  ggplot2::theme_bw()+
  ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 315,hjust=0,vjust=0))


```

```{r miRNA_best_strategy, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE, fig.width = 8, fig.height = 5}
 

  best_strats_mult$strat_corr<- paste(best_strats_mult$strategy , as.character(best_strats_mult$corr_cutoff), sep = " ")
  best_strats_mult$strat_pairs <- as.character(best_strats_mult$TP+ best_strats_mult$FP)

  ggplot2::ggplot(best_strats_mult, ggplot2::aes(x = miRNA, y = Odds_ratio, fill  = strategy)) + 
  ggplot2::geom_bar(stat = "identity", width = 0.8) + 
  ggplot2::geom_text(ggplot2::aes(label=as.character(corr_cutoff)), vjust=-0.3, size=3.5,  colour = "black") +
  ggplot2::geom_text(ggplot2::aes(label = TP), size=3.5, vjust = 1.5, colour = "white")+
  #ggplot2::geom_line(ggplot2::aes(group = strat_corr),size = 0.1) +
  ggplot2::scale_fill_manual(values = cols) + 
  #ggplot2::geom_point(data = best_strats_mult, ggplot2::aes(x = miRNA, y = Odds_ratio),color = "black",size = 2, pch = 21)+ 
  ggplot2::theme_bw() +
  ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 60,hjust=1,vjust=1, size = 18),
                 axis.text.y = ggplot2::element_text(size = 18), 
                 axis.title.y = ggplot2::element_text(size = 18, face = "bold"),
                 axis.title.x = ggplot2::element_text())


```


### **Best Strategy Predicted**

```{r best_strat_mult, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}
best_strats_to_print <- best_strats_mult[,c("miRNA","strategy","Odds_ratio", "corr_cutoff", "p.adjust")]
knitr::kable(best_strats_to_print)
```

### **Strategy summary Predicted**

```{r strat_summary, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}
library(dplyr) #remove
strat_sum <- best_strats_mult %>% group_by(strategy) %>% summarise(miRNA_count = length(miRNA), median_OR = median(Odds_ratio))
knitr::kable(strat_sum)
```


### **Best Strategy Validated**

```{r best_strat_val, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE, eval = FALSE}
best_strats_to_print <- best_strats_val[,c("miRNA","strategy","Odds_ratio","corr_cutoff")]
knitr::kable(best_strats_to_print)
```

### **Strategy summary Validated**

```{r strat_summary_val, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE, eval = FALSE}
strat_sum <- best_strats_val %>% group_by(strategy) %>% summarise(miRNA_count = length(miRNA), median_OR = median(Odds_ratio))
knitr::kable(strat_sum)
```





### **Validated and predicted OR correlation**

```{r OR_pre_val_corr, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE, eval = FALSE}
to_corr <- miRNA_cont_tables[,c("miRNA", "strategy","Odds_ratio","db_group")]

corr_OR <- as.data.frame(stats::reshape(to_corr, idvar = c("miRNA", "strategy"), timevar = "db_group", direction = "wide"))
corr_OR <- corr_OR[!is.na(corr_OR$Odds_ratio.validated) & 
                   !is.na(corr_OR$Odds_ratio.predicted) &
                   corr_OR$Odds_ratio.validated > 0 &
                   corr_OR$Odds_ratio.predicted > 0,]

ggpubr::ggscatter(corr_OR, x = "Odds_ratio.predicted", y = "Odds_ratio.validated" ,add = "reg.line", conf.int = TRUE,cor.coef = TRUE, cor.method = "spearman")

```

### **Validated and predicted OR correlation of best**

```{r OR_pre_val_corr_best, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE, eval = FALSE}
to_corr <- miRNA_cont_tables[apply(miRNA_cont_tables[,c("strategy","corr_cutoff", "miRNA")], 1, paste, collapse = "-") %in%
                            apply(best_strats_val[,c("strategy","corr_cutoff", "miRNA")], 1, paste, collapse = "-"),
                            c("miRNA", "strategy","Odds_ratio","db_group")]

corr_OR <- as.data.frame(stats::reshape(to_corr, idvar = c("miRNA", "strategy"), timevar = "db_group", direction = "wide"))
corr_OR <- corr_OR[!is.na(corr_OR$Odds_ratio.validated) & 
                   !is.na(corr_OR$Odds_ratio.predicted) &
                   corr_OR$Odds_ratio.validated > 0 &
                   corr_OR$Odds_ratio.predicted > 0,]

ggpubr::ggscatter(corr_OR, x = "Odds_ratio.predicted", y = "Odds_ratio.validated" ,add = "reg.line", conf.int = TRUE,cor.coef = TRUE, cor.method = "spearman")

```

### **Integrated strategy comparison**

```{r integration, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}

int_cont_tables$strategy <- set_strats_readable(int_cont_tables$strategy)
int_cont_tables <- int_cont_tables[!grepl("DEG",int_cont_tables$strat) & 
                                   !grepl("E0",int_cont_tables$strat) & 
                                   int_cont_tables$db_group == "multimir",]

int_cont_tables$sig_miRNA <- gsub("/[0-9]","", int_cont_tables$coverage_text)

int_cont_tables <- int_cont_tables[!is.na(int_cont_tables$sig_miRNA),]

ggplot2::ggplot(int_cont_tables, ggplot2::aes(fill = strategy, color = strategy, x = sig_miRNA, y = Odds_ratio)) + ggplot2::geom_point(ggplot2::aes(shape = strategy)) +
ggplot2::scale_shape_manual(values=seq(0,length(unique(int_cont_tables$strategy))))

```

```{r, include = FALSE} 
# Load necessary packages
#require(ggplot2)
#require(knitr)
#require(clusterProfiler)
```

## **Used data in this analysis**
Specifically, in this experiment set, known experiment labels are:

`r paste(knitr::knit(text = paste(sample_classes, collapse = "\n")), collapse = "\n")`

## **General description**
This report contains all the functional information that was requested by the options when functional_Hunter.R was executed.
The functional categories can be:

* KEGG pathways
* GO:
	+ Biological Process
	+ Molecular Function
	+ Cellular Component
* Reactome pathways
* Custom nomenclature

All the functional categories are computed with CluterProfiler and GO caterogires are computed also with TopGo.
Some sections will not show if there are not sinficative results. Each category is analysed using Over representation analysis (ORA) and Gene Set Analysis (GSEA).
The ORA method takes a group of significative DEGs (only DEGs, upregulated DEGs or downregulated DEGs) and performs a hypergeometric test for each term of the selected functional category.
In the case of the GSEA method, all the genes are sorted by their fold-change and the algorithm scan which genes with similar fold-change shares a term of the selected functional category.

```{r "{{gene_set}}_{{funsys}}_ora_enrichResults_config", include = FALSE}
# ###########################################
# ################# READ ME #################
# ###########################################
#
# This template assumes that the following variables exists
# => enrichment_results => enrichResult object
# => This template also need 2 tags for chunk naming gene_set and funsys
# => geneList is a named vector of a gene metric with genes as names and gane_mapping_name is the name of the metric 
```



```{r "{{gene_set}}_{{funsys}}_oraEnrich", results='asis', echo = FALSE, message=FALSE, warning=FALSE}
enrichment_results <- enrichments_for_reports[["{{gene_set}}"]][["{{funsys}}"]]

n_category <- 30
default_width <- set_default_width(enrich_obj = enrichment_results, showCategory=n_category)

if(length(get_categories(enrichment_results))>2){ # Needs, at least, two categories
	# Main label 
	cat("### **{{funsys}} - Over Representation Analysis**\n",
		"The ORA method takes a group of significant genes and performs a Fisher's exact test for each term of the selected functional category.\n\n"
	)

	cat(
		"**Barplot**\n\n", 
		"The most highly signficant categories in ascending order, according to adjusted p-value. 
		The x-axis represents the number of significant genes found within the functional category.\n\n"
       )
	pp <- graphics::barplot(enrichment_results, showCategory=n_category)
	plot_in_div(pp, fig_width = default_width)


	cat("\n\n**Dotplot**\n\n",
        "The most highly signficant categories in descending in categories of gene ratio, defined as the proportion of significant genes that are found in the functional category.
         The x-axis represents the gene ratio and the dot size the number of genes associated with the functional category.\n\n"
       )
	pp <- enrichplot::dotplot(enrichment_results, showCategory=n_category)
	plot_in_div(pp, fig_width = default_width)


	cat("\n\n**Gene-Concept Network**\n\n", 
	  "The network connects the top functional categories (brown nodes) to their associated genes (grey or colored nodes).
 	   The size of the functional category nodes shows the number of connected genes.\n\n"
       )
	enrichments_ORA_symb <- DOSE::setReadable(enrichment_results, OrgDb = current_organism_info$Bioconductor_DB[1], keyType="ENTREZID")
	if(length(get_genes(enrichments_ORA_symb)) <= 150){
		if (!is.null(geneList)){
			pp <- enrichplot::cnetplot(enrichments_ORA_symb, showCategory=n_category, foldChange=geneList, layout = "fr") +
			    	ggplot2::scale_color_gradient2(name = gane_mapping_name, low = "#0000BF", high = "#bf0000", na.value = "#50EA55")
		} else {
			pp <- enrichplot::cnetplot(enrichments_ORA_symb, showCategory=n_category)
		}
		plot_in_div(pp, fig_width = default_width)
	}else{
		cat("\n\nCnetplot is not readable because more than 150 genes has been enriched in this nomenclature, so is not printed\n\n")
	}
	cat("\n\n**Enrich Map plot**\n\n", 
	  "The top functional categories (nodes), connected if they share genes. Edge thickness represents the number of shared genes.
	  Nodes size represents the number of significant genes within the category.\n\n"
	)
	pp <- enrichplot::emapplot(enrichment_results, showCategory=n_category)
	plot_in_div(pp, fig_width = default_width)


	cat("\n\n**Heatplot**\n\n", 
	  "Significant genes (x-axis) and the functional categories in which they appear." 
	)
	if (!is.null(geneList)){
		cat("Colour indicates log2 fold change.\n\n")
		pp <- enrichplot::heatplot(enrichments_ORA_symb, showCategory=n_category, foldChange=geneList)
	} else {
		cat("\n\n")
		pp <- enrichplot::heatplot(enrichments_ORA_symb, showCategory=n_category)
	}
	width_size <- calc_width(enrichments_ORA_symb)
	height_size <- calc_height(enrichments_ORA_symb) 
	plot_in_div(pp, height_size, width_size)

	cat("\n\n**Upsetplot**\n\n", 
	  "Genes are clustered according to shared enriched categories. 
	  The y-axis shows the number of genes belonging to the different clusters (top) and categories to which they belong (bottom).\n\n"
	)
	pp <- enrichplot::upsetplot(enrichment_results, n=n_category)
	plot_in_div(pp, fig_width = default_width)
	#cat('\n\nGenes has been enriched in less than 2 categories, so Upsetplot cannot be printed\n\n')

} else{
	# Main label 
	cat("### **{{funsys}} - Over Representation Analysis**\n",
		"After execute ORA enrichment, **any significative term have been found**")
}
```

---  
title: "ExpHunterSuite: Functional Report"
author: "SysBioLab"
output: 
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container {
        max-width: 90%;

    }

    .plot_real_size {
      overflow: scroll;
      max-height: 600px;
    }

    .plot_real_size img{
      max-width: none;
      max-height: none;
    }

    embed {
   
  }   
</style>

```{r config, include=FALSE, message=FALSE}
require(ggplot2)
require(knitr)
require(clusterProfiler)
require(enrichplot)
require(DOSE)
```

```{r child="func_initial_details.Rmd"}
```
```{r child="func_deg_details.Rmd"}
```
```{r child="func_top_genes.Rmd"}
```

```{r ORA_analysis, echo=FALSE, results='asis', message=FALSE, warning=FALSE}


res_ora <- list()
for(funsys in names(flags_ora)) {
  if(flags_ora[[funsys]] == TRUE) {
    enrich_obj <- func_results$ORA[[funsys]]
    exp_res <- knitr::knit_expand("ora_plots_all_objects.Rmd")
    res_ora[[funsys]] <- knitr::knit(text=exp_res, quiet=TRUE)
  } else {
    res_ora[[funsys]] <- paste0("## **No ORA enrichment found for ", funsys, "**\n")
  }
}
cat(unlist(res_ora), sep = '\n')


res_gsea <- list()
for(funsys in names(flags_gsea)) {
  if(flags_gsea[[funsys]] == TRUE) {
    enrich_obj <- func_results$GSEA[[funsys]]
    exp_res <- knitr::knit_expand("gsea_plots_all_objects.Rmd")
    res_gsea[[funsys]] <- knitr::knit(text=exp_res, quiet=TRUE)
  } else {
    res_gsea[[funsys]] <- paste0("## **No GSEA enrichment found for ", funsys, "**\n")
  }
}
cat(unlist(res_gsea), sep = '\n')


```

## **Values of options passed to the Functional Hunter main function**
First column contains the option names; second column contains the given values for each option.
Note that large data objects (e.g. expression results, 
  organism table and custom annotation files) are not shown.
```{r opt_vals, echo = FALSE, results='asis'}
final_main_params <- func_results$final_main_params[! sapply(func_results$final_main_params, is.list)]
print(knitr::kable(cbind(final_main_params)))
```

```{r "{{funsys}}_plots_ora", echo=FALSE, results='asis', message=FALSE, warning=FALSE}
n_category <- 30
default_width <- set_default_width(enrich_obj = enrich_obj, showCategory=n_category)

if(! exists("attr_vector")) attr_vector <- NULL
# Add main label
cat("## **{{funsys}} - Over Representation Analysis**\n",
		"The ORA method takes a group of significant genes and performs a Fisher's exact test for each term of the selected functional category.\n\n"
)

cat(
	"**Barplot**\n\n", 
	"The most highly signficant categories in ascending order, according to adjusted p-value. 
	The x-axis represents the number of significant genes found within the functional category.\n\n"
)
go_pp <- graphics::barplot(enrich_obj, showCategory=n_category)
invisible(plot_in_div(go_pp, fig_width = default_width))

cat("\n\n**Dotplot**\n\n",
        "The most highly signficant categories in descending in categories of gene ratio, defined as the proportion of significant genes that are found in the functional category.
         The x-axis represents the gene ratio and the dot size the number of genes associated with the functional category.\n\n"
)
go_pp <- enrichplot::dotplot(enrich_obj, showCategory=n_category)
invisible(plot_in_div(go_pp, fig_width = default_width))


if (!is.null(attr_vector)){
	cat("\n\n**Distributions of ",gene_attribute_name,"**\n\n", 
	  "This plot shows the distribution of gene ",gene_attribute_name," that enriched the top enriched functional term/pathways.
	   The X axis repressent the distribution values. The color of the distribution repressents the FDR of the enrichment.
	   The vertical lines represents the first, second and third quartiles.\n\n"
	)

	density_pp <- enrich_density(enrich_obj, attributes = attr_vector, showCategory = n_category) + 
	ggplot2::labs(x = gene_attribute_name, y = NULL, fill = "Enrichment\nFDR") +
	ggplot2::scale_fill_gradient2(low = "#3838F3", high = "#F33838", mid = "white",na.value = "#50EA55", midpoint = pvalcutoff)
	invisible(plot_in_div(density_pp, fig_width = default_width))
}

cat("\n\n**Gene-Concept Network**\n\n", 
	  "The network connects the top functional categories (brown nodes) to their associated genes (grey or colored nodes).
 	   The size of the functional category nodes shows the number of connected genes.\n\n"
)
if(length(get_genes(enrich_obj)) <= max_genes){
	go_pp <- enrichplot::cnetplot(enrich_obj, showCategory=n_category, foldChange=attr_vector) +
	 		 { if(!is.null(attr_vector)) ggplot2::scale_color_gradient2(name = gene_attribute_name, low = "#0000BF", high = "#bf0000", na.value = "#50EA55") }
	invisible(plot_in_div(go_pp, fig_width = 15, fig_height = 15))
}else{
	cat("\n\nMore than", max_genes, "genes were found among the enriched genes. Only the first", max_genes, "genes belonging to the most significant categories are shown plotted. \n\n")
	enrich_obj_maxed <- force_max_genes_enrobj(enrich_obj, maxGenes=max_genes, showCategory=n_category)

	go_pp <- enrichplot::cnetplot(enrich_obj_maxed, showCategory=n_category, foldChange=attr_vector) +
	 		 { if(!is.null(attr_vector)) ggplot2::scale_color_gradient2(name = gene_attribute_name, low = "#0000BF", high = "#bf0000", na.value = "#50EA55") }
	invisible(plot_in_div(go_pp, fig_width = default_width))
}

cat("\n\n**Enrich Map plot**\n\n", 
  "The top functional categories (nodes), connected if they share genes. Edge thickness represents the number of shared genes.
	  Nodes size represents the number of significant genes within the category.\n\n"
)

if (nrow(as.data.frame(enrich_obj)) > 2) {
	go_pp <- enrichplot::emapplot(enrich_obj, showCategory=n_category,  min_edge = 0.0000000000000001)
	invisible(plot_in_div(go_pp, fig_width = default_width))
} else {
	cat("\nThis graph can not be plotted because fewer than two functional categories were enriched\n")
}
cat("\n\n**Heatplot**\n\n", 
	  "Significant genes (x-axis) and the functional categories in which they appear." 
)
go_pp <- enrichplot::heatplot(enrich_obj, showCategory=n_category, foldChange=attr_vector)
width_size <- calc_width(enrich_obj)
height_size <- calc_height(enrich_obj) 
invisible(plot_in_div(go_pp, height_size, width_size))

cat("\n\n**Upsetplot**\n\n", 
	  "Genes are clustered according to shared enriched categories. 
	  The y-axis shows the number of genes belonging to the different clusters (top) and categories to which they belong (bottom).\n\n"
)
if (nrow(as.data.frame(enrich_obj) > 1)){ # Needs, at least, two categories
	go_pp <- enrichplot::upsetplot(enrich_obj, n=n_category)
	invisible(plot_in_div(go_pp, fig_width = default_width))
	cat("\n \n")
}else{
	cat("\n\nGenes has been enriched in less than 2 categories, so Upsetplot cannot be printed\n\n")
}
```

---
author: "SysBioLab"
output:
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container { 
        max-width: 90%;
    
    }

    .plot_real_size {
      overflow: scroll;
      max-height: 600px;
    }

    .plot_real_size img{
      max-width: none;
      max-height: none;
    }

    embed {
   
  }   
</style>


# **CoRmiT: RNAseq - miRNAseq correlation report**

This report summarizes the results of ExpHunterSuite CoRmiT, comparing RNAseq and miRNAseq data and looking for putative miRNA-gene target interactions based on different strategies.

## **Strategies comparison**

miRNAseq and RNAseq data are compared and expression levels correlated using multiple strategies. 
Correlated miRNA-mRNA pairs are known **significant pairs** representing putative miRNA-mRNA target interactions. 
Results are then compared with multiMiR databases.

* Brief description of strategies used: 
  specific_evaluation <- eval_method == "specific"
```

```{r check_sig_specific, echo =FALSE, warning=FALSE, result = "asis", eval = (eval_method == "specific")}
  sig_strategies <- unique(cont_tables$strategy[!is.na(cont_tables$coverage)])
```

```{r print_info, echo = FALSE, warning = FALSE, results = "asis", eval = FALSE}

  sig_strategies <- names(strategies)[! names(strategies) %in% unsig_strategies]
  cat(parse_strat_text(sig_strategies))

  mock_strategies <-   c() #"Eigengene_0_RNA_vs_miRNA_normalized_counts", 
      # "normalized_counts_RNA_vs_miRNA_Eigengene_0", 
      # "DEGs_RNA_vs_miRNA_DEMs",
     # "DEGs_DEMs_permutated")

```

```{r sig_unsig, echo = FALSE, warning = FALSE, results = "asis", eval = FALSE}


cat("

  * Three **control strategies** are also used:
    + **Similar lFC:** All possible permutations between differentially expressed genes (DEGs) and differentially expressed miRNAs (DEMs) with similar log2FC (Possitive control). Correlation is not performed.
    + **Opposite lFC:** All possible permutations between DEGs and DEMs with opposite log2FC (Negative control). Correlation is not performed.
    + **E0_vs_counts:** Correlation between the Eigengene profile for gene module 0 (which contains genes that could not be assigned to any other coexpression module) and miRNA normalized counts (Negative control).

Graphics are only displayed for the strategies that were run and for which significant results were found. 
The **control strategies** are artifical correlation strategies used for comparison with the real strategies in terms of performance.
Their results should not be considered for miRNA-gene target selection.

")
    # + **normalized_counts_RNA_vs_miRNA_Eigengene_0:** Correlation between gene normalized counts and the Eigengene profile for miRNA module 0, which contains miRNAs that could not be assigned to any other module.
  sig_strats <-  paste0("**", paste(sig_strategies, collapse = "**, **"), "**")

  unsig_strats <- paste0("**", paste(unsig_strategies, collapse = "**, **"), "**")

 cat(paste0("In this report, only the strategies ", sig_strats[!sig_strats %in% mock_strategies], " produced signficant results.\n\n"))
 if (length(unsig_strategies) > 0){
  cat(paste0(unsig_strats, " were run, however no signficant correlated pairs were obtained."))
 } 

```

### **Correlation distributions**

Distributions of correlation values between miRNAs and target genes obtained using the different strategies.
Dashed line shows the correlation cut-off below which miRNA-target pairs are considered significant. 
Horizontal white lines within the distributions represent the 90 and 95 percentiles. 
The **DEGs_DEMs_permutated** strategy is not included.


```{r methods_comparison_violin_cor, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}
all_cor_dist_fil <- all_cor_dist[!all_cor_dist$strategy %in% c("DEGs_RNA_vs_miRNA_DEMs_opp", "DEGs_RNA_vs_miRNA_DEMs_sim"),]
pp <- ggplot2::ggplot(all_cor_dist_fil, ggplot2::aes(x = strategy, y = correlation, fill = strategy)) +
  ggplot2::geom_violin(col = "white",trim = FALSE, scale = "width",draw_quantiles = c(0.1, 0.05)) + #ggplot2::ylim(-1.000001, min + 0.00001) +
  ggplot2::geom_hline(yintercept= corr_cutoff, linetype = "dashed", colour = "black") +
  ggplot2::geom_boxplot(width=0.05)+ ggplot2::ylim(-1, 1)+
  ggplot2::coord_flip()+
  ggplot2::theme(axis.text.y = ggplot2::element_blank(), axis.title.y = ggplot2::element_blank()) +
  ggplot2::labs(fill = "Strategy")

pp 
```

### **Correlation p-values distribution**

Distributions of correlation p-values between miRNAs and target genes obtained using the different strategies.
The dashed line represents a p-value of 0.05
The **DEGs_RNA_vs_miRNA_DEMs_opp** and **DEGs_RNA_vs_miRNA_DEMs_sim** strategy are not included.

```{r methods_comparison_violin_p, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}

pp <- ggplot2::ggplot(all_cor_dist_fil, ggplot2::aes(x = strategy, y = pval, fill = strategy)) +
  ggplot2::geom_violin(trim = FALSE, scale = "width") + 
  ggplot2::geom_hline(yintercept= p_val_cutoff, linetype = "dashed", colour = "black") +
  ggplot2::geom_boxplot(width=0.05)+ ggplot2::ylim(-0.01, 1) +
  ggplot2::coord_flip()+
  ggplot2::theme(axis.text.y = ggplot2::element_blank(), axis.title.y = ggplot2::element_blank()) +
  ggplot2::labs(fill = "Strategy")
pp

```

### **Overlapping strategies**

This table shows the number of **significant pairs** overlapping between strategies.

```{r methods_comparison_intersection, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}
 subchunk_count <- 3
  common_pairs <- sig_pairs[["all"]]
  sig_pairs[["all"]] <- NULL
  common_pairs <- as.data.frame(as.table(as.matrix(common_pairs)))
  names(common_pairs) <- c("x_axis", "y_axis",  "intersection")
  common_pairs$text <- as.character(common_pairs$intersection)
  common_pairs$x_axis <- as.character(common_pairs$x_axis)
  common_pairs$y_axis <- as.character(common_pairs$y_axis)
  # common_pairs[common_pairs$x_axis == common_pairs$y_axis, "text"] <- "-"
  pp <- gg_heatmap(data =  common_pairs, 
              x_axis = "x_axis",
              y_axis= "y_axis",
              fill = "intersection",
              text_plot= "text", 
              labs = FALSE)
  subchunk_count <- rechunk(pp, counter = subchunk_count, 
          chunk_options = "echo=FALSE, results = 'asis', warning = FALSE, message = FALSE")

```

### **Overlapping strategies for different database groups**

The following tables shows the number of significant pairs that were found in multiMiR database groups for each strategy, and their overlap.

MultiMiR pairs can be classified based on which kind of database they appear in. The two main groups are **predicted** pairs and experimentally **validated** pairs.
We therefore repeated the steps indicated above for the union of **predicted** databases, **validated** databases, and their intersection **(pred_and_val)**.


```{r methods_comparison_intersection_groups, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}

for (db_group in names(sig_pairs)){
   common_pairs <- sig_pairs[[db_group]]
   check_ovlp <- common_pairs
   diag(check_ovlp) <- 0
   cat(paste0("#### **Overlap of significant pairs found on ", stringr::str_to_title(db_group), " databases**\n\n"))
   if (any(check_ovlp > 0)){
       common_pairs <- as.data.frame(as.table(as.matrix(common_pairs)))
       names(common_pairs) <- c("x_axis", "y_axis",  "intersection")
       common_pairs$text <- as.character(common_pairs$intersection)
       common_pairs$x_axis <- as.character(common_pairs$x_axis)
       common_pairs$y_axis <- as.character(common_pairs$y_axis)
       # common_pairs[common_pairs$x_axis == common_pairs$y_axis, "text"] <- "-"
       pp <- gg_heatmap(data =  common_pairs, 
                   x_axis = "x_axis",
                   y_axis= "y_axis",
                   fill = "intersection",
                   text_plot= "text", 
                   labs = FALSE)
       subchunk_count <- rechunk(pp, counter = subchunk_count, 
               chunk_options = "echo=FALSE, results = 'asis', warning = FALSE, message = FALSE")
   } else {
       cat(paste0("There were not overlap between significant pairs of strategies for ", db_group, " databases.\n\n"))
   }
}
   
```

### **Odds ratios for the different strategies**

The plot shows the significant strategies ordered by Odds ratio (OR). 
OR can be thought of as a measure of the association between a given strategy and pairs in the databases. 
OR > 1 represents association while OR = 1 means independence.
The color represents the p-value from the Fisher exact test.
The X-axis represents the OR and the dot size the number of pairs associated with the group of databases.

```{r dotplot, echo=FALSE, results = 'asis', warning = FALSE, message = FALSE}

significant_ct <- cont_tables[!cont_tables$strategy %in% unsig_strategies,]
splitted_c_tables <- split(significant_ct, significant_ct$db_group)

for (db_group in c("predicted", "validated")){
  filtered_cont <- splitted_c_tables[[db_group]]  
  
  cat(paste0("\n\n#### Comparison with ", stringr::str_to_title(db_group), " pairs\n\n"))
  
  overal_OR <- plot_odds_ratio(filtered_cont)
  plot(overal_OR)
  cat("\n\n")
  if (specific_evaluation) {
     cat("\n\nmedian_OR: miRNAs were evaluated independently. OR and Fisher exact test P value was computed for every miRNA and here is represented the median of the OR for miRNA with a P value lower than 0.05.
In that case Strategies without significant miRNAs were rejected.
\n\n")

     filtered_cont <- filtered_cont[!is.na(splitted_c_tables[[db_group]]$median_OR),]
     if (nrow(filtered_cont) == 0 ) {
       cat(paste0("\n\nAny miRNA overlapped with ", db_group," pairs databases. Plotting general stats.\n\n"))
       next 
     }
  filtered_cont[,"Validated_pairs"] <- filtered_cont$TP
  specific_OR <- ggplot2::ggplot(filtered_cont, ggplot2::aes(x = coverage, y = median_OR)) +
  ggplot2::geom_point(ggplot2::aes(color = strategy, size = Validated_pairs), alpha = 0.3) + 
  ggrepel::geom_text_repel(ggplot2::aes(label = coverage_text)) # +
  # ggplot2::theme(legend.position="bottom") +
  # ggplot2::guides(colour = guide_legend(nrow = 4))

  plot(specific_OR)
    cat("\n\n")

    }
}
```


## **Strategies specific results**

```{r render_partial_reports, echo = FALSE, warning = FALSE}
  curr_out <- ""

  curr_out <- unlist(lapply(sig_strategies[sig_strategies != "DEGs_DEMs_permutated"] ,function(method_name){
      sec_text <- paste(c("### **", method_name ," correlation method**\n"),collapse="") # Header
      sec_text <- c(sec_text, knitr::knit_expand("partial/miRNA_cor_methods.Rmd")) # Body
      return(sec_text)
  }))    

``` 
`r paste(knitr::knit(text = curr_out), collapse = '\n')`





### **Values of options used to run CorMiT**
First column contains the option names; second column contains the given values for each option in this run.
```{r opt_vals, echo = FALSE, warning = FALSE}
knitr::kable(cbind(opt))
```

### **Principal Component Analysis**
This is a PCA plot of the count values normalized following the default method and then they are scaled:

#### **Representation of the samples in the two first dimension of PCA** 

```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'} 
FactoMineR::plot.PCA(pca_data, invisible = "quali" , title = "")
``` 

```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'} 
FactoMineR::plot.PCA(pca_data, invisible = "quali" , title = "", label = "none", habillage = "treat")
```

#### **Representation of the samples and the categories of qualitative valiables in the two first dimension of PCA** 

```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'} 

FactoMineR::plot.PCA(pca_data, title = "")
```

#### **Representation of the variable contribution to the PCA axis 1 and 2**

```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis', eval = !is.null(numeric_factors)} 
factoextra::fviz_pca_var(pca_data, axes = c(1, 2),select.var = list(name=numeric_factors), col.var="steelblue", title="") +
		ggplot2::theme_minimal()

``` 

```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis', eval = (dim_to_keep > 3 & !is.null(numeric_factors)) } 
cat("#### **Representation of the variable contribution to the PCA axis 3 and 4**")

factoextra::fviz_pca_var(pca_data, axes = c(3, 4),select.var = list(name=numeric_factors), col.var="steelblue", title="") +
		ggplot2::theme_minimal()

``` 


```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis', eval = !is.null(numeric_factors)} 
cat("#### **Representation of the individuals and the variable contribution to the PCA axis 1 and 2**")
factoextra::fviz_pca_biplot(pca_data, repel = TRUE,select.var = list(name=numeric_factors), col.var="steelblue", title="") +
		ggplot2::theme_minimal()

```

```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'}
cat(paste("#### **Hierarchical clustering of individuals using first", dim_to_keep, "significant PCA dimensions**",sep=' '))
res.hcpc <- FactoMineR::HCPC(pca_data, graph = FALSE)
plot(res.hcpc , choice="tree", title="")

```

```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'}
cat(paste("#### **PCA representation of 1 and 2 axis with individuals coloured by its cluster membership. The first ", dim_to_keep, "significant PCA dimensions are used for HCPC**",sep=' '))
plot(res.hcpc , axes=c(1,2), choice="map", draw.tree= FALSE, title="")
```


```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'}
dim_data_quant <- dim_data_merged$quantitative
dim_data_quant <- dim_data_quant[dim_data_quant$factor %in% numeric_factors,]
if (nrow(dim_data_quant) >0 ) {
	cat("#### **Representation of correlation and P value of numeric factors and PCA dimensions**")
	dim_data_quant$p.value <-formatC(dim_data_quant$p.value, format = "e", digits = 2)
	dim_data_quant$correlation <- round(dim_data_quant$correlation, digits = 4)
	dim_data_quant$text_corr <- paste(as.character(dim_data_quant$correlation),as.character(dim_data_quant$p.value), sep = "\n")
	gg_heatmap(dim_data_quant, x_axis="dimension", y_axis="factor",fill = "correlation", text_plot="text_corr")

}
```

```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'}

dim_data_qual <- dim_data_merged$qualitative
if (nrow(dim_data_qual) >0 ) {
	cat("#### **Representation of R2 and P value of qualitative factors and PCA dimensions**")
	dim_data_qual$p.value <- formatC(dim_data_qual$p.value, format = "e", digits = 2)
	dim_data_qual$correlation <- round(dim_data_qual$R2, digits = 4)
	dim_data_qual$text_adjust <- paste(as.character(dim_data_qual$R2),as.character(dim_data_qual$p.value), sep = "\n")
	gg_heatmap(dim_data_qual, x_axis="dimension", y_axis="factor",fill = "R2", text_plot="text_adjust")

}
```



```{r , echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'}

dim_data_cat <- dim_data_merged$qual_category
if (nrow(dim_data_cat) >0 ) {
	cat("#### **Representation of estimated coordinated from barycentre and P value of qualitative factors and PCA dimensions**")
	dim_data_cat$p.value <- formatC(dim_data_cat$p.value, format = "e", digits = 2)
	dim_data_cat$text_estimate <- paste(as.character(dim_data_cat$Estimate),as.character(dim_data_cat$p.value), sep = "\n")
	gg_heatmap(dim_data_cat, x_axis="dimension", y_axis="factor",fill = "Estimate", text_plot="text_estimate")

}
```

## **Details of the input data**

### **First group of samples (to be referred to as control in the rest of the report)**
```{r control_table, echo = FALSE, warning=FALSE}
	knitr::kable(index_control_cols, col.names="Sample Names:")
```

### **Second group of samples (to be referred to as treatment in the rest of the report)**
```{r treatment_table, echo = FALSE, warning=FALSE}
	knitr::kable(index_treatmn_cols, col.names="Sample Names:")

```

```{r plots_configuration, echo = FALSE, warning=FALSE}
	# this counter is defined for plotting graphs with plot_in_div function. If not defined as a number, plots can be mixed.
	plot_counter <- 1 
```
Note: A positive log fold change shows higher expression in the treatment group; a negative log fold change represents higher expression in the control group.


## **DEgenes Hunter results**

### **Gene classification by DEgenes Hunter**
DEgenes Hunter uses multiple DE detection packages to analyse all genes in the input count table and labels them accordingly:

* **Filtered out:** Genes discarded during the filtering process as showing no or very low expression.
* **Prevalent DEG:** Genes considered as differentially expressed (DE) by at least `r opt$minpack_common` packages, as specified by the `minpack_common` argument.
* **Possible DEG:** Genes considered DE by at least one of the DE detection packages.
* **Not DEG:** Genes not considered DE in any package.

This barplot shows the total number of genes passing each stage of analysis - from the total number of genes in the input table of counts, to the genes surviving the expression filter, to the genes detected as DE by one package, to the genes detected by at least `r opt$minpack_common` packages.

```{r qc,echo = FALSE}
# Create genes barplot
barplot_df <- data.frame(total=c(
		sum(DE_all_genes[, "genes_tag"] %in% c("PREVALENT_DEG", "POSSIBLE_DEG", "NOT_DEG", "FILTERED_OUT")),
		sum(DE_all_genes[, "genes_tag"] %in% c("PREVALENT_DEG", "POSSIBLE_DEG", "NOT_DEG")),
		sum(DE_all_genes[, "genes_tag"] %in% c("PREVALENT_DEG", "POSSIBLE_DEG")),
		sum(DE_all_genes[, "genes_tag"] %in% c("PREVALENT_DEG"))
		),
	  gene_type=c("Total genes", "Genes passing expression filter", "DEGs considered possible or prevalent", "Prevalent DEGs")
	)
	barplot_df[,"gene_type"] <- factor(barplot_df[,"gene_type"], levels = barplot_df[,"gene_type"][order(barplot_df[,"total"])]) # Reorder to plot from most to least

# Plot genes barplot
pgenes <- ggplot2::ggplot(barplot_df, ggplot2::aes(gene_type, total)) +
		  ggplot2::ylab("Number of genes") + 
		  ggplot2::xlab("") +
		  ggplot2::geom_bar(position="dodge", stat="identity", fill=c("#000034", "red", "orange", "blue"), show.legend=FALSE) + ggplot2::coord_flip() + 
		  ggplot2::geom_text(ggplot2::aes(label = total, y= total + 1500))+ 
		  ggplot2::theme(text = ggplot2::element_text(face="bold", size=10))

plot(pgenes)
```

### **Package DEG detection stats**
This is the Venn Diagram of all possible DE genes (DEGs) according to at least on of the DE detection packages employed:

```{r Venn_d, echo = FALSE, fig.height=12, fig.width=12}
sets <- list()
for(DEG_pack in DEG_pack_columns){
	pack <- unlist(strsplit(DEG_pack, split='_'))[1] # Get the name of the package
	sets[[pack]] <- row.names(DE_all_genes)[which(DE_all_genes[,DEG_pack] == TRUE)] # 
}
items <- unique(unlist(sets))

# Per each item, check sets
belonging <- unlist(lapply(sets, function(set){items %in% set}))
belonging <- matrix(as.numeric(belonging),ncol = length(sets))
# Add set names
colnames(belonging) <- names(sets)
# Calculate intersections
intersections <- limma::vennCounts(belonging)
intersections[1,"Counts"] <- NA
# Plot
limma::vennDiagram(intersections, circle.col = grDevices::rainbow(length(sets)))

```

```{r setup_logFCplots, echo = FALSE}
var_thr <- 0.01
```

```{r logFCplots, echo = FALSE, eval=(!grepl("F", opt$modules) && sum(grepl("logFC_", colnames(DE_all_genes))) > 1), results = "asis"}

  cat(paste0("\n### **Plot showing variability between different DEG detection methods in terms of logFC calculation**"))
  cat(paste0("\nThis graph shows logFC calculated (y-axis) for each package (points) and gene (x-axis). Only genes with variability over ", var_thr, " will be plotted. This representation allows to user to observe the behaviour of each DE package and see if one of them has atypical results.\n"))
  cat(paste0("\nIf there are no genes showing sufficient variance in estimated logFC accross methods, no plot will be produced and a warning message will be given.\n\n"))

  pp <- ht2logFCPlot(DE_all_genes, var_filter = var_thr, title = paste0("(variance threshold >= ",var_thr,")"))
	print(pp)
```


### **FDR gene-wise benchmarking**

Benchmark of false positive calling:

Boxplot of FDR values among all genes with an FDR <= 0.05 in at least one DE detection package

```{r FDR_plot, echo = FALSE}
df <- DE_all_genes[DE_all_genes[, "genes_tag"] == "PREVALENT_DEG",]
df <- df[final_FDR_names]

df$gene_names <- row.names(df)
df_melted_FDR <- reshape2::melt(df, id.vars = "gene_names", measure.vars = final_FDR_names)
df_melted_FDR[,"package_name"] <- unlist(lapply(strsplit(as.character(df_melted_FDR[, "variable"]), split='_'), function(x) x[2])) # Ugly way to remove FDR_ from names

if(dim(df)[1] == 0){
	cat("No Prevalent DEGs found, Bar charts of FDR values for prevalent genes cannot be shown")
} else { 

p_seguros_Int <- ggplot2::ggplot(df_melted_FDR, ggplot2::aes(x = package_name, y = value, color = package_name)) +
    			 ggplot2::geom_boxplot(outlier.colour = grDevices::rgb(0, 0, 0, 0)) + 
		         ggplot2::theme_bw(base_size = 30) + 
		         ggplot2::geom_point(position = ggplot2::position_jitter(w = 0.1), color = "grey50", size = 1) + 
		         ggplot2::geom_hline(ggplot2::aes(yintercept = opt$p_val_cutoff)) + 
		         ggplot2::ylab("1 - precision (FDR)") + 
		         ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)) + 
		         ggplot2::xlab("") + 
		         ggplot2::scale_colour_discrete(guide = "none") + 
		         ggplot2::coord_cartesian(ylim = c(0, opt$p_val_cutoff))
plot(p_seguros_Int)
}
```

### **FDR Volcano Plot showing log 2 fold change vs. FDR**
The red horizontal line represents the chosen FDR threshold of `r opt$p_val_cutoff`. The black lines represent
other values.


```{r FDR_volcano, echo = FALSE, warning=FALSE}

volcano_DE_all_genes <- DE_all_genes
volcano_DE_all_genes <- volcano_DE_all_genes[volcano_DE_all_genes$genes_tag != "FILTERED_OUT",]

if (any(volcano_DE_all_genes$combined_FDR[!is.na(volcano_DE_all_genes$combined_FDR)] == 0)){
	min_FDR <- min(volcano_DE_all_genes$combined_FDR[volcano_DE_all_genes$combined_FDR != 0 &
																									 !is.na(volcano_DE_all_genes$combined_FDR)])
	volcano_DE_all_genes[volcano_DE_all_genes$combined_FDR == 0 &
											 !is.na(volcano_DE_all_genes$combined_FDR), "combined_FDR"] <- min_FDR
}

custom_shapes <- ifelse(volcano_DE_all_genes$genes_tag == "PREVALENT_DEG", 16,
													 ifelse(volcano_DE_all_genes$genes_tag == "POSSIBLE_DEG", 0, 3)) 
names(custom_shapes) <- volcano_DE_all_genes$genes_tag

evp <- EnhancedVolcano::EnhancedVolcano(volcano_DE_all_genes, 
	lab=rownames(volcano_DE_all_genes),  
	x="mean_logFCs", 
	y="combined_FDR",     
	boxedLabels = TRUE,     
	drawConnectors = TRUE,     
	pCutoff = opt$p_val_cutoff,     
	FCcutoff = opt$lfc,  
	cutoffLineType = 3,
  ylab = bquote(~-Log[10] ~ italic(FDR)),
  title = NULL,
  subtitle = NULL,
  legendLabels = c("NS", expression(Log[2] ~ FC), "FDR", expression(FDR ~ and
         ~ log[2] ~ FC)),
  caption=NULL,
  legendPosition = 'right',
  shapeCustom = custom_shapes
) 

fc_thresholds <- c(0.01, 0.05,0.1, 0.2)
all_thresholds <- unique(c(opt$p_val_cutoff, fc_thresholds))
mean_wt = data.frame(FDR = all_thresholds, nlFDR = -log10(all_thresholds))

evp <- evp +
ggplot2::geom_hline(data = mean_wt, ggplot2::aes(yintercept = nlFDR, linetype=factor(FDR)) , color = c("red", rep("black", times=nrow(mean_wt)-1)))
plot(evp)

```


```{r config_PCA_DEA, echo = FALSE, results = "asis", warning = TRUE, message = TRUE, eval = !is.null(PCA_res$DEGs)}

pca_data <- PCA_res$DEGs$pca_data
dim_to_keep <- PCA_res$DEGs$dim_to_keep
dim_data <- PCA_res$DEGs$dim_data
dim_data_merged <- PCA_res$DEGs$dim_data_merged

```

```{r child=file.path(template_folder,"facto_miner.Rmd"), eval = !is.null(PCA_res$DEGs)}

```


The complete results of the DEgenes Hunter differential expression analysis can be found in the "hunter\_results\_table.txt" file in the Common_results folder

## **DE detection package specific results**

Various plots specific to each package are shown below:

```{r DESeq2_conditional_block_sample_rank_head, eval="DESeq2" %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("### **DESeq2 size factor vs. sample rank**")
```

```{r DESeq2_conditional_block_sample_rank_plot, eval="DESeq2" %in% names(all_counts_for_plotting), echo=FALSE, warning = FALSE, message = FALSE}
	effective_lib_size <- DESeq2::sizeFactors(package_objects[['DESeq2']][['DESeq2_dataset']])
	effective_lib_size <- effective_lib_size[match(coverage_df$sample_ID, names(effective_lib_size))] 
	sample_rank <- coverage_df$sample_rank
	size_vs_rank <- data.frame(sample_rank = sample_rank,
								eff_lib_size = effective_lib_size)
	if (!is.null(library_sizes)) {
		xtitle="Sample Rank (total reads)"
	} else {
		xtitle="Sample Rank (total counts)"
    }

    ggplot2::ggplot(size_vs_rank, ggplot2::aes(x = sample_rank, y = eff_lib_size)) +
        ggplot2::geom_point(size = 3) +
        cowplot::theme_cowplot() +
        cowplot::background_grid() +
        ggplot2::labs(x=xtitle, y = "Effective library size \n(DESeq2 normalization factor)") +
        ggplot2::ylim(c(0,NA)) +
        ggplot2::scale_color_brewer(palette="Dark2")

```

```{r DESeq2_conditional_block_sample_rank_text, eval="DESeq2" %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("The effective library size is the factor used by DESeq2 normalization algorithm for each sample. The effective library size must be dependent of raw library size.")
```

```{r DESeq2_conditional_block_ELS_head, eval="DESeq2" %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("### **DESeq2 normalization effects**:

This plot compares the effective library size with raw library size")
```

```{r DESeq2_conditional_block_ELS_plot, eval="DESeq2" %in% names(all_counts_for_plotting), echo=FALSE, warning = FALSE, message = FALSE}
	effective_lib_size <- DESeq2::sizeFactors(package_objects[['DESeq2']][['DESeq2_dataset']])
	sums <- colSums(raw_filter)
	norm_bias <- data.frame(sums = sums, 
							eff_lib_size = effective_lib_size,
							Treat = design_vector)
	rownames(norm_bias) <- names(sums)

	plot <- ggplot2::ggplot(norm_bias, ggplot2::aes(x = sums, y = eff_lib_size, group = Treat)) + 
		ggplot2::geom_point(ggplot2::aes(shape = Treat, color = Treat, size = 8)) +
		ggplot2::geom_smooth(method = "lm", fill = NA, ggplot2::aes(color = Treat))+
		ggplot2::xlab("Library size (N reads)") +
        ggplot2::ylab("Effective library size \n(DESeq2 normalization factor)") +
		ggplot2::theme_classic() +
		ggplot2::theme(axis.text = ggplot2::element_text(size= 10), 
			axis.title = 	ggplot2::element_text(size = 8, face = "bold"),
			legend.title = ggplot2::element_text(size = 8, face = "bold")) +
		ggplot2::guides(size = FALSE)

	print(plot)
  
```

```{r DESeq2_conditional_block_ELS_text, eval="DESeq2" %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("The effective library size is the factor used by DESeq2 normalization algorithm for each sample. The effective library size must be dependent of raw library size.")
```


```{r DESeq2_conditional_block_ma_head, eval='DESeq2' %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("### **DESeq2 MA plot**:

This is the MA plot from DESeq2 package:")

```

```{r DESeq2_conditional_block_ma_plot, eval='DESeq2' %in% names(all_counts_for_plotting), echo=FALSE}
	DESeq2::plotMA(package_objects[['DESeq2']][['de_deseq2']], cex.lab=1.6, cex.axis=1.5)
```

```{r DESeq2_conditional_block_ma_text, eval='DESeq2' %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

	
cat("In DESeq2, the MA-plot (log ratio versus abundance) shows the log2 fold changes are attributable to a given variable over the mean of normalized counts. Points will be colored red if the adjusted Pvalue is less than 0.1. Points which fall out of the window are plotted as open triangles pointing either up or down.

A table containing the DESeq2 DEGs is provided: in Results\\_DESeq2/DEgenes\\_DESEq2.txt

A table containing the DESeq2 normalized counts is provided in Results\\_DESeq2/Normalized\\_counts\\_DESEq2.txt")

```

```{r DESeq2_only_prev_norm_head, eval="DESeq2" %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("### **Differences between samples by PREVALENT DEGs normalized counts**:
		
Counts of prevalent DEGs were normalizated by DESeq2 algorithm. This count were scaled by log10 and plotted in a heatmap.")


```

```{r DESeq2_only_prev_norm_plot, eval="DESeq2" %in% names(all_counts_for_plotting), echo = FALSE, fig.height=12, fig.width=12}
	
	DE_prevalent_genes <- rownames(DE_all_genes[DE_all_genes[, "genes_tag"] %in% c("PREVALENT_DEG"),])
	if(length(DE_prevalent_genes) > 1){
		deseq_prevalent_normalized <- deseq_normalized_data[rownames(deseq_normalized_data) %in% DE_prevalent_genes,]  
		deseq_normalized_data_mtx <- log10(data.matrix(deseq_prevalent_normalized) + 1)
	    gplots::heatmap.2(deseq_normalized_data_mtx, col = col, labRow = FALSE, dendrogram = "column", trace = "none", srtCol=45,  adjCol = c(1,1))

	}else{
        cat("Lower than 2 prevalent differential expression were found")
	}
	
```


```{r edgeR_conditional_block_head, eval='edgeR' %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("### **edgeR MA plot**

This is the MA plot from edgeR package:")

```

```{r edgeR_conditional_block_plot, eval='edgeR' %in% names(all_counts_for_plotting), echo=FALSE}

with(all_counts_for_plotting[['edgeR']], plot(logCPM, logFC, pch=20, main='edgeR: Fold change vs abundance', cex.lab=1.5, cex.axis=1.5))
with(subset(all_counts_for_plotting[['edgeR']], FDR < opt$p_val_cutoff), points(logCPM, logFC, pch=20, col='red'))
abline(h=c(-1,1), col='blue')
	
```

```{r edgeR_conditional_block_body, eval='edgeR' %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("Differential gene expression data can be visualized as MA-plots (log ratio versus abundance) where each dot represents a gene. The differentially expressed genes are colored red and the non-differentially expressed ones are colored black.

A table containing the edgeR DEGs is provided in Results\\_edgeR/DEgenes\\_edgeR.txt

A table containing the edgeR normalized counts is provided in Results\\_edgeR/Normalized\\_counts\\_edgeR.txt")

```

```{r limma_conditional_block_head, eval='limma' %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("### **limma Volcano plot**

Volcano plot of log2-fold change versus -log10 of adjusted p-values for all genes according to the analysis with limma:")

```

```{r limma_conditional_block_plot, eval='limma' %in% names(all_counts_for_plotting), echo=FALSE, warning=FALSE}

limma_sig <- row.names(all_counts_for_plotting[['limma']]) %in% row.names(DE_all_genes[DE_all_genes$limma_DEG == TRUE, ]) + 1
plot(x=all_counts_for_plotting[['limma']]$logFC, 
	y=-log10(all_counts_for_plotting[['limma']]$adj.P.Val),
	main= c('groupsB-groupsA'), xlab='logFC', ylab='-log10 adjusted p-value',
	col=c('blue', 'red')[limma_sig],
	pch=20,cex.lab=1.6, cex.axis=1.5)
abline(v= opt$lfc, col='cyan')	
abline(v= -(opt$lfc), col='cyan')	
abline(h=-log10(opt$p_val_cutoff), col='red', lty='dashed')

```

```{r limma_conditional_block_text, eval='limma' %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("A table containing the limma DEGs is provided in Results\\_limma/DEgenes\\_limma.txt")


cat("A table containing the limma normalized counts is provided in Results\\_limma/Normalized\\_counts\\_limma.txt")
```

```{r NOISeq_conditional_block_head, eval='NOISeq' %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}
cat("### **NOISeq Expressionplot**

This is the summary plot for (M,D) values (black) and the differentially expressed genes (red) from the NOISeq package (Image extracted from {'ExpressionPlot.pdf'} file):"

)
```

```{r NOISeq_conditional_block_plot, eval='NOISeq' %in% names(all_counts_for_plotting), echo=FALSE, warning=FALSE, message=FALSE, results='hide',}
    q_value <- 1-opt$p_val_cutoff # Differential expression threshold for creating plot
	NOISeq::DE.plot(package_objects[['NOISeq']], q = q_value, graphic = 'MD', cex.lab=1.4, cex.axis=1.4)

```

```{r NOISeq_conditional_block_text, eval='NOISeq' %in% names(all_counts_for_plotting), echo=FALSE, results='asis'}

cat("A table containing the NOISeq DEGs is provided in Results\\_NOISeq/DEgenes\\_NOISeq.txt.

A table containing the NOISeq normalized counts is provided in Results\\_NOISeq/Normalized\\_counts\\_NOISeq.txt"
	
)
```

## **Detailed package results comparation**
This is an advanced section in order to compare the output of the packages used to perform data analysis. The data shown here does not necessarilly have any biological implication. 

### **P-value Distributions**
Distributions of p-values, unadjusted and adjusted for multiple testing (FDR)

```{r pval_distr, echo = FALSE, warning = FALSE}

df <- DE_all_genes[DE_all_genes$genes_tag != "FILTERED_OUT", ]
df$gene_names <- row.names(df)
df_melted_pval <- reshape2::melt(df, id.vars = "gene_names", measure.vars = final_FDR_names)
df_melted_FDR <- reshape2::melt(df, id.vars = "gene_names", measure.vars = final_pvalue_names)
# Now plot
ggplot2::ggplot(df_melted_pval, ggplot2::aes(x=value)) + 
    ggplot2::geom_density() + ggplot2::facet_grid(. ~variable)
ggplot2::ggplot(df_melted_FDR, ggplot2::aes(x=value)) + 
    ggplot2::geom_density() + ggplot2::facet_grid(. ~variable)
```

### **FDR Correlations**
Correlations of adjusted p-values, adjusted for multiple testing (FDR) and for log Fold Change.

```{r pval_corr, echo = FALSE, warning = FALSE, eval = length(all_counts_for_plotting) > 1}

PerformanceAnalytics::chart.Correlation(df[, c("combined_FDR", final_FDR_names)], histogram=TRUE, pch=19, log="xy", na.action = na.omit)
PerformanceAnalytics::chart.Correlation(df[, final_pvalue_names], histogram=TRUE, pch=19, log="xy", na.action = na.omit)
PerformanceAnalytics::chart.Correlation(df[, c("mean_logFCs", final_logFC_names)], histogram=TRUE, pch=19, log="xy", na.action = na.omit)

```

---  
title: "ExpHunterSuite: Module Specific Report"
author: "SysBioLab"
output: 
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container {
        max-width: 90%;

    }

    .plot_real_size {
      overflow: scroll;
      max-height: 600px;
    }

    .plot_real_size img{
      max-width: none;
      max-height: none;
    }

    embed {
   
  }   
</style>

```{r func_initial_details, eval=exists("cl_flags_ora"), echo=FALSE, results='asis', message=FALSE, warning=FALSE}
initial_out <- knitr::knit_expand(file.path(template_folder, "func_initial_details.Rmd"))
initial <- knitr::knit(text=initial_out, quiet=TRUE)
cat(initial, sep='\n')
```
```{r func_deg_details, echo=FALSE, results='asis', message=FALSE, warning=FALSE}
degh_out <- knitr::knit_expand(file.path(template_folder, "func_deg_details.Rmd"))
degh <- knitr::knit(text=degh_out, quiet=TRUE)
cat(degh, sep='\n')
```

```{r load_correlation_stats, echo=FALSE, results='asis', message=FALSE, warning=FALSE}
cor_out <- knitr::knit_expand(file.path(template_folder, "partial/corr_cl_stats.Rmd"))
rescorr <- knitr::knit(text=cor_out, quiet=TRUE)
cat(rescorr,sep='\n')
```

```{r ORA_analysis, eval=exists("cl_flags_ora"), echo=FALSE, results='asis', message=FALSE, warning=FALSE}
gene_set <- "ORA"
default_width <- 5
res <- list()
for(funsys in names(cl_flags_ora)) {
  if(cl_flags_ora[[funsys]] == TRUE) {
    func_clusters <- func_results$WGCNA_ORA_expanded[[funsys]]
    enrich_obj <- func_clusters[[which(names(func_clusters) == cl)]]
    gene_attribute_name <- "fold change"
    exp_res <- knitr::knit_expand("ora_plots_all_objects.Rmd")
    res[[funsys]] <- knitr::knit(text=exp_res, quiet = TRUE)
  } else {
    res[[funsys]] <- paste0("## **No enrichment found for ", funsys, "**\n")
  }
}
cat(unlist(res), sep = '\n')
```

```{r PCIT_conditional_block_head, eval=!is.null(DE_all_genes[['rif1']]), echo=FALSE, results='asis'}
cat("### **PCIT Results**

Plots of some of the various metrics computed in the PCIT implementation. Better explanation and more plots to be added at a later date:"

)
```

```{r PCIT_conditional_block_plot, eval=!is.null(DE_all_genes[['rif1']]), echo=FALSE, warning=FALSE, message=FALSE}
    ggplot2::ggplot(DE_all_genes, ggplot2::aes(x=ctrl_cn)) +
              ggplot2::geom_density()
    ggplot2::ggplot(DE_all_genes, ggplot2::aes(x=treat_cn)) +
              ggplot2::geom_density()
    ggplot2::ggplot(DE_all_genes, ggplot2::aes(x=diff_cn)) +
            ggplot2::geom_density()
    ggplot2::ggplot(DE_all_genes, ggplot2::aes(x=diffK)) +
            ggplot2::geom_density()

    av_de <- ggplot2::ggplot(DE_all_genes, ggplot2::aes(x=average, y=de)) +
            ggplot2::geom_point()
    av_de <- ggExtra::ggMarginal(
      av_de,
      type = 'density',
      margins = 'both',
      size = 5,
      colour = '#000000',
      fill = '#A6A39E'
    )

    de_diffK <- ggplot2::ggplot(DE_all_genes, ggplot2::aes(x=de, y=diffK)) +
            ggplot2::geom_point()
    de_diffK <- ggExtra::ggMarginal(
      de_diffK,
      type = 'density',
      margins = 'both',
      size = 5,
      colour = '#000000',
      fill = '#A6A39E'
    )

    de_pif <- ggplot2::ggplot(DE_all_genes, ggplot2::aes(x=de, y=pif)) +
            ggplot2::geom_point()
    de_pif <- ggExtra::ggMarginal(
      de_pif,
      type = 'density',
      margins = 'both',
      size = 5,
      colour = '#000000',
      fill = '#A6A39E'
    )

    de_rif1 <- ggplot2::ggplot(DE_all_genes, ggplot2::aes(x=de, y=rif1)) +
            ggplot2::geom_point()
    de_rif1 <- ggExtra::ggMarginal(
      de_rif1,
      type = 'density',
      margins = 'both',
      size = 5,
      colour = '#000000',
      fill = '#A6A39E'
    )

    de_rif2 <- ggplot2::ggplot(DE_all_genes, ggplot2::aes(x=de, y=rif2)) +
            ggplot2::geom_point()
    de_rif2 <- ggExtra::ggMarginal(
      de_rif2,
      type = 'density',
      margins = 'both',
      size = 5,
      colour = '#000000',
      fill = '#A6A39E'
    )

    rif1_rif2 <- ggplot2::ggplot(DE_all_genes, ggplot2::aes(x=rif1, y=rif2)) +
            ggplot2::geom_point()
    rif1_rif2 <- ggExtra::ggMarginal(
      rif1_rif2,
      type = 'density',
      margins = 'both',
      size = 5,
      colour = '#000000',
      fill = '#A6A39E'
    )
    gridExtra::grid.arrange(av_de, de_diffK, de_pif, de_rif1, de_rif2, rif1_rif2)

```

```{r "{{funsys}}_plots_gsea", echo=FALSE, message=FALSE, warning=FALSE, results='asis'}

n_category <- 30
default_width <- set_default_width(enrich_obj = enrich_obj, showCategory=n_category)

	cat(paste("\n\n## **{{funsys}} - Gene Set Enrichment Analysis**\n",sep=""),
		"In this method, all the genes are sorted by their fold-change and the algorithm scan which genes with similar fold-change shares a term of the selected functional category.\n\n"
	)

	cat("\n\n**Dotplot**\n\n",
		"The plot shows the top functional terms in descendant order by gene ratio. This ratio is the proportion of the known genes for a given functional term that are identified in the expression data. The color represents the associated adjusted pvalue. 
		The X axis represents the gene ratio and the dot size the number of DEG genes associated to the functional term.\n\n"
	)
	go_pp <- enrichplot::dotplot(enrich_obj, showCategory=n_category)
	invisible(plot_in_div(go_pp, fig_width = default_width))

	cat("\n\n**Gene-Concept Network**\n\n", 
		"The network connects the top functional terms (brown nodes) to their associated genes (grey or colored nodes).
		The size of the functional terms shows the number of connected genes.\n\n"
	)
	# We must do it this way until we can figure out what's up with ridgePlot
	enrich_go_gsea_symb <- DOSE::setReadable(enrich_obj, OrgDb = current_organism_info$Bioconductor_DB[1], keyType="ENTREZID")

	if(length(get_genes(enrich_go_gsea_symb)) <= 200){
		go_pp <- enrichplot::cnetplot(enrich_go_gsea_symb, showCategory=n_category, foldChange=attr_vector) +
				   ggplot2::scale_color_gradient2(name = "fold change", low = "#0000BF", high = "#bf0000", na.value = "#50EA55")
		invisible(plot_in_div(go_pp, fig_width = default_width))
	}else{
		cat("\n\nCnetplot is not readable because more than 200 genes has been enriched in this nomenclature, so is not printed\n\n")
	}

	cat("\n\n**Enrich Map plot**\n\n", 
		"The network connects the top functional terms (nodes) between them trought their associates genes (grey edges, thickness represents the number of shared genes.).
		The size of the functional terms shows the number of connected genes and the color the adjusted pvalue of the functional term.\n\n"
	)
	if(nrow(as.data.frame(enrich_obj) > 1)) {

		go_pp <- enrichplot::emapplot(enrich_obj, showCategory=n_category)
		invisible(plot_in_div(go_pp, fig_width = default_width))
	} else {
		cat("\nThis graph can not be plotted because only one functional category was enriched\n")
	}
	cat("\n\n**Heatplot**\n\n", 
		"Top functional terms are show in the Y axis whereas in the X axis are show the associated genes to them. The coloured squares indicates the fold change value.\n\n"
	)
	go_pp <- enrichplot::heatplot(enrich_go_gsea_symb, showCategory=n_category, foldChange=attr_vector)
	width_size <- calc_width(enrich_go_gsea_symb)
	height_size <- calc_height(enrich_go_gsea_symb) 
	invisible(plot_in_div(go_pp, height_size, width_size))

	cat("\n\n**Ridgeplot**\n\n",
		"Top functional terms are show in the Y axis and for each one of them a fold-change distribution is shown.
		The distribution colours indicates the adjusted pvalue.\n\n"
	)
	go_pp <- enrichplot::ridgeplot(enrich_obj, showCategory=n_category)
	invisible(plot_in_div(go_pp, fig_width = default_width))

	cat("\n\n**Rank plot**\n\n", 
		"The enrichment score evolution trought the ranked genes by fold-change are show for the top functional terms. 
		Y axis shows the enrichment score and the X axis shows the genes ranked by fold-change.\n\n"
	)
	go_pp <- enrichplot::gseaplot2(enrich_obj, geneSetID=ifelse(nrow(enrich_obj@result) < 4, seq(nrow(enrich_obj@result)), seq(4)), pvalue_table = TRUE) 
	invisible(plot_in_div(go_pp, fig_width = default_width))
	cat("\n \n")
```

---
author: "SysBioLab"
output:
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container {
        max-width: 90%;
    }

    .plot_real_size {
    	overflow: scroll;
  	}

    .plot_real_size img{
    	max-width: none;
    	max-height: none;
    }

    embed {
   		width: 800px;
  		height: 800px;
  }   
</style>

```{r config, include = FALSE}
 
# Load necessary packages
require(ggplot2)
require(knitr)

# Prepare recursive text

# Prepare all HPO reports container
out <- NULL
n_category <- 30
cl <- miRNA #this is for make compatible ora_enrichResult.Rmd 

```




# **DEgenes Hunter - miRNA `r miRNA` - Functional analysis report** 


## **General description**
Functional information for the `r miRNA` putative target genes.

Over representation analysis (ORA) was performed for the following functional systems:

* KEGG pathways
* GO:
	+ Biological Process
	+ Molecular Function
	+ Cellular Component
* Reactome pathways
* Custom nomenclature

The ORA method performs an enrichment of functional terms in putative target genes by a Fisher exact test.


```{r expression_profiles, echo = FALSE, message = FALSE, warning = FALSE, eval = !is.null(opt$RNAseq_folder) && !is.null(opt$RNAseq_folder) }

# Load genes expressions
targets <- unique(raw_data[raw_data$miRNA == miRNA, "Target_ID"])

targets_profiles <- RNAseq$normalized_counts[,colnames(RNAseq$normalized_counts) %in% targets]

# Prepare regular cluster info

miRNA_id <- unique(raw_data[raw_data$miRNA == miRNA, "miRNA_ID"])
miRNA_id <- miRNA_id[!is.na(miRNA_id)] # sometimes a miRNA has no mapping in multimiR
miRNA_mod <- miRNAseq$DH_results[miRNAseq$DH_results$gene_name == miRNA_id, "Cluster_ID"]
# save(list = ls(all.names = TRUE),file = "/home/bio267lab/proyectos/target_miRNA_2020/test_func.RData")
#   save(enr_go, file = "/home/bio267lab/proyectos/target_miRNA_2020/test_func.RData")

if (miRNA_strat == "d") {
  # save(miRNAseq,miRNA_id, file = "/mnt/scratch/users/bio_267_uma/josecordoba/NGS_projects/LaforaRNAseq/target_miRNA/test.RData")
  # q()
  # Count <- miRNAseq[['normalized_counts']][, miRNA_id]
  Count <- scale_data_matrix(as.matrix(miRNAseq[['normalized_counts']][, miRNA_id]), norm_by_col = TRUE)[,1]
  miRNA_profile <- as.data.frame(Count)
  miRNA_profile$Sample <- rownames(miRNA_profile)
  miRNA_profile$type <- paste0(miRNA_id, " normalized profile")


} else if (miRNA_strat == "h") {

  Count <- hub_miRNAs[,as.character(miRNA_mod)]
  miRNA_profile <- as.data.frame(Count)
  miRNA_profile$Sample <- rownames(miRNA_profile)
  miRNA_profile$type <- paste0("Hub gene of module ", miRNA_mod)

} else if (miRNA_strat == "E") {

  # miRNAseq$Eigengene <- as.data.frame(as.table(miRNAseq$Eigengene), stringsAsFactors = FALSE)
  # colnames(miRNAseq$Eigengene) <- c("Sample","Cluster_ID","Count") 
  # tgt_eigvalues_gnorm <- miRNAseq$Eigengene
  # tgt_eigvalues_gnorm$Count <- (tgt_eigvalues_gnorm$Count + 1) / 2 

  miRNA_profile <- tgt_eigvalues_gnorm[tgt_eigvalues_gnorm$Cluster_ID == miRNA_mod, c("Sample", "Count")]
  miRNA_profile$type <- paste0("Eigengene of module ", miRNA_mod)

}

# cl_profile <- cl_eigvalues_gnorm[cl_eigvalues_gnorm$Cluster_ID == ,]
# message(cl_profile)
# cl_profile <- cl_eigvalues_gnorm[grepl(paste(c("ME",{{cl}},"$"),collapse=""), cl_eigvalues_gnorm$Cluster_ID),]
# message(cl_profile)
# q()
# Plot
# targets_profiles <- RNAseq$normalized_counts[tgt_indx, ]
        # save(enr_ORA_expanded, file = "/mnt/scratch/users/bio_267_uma/josecordoba/NGS_projects/LaforaRNAseq/target_miRNA/test.RData")

# save(miRNA_profile, targets_profiles, file = "/mnt/scratch/users/bio_267_uma/josecordoba/NGS_projects/LaforaRNAseq/target_miRNA/test.RData")
# q()
targets_profiles <- as.data.frame(as.table(scale_data_matrix(data_matrix = targets_profiles, norm_by_col = TRUE)))
        colnames(targets_profiles) <- c("Sample","Gene","Count")
# save(targets, RNAseq,file = "/mnt/scratch/users/bio_267_uma/josecordoba/NGS_projects/LaforaRNAseq/target_miRNA/test.RData")
# q()
pp <- ggplot2::ggplot(mapping = ggplot2::aes(x = Sample)) + 
    ggplot2::geom_line(data = targets_profiles, alpha = 0.05, mapping = ggplot2::aes(y = Count, group = Gene)) +
    ggplot2::geom_line(data = miRNA_profile, mapping = ggplot2::aes(y = Count, group = type), color = "blue", linetype = "dashed", size = 0.5) +
    # scale_y_continuous(sec.axis = sec_axis(~.*2-1, name = "miRNA-targets correlation")) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 30, hjust = 1))
plot(pp)
```


```{r load_ORA_results, echo = FALSE, eval = exists("enrichments_ORA_expanded")}
# Per each category defined, perform enrichment
curr_out <- unlist(lapply(names(enrichments_ORA_expanded),function(cat){
  sec_text <- paste(c("## **",cat," enrichment results**\n"),collapse="") # Header
  sec_text <- c(sec_text,knitr::knit_expand("partial/ora_enrichResult.Rmd")) # Body
  return(sec_text)
}))
```
`r paste(knitr::knit(text = curr_out), collapse = '\n')`


```{r load_ORA_customResults, echo = FALSE}
curr_out <- ""

if(exists("custom_cls_ORA")){
  # Per each custom category defined, perform enrichment
  curr_out <- unlist(lapply(names(custom_cls_ORA),function(cat){
    sec_text <- paste(c("## **",basename(cat)," custom enrichment results**\n"),collapse="") # Header
    sec_text <- c(sec_text,knitr::knit_expand("partial/ora_customEnrichResult.Rmd")) # Body
    return(sec_text)
  }))
}
```
`r paste(knitr::knit(text = curr_out), collapse = '\n')`

Statistics about input results obtained from DEGenes Expression Hunter are:
```{r table_exp_flags_other, echo = FALSE}
  cl_table <- as.data.frame(table(DEGH_results$genes_tag))
  names(cl_table) <- c("Gene_tag","Genes")
  knitr::kable(cl_table)
```

## **Data quality control (QC)**

### **Correlation between samples:**

Here we show scatterplots comparing expression levels for all genes between the different samples, for i) all controls, ii) all treatment samples and iii) for all samples together.

These plots will only be produced when the total number of samples to compare within a group is less than or equal to 10.

```{r corr_plot_control, echo = FALSE, fig.height=12, fig.width=12, eval=length(index_control_cols) <= 10, results='asis', warning = FALSE}

cat("#### **Correlation between control samples:**
Replicates within the same group tend to have Pearson correlation coefficients >= 0.96. Lower values may indicate problems with the samples.
")

PerformanceAnalytics::chart.Correlation(raw_filter[index_control_cols], histogram=TRUE, pch=19, log="xy")

```

```{r corr_plot_treatment, echo = FALSE, fig.height=12, fig.width=12, eval=length(index_treatmn_cols) <= 10, results='asis', warning = FALSE}

cat("#### **Correlation between treatment samples:**
Replicates within the same group tend to have Pearson correlation coefficients >= 0.96. Lower values may indicate problems with the samples.
")

PerformanceAnalytics::chart.Correlation(raw_filter[index_treatmn_cols], histogram=TRUE, pch=19, log="xy")


```


```{r corr_plot_all , echo = FALSE, fig.height=12, fig.width=12, eval=length(index_control_cols) + length(index_treatmn_cols) <= 10, results='asis', warning = FALSE}


cat("#### **Correlation between samples: All vs all replicates**

Correlation coefficients tend to be slightly higher between replicates from the same group than between replicates from different groups. If this is not the case, it may indicate mislabelling or other potential issues.

")

PerformanceAnalytics::chart.Correlation(raw_filter, histogram=TRUE, pch=19, log="xy")

```

### **Heatmap and clustering showing correlation between replicates**
**BROWN: higher correlation; YELLOW: lower**

```{r heatmap_corr, eval=!"externalDEA" %in% names(all_counts_for_plotting), echo = FALSE, fig.height=12, fig.width=12}
col<- colorRampPalette(c("#FFFFCC", "#FD8D3C", "#800026"))(20)
res <- cor(raw_filter)
gplots::heatmap.2(x = res, col = col, symm = TRUE, margins = rep(max(nchar(colnames(res))), 2), scale = "none", trace = "none")
```

```{r config_PCA, echo = FALSE, results = "asis", warning = TRUE, message = TRUE}
pca_data <- PCA_res$all_genes$pca_data
dim_to_keep <- PCA_res$all_genes$dim_to_keep
dim_data <- PCA_res$all_genes$dim_data
dim_data_merged <- PCA_res$all_genes$dim_data_merged

```

```{r child=file.path(template_folder,"facto_miner.Rmd")}

```



### **Visualizing normalization results**

These boxplots show the distributions of count data before and after normalization (shown for normalization method `r names(all_data_normalized)[1]`):

#### Representation of cpm unfiltered data:

```{r density_distribution_cpm, echo = FALSE, warning=FALSE}
cpm_no_zeros <- cpm_table[! rowSums(cpm_table) == 0, ]
log2cpm <- log2(cpm_no_zeros + 1)
cpm_long <- reshape2::melt(log2cpm, measure.vars=colnames(log2cpm))

ggplot2::ggplot(cpm_long, ggplot2::aes(x = value, colour=Var2)) + ggplot2::geom_density() +
ggplot2::xlab(expression(log[2](cpm + 1))) +
ggplot2::geom_vline(xintercept = log2( opt$reads + 1))

```

#### Before normalization:

```{r density_distribution_raw, echo = FALSE, warning=FALSE}
log2raw_counts <- log2(raw_filter + 1)
raw_long <- reshape2::melt(log2raw_counts, measure.vars=colnames(log2raw_counts))

ggplot2::ggplot(raw_long, ggplot2::aes(x = value, colour=variable)) + ggplot2::geom_density() +
ggplot2::xlab(expression(log[2](count + 1)))

```

#### After normalization:

```{r density_distribution_norm, echo = FALSE, warning=FALSE}

log2norm_counts <- log2(all_data_normalized[["default"]] + 1)
norm_long <- reshape2::melt(log2norm_counts, measure.vars=colnames(log2norm_counts))

ggplot2::ggplot(norm_long, ggplot2::aes(x = value, colour=variable)) + ggplot2::geom_density() +
ggplot2::xlab(expression(log[2](count + 1)))

```

### **Count metrics by sample ranks**

```{r coverage_plot_head, echo = FALSE, warning = FALSE, results='asis'}
	if (!is.null(library_sizes)) {
		cat('#### **Sample rank versus total reads in raw FASTQ**')
	}else{
		cat('#### **Sample rank versus total counts**')
	}
```

```{r coverage_plot, echo = FALSE, warning = FALSE}
    if (!is.null(library_sizes)) {
        xtitle="Sample Rank (total reads)"
    } else {
        xtitle="Sample Rank (total counts)"
    }
    ggplot2::ggplot(coverage_df, ggplot2::aes(x = sample_rank, y = total_counts)) +
        ggplot2::geom_point(size = 3) +
        cowplot::theme_cowplot() +
        cowplot::background_grid() +
        ggplot2::labs(title = "Read Counts", x=xtitle, y = "Reads Counted") +
        ggplot2::ylim(c(0,NA)) +
        ggplot2::scale_color_brewer(palette="Dark2")
```
```{r coverage_plot_text, echo = FALSE, warning = FALSE, results='asis'}
	if (!is.null(library_sizes)) {
		cat('\n\n Sample rank is the position a sample holds after sorting by total reads')
	}else{
		cat('\n\n Sample rank is the position a sample holds after sorting by total counts')
	}
```

```{r coverage_plot_frac_head, eval = !is.null(coverage_df$counted_frac),  echo = FALSE, warning = FALSE, results='asis'}
	cat('#### **Fraction counts read sorted by rank**')
```

```{r coverage_plot_frac, eval = !is.null(coverage_df$counted_frac),  echo = FALSE, warning = FALSE}
    ggplot2::ggplot(coverage_df, ggplot2::aes(x = sample_rank, y = counted_frac)) +
        ggplot2::geom_point(size = 3) +
        cowplot::theme_cowplot() +
        cowplot::background_grid() +
        ggplot2::labs(title = "Read Counts", x="Sample Rank (total reads)", y = "% counts read") +
        ggplot2::ylim(c(0,NA)) +
        ggplot2::scale_color_brewer(palette="Dark2")
```

```{r coverage_plot_frac_text, eval = !is.null(coverage_df$counted_frac),  echo = FALSE, warning = FALSE, results='asis'}
	cat('Counted fraction is the percentage of reads that map to a gene (only available if total reads provided)')
```

### **Statistics of expressed genes**

```{r exp_genes_plot, echo = FALSE, warning = FALSE}
		colors <- c("Unique" = "blue", "Union" = "red", "Intersection" = "green")
		ggplot2::ggplot(data = exp_genes_df, ggplot2::aes(x = count_rank)) +
		    ggplot2::geom_line(ggplot2::aes(y = expressed_genes, color = "Unique")) +
		    ggplot2::geom_line(ggplot2::aes(y = union_expressed_genes, color = "Union")) +
		    ggplot2::geom_line(ggplot2::aes(y = inters_expressed_genes, color = "Intersection")) +
		    ggplot2::labs(x = "Sample rank",
		                  y = "Number of genes",
		                  color = "Legend") +
		    ggplot2::expand_limits(y=0) +
		    ggplot2::scale_color_manual(values = colors)
```

Samples are ranked by total expressed genes. Union of expressed genes represents the cumulative total expressed genes (sum of
all genes expressed in any sample up to current sample, expected to increase with sample rank). Intersection of expressed genes
represents the cumulative intersection of expressed genes (sum of genes expressed in all samples up to current sample, expected
to decrease with sample rank).

#### **Mean count distribution by filter**

```{r mean_counts_plot, echo = FALSE, warning = FALSE}
    ggplot2::ggplot(mean_counts_df, ggplot2::aes(x = counts, col = filter)) +
	ggplot2::geom_density(ggplot2::aes(y=..count..)) +
	ggplot2::scale_x_log10() +
	ggplot2::labs(x = "Mean counts per gene", y = "Frequency") +
    ggplot2::guides(col = ggplot2::guide_legend(title = NULL)) +
    ggplot2::scale_color_brewer(palette = "Paired") +
    cowplot::theme_cowplot() +
    ggplot2::theme(legend.position = "top",
          legend.justification="center",
          legend.background = ggplot2::element_rect(color = NA))
```

This plot represents the mean counts distribution per gene, classified by filters

### Gene counts variance distribution

Variance of gene counts across samples are represented. Genes with lower variance than selected threshold (dashed grey line) were filtered out. 

```{r variance_dist, echo = FALSE, eval=!grepl("F", opt$modules), warning = FALSE, message = FALSE}

gene_count_var <- data.frame(variance =var_filter[["variance_dis"]])

ggplot2::ggplot(gene_count_var, ggplot2::aes(x = variance))+
ggplot2::geom_density() + 
ggplot2::geom_vline(xintercept = var_filter[["thr"]], linetype="dashed", 
                color = "grey")
```

```{r all_norm_head, echo=FALSE, results='asis'}

cat("### **Samples differences by all counts normalized**:
		
All counts were normalizated by default (see options below) algorithm. This count were scaled by log10 and plotted in a heatmap.")

```

```{r heatmap_all_norm_plot), echo = FALSE, fig.height=12, fig.width=12}
	
		deseq_normalized_data <- all_data_normalized[['default']]
	if (nrow(all_data_normalized[['default']]) <= 50000) { 
		deseq_normalized_data_mtx <- log10(data.matrix(deseq_normalized_data) + 1)

		plot <- gplots::heatmap.2(deseq_normalized_data_mtx, col = col, labRow = FALSE, dendrogram = "column", trace = "none", srtCol=45,  adjCol = c(1,1))
	} else {
		cat("\n\nThis plot can not be generated because more than 50 000 genes have passed the filters and it can cause memory problems.\n\n")
	}

```

```{r top_n_expression, results='asis', echo=FALSE, warning=FALSE, message=FALSE}
	cat("\n\n### **Percentages of reads per sample mapping to the most highly expressed genes**\n\n")
	make_top_n_expression_table(count_data=raw_filter, n=5)

```

```{r "{{cl}}_{{cat}}_ora_enrichResults_config", include = FALSE}
# ###########################################
# ################# READ ME #################
# ###########################################
#
# This template assumes that the following variables exists
# => current_organism_info$Bioconductor_DB[1] :: organism database to translate genes
# => cl :: cluster to be checked
# => cat :: category to be checked
#
# This templates include the following chunk names:
# => <cl>_<cat>_ora_enrichmentsResult
# => <cl>_<cat>_oraEnrich
 
# Load necessary packages
# require(ggplot2)
# require(knitr)
# require(clusterProfiler)
# require(enrichplot)

# Prepare recursive text
chunks <- function(code, options = ""){paste(paste("```{r ",options,"}",sep = ""),code,"```",sep="\n")}
render <- function(text){paste(knitr::knit(text = paste(text,collapse = "\n")),collapse = "\n")}

# Prepare all HPO reports container
out <- NULL
 
# Load necessary functions
if("ExpHunterSuite" %in% (.packages())){ # Package mode, load private functions
	set_default_width <- ExpHunterSuite:::set_default_width
	get_categories <- ExpHunterSuite:::get_categories
	plot_in_div <- ExpHunterSuite:::plot_in_div
	get_genes <- ExpHunterSuite:::get_genes
	calc_width <- ExpHunterSuite:::calc_width
	calc_height <- ExpHunterSuite:::calc_height 
}

```



```{r "{{cl}}_{{cat}}_oraEnrich", results='asis', echo = FALSE, message=FALSE, warning=FALSE}

n_category <- 30
default_width <- set_default_width(enrich_obj = enrichments_ORA_expanded[["{{cat}}"]][["{{cl}}"]], showCategory=n_category)
if(length(get_categories(enrichments_ORA_expanded[["{{cat}}"]][["{{cl}}"]]))>=2){ # Needs, at least, two categories
	# Main label 
	cat("### **{{cat}} - Over Representation Analysis**\n",
		"The ORA method takes a group of significant genes and performs a Fisher's exact test for each term of the selected functional category.\n\n"
	)

	cat(
		"**Barplot**\n\n", 
		"The most highly signficant categories in ascending order, according to adjusted p-value. 
		The x-axis represents the number of significant genes found within the functional category.\n\n"
       )
	pp <- graphics::barplot(enrichments_ORA_expanded[["{{cat}}"]][["{{cl}}"]], showCategory=n_category)
	plot_in_div(pp, fig_width = default_width)




	cat("\n\n**Dotplot**\n\n",
        "The most highly signficant categories in descending in categories of gene ratio, defined as the proportion of significant genes that are found in the functional category.
         The x-axis represents the gene ratio and the dot size the number of genes associated with the functional category.\n\n"
       )
	pp <- enrichplot::dotplot(enrichments_ORA_expanded[["{{cat}}"]][["{{cl}}"]], showCategory=n_category)
	plot_in_div(pp, fig_width = default_width)



	cat("\n\n**Gene-Concept Network**\n\n", 
	  "The network connects the top functional categories (brown nodes) to their associated genes (grey or colored nodes).
 	   The size of the functional category nodes shows the number of connected genes.\n\n"
       )
	enrichments_ORA_symb <- DOSE::setReadable(enrichments_ORA_expanded[["{{cat}}"]][["{{cl}}"]], OrgDb = current_organism_info$Bioconductor_DB[1], keyType="ENTREZID")
	if(length(get_genes(enrichments_ORA_symb)) <= 150){
		pp <- enrichplot::cnetplot(enrichments_ORA_symb, showCategory=n_category, foldChange=geneList) +
			  ggplot2::scale_color_gradient2(name = "fold change", low = "#0000BF", high = "#bf0000", na.value = "#50EA55")
		plot_in_div(pp, fig_width = default_width)
	}else{
		cat("\n\nCnetplot is not readable because more than 150 genes has been enriched in this nomenclature, so is not printed\n\n")
	
	}

	cat("\n\n**Enrich Map plot**\n\n", 
	  "The top functional categories (nodes), connected if they share genes. Edge thickness represents the number of shared genes.
	  Nodes size represents the number of significant genes within the category.\n\n"
	)
	pp <- enrichplot::emapplot(enrichments_ORA_expanded[["{{cat}}"]][["{{cl}}"]], showCategory=n_category)
	plot_in_div(pp, fig_width = default_width)



	cat("\n\n**Heatplot**\n\n", 
	  "Significant genes (x-axis) and the functional categories in which they appear. Colour indicates log2 fold change.\n\n"
	)

	pp <- enrichplot::heatplot(enrichments_ORA_symb, showCategory=n_category, foldChange=geneList)
	width_size <- calc_width(enrichments_ORA_symb)
	height_size <- calc_height(enrichments_ORA_symb) 
	plot_in_div(pp, height_size, width_size)

	cat("\n\n**Upsetplot**\n\n", 
	  "Genes are clustered according to shared enriched categories. 
	  The y-axis shows the number of genes belonging to the different clusters (top) and categories to which they belong (bottom).\n\n"
	)
	pp <- enrichplot::upsetplot(enrichments_ORA_expanded[["{{cat}}"]][["{{cl}}"]], n=n_category)
	plot_in_div(pp, fig_width = default_width)
	cat('\n\nGenes has been enriched in less than 2 categories, so Upsetplot cannot be printed\n\n')

} else{
	# Main label 
	cat("### **{{cat}} - Over Representation Analysis**\n",
		"After execute ORA enrichment, **any significative term have been found**")
}
```

---
title: "The Expression Hunter Suite"
author: "James R. Perkins"
date: "`r format(Sys.Date(), '%m/%d/%Y')`"
output:
  rmarkdown::html_document:
    highlight: pygments
    toc: true
    fig_width: 5
vignette: >
  %\VignetteIndexEntry{The Expression Hunter Suite}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Version Info

 **R version**: `r R.version.string`

 **Bioconductor version**: `r BiocManager::version()`

 **Package version**: `r packageVersion("ExpHunterSuite")`

# Introduction

ExpHunterSuite implements a comprehensive protocol for the analysis of 
transcriptional data using established *R* packages and combining their 
results. It covers all key steps in DEG detection, CEG detection and functional 
analysis for RNA-seq data. It has been implemented as an R package 
containing functions that can be run interactively. In addition, it also 
contains scripts that wrap the functions and can be run directly from the 
command line.

#Standard Package Usage

In this section we will describe how the functions in ExpHunterSuite can be used
interactively or joined together in user-written scripts. We will also describe
how the output reports can be generated from this data.

## Differential Expression Analysis

The most basic use of the package is to perform differential expression (DE) 
gene analysis. ExpHunterSuite will, following some initial preprocessing, run 
the different methods, combine the results, and produce an output report, as 
well as a single output table containing the results of all of the methods used,
and their combined scores. The combined scores consist of the mean logFC value 
and the combined adjusted p-value (FDR) values, calculated by Fishers method.

To use ExpHunterSuite with only a single DE package, one can run the following 
command:

```{r DEA_one_pack, echo=TRUE, results="hide", message=FALSE, warning=FALSE}
library(ExpHunterSuite)
data(toc)
data(target)
degh_out_one_pack <- main_degenes_Hunter(raw=toc, 
                                         target=target,
                                         modules="D") # D for DESeq2
```
Where toc is a data.frame of aligned reads per samples and target is a 
data.frame relating each sample to its sample meta data. Here we include a 
minimal example of the target file whcih includes the samples (CTL/epm2a), 
the samples condition (Ctrl or Treat):

```{r input_data, echo=TRUE, results="as.is", message=FALSE, warning=FALSE}
head(toc)
head(target)
```


The files containing this data are contained within the extData package 
directory and can be accessed in the following manner. We will come back to 
them in the section on command line usage.

```{r input_files, echo=TRUE, results="as.is"}
system.file("extData", "table_of_counts.txt", package = "ExpHunterSuite")
system.file("extData", "target.txt", package = "ExpHunterSuite")
```

To use it with multiple packages, one can run the following command:

```{r DEA_multi_pack, echo=TRUE, results="hide", message=FALSE, warning=FALSE}
degh_out_multi_pack <- main_degenes_Hunter(raw=toc, 
                                    target=target, 
                                    modules="DEL") # D:DESeq2 E:EdgeR, L:limma
```

The output is a list, which includes, in slot *DE_all_genes* a data.frame 
containing, for each gene, 
logFC/p-values/adjusted p-values for the different DE methods implemented:

```{r standard_DEA_results, echo=TRUE}
head(degh_out_multi_pack$DE_all_genes)
```

It also contains information on whether the genes are considered to be DE, in 
the column *genes_tag* The tag *PREVALENT_DEGS* refers to those genes that are 
considered significant in at least n of the DE methods used
*POSSIBLE_DEGS* are those considered significant by at least one method. As 
such, *PREVALENT_DEGS* and *POSSIBLE_DEGS* will be the same 
when n = 1. N is controlled by the argument *minlibraries*.

To be considered significant for a given method, a gene must have an 
adjusted-pvalue < 0.05 and |logFC| > 1; these values are adjustable 
using the arguments *p_val_cutoff* and *lfc*.

The *genes_tag* columns includes the labels *NOT_DEGS* and *FILTERED_OUT* to 
refer to those genes not detected as DE by at least one DE method 
and those that do not pass the initial low-count filtering step, controlled by 
parameters *reads* and *minlibraries*.

There is another column, *combined_FDR* – which is POS/NEG depending on whether 
the combined adjusted p-value as described above is less 
than or equal to 0.05 (or whatever the value of the argument *p_val_cutoff* is).

### More complex model designs.

In order to control for specific variables (such as individuals in paired 
designs, potential confounding factors such as age, etc.), 

For example, if we consider our previous experiment, but add an extra column to 
the target, indicating different age groupings for the 
samples we obtain the following:

```{r standard_DEA_model_target, echo=TRUE, results="as.is"}
target_multi <- data.frame(target,
  age_group = c("adult", "child", "adult", "child", "adult", "adult", "child"))
target_multi
```

We may wish to control for the effects of age_group on the experiment.

This can be achieved using the argument *model_variables*. The variables given 
to this argument will be used in the model when calculating
differential expression between the Treat and Ctrl samples:

```{r standard_DEA_model_execute, results="hide", message=FALSE, warning=FALSE}
degh_out_model <- main_degenes_Hunter(raw=toc, target=target_multi, 
                                      modules="D", model_variables="age_group")
```

This works by using the variable age_group to create a linear model formula to 
be passed to the different DE methods (with the exception of NOISeq). 

The output has the same structure as the original analysis.

Custom model designs can also be specified in the *model_variables* argument, 
based on the R model syntax, see *help("formula")* for more details. If a 
custom formula is used, the *custom_model* argument must be set to true.

## Co-expression Analysis

Co-expression analysis is included via the R package Weighted correlation 
network analysis (WGCNA). The idea is to look for groups (modules) of genes 
showing correlated expression.
The groups can then be correlated with experimental factors, such as treatment 
vs. non treatment, as well as other groupings such as the age grouping mentioned
earlier, or numeric factors such as known values of metabolites related to the 
experiment. 

WGCNA is activated using by adding "W" to the *modules* argument. The traits to
 be correlated with the modules are specified using the *string_factors* and 
 *numeric_factors* options:

```{r standard_CEA, results="hide", message=FALSE, warning=FALSE}
degh_out_coexp <- main_degenes_Hunter(raw=toc, target=target_multi, 
                                      modules="DW", string_factors="age_group")
```

Please note that WGCNA requires a normalized expression matrix as input, as such
it cannot be run alone, it must be run alongside at least one DE method, which 
is specified with the argument *WGCNA_norm_method*.

## Functional Analysis - Expression Data

Functional analysis can be performed on the results of the DEG/WGCNA analysis to
look for over representation (enrichement) for groups of genes amongst the DEGs 
and/or modules of genes obtained.

For differentially expressed genes, the following code, which takes as input the 
output object of running degenes hunter for DE analysis, can be used.

Currently only overrepresentation analysis using the clusterProfiler package is
implemented. 

```{r standard_FA, results="as.is",  eval=FALSE}

fh_out_one_pack <- main_functional_hunter( #Perform enrichment analysis
        degh_out_one_pack,
        model_organism = 'Mouse', # Use specified organism database 
        enrich_dbs = c("MF", "BP", "CC", "Kegg", "Reactome"), # Enrichment analysis for GO, KEGG and Reactome
        enrich_methods = "o" # Use overepresentation analysis only
)

```

This will produce a list object as output. The members of this list are named:
final_main_params - this contains a list with full details of the parameters used

ORA - this list contains the results of the analysis. This is a named list of 
enrichResult objects, one for each gene annotation database used (e.g. GO BP,
Reactome, KEGG, or in the case of a custom gmt annotation file, the file name will 
be used).

DEGH_results_annot - this is a dataframe containing the results of the 
DEG/coexpression analysis.

When coexpression analysis has also been run using the main_degenes_Hunter function
the following code will 

```{r coexp_FA, results="as.is", eval=FALSE}

fh_out_coexp <- main_functional_hunter( # Perform enrichment analisys
        degh_out_coexp,
        model_organism = 'Mouse', # Use specified organism database 
        enrich_dbs = c("MF", "BP", "CC", "Kegg", "Reactome"), # Enrichment analysis for GO, KEGG and Reactome
        enrich_methods = "o" # Use overepresentation analysi only
)

```

## Functional Analysis - general

There is also a function to implement functional enrichment more generally, 
using as input a lists of genes instead of the output of the main_degenes_Hunter 
function. Each line in the file should contain a list of genes, with the format:
identifier<tab>gene1,gene2,...

```{r c2e, echo=TRUE,  eval=FALSE, results="hide"}
input_file <- system.file("extData", "cluster_genes.txt", package = "ExpHunterSuite")
print(readLines(input_file,n=2))

organisms_table <- get_organism_table()
current_organism_info <- organisms_table[rownames(organisms_table) %in% "Mouse",]
org_db <- get_org_db(current_organism_info)

enr_lists <- main_clusters_to_enrichment(input_file, org_db=org_db, 
    current_organism_info=current_organism_info, gene_keytype="ENSEMBL")
```

## Obtaining Reports

To obtain highly detailed html reports including multiple plots to visualize 
the data and the results of the different analysis methods, 
the following commands can be used for main_degenes_Hunter and main_functional_hun:

```{r write_reports, echo=TRUE, eval=FALSE, results="hide"}

write_expression_report(exp_results=degh_out_coexp)
write_enrich_files(func_results=fh_out_one_pack)
write_functional_report(hunter_results=degh_out_coexp, 
                        func_results=fh_out_coexp)
```

In all cases, the output folder for each report can be specified with the 
*output_files* option.

## Functional Analysis - special options for reports

There are several options when using GO with functional analysis whereby you can exploit
the hierarchical nature of this ontology to alter the results. 

simplify - simplify the merged enrichments by removing redundant terms, based on semantic
similarity, using the simplify() function from clusterProfiler. Uses the Wang method for
semantic similarity, with a cutoff of 0.7.

clean_parentals - remove parental GO terms from merged enrichments for each cluster/module. 
I.e. when the terms in a cluster or module have a parent-child relationship, the child term
is kept. Note that different clusters in the merged enrichment might have GOs terms that have 
a parent-child relationship.

There is also a summary method, which is accessed by adding "S" to the 
write_clusters_to_enrichment function or by adding a non-null "sim_thr" value to the 
main_functional_hunter funtion. This function is used to reduce the number of enriched 
GO terms by clustering similar GO terms and then, for each cluster, choosing either the most
signficant term or the most ancestral term, specified using the summary_common_name option.

group_results - The option  can be used to group categories with similar names in the emap plots
produced in the reports (the graphs that show the connections between terms) and in the plots
produced by write_functional_report with mode "P".

max_genes - the maximum number of genes to plot in the cnet plots - the plots in the reports 
that shows categories connected to each other via shared genes.



# Command-line Package Usage

The package also includes a number of scripts, in the folder *inst/scripts*, 
which can be used to run the above functions from the command line.

We recommend the user first creates a folder in which to install the 
ExpHunterSuite command line scripts, then copies the scripts there and make 
them command line accesible using these commands:

```bash
mkdir install_folder
Rscript -e "ExpHunterSuite::install_DEgenes_hunter('install_folder')"
export PATH=path_to_install_folder:$PATH
```

This export PATH can also be added to the .bashrc or .bash_profile files.

The user can then run the protocol from the command line with scripts such as 
the following, which will implement the functions and create the 
output reports, all from a single script.

```bash
degenes_Hunter.R -t $TARGET_FILE -i $TOC -o $EXP_RESULTS
functional_Hunter.R -i $EXP_RESULTS -m Organism -o FUNC_RESULTS
```

Full details of the arguments to give the the script can be found by running 
*degenes_Hunter.R -h* or *functional_Hunter.R -h*. More examples are given in 
the README file for this packet


## Usage

There are two ways for using DEgenes Hunter: as command line scripts or
as an R package. We will explain the procedure to perform both 1.
differential expression and 2. functional analysis using the command
line.

### 1\. Command line scripts for differential expression analysis.

Once installed, DEgenes Hunter performs the expression analysis from a
raw count table. For this, the user must first create a targets file,
including for each *sample* its name in the count table, it *treat*
condition (Treat or Ctrl).This file must contain this information
separated by
    tabs.

    Note: we recommend to use ENSEMBL identifiers for the functional analysis.

Here we include an example in which the targets file must include the
samples (CTL, TreatA and TreatB), the samples condition (Ctrl or Treat)
and to which age\_group they belong (adult or child). The correction
including additional factors (-v and -M) or co-expression analysis using
extra measures (-S and -C) require additional information that must be
included in targets file. Extra measures are named as *traits*. These
options use the *traits* column names as arguments.

| sample    | treat | age\_group |
| --------- | ----- | ---------- |
| CTL\_1    | Ctrl  | adult      |
| CTL\_2    | Ctrl  | child      |
| TreatA\_1 | Treat | adult      |
| TreatA\_2 | Treat | child      |
| TreatB\_1 | Treat | adult      |
| TreatB\_2 | Treat | adult      |

Once generated, the expression analysis can be performed using
degenes\_Hunter.R script. For this, we must call degenes\_Hunter.R and
give it the following input arguments.

Here we show an example of basic
usage:

``` bash
    degenes_Hunter.R -t path_to_target_file -i path_to_counts_table -o path_to_results
```

Mandatory arguments:

    -i | -t 
    (mandatory) Specify the path to the input counts/mapping table and to the targets file.
    
    -i Input read counts file.
    -t Targets file.

Differential expresion analysis arguments:

    -o Output path.
      (optional) Output folder. 
      Default = "./hunter_DE_results"
    -r any integer.
      (optional) Number of minimum mapped reads required in order to not be filtered out. Lesser number of reads are discarded. 
      0 = No filtering. 
      By default, reads less than 2 are discarded.
    -l any integer <= samples provided in the experiment.
      (optional) Minimum number of libraries that must have reads of a transcript in order to not to be filtered. 
      By default, minimum libraries required are 2.
    -p value between 0.01 and 0.1
      (optional) Adjusted p-value for the differential expression analysis. 
      Default = 0.05
    -f float
    (optional) Minimum log2 fold change in expression. Please, consider this is on a log2 scale, so a value of 1 would mean a 2 fold change.
      Default = 1.
    -q value between 0.95 and 0.99
      (optional) q value threshold for NOISeqBIO analysis. 
      Default = 0.95 (recommended)
    -a "BH" | "bonferroni" | "holm" | "hochberg" | "hommel" | "BY"
      (optional) adjust method for the combined nominal p-values. 
      By default the BH method is performed.
    -n name of your experiment
      (optional) Your experiment name. 
      Default = Experiment1
    -m D | E | L | N | W
      (optional) Differential expression packages to analyse data with.
      D = DESeq2, E = edgeR, L = limma, N = NOISeq (NOISeqBIO function within NOISeq package is used), W = WGCNA (this activates the co-expression analysis).
      Default = DELN.
    -c 1-4
      (optional) Minimum number of packages to consider a gene as 'Prevalent' DEG.
      Default = 4.
    -e External DEG data file.
      (optional) External file with pre-analysed DE data. Must consist of three columns containing p-value, logFC and FDR/p-adjust. Please, respect the columns order.
      Default = NULL.
    -v model variables
      (optional) Variables to include in the model. Must be comma separated and each variable must be a column in the target file.
      Default = NULL.
    -M model_text
      Text for this variable will be given directly to the model construction, overwriting the previous configuration.

Co-expression analysis arguments:

    -b Any integer.
      Maximum block size value to be given to the WGCNA blockwiseModules function as the maxBlockSize argument.
      Default = 5000.
    --WGCNA_norm_method NOISeq | DESeq2 | edgeR | limma
      Method used to normalized the table of counts for WGCNA. Must also run this method in the --modules argument. Raw counts are used if an empty string is given.
      Default=DESeq2
    --WGCNA_deepsplit 1-4
      This option controls the module building process and is defined as 1,2,3 and 4 values. 1 for rough clustering and 4 for accurate clustering.
      Default = 2.
    --WGCNA_min_genes_cluster integer
      Minimum number of genes to keep a cluster.
      Default = 20.
    --WGCNA_detectcutHeight 0 - 1 float
      Cut height to split modules.
      Default = 0.995.
    --WGCNA_mergecutHeight 0 - 1 float
      Value to merge two similar modules: Maximum dissimilarity (i.e., 1-correlation).
      Default = 0.25.
    -w
      Run WGCNA for treated only, control only, and both as 3 separate runs. Needed if using PCIT. If false, WGCNA runs once, on the table including treament and control.
    --WGCNA_blockwiseNetworkType unsigned | signed | signed hybrid
      NetworkType option to be passed to blockwiseModules function
      Default = signed.
    --WGCNA_blockwiseTOMType none | unsigned | signed | signed Nowick | unsigned 2 | signed 2 | signed Nowick 2
      TOMType option to be passed to blockwiseModules function. 
      Default = signed.
    -S comma sepparated text
      Columns in the target file to be used as categorical factors for the correlation analysis. If more than one to be used, should be comma separated
    -N comma sepparated text
      Columns in the target file to be used as numeric (continuous) factors for the correlation analysis. If more than one is specified, they must separated by commas.

Results files will be included in the
    output\_path:

    * DEG_report.html: file that encompass and summarizes all the information provided by the expression analysis. 
    * control_treatment.txt: file that includes information about the samples classification as determined in the targets file.
    * filtered_count_data.txt: filtered counts table used the differential expression analysis. Filtering has been performed according to -r, -l and -F options.
    * opt_input_values.txt: summary of the parameters used for the differential expression analysis

This folder will also include a Common\_results folder with a file
(table) with all methods used for the differential expression analysis
and their logFC, FDR and p-value calculated, the number of DEGs and
values for combined\_FDR, FDR\_labeling, mean\_logFCs and genes\_tag,
and results for the WGCNA analysis (if established): Cluster\_ID and
Cluster\_MM (MM: module membership).

In addition, the results folder will include subfolders generated in
accordance to the methods used for the differential expression analysis
(results\_DESeq2, Results\_edgeR, Results\_limma, Results\_NOISeq). All
these folder include two files, one with the normalized counts for all
samples and another one with the results given by each package.

In the case of performing the co-expression analysis with WGCNA, it will
be created a Results\_WGCNA folder including tables with correlations
between modules, genes and traits.

#### Non-canonical usage scenarios:

##### A. Differential expression corrected by extra factors

RNA-seq samples can have several additional attributes in addition to
the treatment/control status required to detect differential expression.
In the previous example, one of these variables is defined in the
targets file as age\_group. It is possible to include these attribute in
the model design as additional factors to control for. In these cases,
the differential expression model can be completed by adding the
additional argument -v <factors>, e.g. -v age\_group

An example of code for this analysis is the
follows:

``` bash
    degenes_Hunter.R -t path_to_target_file -i path_to_counts_table -v age_group -o path_to_results
```

##### B. Genes co-expression analysis with WGCNA

Co-expression analysis has been included in DEgenes Hunter to detect
gene modules with related biological functions. WGCNA can be activated
using -m “W” option. Additional traits can be correlated with module
mean profile (use -S for discrete columns and -N for continuous column).
Here we show an example of using WGCNA with restrictive
options:

``` bash
    degenes_Hunter.R -m WDELN -c 4 -f 1 --WGCNA_mergecutHeight 0.1 --WGCNA_min_genes_cluster 15 --WGCNA_detectcutHeight 0.995 -S age_group -t path_to_targets_file -i path_to_counts_table -o path_to_results
```

##### C. Analysing pre-normalized data with WGCNA

DEgenes Hunter requires a table of counts with integers. However, in
some situtations, the user may wish to reanalyse a dataset consisting of
non-integers, such as microarray data or pre-normalized data.

In this situation, the user can run WGCNA using the data in the table of
counts directly, without performing normalization. To do this, they must
run degenes\_hunter.R with the argument –WGCNA\_norm\_method equal to
“none” and the argument –modules must include “WL”, i.e. specify limma
is the only algorithm that will accept normalised values. However the DE
results will likely not make much
sense.

``` bash
    degenes_Hunter.R --WGCNA_norm_method none -m WL -c 1 -f 1 -S age_group -t path_to_targets_file -i path_to_normalized_table -o path_to_results
```

##### D. Using a pre-calculated list of DEA genes

In some cases, as well as using pre-normalized count data, we wish to
use a precalculated list of DE genes. This can be useful if we want to
run functional enrichment but not the DE analysis modules.

To do so, the user must provide a preeanalyzed list of DE analysis
results. This should consist of four columns, with the following names
and corresponding information: Entrez (or other gene id supported by
functional hunter), P.Value, logFC and adj.P.Val, in that order.

In such a scenario, the user can provide a target and counts file, in
which case the DE output reports will be generated using this
information. They can also choose not to provide them, in which case the
DE output report will be rather limited.

To run DEgenes Hunter using a pre-calculated gene list, the following
command can be
used:

``` bash
    degenes_Hunter.R  -m "F" -t path_to_targets_file -i path_to_normalized_table -e path_to_precalculated_deg_file -o path_to_results
```

##### E. Multifactorial (2x2) analysis to look for interactions between factors and effects in distinct groups

Currently only a 2x2 factorial design is possible for interactions, and
2xn for group effects.

In the case of a 2x2 design, if we consider the following experimental
design, similar to the one shown above:

| sample     | treat | age\_group |
| ---------- | ----- | ---------- |
| ad\_CTL\_1 | Ctrl  | ad         |
| ad\_CTL\_2 | Ctrl  | ad         |
| ad\_CTL\_3 | Ctrl  | ad         |
| ch\_CTL\_1 | Ctrl  | ch         |
| ch\_CTL\_2 | Ctrl  | ch         |
| ch\_CTL\_3 | Ctrl  | ch         |
| ad\_TRT\_1 | Treat | ad         |
| ad\_TRT\_2 | Treat | ad         |
| ad\_TRT\_3 | Treat | ad         |
| ch\_TRT\_1 | Treat | ch         |
| ch\_TRT\_2 | Treat | ch         |
| ch\_TRT\_3 | Treat | ch         |

We can look for an interaction between treatment and age\_group, the
effects of treatment in a specific age group, or the differences between
adults and children among untreated or treated samples.

Interaction can be thought of seeing whether the effect of treatment is
different between age groups.

The required contrast (i.e. interaction or effect) must be specified in
the following manner, via the flag –multifactorial:

“FactorA,FactorB:contrast”

In the case of an interaction between the factors, the contrast should
be specified as “interaction,baseA,baseB”, where baseA and baseB should
be the base levels for each factor. The resulting logFC values detected
by this contrast would represent \[numA\_numB - baseA\_numB\] -
\[numA\_baseB - baseA\_baseB\] with numA/B representing the non-base
levels for the factorA.

So, for our example, if one wished to see the interaction between
treatment and age\_group they should use the
following:

``` bash
degenes_Hunter.R -m "DEL" -i path_to_normalized_table -t path_to_targets_file -o results_2x2_interaction --multifactorial "treat,age_group:interaction,Ctrl,ch"
```

In the case of the effects of one factor in a group of samples specified
by another factor, the contrast should be specified in the form
"effect,baseA,groupB", where the baseA should be the level in FactorA
that should be used as the base for FC calculation, and groupB
represents the level in Factor B that is the group we are looking for
the change in.

So, for our example if one wished to see the effect of treatment in
children only, they should use the
following:

``` bash
degenes_Hunter.R -m "DEL" -i path_to_normalized_table -t path_to_targets_file -o results_2x2_effect_treat_ch --multifactorial "treat,age_group:effect,Ctrl,ch"
```

Similarly, if they wished to see the difference between age groups in
the ctrl samples, they should use the
following:

``` bash
degenes_Hunter.R -m "DEL" -i path_to_normalized_table -t path_to_targets_file -o results_2x2_effect_age_ctrl --multifactorial "age_group,treat:effect,ch,Ctrl"
```

Note FactorB in the effects contrast can have more than 2
groups.

##### F. Multifactorial nested (2x2xn and 2xnxn) analysis to look for interactions between factors and effects in distinct groups where the groups contain paired samples (e.g. the same patient/control before and after treatment)

*NOTE: this can only be used with DESeq2 for now*

In some cases we are interested in the differences that occur between
case and control samples that occur in one group of samples but not in
another, similar to the “interaction” contrast described previously,
however we have an added complication: the samples within each group are
paired. A typical example would be an experiment in which we have
patients and healthy controls, and we want to see how a treatment
affects patients, compared to controls. In this case, the control
samples are the untreated individuals, and the case samples are the
treated individuals. The groups are patients vs. controls. As such we
have a 2x2 interaction design. However, if the individuals are paired,
i.e. the treated and untreated samples come from the same individual, we
can add this to the experiment design so that it can be used in the DEG
detection analysis:

| sample        | treat | pat\_or\_hc | ind\_id |
| ------------- | ----- | ----------- | ------- |
| pat\_ctrl\_1  | Ctrl  | patient     | p1      |
| pat\_ctrl\_2  | Ctrl  | patient     | p2      |
| pat\_ctrl\_3  | Ctrl  | patient     | p3      |
| pat\_ctrl\_4  | Ctrl  | patient     | p4      |
| pat\_treat\_1 | Treat | patient     | p1      |
| pat\_treat\_2 | Treat | patient     | p2      |
| pat\_treat\_3 | Treat | patient     | p3      |
| pat\_treat\_4 | Treat | patient     | p4      |
| hc\_ctrl\_1   | Ctrl  | healthy     | h1      |
| hc\_ctrl\_2   | Ctrl  | healthy     | h2      |
| hc\_ctrl\_3   | Ctrl  | healthy     | h3      |
| hc\_ctrl\_4   | Ctrl  | healthy     | h4      |
| hc\_treat\_1  | Treat | healthy     | h1      |
| hc\_treat\_2  | Treat | healthy     | h2      |
| hc\_treat\_3  | Treat | healthy     | h3      |
| hc\_treat\_4  | Treat | healthy     | h4      |

As can be observed, the same individual appears twice in the design -
corresponding to samples before and after treatment (Ctrl and Treat can
of course also refer to e.g. sample from different tissues, etc.,)

This design has the advantage of allowing us to compare the change in
gene expression to the starting point of each individual, which may
vary.

In this case, we can look for differences between groups (like the
interaction shown above for the unpaired design) using the following
custom\_model:

``` bash
--multifactorial "pat_or_hc,ind_id:nested_int,Ctrl,patient"
```

And we can look for changes occurring in the patients group using:

``` bash
--multifactorial "pat_or_hc,ind_id:nested_effect,Ctrl,patient"
```

And we can look for changes occurring in the control subjects group
using:

``` bash
--multifactorial "pat_or_hc,ind_id:nested_effect,Ctrl,control"
```

Note that the changes must always be between Ctrl and Treat samples from
the Treatment column, for interactions there must only be two groups,
and each sample in each group must appear twice with the same patient
ID.

### 2\. Command line scripts for functional enrichment analysis.

To perform the functional enrichment analysis we will use the
functional\_Hunter.R script. This tool will use the hypergeometric test
to enrich genes in functions and pathways from GO, KEGG and Reactome.
Depending on the expression analysis performed, the DEgenes Hunter
functional analysis tool, functional\_Hunter.R, will execute different
enrichments:

    * When differential expression analysis is launched, all prevalent DEGs will be used to perform the functional enrichment. An html summary will be returned.
    * If co-expression analysis is set up, genes from each WGCNA independent module will be used to perform the functional enrichment. An html summary for each module and a global module enrichments summary will be returned.

Here we show an example of basic usage:

``` sh
    functional_Hunter.R -i path_to_results -m Organism -o path_to_output
```

Mandatory arguments:

    -i | -m | -o 
    (required) Specify the path to the degenes_Hunter.R output folder, the model organism to use and the path to the output folder.
    
    -i path
      Path to the ExpHunterSuite differential expression results.
    -m organism
      Ortologue species to be used as model organism to perform the functional analysis with. Run 'functional_Hunter.R' -L to display all available organisms.
    -o Output path.

Optional input arguments:

    -t input_ID
      Input gene IDs of counts table. Available IDs are: ENSEMBL (E), entrezgene (e), TAIR/Arabidopsis (T), Gene Names (G). 
      Default = E.
    -L 
      (optional) List all organisms provided.
    -a tab_file
      (optional) Path to file with own annotations for functional analysis.
    -f G | g | K | R
      Nomenclature and enrichment method(s) to use (topGO: G = GO | clusterProfiler: K = KEGG, g = GO, R = Reactome).
      Default = gKR.
    -G M | B | C
     Gene Ontology sub-classification to perform functional enrichment.
      M = Molecular Function (MF), B = Biological Process (BP), C = Celular Components (CC)
      Default = MBC.
    -A analysis_type
      Analysis performance (g = Gene Set Enrichment Analysis, o = Over Representation Analysis). 
      Default = go.
    -P float
      Enrichment p-value threshold. 
      Default = 0.1.
    -Q float
      Enrichment q-value threshold. 
      Default = 0.2.
    -c integer
      Cores to be used to parallelize clusters enrichments. 
      Default = 1
    -C files
      Files with custom functional annotation database (in GMT format) separated by commas (,)
    -r mode
       Flags to activate remote query from enrichments and genes translation. Use (b) to launch biomaRt translation; (k) to use KEGG remote database. Requires internet connection.
       Default = NULL
    -q 
    (optional) If indicated, biomaRt query is saved in an .RDS file.

#### DEgenes Hunter functional enrichment examples of use

Here we show an example of use for DEgenes Hunter functional enrichment,
changing some input parameters.

*Functional enrichment in GO biological processes (-G B) using topGO (-f
G) for H. sapiens (-m Human), using a overrepresentation analysis (-A
o). P-value threshold set to 0.1 (-P 0.1). ctrl\_vs\_mut is the input
folder with data from the functional expression analysis performed with
degenes\_Hunter.R (-i). Gene identifiers provided as entrez codes (-t
E). Execution parallelized using 6 cores (-c
    6).*

    functional_Hunter.R -f G -G B -A o -P 0.1 -m Human -i ctrl_vs_mut -t E -c 6 -o functional_enrichment

```{r, include = FALSE} 
# Load necessary packages
require(ggplot2)
require(knitr)
require(clusterProfiler)
```


```{r table_top_genes, echo = FALSE, results = 'asis'}
DEG_cols <- grep("_DEG$", names(DEGH_results), value=TRUE)
FC_cols <- grep("^logFC", names(DEGH_results), value=TRUE)

top_bot_genes_list <- lapply(DEG_cols, function(DEG_col) {
	method_name <- strsplit(DEG_col, "_")[[1]][1]
	FC_col <- paste("logFC", method_name, sep="_")
	DEGH_results_method <- DEGH_results[DEGH_results[,DEG_col] == TRUE,]
	FCs <- DEGH_results_method[, FC_col]

	ordered_FC_vals <- order(DEGH_results_method[, FC_col])
	index_bottom_15 <- intersect(head(ordered_FC_vals, 15), which(FCs<0))
	index_top_15 <- intersect(tail(ordered_FC_vals, 15), which(FCs>0))
	top_15_IDs <- row.names(DEGH_results_method[index_top_15, ])
	bot_15_IDs <- row.names(DEGH_results_method[index_bottom_15, ])
	return(list(top_15_IDs, bot_15_IDs))
})

all_top_IDs <- unique(unlist(sapply(top_bot_genes_list, "[", 1)))
all_bot_IDs <- unique(unlist(sapply(top_bot_genes_list, "[", 2)))
topgenes <- DEGH_results[all_top_IDs, c("input_IDs", FC_cols, "mean_logFCs", "SYMBOL")]
botgenes <- DEGH_results[all_bot_IDs, c("input_IDs", FC_cols, "mean_logFCs", "SYMBOL")]

if(exists("annot_table")) {
    if(!is.null(annot_table)) {
	    topgenes$original_ids <- row.names(topgenes)
	    botgenes$original_ids <- row.names(botgenes)
    }
}

# Message
cat("## **Top genes**\n")
cat(paste(c("Table of signifcant genes. Variables taken into account are being shown into table (except gene symbols/ID). Top of positive (",nrow(topgenes)," items) and negative (",nrow(botgenes)," items) are being shown in two different tables\n"), collapse = ""))
```

```{r render_topgenes, echo = FALSE, results = 'asis'}
cat("\n### **Top positive**\n")
# Render
DT::datatable(topgenes, filter = 'top', rownames = FALSE, extensions = c('Buttons','ColReorder'),
                    options = list(
                    	colReorder = TRUE,
                    	dom = 'lftBip',
                        buttons = c('copy', 'csv', 'excel')
                    ))
```
```{r render_botgenes, echo = FALSE, results = 'asis'}
cat("\n### **Top negative**\n")
# Render
DT::datatable(botgenes, filter = 'top', rownames = FALSE, extensions = c('Buttons','ColReorder'),
                    options = list(
                    	colReorder = TRUE,
                    	dom = 'lftBip',
                        buttons = c('copy', 'csv', 'excel')
                    ))

```

---
title: "ExpHunterSuite: Differential Expression Report"
output:
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container {
        max-width: 90%;

    }

    .plot_real_size {
    	overflow: scroll;
  		max-height: 600px;
  	}

    .plot_real_size img{
    	max-width: none;
    	max-height: none;
    }

    embed {
   
  }   
</style>


```{r child=file.path(template_folder,"expression_qc.Rmd")}
```

```{r child=file.path(template_folder,"dea_results.Rmd"), eval = any(grepl("[DENL]",opt$modules))}
``` 

```{r child=file.path(template_folder,"WGCNA_results.Rmd"), eval = grepl("W", opt$modules)}
``` 
 
```{r child=file.path(template_folder,"diffcoexp_results.Rmd"), eval = grepl("X", opt$modules)}
``` 
 
```{r child=file.path(template_folder,"PCIT_results.Rmd"), eval = grepl("P", opt$modules)}
``` 

### **Values of options used to run DEGenesHunter**
First column contains the option names; second column contains the given values for each option in this run.
```{r opt_vals, echo = FALSE, warning = FALSE}
knitr::kable(cbind(opt))
```

```{r  "{{funsys}}_compareCluster", echo=TRUE, results='asis', message=FALSE, echo=FALSE}
default_width <- set_default_width(enrich_obj = enrich_obj, showCategory = n_category)

cat(paste0("## **", funsys, " Over Representation Analysis**"), "\n",
	"The ORA method takes a group of significative DEGs (only DEGs, upregulated DEGs or downregulated DEGs) and performs a hypergeometric test for each term of the selected functional category.\n\n")

plot_enrResult_DT(enrich_obj)


cat("\n\n**Dotplot**\n\n",
        "The most highly signficant categories in descending in categories of gene ratio, defined as the proportion of significant genes that are found in the functional category.
         The x-axis represents the gene ratio and the dot size the number of genes associated with the functional category.\n\n"
)
pp <- enrichplot::dotplot(enrich_obj, showCategory=showCategories) 

pp <- set_standard_size(pp)
invisible(plot_in_div(pp, calc_height(enrich_obj), calc_width(enrich_obj)))

cat("\n\n**Heatplot**\n\n", 
	  "Significant genes (x-axis) and the functional categories in which they appear." 
)
width_size <- calc_width(enrich_obj)
height_size <- calc_height(enrich_obj) 
pp <- clusters_heatplot(enrich_obj@compareClusterResult) 
invisible(plot_in_div(pp, height_size, width_size))

cat("\n\n**Gene-Concept Network**\n\n", 
	  "The network connects the top functional categories (brown nodes) to their associated genes (grey or colored nodes).
 	   The size of the functional category nodes shows the number of connected genes.\n\n"
)
if (nrow(as.data.frame(enrich_obj)) > 2) {

 cnet_pp <- enrichplot::cnetplot(enrich_obj, showCategory=showCategories) 
 invisible(plot_in_div(cnet_pp, fig_width = 15, fig_height = 15))
} else {
	cat("\nThis graph can not be plotted because fewer than two functional categories were enriched\n")
}


cat("\n\n**Enrich Map plot**\n\n", 
  "The top functional categories (nodes), connected if they share genes. Edge thickness represents the number of shared genes.
	  Nodes size represents the number of significant genes within the category. Pies proportion represent the proportion of genes belonging to each cluster.\n\n"
)


if (nrow(as.data.frame(enrich_obj)) > 2) {
	if(group_results == TRUE)
	  pp <- enrichplot::emapplot(enrich_obj, showCategory=showCategories, pie.params = list(pie ="Count"),  layout.params = list(layout = "nicely"), 
                  shadowtext = FALSE, node_label = "group",  edge.params = list(min = 0.0000000000000001), group_category = TRUE, 
                  nCluster = min(floor(nrow(enrich_obj)/7), 20), nWords = 6, repel = TRUE)
	else 
	  pp <- enrichplot::emapplot(enrich_obj, showCategory= showCategories, pie.params = list(pie ="Count"), layout.params = list(layout = "nicely"), 
                  shadowtext = FALSE, repel = TRUE, edge.params = list(min = 0.0000000000000001))
	invisible(plot_in_div(pp, fig_width = calc_width(enrich_obj)))

} else {
	cat("\nThis graph can not be plotted because fewer than two functional categories were enriched\n")
}

if (nrow(as.data.frame(enrich_obj)) > 1000000000000) {

cat("\n\n**Cluster concept network plot**\n\n", 
  "The top functional categories (nodes), connected if they are shared among clusters. Edge thickness represents the number of shared genes.
	  Nodes size represents the number of clusters sharing the functional category. Pies proportion represent the proportion of clusters with that enriched category.\n\n"
)


# clnetplot_pp <- clnetplot(enrich_obj, showCategory=showCategories) 

# invisible(plot_in_div(clnetplot_pp, fig_width = 15, fig_height = 15))
} else {
	cat("\nThis graph can not be plotted because fewer than two functional categories were enriched\n")
}
```

```{r WGCNA_conditional_block_head, eval=grepl("W", opt$modules), echo=FALSE, results='asis'}
cat("## **WGCNA Results**

WGCNA was run to look for modules (clusters) of coexpressed genes. These modules were then compared with the sample factors to look for correlation. 
If no sample factors were specified, this comparison was performed with treatment/control labels.

The following graphic shows the power value chosen for building clusters. The power is chosen by looking at the characteristics of the network produced."

)
```

```{r WGCNA_conditional_block_plot_power_choice, eval=grepl("W", opt$modules), echo=FALSE, warning=FALSE, message=FALSE, fig.height=10, fig.width=15}
    replayPlot(results_WGCNA[['plot_objects']][['power_threshold_effects']])
```



```{r WGCNA_conditional_block_head_cluster_membership, eval=grepl("W", opt$modules), echo=FALSE, results='asis'}
cat("In total there were ", max(unique(results_WGCNA$gene_cluster_info$Cluster_ID)) , "clusters. 
	The following plot shows the number of genes per cluster:"
)
```

```{r WGCNA_conditional_block_plot_cluster_membership, eval=grepl("W", opt$modules), echo=FALSE, warning=FALSE, message=FALSE, fig.height=10, fig.width=15}
	genes_per_cluster <- table(results_WGCNA$gene_cluster_info$Cluster_ID)
	col_clusters_barplot <- results_WGCNA[['plot_objects']][['sorted_colours']]
	names(col_clusters_barplot) <- names(genes_per_cluster)
	ggplot2::ggplot(as.data.frame(genes_per_cluster), ggplot2::aes(Var1, Freq, fill=Var1)) + 
    ggplot2::geom_bar(stat="identity") + ggplot2::scale_fill_manual(name = "Cluster ID", values=col_clusters_barplot) + 
    ggplot2::labs(x = "Cluster ID (0 = unclustered)", y = "Frequency", main="Number of Genes per Cluster") + ggplot2::theme(legend.position = "none")
```

```{r WGCNA_conditional_dist_cluster_membership, eval=grepl("W", opt$modules), echo=FALSE, results='asis'}
cat("### Module Membership distribution\n"
)
```

```{r WGCNA_dist_cluster_membership, eval=grepl("W", opt$modules), echo=FALSE, warning=FALSE, message=FALSE, fig.height=10, fig.width=15}
		"%>%" <- magrittr::"%>%"

	mm_dist <- DE_all_genes[DE_all_genes$Cluster_ID != 0, c("Cluster_MM", "Cluster_ID")]
	mm_dist$Cluster_ID <- as.character(mm_dist$Cluster_ID)
	mm_summary <- mm_dist %>% 
					dplyr::group_by(Cluster_ID) %>% 
					dplyr::summarise(MM_sd = sd(Cluster_MM),
							MM_mean = mean(Cluster_MM),
							gene_count = length(Cluster_MM))
	mm_summary <- as.data.frame(mm_summary)				
	mm_tables_equivalence <- match(mm_dist$Cluster_ID, mm_summary$Cluster_ID)
	mm_dist <- cbind(mm_dist, mm_summary[mm_tables_equivalence, c("MM_sd", "MM_mean", "gene_count")])
	mm_dist <- mm_dist[!is.na(mm_dist$Cluster_ID),]		
	mm_dist$Cluster_ID <- factor(mm_dist$Cluster_ID, levels = as.character(sort(unique(as.numeric(mm_dist$Cluster_ID)))))
	# mm_dist<- mm_dist[order(as.numeric(mm_dist$Cluster_ID)),]	
	gg <- ggplot2::ggplot(mm_dist, ggplot2::aes(x = Cluster_ID, group = Cluster_ID, y = Cluster_MM, fill = gene_count)) + 
			ggplot2::geom_violin(trim = TRUE, scale = "width") + ggplot2::ylim(c(-0.00001, 1.00001)) +
  			ggplot2::geom_boxplot(width=0.05)+  
  			ggplot2::scale_fill_gradient(low = "#9FCBFC", high =  "#0067DC")
	gg

```



```{r WGCNA_conditional_block_plot_factors, eval=grepl("W", opt$modules), echo=FALSE, warning=FALSE, message=FALSE, fig.height=10, fig.width=15, results = 'asis'}
cat("### **Cluster assignment vs lower module membership (MM) **")

cat("\nThis plot shows, for each gene, the cluster ID ascertained by WGCNA, vs. the cluster whose eigen gene has the highest correlation (module membership/MM).")
	
	plot_counter <- plot_in_div(results_WGCNA[['plot_objects']][['cluster_vs_MM']], counter = plot_counter)
	cluster_count <- get_clusters_count(results_WGCNA)
	features_count <- get_features_count(results_WGCNA)
	
cat("### **Cluster vs. factors correlation**")

cat("\nThe following plots show the correlation between the different modules and specified factors. This is done using eigengenes, 
which can be broadly thought of as the average expression pattern for the genes in a given cluster. MEn refers to the eigengene for cluster n.\n")

cat("\nThis plot shows the correlation between clusters (eigen genes) and factors directly.")

	pp <- gg_heatmap(data =  results_WGCNA[['plot_objects']][['trait_vs_module']], 
						x_axis = "Trait",
						y_axis= "Module",
						fill = "Correlation",
						text_plot= "Text_correlation")
	plot_counter <- plot_in_div(pp, fig_height= (5 + (0.8 * cluster_count)), fig_width=(3 + (features_count * 0.8)), cex = 0.5, counter = plot_counter)

cat("### **WGCNA Eigen values clustering**")

cat("\nWGCNA dendogram showing distances between these eigengenes along with the factors. Distances has been calculated using **signed** correlation so more near elements, more positive correlation between elements.") 
	
	plot_counter <- plot_in_div(WGCNA::plotEigengeneNetworks(results_WGCNA[['plot_objects']][['trait_and_module']], "Eigengene dendrogram", marDendro = c(0,4,2,0), plotHeatmaps = FALSE),
		fig_width= 4+ calc_width_clusters((features_count + cluster_count), multiplier = 0.15),  min_size = 5, counter = plot_counter)

cat("### **Eigen values clustering (Absolute correlation)**")

cat("\nWGCNA like dendogram showing distances between these eigengenes along with the factors. Distances has been calculated using **absolute** correlation so more near elements, more absolute correlation between elements.")

	corME <- cor(results_WGCNA[['plot_objects']][['trait_and_module']], use="p")
	disME_abs <- as.dist(1-abs(corME))
	clust_abs = fastcluster::hclust(disME_abs, method = "average")
	
	plot_counter <- plot_in_div(plot(clust_abs, sub="", xlab="", ylab="", ylim=c(0,1)),
		fig_width= 4+ calc_width_clusters((features_count + cluster_count), multiplier = 0.15),  min_size = 5, counter = plot_counter)


corME2 <- corME
rownames(corME2) <- gsub("^ME", "", rownames(corME2))
colnames(corME2) <- gsub("^ME", "", colnames(corME2))

mod_traits_net <- corM2igraph(corME2, cor_abs_thr = 0.8)

mod_net <- igraph::graph_from_data_frame(d=mod_traits_net$edges, vertices = mod_traits_net$nodes, directed = FALSE)
# layout <- igraph::layout_with_fr(mod_net, weights = mod_traits_net$edges$corr * 3 )

cat("\n### **Correlation network between modules and factors**\n\n")

if (nrow(mod_traits_net$nodes < 2)){

	cat("This plot shows modules (black) and factors (green) as nodes. Correlations coefficients over 0.8 (red) and under -0.8 (blue) are represented as edges\n")
	"%>%" <- magrittr::"%>%"
	options(ggrepel.max.overlaps = Inf)
	tg <- tidygraph::as_tbl_graph(mod_net) %>% tidygraph::activate(nodes) %>% dplyr::mutate(label=name)
	igraph::E(tg)$weight <- igraph::E(tg)$corr
	test_graph <- tg %>% 
	ggraph::ggraph(layout = "stress")+  
	ggraph::geom_edge_fan(ggplot2::aes(colour = ifelse(mod_traits_net$edges$corr_type == "corr", "#FF7272","#9595FF"))) +
	ggraph::geom_node_text(ggplot2::aes(label = name), 
	       repel = TRUE, point.padding  = grid::unit(0.2, "lines"), 
	       colour=ifelse(grepl("_",mod_traits_net$nodes$nodes), "#43CF4C", "black"))+
	ggraph::theme_graph(background = 'white')


	plot_counter <- plot_in_div(test_graph)
	options(ggrepel.max.overlaps = 10)

} else {
	cat("Only 2 modules or factors correlates with higher ansolute correlation coefficient than 0.85.")
}


cat("\n### **Correlation between all clusters and factors**\n")

	correlation <- cor(results_WGCNA[['plot_objects']][['trait_and_module']], use = "p")
	correlation_df <- as.data.frame(correlation)
	pp <- gg_heatmap(data_table = correlation_df, 
    						fill = "Correlation",
    						labs = FALSE,
    						input = "matrix",
    						x_angle = 25)
	cor_elements <- features_count + cluster_count
	plot_counter <- plot_in_div(pp,	5 + (cor_elements *0.2), 5 + (cor_elements *0.2), counter = plot_counter)

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics_functions.R
\name{get_stats_from_cm}
\alias{get_stats_from_cm}
\title{Obtain full set of measures from a confusion matrix}
\usage{
get_stats_from_cm(cm)
}
\arguments{
\item{cm}{confusion matrix}
}
\value{
a data frame with several measures
}
\description{
Obtain full set of measures from a confusion matrix
}
\examples{
cm <- matrix(c(4,3,2,1), ncol = 2)
rownames(cm) <- c(TRUE,FALSE)
colnames(cm) <- c(TRUE,FALSE)
stats <- get_stats_from_cm(cm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_analysis_library.R
\name{scale_data_matrix}
\alias{scale_data_matrix}
\title{Scale a matrix using minimum-maximum method}
\usage{
scale_data_matrix(data_matrix, norm_by_col = FALSE)
}
\arguments{
\item{data_matrix}{to be scaled}

\item{norm_by_col}{boolean flag: if true scaling will be performed
by columns intead of by rows. Default: FALSE}
}
\value{
scaled matrix
}
\description{
Scale a matrix using minimum-maximum method
}
\keyword{method}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics_functions.R
\name{get_stats}
\alias{get_stats}
\title{Function to calculate full set of measures from a confusion data frame}
\usage{
get_stats(df)
}
\arguments{
\item{df}{confusion dataframe}
}
\value{
a data frame with several measures
}
\description{
Function to calculate full set of measures from a confusion data frame
}
\examples{
df <- data.frame(TP = 4, FP = 3, TN = 2, FN = 1)
stats <- get_stats(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/installFunc.R
\name{install_DEgenes_hunter}
\alias{install_DEgenes_hunter}
\title{Hunter installation Function}
\usage{
install_DEgenes_hunter(to)
}
\arguments{
\item{to}{Path to folder in which copy Hunter scripts}
}
\value{
void
}
\description{
This function allows you to install scripts.
}
\examples{
install_DEgenes_hunter("./")
}
\keyword{installation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_parse_pseudocounts.R
\name{parse_pseudocounts}
\alias{parse_pseudocounts}
\title{main_parse_pseudocounts
Functions that loads pseudommappers results and complite it into a table of counts}
\usage{
parse_pseudocounts(input_folder, mapper, output_type, annotation_file)
}
\arguments{
\item{input_folder}{folder with pseudocounts files}

\item{mapper}{string pseudomapping program}

\item{output_type}{string indicate i fthe output must be 'G' genes, 'T' transcripts or both}

\item{annotation_file}{gtf or gff path, requested for parsin salmon results}
}
\description{
main_parse_pseudocounts
Functions that loads pseudommappers results and complite it into a table of counts
}
\examples{
\dontrun{
	parse_pseudocounts(input_folder = "Path_to_folder",
						mapper = "salmon",
						annotation_file = "path_to_gtf")	
 }
}
\keyword{method}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_analysis_library.R
\name{multienricher_topGO}
\alias{multienricher_topGO}
\title{Perform topGO enrichment analysis of a list of genes}
\usage{
multienricher_topGO(
  all_funsys,
  genes_list,
  universe = NULL,
  organism_info,
  gene_id = "entrez",
  algorithm = "classic",
  statistic = "fisher",
  nodeSize = 5,
  task_size = 1,
  workers = 1
)
}
\arguments{
\item{all_funsys}{vector of funsys to use (e.g. MF, Reatcome)}

\item{genes_list}{vector of genes or list of vectors, to be enriched}

\item{universe}{background of genes to use as universe}

\item{organism_info}{from the annotation table: infor on db names etc}

\item{gene_id}{what identifier do the genes use - must be entrez for}

\item{algorithm}{alrgorithm for decting enriched functions - see topGO docs}

\item{statistic}{method for detecting enriched functions}

\item{nodeSize}{related to the creation of the GOdata object}

\item{task_size}{for parallelization}

\item{workers}{for parallelization}
}
\description{
Perform topGO enrichment analysis of a list of genes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics_functions.R
\name{extract_from_cm}
\alias{extract_from_cm}
\title{Extract value from confusion matrix}
\usage{
extract_from_cm(cm, real, pred, default = 0)
}
\arguments{
\item{cm}{confusion matrix}

\item{real}{slot name}

\item{pred}{slot name}

\item{default}{to return if value is not available}
}
\value{
value required or default if it is not available
}
\description{
Extract value from confusion matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{target}
\alias{target}
\title{Target data containing sample information for the toc data}
\format{
A data.frame of sample information. Rows are samples, columns are
different experimental variables/factors
}
\usage{
data(target)
}
\description{
Target data containing sample information for the toc data
}
\examples{
data(target)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics_functions.R
\name{ppv}
\alias{ppv}
\title{Calculates Positive Predictive Value from a measures dataframe}
\usage{
ppv(df)
}
\arguments{
\item{df}{dataframe with measures}
}
\value{
calculated metric
}
\description{
Calculates Positive Predictive Value from a measures dataframe
}
\examples{
df <- data.frame(TP = 4, FP = 3, TN = 2, FN = 1)
df$Precision <- ppv(df)
}
\keyword{metrics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_rtable2measures.R
\name{rtable2measures}
\alias{rtable2measures}
\title{Calculates emasures from result tables}
\usage{
rtable2measures(
  htfile,
  realprediction,
  aucfile = NULL,
  experiment = NULL,
  logFC_threshold = 1,
  pval_threshold = 0.01
)
}
\arguments{
\item{htfile}{hunter table file}

\item{realprediction}{real predictions file}

\item{aucfile}{output AUC plot file}

\item{experiment}{experiment info file to be loaded}

\item{logFC_threshold}{logFC threshold}

\item{pval_threshold}{p-value threshold}
}
\value{
metrics table
}
\description{
Calculates emasures from result tables
}
\examples{
htfile <- system.file("extData", 
"hunter_results_table.txt", package = "ExpHunterSuite")
realprediction <- system.file("extData", 
"ground_truth_DE.txt", package = "ExpHunterSuite")
rtable2measures(htfile, realprediction)
}
\keyword{metrics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/general_functions.R
\name{parallel_list}
\alias{parallel_list}
\title{Parallelize function execution in list with multiple elements}
\usage{
parallel_list(X, FUNC, workers = 2, task_size = 1, ...)
}
\arguments{
\item{X}{R list}

\item{FUNC}{Function to execute in each element of the list}

\item{workers}{Number of process to use in parallel execution}

\item{task_size}{Number of list elements to be executed on each
BiocParallel task}

\item{...}{Additional arguments to BiocParallel or to FUNC}
}
\value{
list
}
\description{
Parallelize function execution in list with multiple elements
}
\keyword{performance}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_analysis_library.R
\name{get_organism_table}
\alias{get_organism_table}
\title{Table with information abaut all organism available}
\usage{
get_organism_table(file = NULL)
}
\arguments{
\item{file}{to be loaded. If none given, internal organism table loaded}
}
\value{
organism table
}
\description{
Table with information abaut all organism available
}
\examples{
ot <- get_organism_table()
}
\keyword{method}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_analysis_library.R
\name{trycatch_pairwise_termsim}
\alias{trycatch_pairwise_termsim}
\title{if pairwise_termsim throws an error, remove dup cat descriptions
Only seems to be a problem with Reactome - to study further
Further investigation re: number of cats also required}
\usage{
trycatch_pairwise_termsim(enr)
}
\arguments{
\item{enr}{enrichment object to be studied}
}
\value{
enrichment object after add termsim info
}
\description{
if pairwise_termsim throws an error, remove dup cat descriptions
Only seems to be a problem with Reactome - to study further
Further investigation re: number of cats also required
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_handling.R
\name{download_latest_kegg_db}
\alias{download_latest_kegg_db}
\title{download kegg db for a given organism}
\usage{
download_latest_kegg_db(current_organism_info, file)
}
\arguments{
\item{current_organism_info}{organism info for which to download the file}

\item{file}{where to save the file}
}
\description{
download kegg db for a given organism
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_report.R
\name{write_functional_report}
\alias{write_functional_report}
\title{Write Main DEgenes Hunter functional report
This function allows you to report the Functional analysis.}
\usage{
write_functional_report(
  hunter_results,
  func_results,
  output_files = getwd(),
  fc_colname = "mean_logFCs",
  organisms_table = NULL,
  template_folder = file.path(find.package("ExpHunterSuite"), "templates"),
  cores = 2,
  task_size = 1,
  report = "fc",
  showCategories = 30,
  group_results = FALSE,
  max_genes = 200,
  corr_threshold = 0.8,
  pvalcutoff = 0.05
)
}
\arguments{
\item{hunter_results}{DEG analysis results}

\item{func_results}{functional results}

\item{output_files}{output folder.}

\item{fc_colname}{(OPTIONAL) main logFC colname (into hunter_results
dataframe)}

\item{organisms_table}{(OPTIONAL) configuration table for given organism.
Use see get_organism_table()}

\item{template_folder}{(OPTIONAL) RMD templates folder}

\item{cores}{(OPTIONAL) cores for parallel features}

\item{task_size}{number of elements per packages used}

\item{report}{string with reports to be written. Allowed: clusters (c)}

\item{showCategories}{number of categories in the figures (per cluster)}

\item{group_results}{experimental - whether to group results in the emap plot
and functional (f). Default = "fc"}

\item{max_genes}{maximum number of genes to plot in cnet plot}

\item{corr_threshold}{minimun module eigengene-trait vector absolute Pearson R value}

\item{pvalcutoff}{maximum module eigengene-trait vector correlation P value}
}
\value{
void
}
\description{
Write Main DEgenes Hunter functional report
This function allows you to report the Functional analysis.
}
\examples{
# Load func and DE results
data(degh_output)
func_results <- list() 
func_results <- main_functional_hunter(degh_output, "Mouse")
write_functional_report(degh_output, func_results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_handling.R
\name{write_enrich_files}
\alias{write_enrich_files}
\title{Write enrichment files related to functional_hunter results list}
\usage{
write_enrich_files(func_results, output_path = getwd())
}
\arguments{
\item{func_results}{functional enrichment results}

\item{output_path}{output folder path}
}
\value{
void
}
\description{
Write enrichment files related to functional_hunter results list
}
\examples{
# Use a @functional_hunter or @multienricher result object to 
# create real files
write_enrich_files(list(),"./")
}
\keyword{export}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_handling.R
\name{get_kegg_db_path}
\alias{get_kegg_db_path}
\title{get path for KEGG db for downloading/usage}
\usage{
get_kegg_db_path(kegg_data_file, current_organism_info)
}
\arguments{
\item{kegg_data_file}{file path. if null, inst/kegg_data_files will be used}

\item{current_organism_info}{information for the organism, including KEGG code}
}
\description{
get path for KEGG db for downloading/usage
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_degenes_Hunter.R
\name{main_degenes_Hunter}
\alias{main_degenes_Hunter}
\title{Main DEgenes Hunter Function}
\usage{
main_degenes_Hunter(
  raw = NULL,
  pseudocounts = FALSE,
  target = NULL,
  count_var_quantile = 0,
  external_DEA_data = NULL,
  output_files = getwd(),
  reads = 2,
  minlibraries = 2,
  filter_type = "separate",
  p_val_cutoff = 0.05,
  lfc = 1,
  modules = "DELNW",
  minpack_common = 4,
  model_variables = "",
  numerics_as_factors = TRUE,
  string_factors = NULL,
  numeric_factors = NULL,
  WGCNA_memory = 5000,
  WGCNA_norm_method = "DESeq2",
  WGCNA_deepsplit = 2,
  WGCNA_min_genes_cluster = 20,
  WGCNA_detectcutHeight = 0.995,
  WGCNA_mergecutHeight = 0.25,
  WGCNA_all = FALSE,
  WGCNA_blockwiseNetworkType = "signed",
  WGCNA_blockwiseTOMType = "signed",
  WGCNA_minCoreKME = 0.5,
  WGCNA_minCoreKMESize = NULL,
  WGCNA_minKMEtoStay = 0.2,
  WGCNA_corType = "pearson",
  multifactorial = "",
  library_sizes = NULL
)
}
\arguments{
\item{raw}{Dataframe with raw counts per sample and feature}

\item{pseudocounts}{boolean, activate if the input contains pseudocounts}

\item{target}{set with control and treatment information}

\item{count_var_quantile}{quantile variance threshold}

\item{external_DEA_data}{external DEA set}

\item{output_files}{otput files path}

\item{reads}{set of reads}

\item{minlibraries}{minimum of libraries to use a set of data}

\item{filter_type}{filter type to ba applied. Allowed: "separate" or "global"}

\item{p_val_cutoff}{p-value threshold}

\item{lfc}{minimum logFold Change}

\item{modules}{modules to be executed. Allowed: DESeq2 (D), NOISeq (N),
Limma (L), EdgeR (E), WGCNA (W)}

\item{minpack_common}{minimum of pack that must be significant to tag
a gene as significant}

\item{model_variables}{custom model}

\item{numerics_as_factors}{transform numeric values to factors. Default: TRUE}

\item{string_factors}{string factors for WGCNA}

\item{numeric_factors}{numeric factors for WGCNA}

\item{WGCNA_memory}{see WGCNA package}

\item{WGCNA_norm_method}{see WGCNA package}

\item{WGCNA_deepsplit}{see WGCNA package}

\item{WGCNA_min_genes_cluster}{see WGCNA package}

\item{WGCNA_detectcutHeight}{see WGCNA package}

\item{WGCNA_mergecutHeight}{see WGCNA package}

\item{WGCNA_all}{see WGCNA package}

\item{WGCNA_blockwiseNetworkType}{see WGCNA package}

\item{WGCNA_blockwiseTOMType}{see WGCNA package}

\item{WGCNA_minCoreKME}{see WGCNA package}

\item{WGCNA_minCoreKMESize}{see WGCNA package}

\item{WGCNA_minKMEtoStay}{see WGCNA package}

\item{WGCNA_corType}{see WGCNA package}

\item{multifactorial}{specify interaction/effect when multifactorial design}

\item{library_sizes}{NULL or a dataframe with sample names and library sizes}
}
\value{
expression analysis result object with studies performed
}
\description{
This function allows you to perform the DEG analysis with different
algorithms.
}
\examples{
data(toc)
data(target)
degh_out <- main_degenes_Hunter(raw=toc, target=target, modules="D")
}
\keyword{method}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{toc}
\alias{toc}
\title{Table of counts dummy dataset for DEgeneHunter examples}
\format{
A truncated data.frame of aligned read counts for genes (rows) and
samples (columns)
}
\usage{
data(toc)
}
\description{
Table of counts dummy dataset for DEgeneHunter examples
}
\examples{
data(toc)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_handling.R
\name{target_generation}
\alias{target_generation}
\title{Function to generate targets file
Load target file if it exists, otherwise use the -C and -T flags.
Note target takes precedence over target.}
\usage{
target_generation(from_file = NULL, ctrl_samples = NULL, treat_samples = NULL)
}
\arguments{
\item{from_file}{input targets file}

\item{ctrl_samples}{control samples}

\item{treat_samples}{target samples}
}
\value{
targets info loaded
}
\description{
Function to generate targets file
Load target file if it exists, otherwise use the -C and -T flags.
Note target takes precedence over target.
}
\examples{
# To generate target or load from file. 
# Example shows generation based on names of treament/control samples.
# These should match the column names in the table of counts.
case_sample_names <- paste("case", seq(5), sep="_")
control_sample_names <- paste("control", seq(5), sep="_")
target <- target_generation(ctrl_samples=control_sample_names, 
                            treat_samples=case_sample_names)
}
\keyword{design}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_report.R
\name{write_clusters_to_enrichment}
\alias{write_clusters_to_enrichment}
\title{Write Main clusters to enrichment output
This function allows you to report the Functional analysis.}
\usage{
write_clusters_to_enrichment(
  output_path = "results",
  output_file = "results",
  mode = "PR",
  enrichments_ORA = NULL,
  task_size = 1,
  workers = 1,
  template_folder = file.path(find.package("ExpHunterSuite"), "templates"),
  top_categories = NULL,
  group_results = FALSE,
  n_category = 30,
  sim_thr = 0.7,
  summary_common_name = "ancestor",
  pvalcutoff = 0.1,
  gene_attributes = NULL,
  gene_attribute_name = NULL,
  max_genes = 200,
  simplify = FALSE,
  clean_parentals = FALSE
)
}
\arguments{
\item{output_path}{output folder}

\item{output_file}{output file name for heatmaps}

\item{mode}{type of output to produce - P for plots, R for reports, and S for summarized heatmaps}

\item{enrichments_ORA}{list of enrich results for all clusters}

\item{task_size}{number of elements per packages used}

\item{workers}{(OPTIONAL) cores for parallel features}

\item{template_folder}{(OPTIONAL) RMD templates folder}

\item{top_categories}{numbers of categories from each cluster to use for merge}

\item{group_results}{experimental - whether to group results in the emap plot}

\item{n_category}{number of categories in the figures (per cluster)}

\item{sim_thr}{value to use when combining similar categories in summary mode}

\item{summary_common_name}{'significant' to use the most significant term to label each summarized group
'ancestor' to use the common ancestor of the group"}

\item{pvalcutoff}{used to select terms for summarizing}

\item{gene_attributes}{named list of attributes e.g. FCs for emap plot coloured nodes (genes)}

\item{gene_attribute_name}{name for the legend in the emap plot for the nodes (genes)}

\item{max_genes}{maximum number of genes to plot in cnet plot}

\item{simplify}{Activate for process ClusterProfiler results with simplify function}

\item{clean_parentals}{Activate to reduce significant terms in GO by removin the parentals}
}
\value{
void
}
\description{
Write Main clusters to enrichment output
This function allows you to report the Functional analysis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{degh_output}
\alias{degh_output}
\title{Output data from the main_degenes_Hunter function}
\format{
An output object from the main_degenes_Hunter function.
}
\usage{
data(degh_output)
}
\description{
Output data from the main_degenes_Hunter function
}
\examples{
data(degh_output)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/general_functions.R
\name{get_unexported_function}
\alias{get_unexported_function}
\title{Method used to import hidden functions legally for CRAN check}
\usage{
get_unexported_function(pkg, fun)
}
\arguments{
\item{pkg}{package}

\item{fun}{function to be luaded}
}
\value{
function requried
}
\description{
Method used to import hidden functions legally for CRAN check
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_handling.R
\name{load_synth_dataset}
\alias{load_synth_dataset}
\title{Loads Hunter DEG analysis table and simulated TRUE DEG vector. Concat in
training/testing ML data frame format and returns it.}
\usage{
load_synth_dataset(
  folder = NULL,
  hunter_table_file = "hunter_results_table.txt",
  prediction_file_regex = "_predv"
)
}
\arguments{
\item{folder}{where files are placed}

\item{hunter_table_file}{DEG analys result table file name}

\item{prediction_file_regex}{regex to find TRUE DEG file generated}
}
\value{
a data frame with DEG result and real predictions column
}
\description{
Loads Hunter DEG analysis table and simulated TRUE DEG vector. Concat in
training/testing ML data frame format and returns it.
}
\examples{
# Used to load a synth + DE analysis result folder
ht <- load_synth_dataset() 
# Will return an empty object, you need a correct folder
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_degsynth.R
\name{degsynth}
\alias{degsynth}
\title{Main function to generate synthetic data using an specific exponential
distribution for logFC}
\usage{
degsynth(
  outfile = NULL,
  inputfile = NULL,
  replicates = 3,
  ngenes = 20000,
  DEGs_proportion = 0.2,
  FC_min = 1.3,
  FC_max = 3,
  P_up = 1,
  group = NULL
)
}
\arguments{
\item{outfile}{output file}

\item{inputfile}{input file}

\item{replicates}{number of replicates}

\item{ngenes}{number of genes}

\item{DEGs_proportion}{numeric (0,1) proportion of DEG genes to be simulated}

\item{FC_min}{minimum Fold-Change}

\item{FC_max}{maximum Fold-Change}

\item{P_up}{proportion og up-regulated genes}

\item{group}{optional group identifiers seprated by commas}
}
\value{
synthetic data generated
}
\description{
Main function to generate synthetic data using an specific exponential
distribution for logFC
}
\examples{
synthetic_dataset <- degsynth()
# Returns simulated count dataset and indication of which are DE/not DE
}
\keyword{synthetic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_clusters_to_enrichment.R
\name{main_clusters_to_enrichment}
\alias{main_clusters_to_enrichment}
\title{Perform enrichment analysis on multiple lists of genes}
\usage{
main_clusters_to_enrichment(
  input_file = NULL,
  gene_attribute_file = NULL,
  org_db = NULL,
  gene_keytype = "ENTREZID",
  temp_file = "enr_tmp.RData",
  force = FALSE,
  all_funsys = c("BP", "MF", "CC"),
  task_size = 1,
  current_organism_info = NULL,
  workers = 1,
  pvalcutoff = 0.1,
  qvalcutoff = 0.2,
  all_custom_gmt = NULL,
  kegg_data_file = NULL,
  universe = NULL
)
}
\arguments{
\item{input_file}{input file path. File must contain 1 cluster per line clusterA ..tab.. gene1,gene2}

\item{gene_attribute_file}{file to map genes to attributes e.g. fold change}

\item{org_db}{org_db object}

\item{gene_keytype}{ID used in the input file}

\item{temp_file}{enrichments will be read from this file if it exists, written to it if it doesnt}

\item{force}{if true, to run enrichments even if the temp_file exists}

\item{all_funsys}{list of GOs/dbs to use for enrichment analysis}

\item{task_size}{number of elements per packages used}

\item{current_organism_info}{organism info from text table,}

\item{workers}{cores to be used if parallel features are going to be used}

\item{pvalcutoff}{adjusted p-value threshold to use for clusterProfiler,}

\item{qvalcutoff}{q-value threshold to use for clusterProfiler,}

\item{all_custom_gmt}{list of custom annotation sets}

\item{kegg_data_file}{if KEGG included in enrich_dbs path needs}

\item{universe}{background of genes to use as universe}
}
\description{
Perform enrichment analysis on multiple lists of genes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_handling.R
\name{write_df_list_as_tables}
\alias{write_df_list_as_tables}
\title{Function to write expression package results.
Write to disk the results of each diff expression package}
\usage{
write_df_list_as_tables(df_list, prefix, root = getwd())
}
\arguments{
\item{df_list}{dataframes list}

\item{prefix}{prefix concatenated to output file name}

\item{root}{path where write}
}
\value{
void
}
\description{
Function to write expression package results.
Write to disk the results of each diff expression package
}
\examples{
# Typically used with the @all_counts_for_plotting slot of the main output
deg_res <- list(aa=data.frame(a=seq(5),b=seq(5)), 
                bb=data.frame(a=seq(6,10),b=seq(6,10)))
write_df_list_as_tables(deg_res, prefix="test", root=tempdir())
}
\keyword{output}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_functions.R
\name{ht2logFCPlot}
\alias{ht2logFCPlot}
\title{Function to generate scatter plot for each gene in a hunter table
and show logFC per package
taking into account the variablity between them}
\usage{
ht2logFCPlot(
  ht,
  var_filter = 0.001,
  title = "Filtered logFC",
  top = 50,
  alpha = 0.5
)
}
\arguments{
\item{ht}{: hunter table dataframe}

\item{var_filter}{: variability threshold to show gene into this
graph (low variability will be removed)}

\item{title}{: plot title}

\item{top}{: plots only the top N items with more variance.
NULL will not filter}

\item{alpha}{: transparency of dots}
}
\value{
plot ready to be rendered
}
\description{
Function to generate scatter plot for each gene in a hunter table
and show logFC per package
taking into account the variablity between them
}
\author{
Fernando Moreno Jabato <jabato(at)uma(dot)com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_analysis_library.R
\name{multienricher_gsea}
\alias{multienricher_gsea}
\title{Perform gsea enrichment analysis of a list of genes}
\usage{
multienricher_gsea(
  all_funsys = NULL,
  genes_list,
  organism_info,
  org_db = NULL,
  task_size = 1,
  workers = 1,
  pvalueCutoff = 0.05,
  pAdjustMethod = "BH",
  kegg_file = NULL,
  custom_sets = NULL,
  ...
)
}
\arguments{
\item{all_funsys}{vector of funsys to use (e.g. MF, Reatcome)}

\item{genes_list}{vector of genes or list of vectors, to be enriched}

\item{organism_info}{from the annotation table: infor on db names etc}

\item{org_db}{org db file (optional) - normally obtained from organism_info}

\item{task_size}{for parallelization}

\item{workers}{for parallelization}

\item{pvalueCutoff}{p-value finding enriched categories}

\item{pAdjustMethod}{adjustment method for p-values}

\item{kegg_file}{must be provided if kegg annotation required}

\item{custom_sets}{custom set object (processed by load_and_parse_gmt)}

\item{...}{other arguments passed to the enrichment function}
}
\description{
Perform gsea enrichment analysis of a list of genes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_report.R
\name{write_expression_report}
\alias{write_expression_report}
\title{Write Main DEgenes Hunter report}
\usage{
write_expression_report(
  exp_results,
  output_files = getwd(),
  template_folder = NULL,
  opt = NULL
)
}
\arguments{
\item{exp_results}{DEG analysis results object}

\item{output_files}{output file path}

\item{template_folder}{RMD templates folder}

\item{opt}{option list}
}
\value{
void
}
\description{
This function allows you to report the DEG analysis.
}
\examples{
# Load DE analysis results
degh_output <- list() # data(degh_output)
write_expression_report(degh_output)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_analysis_library.R
\name{load_and_parse_gmt}
\alias{load_and_parse_gmt}
\title{Load a GMT format file and return a dataframe in correct format}
\usage{
load_and_parse_gmt(gmt_file)
}
\arguments{
\item{gmt_file}{file to be loaded}
}
\value{
GMT loaded info
}
\description{
Load a GMT format file and return a dataframe in correct format
}
\examples{
gmt_file <- system.file("extData", 
"toy_categories_1.gmt", package = "ExpHunterSuite")
load_and_parse_gmt(gmt_file)
}
\keyword{file}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_degsynth.R
\name{scale_range}
\alias{scale_range}
\title{Funtion used to scale a vector of numeric values from
(x_min,x_max) range to
(y_min,y_max) range.
f : R -> R
(x_min, x_max) -> (y_min, y_max)
vect - x_min
f(vect,y_min,y_max) =  ------------- x (y_max - y_min) + y_min
x_max - x_min}
\usage{
scale_range(vect, nmin, nmax)
}
\arguments{
\item{vect}{vector to be transformated}

\item{nmin}{new minimum}

\item{nmax}{new maximum}
}
\value{
transformated vector to new range
}
\description{
Funtion used to scale a vector of numeric values from
(x_min,x_max) range to
(y_min,y_max) range.
f : R -> R
(x_min, x_max) -> (y_min, y_max)
vect - x_min
f(vect,y_min,y_max) =  ------------- x (y_max - y_min) + y_min
x_max - x_min
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_functional_hunter.R
\name{main_functional_hunter}
\alias{main_functional_hunter}
\title{Perform Functional analysis with different
enrichment corpora.}
\usage{
main_functional_hunter(
  hunter_results,
  model_organism,
  annot_table = NULL,
  organisms_table = get_organism_table(),
  input_gene_id = "ENSEMBL",
  custom = NULL,
  enrich_dbs = c("MF", "Reactome"),
  kegg_data_file = "",
  enrich_methods = "ORA",
  annotation_source = "orgdb",
  pthreshold = 0.1,
  qthreshold = 0.2,
  cores = 1,
  task_size = 1,
  output_files = "results",
  fc_colname = "mean_logFCs",
  universe = NULL,
  clean_parentals = FALSE,
  simplify = FALSE,
  top_categories = 50,
  sim_thr = NULL,
  summary_common_name = "ancestor",
  clusters_flag = FALSE
)
}
\arguments{
\item{hunter_results}{output of main_degenes_hunter - list with DEG analysis results}

\item{model_organism}{organisms which genes are being studied}

\item{annot_table}{(OPTIONAL) annotation table to translate gene IDs}

\item{organisms_table}{(OPTIONAL) configuration table for given organism.
Use see get_organism_table()}

\item{input_gene_id}{(OPTIONAL) type og gene IDs given. Allowed: ENSEMBL (E)
, entrezgene (e), TAIR/Arabidopsis (T), Gene Names (G).}

\item{custom}{(OPTIONAL) list of dataframes with GMT corpus}

\item{enrich_dbs}{annotation dbs to be used: BP,MF,CC,Reactome,KEGG
\itemize{
\item MF and Reactome default
}}

\item{kegg_data_file}{if KEGG included in enrich_dbs path needs}

\item{enrich_methods}{enrichment methods to be used: ORA or GSEA}

\item{annotation_source}{annote with orgdbs or Biomart? Currently on orgdbs}

\item{pthreshold}{pvalue threshold}

\item{qthreshold}{qvalue threshold}

\item{cores}{cores to be used if parallel features are going to be used.
Default: 1}

\item{task_size}{number of elements per packages used}

\item{output_files}{output folder}

\item{fc_colname}{main logFC colname (into hunter_results dataframe)}

\item{universe}{whether to use all genes as the background, or expressed only}

\item{clean_parentals}{clean parental terms in merged enrichment}

\item{simplify}{simplify the merged enrichments}

\item{top_categories}{numbers of categories from each cluster to use for merge}

\item{sim_thr}{value to use when combining similar categories in summary mode}

\item{summary_common_name}{'significant' to use the most significant term to label each summarized group}

\item{clusters_flag}{execute clusters enrichments}
}
\value{
functional result object with enrichments performed
}
\description{
Perform Functional analysis with different
enrichment corpora.
}
\examples{
# Load DE analysis result
data(degh_output)
func_results <- main_functional_hunter(hunter_results = degh_output,
model_organism = "Mouse", enrich_dbs = "MF")
}
\keyword{method}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_degsynth.R
\name{fcfunc}
\alias{fcfunc}
\title{Custom function to generate exponentially degradated foldchange range}
\usage{
fcfunc(means, fcmin = 1.4, fcmax = 3, meanlog = 1, sdlog = 0.8)
}
\arguments{
\item{means}{means vector}

\item{fcmin}{minimum foldchange}

\item{fcmax}{maximum foldchange}

\item{meanlog}{param of rlnorm function}

\item{sdlog}{param of rlnorm function}
}
\value{
a vector of foldchanges to be applied
}
\description{
Custom function to generate exponentially degradated foldchange range
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_treatment_control.R
\name{STC}
\alias{STC}
\title{Function to Simulate Treatment-Control paired experiments for genes
expression count tables. Pipeline inspired on TCC simulateReadCounts function}
\usage{
STC(
  Ngene = 1000,
  PDEG = 0.2,
  DEG.foldchange = 2,
  replicates = 3,
  bcount = NULL,
  group = NULL
)
}
\arguments{
\item{Ngene}{number of genes to be included in the experiment}

\item{PDEG}{Percentage of Differentially Expressed Genes. If one number
is specified, only Up-regulated genes will be simulated. If a vector of two
number is given, proportions will be applied for Up and Down regualted
simulation respectively}

\item{DEG.foldchange}{foldchange to be applied. Can be a number or a
function which must accept only a vector and return a vector of the same
length with the fold-change values. Warning: NOT LOG FOLD CHANGE}

\item{replicates}{to be ge generated of each group (Treatment and Control)}

\item{bcount}{matrix of genes counts used to simulate. If NULL,
toc dataset will be loaded}

\item{group}{columns to be used as group replicates. If NULL,
all columns will be used.}
}
\value{
a list with COUNT) a simulated gene counts matrix with simulated
genes and sample names and TRUEDEG) a vector with real up/down regulated
genes
}
\description{
Function to Simulate Treatment-Control paired experiments for genes
expression count tables. Pipeline inspired on TCC simulateReadCounts function
}
\author{
Fernando Moreno Jabato <jabato(at)uma(dot)es>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_handling.R
\name{load_WGCNA_results}
\alias{load_WGCNA_results}
\title{Loads stored WGCNA results in DEgenes Hunter expression analysis package
folder and returns in compact object}
\usage{
load_WGCNA_results(path, main_deg_table)
}
\arguments{
\item{path}{of expression analysis results (Hunter folder)}

\item{main_deg_table}{dataframe with main DEG analysis results}
}
\value{
WGCNA results loaded
}
\description{
Loads stored WGCNA results in DEgenes Hunter expression analysis package
folder and returns in compact object
}
\keyword{input}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_analysis_library.R
\name{get_org_db}
\alias{get_org_db}
\title{Obtain an org_db object from a current_organism_info, obtained from
subsetting the results of get_organism_table()}
\usage{
get_org_db(current_organism_info)
}
\arguments{
\item{current_organism_info}{row from output of get_organism_table()}
}
\value{
org_db
}
\description{
Obtain an org_db object from a current_organism_info, obtained from
subsetting the results of get_organism_table()
}
\keyword{org_db}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_analysis_library.R
\name{translate_from_table}
\alias{translate_from_table}
\title{Translates a given gene ID using a dictionary. Note: one unknown ID can
corresponds to many known ids.}
\usage{
translate_from_table(ids_to_translate, annot_table)
}
\arguments{
\item{ids_to_translate}{set of IDs to be translated}

\item{annot_table}{dictionary to translate IDs}
}
\value{
translated IDs or NA if it's not possible to translate
}
\description{
Translates a given gene ID using a dictionary. Note: one unknown ID can
corresponds to many known ids.
}
\keyword{translate}
